@@ -76,6 +76,9 @@ decide whether to connect immediatly or not. *)
     mutable op_client_dprint_html : ('a -> CommonTypes.ui_conn ->
     CommonTypes.file -> string -> bool);
 
+(* used to print network specific client infos *)
+    mutable op_client_print_info : ('a -> CommonTypes.ui_conn -> unit);
+
     mutable op_client_debug : ('a -> bool -> unit);
 
     mutable op_client_can_upload : ('a -> int -> unit);
@@ -159,6 +162,10 @@ let client_dprint_html (client: client) o file str =
   let client = as_client_impl client in
   client.impl_client_ops.op_client_dprint_html client.impl_client_val o file str
 
+let client_print_info (client: client) o =
+  let c = as_client_impl client in
+  c.impl_client_ops.op_client_print_info c.impl_client_val o
+
 let client_connect client=
   let client = as_client_impl client in
   client.impl_client_ops.op_client_connect client.impl_client_val
@@ -188,7 +195,7 @@ let ni n m =
   s
 
 let fni n m =   failwith (ni n m)
-  let ni_ok n m = ignore (ni n m)
+let ni_ok n m = ignore (ni n m)
 
 let clients_ops = ref []
 
@@ -207,6 +214,7 @@ let new_client_ops network =
       op_client_bprint = (fun _ _ -> ni_ok network "client_bprint");
       op_client_dprint = (fun _ _ _ -> ni_ok network "client_dprint");
       op_client_dprint_html = (fun _ _ _ _ -> fni network "client_dprint_html");
+      op_client_print_info = (fun _ _ -> fni network "client_print_info");
       op_client_can_upload = (fun _ _ -> ni_ok network "client_can_upload");
       op_client_enter_upload_queue = (fun _ -> ni_ok network "client_enter_upload_queue");
     } in
@@ -443,7 +451,7 @@ let client_print c o =
 	("", "sr", Printf.sprintf "%d" (client_num c));
 	("", "sr", n.network_name);
 	("", "sr", ips);
-	] @ (if !Geoip.active then [(cn, "sr", cc)] else []) @ [
+	] @ (if !Geoip.active then [(cn, "sr", CommonPictures.flag_html cc)] else []) @ [
 	(String.escaped info.G.client_name, "sr", client_short_name info.G.client_name); ]);
     end
   else begin
@@ -467,16 +475,15 @@ let clear_upload_slots () =
     try
       let i = client_info c in
       let ctime = ((BasicSocket.last_time ()) - i.GuiTypes.client_connect_time) / 60 in
-      if i.GuiTypes.client_uploaded = Int64.zero && ctime > 1 then
+      if i.GuiTypes.client_session_uploaded = Int64.zero && ctime > 1 then
         begin
 	  client_disconnect c;
-          if !verbose then lprintf_nl "disconnected client %d: [%s %s] %s after %d %s of silence."
+          if !verbose then lprintf_nl "disconnected client %d: [%s %s] %s after %d minute%s of silence."
 	    (client_num c)
 	    (GuiTypes.client_software i.GuiTypes.client_software i.GuiTypes.client_os)
 	    i.GuiTypes.client_release
 	    i.GuiTypes.client_name
-	    ctime
-	    (if ctime = 1 then "minute" else "minutes")
+	    ctime (Printf2.print_plural_s ctime)
 	end
     with _ -> ()
   ) !uploaders
@@ -491,7 +498,6 @@ let impl_client_info impl =
      T.client_tags = [];
      T.client_name = "";
      T.client_network = 0;
-     T.client_files = None;
      T.client_rating = 0;
      T.client_chat_port = 0;
      T.client_connect_time = BasicSocket.last_time ();
@@ -500,8 +506,11 @@ let impl_client_info impl =
      T.client_os = None;
      T.client_release = "";
      T.client_emulemod = "";
-     T.client_downloaded = 0L;
-     T.client_uploaded = 0L;
+     T.client_total_downloaded = 0L;
+     T.client_total_uploaded = 0L;
+     T.client_session_downloaded = 0L;
+     T.client_session_uploaded = 0L;
      T.client_upload = None;
      T.client_sui_verified = None;
+     T.client_file_queue = [];
   }
