@@ -27,8 +27,13 @@ open CommonClient
 open CommonServer
 open CommonNetwork
 open CommonOptions
+open CommonUserDb
 open CommonTypes
 open CommonFile
+open Gettext
+
+let _s x = _s "CommonComplexOptions" x
+let _b x = _b "CommonComplexOptions" x
 
 let log_prefix = "[cCO]"
 
@@ -64,16 +69,17 @@ module FileOption = struct
       match v with
         Options.Module assocs ->
           let get_value name conv = conv (List.assoc name assocs) in
+	  let get_value_nil name conv =
+	    try conv (List.assoc name assocs) with Not_found -> [] in
+          let filename = get_value "file_filename" value_to_string in
           let network = try get_value "file_network" value_to_string
             with _ -> "Donkey" in
           let network = 
             try network_find_by_name network with e ->
                 lprintf_nl
 		  "Error %s for network %s while parsing file %s"
-		    (Printexc2.to_string e)
-		    network
-		    (get_value "file_filename" value_to_string);
-    lprintf_nl "This core is lacking support for network %s, exiting" network;
+		    (Printexc2.to_string e) network filename;
+                lprintf_nl "This core is lacking support for network %s, exiting" network;
                 exit_properly 70
           in
           let file_state = 
@@ -96,20 +102,74 @@ module FileOption = struct
               impl.impl_file_age <- 
                 normalize_time (get_value "file_age" value_to_int)
             with _ -> ());
+
+          (try
+              impl.impl_file_release <- 
+                get_value "file_release" value_to_bool
+            with _ -> ());
+
+	  let file_user =
+	    try
+	      let u = get_value "file_owner" value_to_string in
+		begin
+		  try
+		    user2_user_find u
+		  with Not_found ->
+		    lprintf_nl "file_owner %s of %s does not exist, changing to %s"
+		      u filename admin_user.user_name;
+		    admin_user
+		end
+	    with Not_found ->
+		lprintf_nl "file_owner of %s is empty, changing to %s"
+		  filename admin_user.user_name;
+		admin_user
+	  in
+	  set_file_owner file file_user;
+
+	  let file_group =
+	    let dgroup = user2_print_user_default_group file_user in
+	    try
+	      match (get_value "file_group" stringvalue_to_option) with
+		None -> None
+	      | Some g ->
+		  begin
+		    try
+		      let g = user2_group_find g in
+		      if List.mem g file_user.user_groups then
+		        Some g
+		      else
+			begin
+			  lprintf_nl "file_owner %s is not member of file_group %s, changing file_group of %s to user_default_group %s"
+			    file_user.user_name g.group_name filename dgroup;
+			  file_user.user_default_group
+			end
+		    with Not_found ->
+		      lprintf_nl "file_group %s of %s not found, changing to user_default_group %s of user %s"
+			g filename dgroup file_user.user_name;
+		      file_user.user_default_group
+		  end
+	    with Not_found ->
+		lprintf_nl "file_group of %s is empty, changing to user_default_group %s of user %s"
+		  filename dgroup file_user.user_name;
+		file_user.user_default_group
+	  in
+	  set_file_group file file_group;
+
           set_file_state file file_state;       
 
           (try
-              set_file_best_name file
-              (get_value "file_filename" value_to_string)
+              set_file_best_name file filename "" 0
             with _ -> ());
+
+          (try
+	    List.iter (fun s -> add_file_filenames file s)
+	      (get_value_nil "file_filenames" (value_to_list value_to_string))
+            with _ -> ());
+
           set_file_priority file priority;
 
-          if !verbose then lprintf_nl "New %s file %s"
-	      (match file_state with
-                 FileDownloading -> "downloading"
-               | FileDownloaded -> "downloaded"
-               | _ -> "other")
-	     (file_best_name file);
+          if !verbose && !CommonGlobals.is_startup_phase then
+            lprintf_nl "Started download of %s" (file_best_name file);
 
           file
       | _ -> assert false
@@ -123,7 +183,12 @@ module FileOption = struct
         ("file_priority", int_to_value (file_priority file)) ::
         ("file_state", state_to_value (file_state file)) ::
         ("file_filename", string_to_value (file_best_name file)) ::
+	("file_filenames", List
+	(List.map string_to_value impl.impl_file_filenames)) ::
         ("file_age", IntValue (Int64.of_int impl.impl_file_age)) ::
+        ("file_release", bool_to_value impl.impl_file_release) ::
+        ("file_owner", string_to_value (file_owner file).user_name) ::
+        ("file_group", option_to_stringvalue (match file_group file with Some g -> Some g.group_name | None -> None)) ::
           (file_to_option file)
         )
           
@@ -266,7 +331,7 @@ module ServerOption = struct
           in
           let network = 
             try network_find_by_name network with e ->
-                lprintf_nl "Network %s not supported" network;
+                lprintf_nl "Loading servers, network %s not supported, deleting server" network;
                 raise e
               in
           let server = network_server_of_option network assocs in
@@ -498,7 +563,7 @@ let special_queries = define_option searches_section
     "-movies", "avi -minsize 650000000 -1cd";
     "-mp3s", "mp3 -minsize 3000000 -maxsize 10000000";
     "-albums", "album -minsize 30000000 -maxsize 150000000";
-    "-nosex", "-without xxx";
+    "-nosex", "-not xxx";
   ]
   
 let customized_queries =
@@ -650,7 +715,7 @@ let sharing_only_directory = {
     sharing_directories = false;
     sharing_extensions = [];
     sharing_recursive = false;
-    sharing_minsize = Int64.one;
+    sharing_minsize = zero;
     sharing_maxsize = Int64.max_int;
   }  
   
@@ -659,7 +724,7 @@ let sharing_incoming_directories = {
     sharing_directories = true;
     sharing_extensions = [];
     sharing_recursive = false;
-    sharing_minsize = Int64.one;
+    sharing_minsize = zero;
     sharing_maxsize = Int64.max_int;
   }  
   
@@ -667,8 +732,8 @@ let sharing_incoming_files = {
     sharing_incoming = true;
     sharing_directories = false;
     sharing_extensions = [];
-    sharing_recursive = false;
-    sharing_minsize = Int64.one;
+    sharing_recursive = true;
+    sharing_minsize = zero;
     sharing_maxsize = Int64.max_int;
   }
 
@@ -677,7 +742,7 @@ let sharing_directories = {
       sharing_directories = true;
       sharing_extensions = [];
       sharing_recursive = false;
-      sharing_minsize = Int64.one;
+      sharing_minsize = zero;
       sharing_maxsize = Int64.max_int;
     }
   
@@ -712,7 +777,7 @@ let sharing_strategies = define_option searches_section
       sharing_directories = false;
       sharing_extensions = [];
       sharing_recursive = true;
-      sharing_minsize = Int64.one;
+      sharing_minsize = zero;
       sharing_maxsize = Int64.max_int;
     };
     
@@ -744,7 +809,7 @@ let _ =
       
   )
 
-let sharing_strategies name =
+let sharing_strategy name =
   match name with
   | "incoming_files" -> sharing_incoming_files
   | "incoming_directories" -> sharing_incoming_directories
@@ -776,16 +841,18 @@ module SharedDirectoryOption = struct
             in
             let shdir_priority = get_value_safe "priority" value_to_int 0
             in
+(*
             let shdir_networks = get_value_safe "networks"
                 (value_to_list value_to_string) [] 
             in
+*)
             let shdir_strategy = get_value_safe "strategy"
                 value_to_string "only_directory"
             in
             {
               shdir_dirname = shdir_dirname; 
               shdir_strategy = shdir_strategy; 
-              shdir_networks = shdir_networks; 
+              shdir_networks = []; (* shdir_networks; *)
               shdir_priority = shdir_priority; 
             }
           end
@@ -811,8 +878,10 @@ module SharedDirectoryOption = struct
     let shared_directory_to_value s =
       let list = [
           "dirname", filename_to_value s.shdir_dirname;
+(*
           "networks", 
           list_to_value string_to_value s.shdir_networks;
+*)
           "strategy", 
           string_to_value  s.shdir_strategy;
           "priority", int_to_value s.shdir_priority;
@@ -825,8 +894,21 @@ module SharedDirectoryOption = struct
         shared_directory_to_value
 
   end
-    
-  
+
+let default_incoming_files = {
+  shdir_dirname = Filename.concat "incoming" "files";
+  shdir_priority = 0;
+  shdir_networks = [];
+  shdir_strategy = "incoming_files";
+  }
+
+let default_incoming_directories = {
+  shdir_dirname = Filename.concat "incoming" "directories";
+  shdir_priority = 0;
+  shdir_networks = [];
+  shdir_strategy = "incoming_directories";
+  }
+
 let shared_directories = 
   define_option CommonOptions.path_section ["shared_directories" ] 
   "                Incoming and shared directories.
@@ -838,8 +920,8 @@ let shared_directories =
    Finished BT multifile downloads are committed to the first directory
    with strategy incoming_directories. Other downloads are committed
    to the first directory with the strategy incoming_files.
-   If more than one directory has one of the incoming_* strategies
-   it will be ignored on commit, but they are shared nonetheless.
+   MLdonkey searches all shared_directories with incoming_* strategies
+   on commit and uses the first one with enough free diskspace.
    Other strategies can be found in searches.ini, section customized_sharing."
     (list_option SharedDirectoryOption.t) 
   [
@@ -849,64 +931,91 @@ let shared_directories =
       shdir_networks = [];
       shdir_strategy = "all_files";
     };
-    {
-      shdir_dirname = "incoming/files";
-      shdir_priority = 0;
-      shdir_networks = [];
-      shdir_strategy = "incoming_files";
-    };
-    {
-      shdir_dirname = "incoming/directories";
-      shdir_priority = 0;
-      shdir_networks = [];
-      shdir_strategy = "incoming_directories";
-    }
+    default_incoming_files;
+    default_incoming_directories;
   ]
 
 
 let search_incoming_files () =
-  try
-    List.find (fun s -> s.shdir_strategy = "incoming_files") 
-    !!shared_directories
-  with Not_found ->
-        let dirname = Filename.concat "incoming" "files" in
-        let s = {
-        shdir_dirname = dirname;
-        shdir_priority = 0;
-        shdir_networks = [];
-        shdir_strategy = "incoming_files";
-          }
-        in
-        shared_directories =:= s :: !!shared_directories;
-        s
-
-let incoming_files () =
-  let dir = search_incoming_files () in
-    Unix2.safe_mkdir dir.shdir_dirname;
-    Unix2.can_write_to_directory dir.shdir_dirname;
-    dir
+  let list =
+    List.filter (fun s -> s.shdir_strategy = "incoming_files") !!shared_directories
+  in
+  match list with
+  | [] -> shared_directories =:= default_incoming_files :: !!shared_directories;
+        [default_incoming_files]
+  | l -> l
 
 let search_incoming_directories () =
-  try
-    List.find (fun s -> s.shdir_strategy = "incoming_directories") 
-    !!shared_directories
-  with Not_found ->
-      let dirname = Filename.concat "incoming" "directories" in
-      let s = {
-          shdir_dirname = dirname;
-          shdir_priority = 0;
-          shdir_networks = [];
-          shdir_strategy = "incoming_directories";
-        }
-      in
-      shared_directories =:= s :: !!shared_directories;
-      s
+  let list =
+    List.filter (fun s -> s.shdir_strategy = "incoming_directories") !!shared_directories
+  in
+  match list with
+  | [] -> shared_directories =:= default_incoming_directories :: !!shared_directories;
+        [default_incoming_directories]
+  | l -> l
+
+let incoming_dir usedir ?user ?needed_space ?network () =
+
+  let directories =
+    if usedir then
+      search_incoming_directories ()
+    else
+      search_incoming_files ()
+  in
+
+  let dirname_user =
+    match user with
+    | None -> ""
+    | Some user -> user.user_commit_dir
+  in
+
+(*
+  let dirname_network =
+    match network with
+    | None -> ""
+    | Some network -> network
+  in
+*)
+(* todo: make the dir naming order user configurable *)
+  let compute_dir_name dir =
+    let dirname = Filename2.normalize (Filename.concat dir dirname_user) in
+(*    let dirname = Filename.concat dirname dirname_network in *)
+    dirname
+  in
+
+  let checkdir =
+    let module U = Unix.LargeFile in
+    try
+      List.find (fun d ->
+	let dirname = compute_dir_name d.shdir_dirname in
+(* check if temp_directory and incoming are on different partitions *)
+	try
+          if (U.stat dirname).U.st_dev <> (U.stat !!temp_directory).U.st_dev then
+            begin
+              match needed_space with
+              | None -> true
+              | Some needed_space ->
+                  match Unix32.diskfree dirname with
+                    Some v -> v >= needed_space
+	          | _ -> true
+            end
+          else true
+	with _ -> true
+        ) directories
+    with Not_found -> raise Incoming_full;
+  in
+
+  let newdir = {
+      shdir_dirname = (compute_dir_name checkdir.shdir_dirname);
+      shdir_priority = checkdir.shdir_priority;
+      shdir_networks = checkdir.shdir_networks;
+      shdir_strategy = checkdir.shdir_strategy;
+    }
+  in
+  Unix2.safe_mkdir newdir.shdir_dirname;
+  Unix2.can_write_to_directory newdir.shdir_dirname;
+  newdir
 
-let incoming_directories () =
-  let dir = search_incoming_directories () in
-    Unix2.safe_mkdir dir.shdir_dirname;
-    Unix2.can_write_to_directory dir.shdir_dirname;
-    dir
 
 let _ =
 (* Check the definition of the incoming_files and incoming_directories in
@@ -915,8 +1024,8 @@ shared_directories *)
   option_hook shared_directories (fun _ ->
       if not !verification then begin
           verification := true;
-          ignore (incoming_files ());
-          ignore (incoming_directories ());
+          ignore (incoming_dir false ());
+          ignore (incoming_dir true ());
           verification := false
         end
   )
@@ -930,6 +1039,7 @@ shared_directories *)
 
 let load () =
   Options.load files_ini;
+  shorten_all_file_filenames !!max_filenames;
   Options.load servers_ini;
   Options.load searches_ini;
   Options.load results_ini;
@@ -965,13 +1075,13 @@ let save () =
         Options.save_with_help results_ini;
         results =:= [];
     end;
-  lprintf_nl "Options correctly saved"
+  lprintf_nl (_b "Options correctly saved")
   end
 
 let save_sources () =
   if !allow_saving_ini_files then begin
   networks_iter (fun n -> network_save_sources n);
-  lprintf_nl "Sources correctly saved"
+  lprintf_nl (_b "Sources correctly saved")
   end
 
 open Zip
@@ -981,11 +1091,13 @@ let backup_zip archive files =
     Unix2.tryopen_write_zip archive (fun oc ->
       List.iter (fun file ->
 	try
-          let s = Unix.stat file in
-	  Zip.copy_file_to_entry file oc ~level:9 ~mtime:s.Unix.st_mtime file
+          let module U = Unix.LargeFile in
+          let s = U.stat file in
+	  Zip.copy_file_to_entry file oc ~level:9 ~mtime:s.U.st_mtime file
 	with e ->
 	  failwith (Printf.sprintf "Zip: error %s in %s" (Printexc2.to_string e) file)
-      ) files)
+      ) files);
+    (try Unix.chmod archive 0o600 with _ -> ())
   with e ->
     failwith (Printf.sprintf "Zip: error %s in %s" (Printexc2.to_string e) archive)
 
@@ -1004,10 +1116,10 @@ let backup_tar archive files =
 	      failwith (Printf.sprintf "Tar: file %s too big, limit %d byte" arg Sys.max_string_length);
 	    let header = 
 	      { Tar.t_name = arg;
-	      t_mode = 0o644;
+	      t_mode = stat.Unix.st_perm;
 	      t_uid = stat.Unix.st_uid;
 	      t_gid = stat.Unix.st_gid;
-	      t_size = 0;
+	      t_size = stat.Unix.st_size;
 	      t_mtime = Int32.of_float stat.Unix.st_mtime;
 	      t_chksum = 0;
 	      t_typeflag = REGULAR;
@@ -1023,20 +1135,17 @@ let backup_tar archive files =
 	    Pervasives.really_input ic s 0 size;
 	    header, s) in
 	Tar.output otar header s
-      with e ->
-	let error = Printexc2.to_string e in
-	if error = "Gzip.Error(\"error during compression\")"
-	  && Autoconf.windows && arg = "fasttrack.ini" then begin
-	    (* for whatever reason this error is raised on Windows,
-               but fasttrack.ini is stored correctly *)
-	    if !verbose then
-        lprintf_nl "Tar: Windows specific pseudo error %s in %s" error arg
-	  end
-	else begin
+      with
+      | (Gzip.Error "error during compression") as e when Autoconf.windows && arg = "fasttrack.ini" ->
+	  (* for whatever reason this error is raised on Windows,
+             but fasttrack.ini is stored correctly *)
+	  if !verbose then
+            lprintf_nl "Tar: Windows specific pseudo error %s in %s" (Printexc2.to_string e) arg
+      | e ->
 	  failed_files := arg :: !failed_files;
-    lprintf_nl "Tar: error %s in %s" error arg
-        end
+	  lprintf_nl "Tar: error %s in %s" (Printexc2.to_string e) arg
     ) files);
+    (try Unix.chmod archive 0o600 with _ -> ());
     if !failed_files <> [] then
       failwith (Printf.sprintf "Tar: error backing up %s"
 	(String.concat " " (List.rev !failed_files)))
@@ -1077,7 +1186,7 @@ let backup_options () =
       end
     with e -> lprintf_nl "Exception %s while options backup" (Printexc2.to_string e); raise e
   end;
-  lprintf_nl "Options backup as %s correctly saved" format
+  lprintf_nl (_b "Options backup as %s correctly saved") format
              
 let _ =
   CommonBlocking.add_update_hook CommonServer.check_blocked_servers;
@@ -1099,6 +1208,9 @@ let _ =
       CommonBlocking.set_geoip_dat !!geoip_dat
     with _ -> ()
   );
+  option_hook max_filenames (fun _ ->
+    shorten_all_file_filenames !!max_filenames
+  );
   option_hook max_opened_connections (fun _ ->
   if !verbose then lprintf_nl
     "checking max_opened_connections = %d for validity" !!max_opened_connections;
@@ -1120,8 +1232,10 @@ let max_filedescs = (max_all_sockets - max_sockets) / 2 *)
     max_opened_connections =:= min_conns
   end;
 
+  let reserved_fds = 40 in (* ini files, dynamic libs, etc. *)
+
   let total_files = (* maximum number of files in use at the same time *)
-    (maxi (List.length !!files) !!max_concurrent_downloads) + !!max_upload_slots + 20 (* ini files etc. *)
+    (max (List.length !!files) !!max_concurrent_downloads) + !!max_upload_slots + reserved_fds
   in
 
   let wanted_socks = !!max_opened_connections + total_files in
@@ -1138,7 +1252,7 @@ let max_filedescs = (max_all_sockets - max_sockets) / 2 *)
   else
     begin
       let new_max_opened_connections =
-        maxi (max_all_sockets - total_files) (max_all_sockets / 2)
+        max (max_all_sockets - total_files) (max_all_sockets / 2)
       in
       lprintf_nl "max_opened_connections is set too high (%d), reducing to %d"
         !!max_opened_connections new_max_opened_connections;
@@ -1152,7 +1266,7 @@ let max_filedescs = (max_all_sockets - max_sockets) / 2 *)
   TcpBufferedSocket.set_max_opened_connections
     (fun _ -> !!max_opened_connections);
 
-  Unix32.max_cache_size := total_files - 20;
+  Unix32.max_cache_size := total_files - reserved_fds;
   calc_real_max_indirect_connections ()
 )
 
