@@ -98,6 +98,8 @@ let networks_string = ref ""
 
 let patches_string = ref ""
 
+let is_startup_phase = ref true
+
 let version () =
   Printf.sprintf "MLNet %s: Multi-Network p2p client (%s)"
     Autoconf.current_version !networks_string
@@ -147,11 +149,6 @@ let find_port server_name bind_addr port_option handler =
     iter !!port_option
   else None
 
-let one_day = 3600. *. 24.
-let half_day = one_day /. 2.
-
-let minutes25 = 25 * 60
-
 let new_connection_control () = {
     control_last_ok = 0;
     control_state = 0;
@@ -160,7 +157,7 @@ let new_connection_control () = {
   }
 
 let new_connection_control_recent_ok () = {
-    control_last_ok = last_time () - minutes25;
+    control_last_ok = last_time () - (Date.minute_in_secs * 25);
     control_state = 0;
     control_last_try = 0;
     control_min_reask = !!min_reask_delay;
@@ -263,6 +260,7 @@ let exit_properly n = Pervasives.exit n
 let user_socks = ref ([] : TcpBufferedSocket.t list)
 let dialog_history = ref ([] : (int * string * string) list )
 
+exception Incoming_full
 
 let want_and_not andnot f none value =
 (*   lprintf "want_and_not [%s]\n" value; *)
@@ -337,49 +335,6 @@ let rec find_tag name tags =
   (* first GUI have gui_num = 2, since newly created objects have _update = 1 *)
 let gui_counter = ref 2
 
-  (*
-let ip_of_addr addr f =
-  if addr.addr_name <> "" then
-    if addr.addr_age + !!ip_cache_timeout < last_time () then begin
-        Ip.async_ip  addr.addr_name (fun ip ->
-            addr.addr_ip <- ip;
-            addr.addr_age <- last_time ();
-            f ip)
-      end else
-      f addr.addr_ip
-  else
-    f addr.addr_ip
-
-let sync_ip_of_addr addr =
-  if addr.addr_name <> "" then
-    if addr.addr_age + !!ip_cache_timeout < last_time () then begin
-        let ip = Ip.from_name  addr.addr_name in
-        addr.addr_ip <- ip;
-        addr.addr_age <- last_time ();
-        ip
-      end else
-      addr.addr_ip
-  else
-    addr.addr_ip
-
-let new_addr_ip ip = {
-    addr_ip = ip; addr_name = Ip.to_string ip; addr_age = 0;
-  }
-
-let new_addr_name name = {
-    addr_ip = Ip.null; addr_name = name; addr_age = 0
-  }
-
-let string_of_addr addr =
-  if addr.addr_name = "" then Ip.to_string addr.addr_ip else addr.addr_name
-
-let addr_of_string s =
-  let ip = try Ip.of_string s with _ -> Ip.null in
-  if ip <> Ip.null then new_addr_ip ip else new_addr_name s
-
-let addr_is_ip addr = addr.addr_name = ""
-    *)
-
 let upload_counter = ref Int64.zero
 let download_counter = ref Int64.zero
 let nshared_files = ref 0
@@ -398,6 +353,7 @@ let string_of_field t =
   | Field_Completesources -> "completesources"
   | Field_Filename -> "filename"
   | Field_Size -> "size"
+  | Field_Size_Hi -> "size_hi"
   | Field_Uid -> "uid"
   | Field_Bitrate -> "bitrate"
   | Field_Codec -> "codec"
@@ -405,6 +361,7 @@ let string_of_field t =
   | Field_Lastseencomplete -> "lastcompl"
   | Field_Medialength -> "mlen"
   | Field_Mediacodec -> "mediacodec"
+  | Field_KNOWN s -> s
   | Field_UNKNOWN s -> s
 
 let field_of_string t =
@@ -419,6 +376,7 @@ let field_of_string t =
   | "completesources" -> Field_Completesources
   | "filename" -> Field_Filename
   | "size" -> Field_Size
+  | "size_hi" -> Field_Size_Hi
   | "uid" -> Field_Uid
   | "bitrate" -> Field_Bitrate
   | "codec" -> Field_Codec
@@ -426,10 +384,11 @@ let field_of_string t =
   | "lastcompl" -> Field_Lastseencomplete 
   | "mlen" -> Field_Medialength
   | "mediacodec" -> Field_Mediacodec
-  | _ -> Field_UNKNOWN t
+  | _ -> Field_KNOWN t
 
 let escaped_string_of_field tag =
   match tag.tag_name with
+  | Field_KNOWN s -> String.escaped s
   | Field_UNKNOWN s -> String.escaped s
   | t -> string_of_field t
 
@@ -438,6 +397,10 @@ let string_of_tag tag =
   Printf.sprintf "  \"%s\" = %s" (escaped_string_of_field tag)
   (string_of_tag_value tag.tag_value)
 
+let hexstring_of_tag tag =
+  Printf.sprintf "  \"%s\" = %s" (String2.hex_string_of_string (escaped_string_of_field tag))
+  (string_of_tag_value tag.tag_value)
+
 let rec print_tags tags =
   match tags with
     [] -> ()
@@ -749,11 +712,6 @@ let partial_chunk c =
   | VerificationBitmap.State_complete | VerificationBitmap.State_verified -> 
       false
 
-module Connections = struct
-
-  end
-
-
 let parse_magnet url =
   let url = Url.of_string url in
   if url.Url.short_file = "magnet:" then
@@ -916,12 +874,12 @@ module StringIntern = Weak.Make(struct
 let intern_table = StringIntern.create 1000
 let intern s = StringIntern.merge intern_table s
 
-let print_command_result o buf result =
+let print_command_result o result =
   if use_html_mods o then
-    html_mods_table_one_row buf "serversTable" "servers" [
+    html_mods_table_one_row o.conn_buf "serversTable" "servers" [
       ("", "srh", result); ]
   else
-    Printf.bprintf buf "%s" result
+    Printf.bprintf o.conn_buf "%s" result
 
 let _ =
   Heap.add_memstat "CommonGlobals" (fun level buf ->
