@@ -40,6 +40,7 @@ open CommonResult
 open CommonServer
 open CommonTypes
 open CommonComplexOptions
+open CommonUserDb
 
 let log_prefix = "[cInt]"
 
@@ -84,19 +85,21 @@ let canonize_basename name =
         Charset.add_uchar buf uc'
       end
   done;
-  if Autoconf.windows then
+  if not Autoconf.windows then
     Charset.to_locale (Buffer.contents buf)
   else
-    Buffer.contents buf
+    Buffer.contents buf (* Windows uses patched Ocaml which always uses Unicode chars *)
 
 let last_sent_dir_warning = Hashtbl.create 10
 
+let hdd_full_log_closed = ref false
 let all_temp_queued = ref false
 
-let send_dirfull_warning dir line1 =
-  lprintf_nl "WARNING: Directory %s is full, %s" dir line1;
-  Printf.fprintf Pervasives.stderr "\nWARNING: Directory %s is full, %s\n" dir line1;
-  Pervasives.flush Pervasives.stderr;
+let send_dirfull_warning dir full line1 =
+  let status = if full then "is full" else "has enough space again" in
+  lprintf_nl "WARNING: Directory %s %s, %s" dir status line1;
+  if (not (keep_console_output ())) then
+    Printf.eprintf "\n%s WARNING: Directory %s %s, %s\n%!" (log_time ()) dir status line1;
   if !!hdd_send_warning_interval <> 0 then
     let current_time = last_time () in
     let time_threshold =
@@ -108,12 +111,12 @@ let send_dirfull_warning dir line1 =
       with Not_found -> true in
 
     if send_mail_again then begin
-      Hashtbl.replace last_sent_dir_warning dir current_time;
+      if full then Hashtbl.replace last_sent_dir_warning dir current_time;
       CommonEvent.add_event (Console_message_event
-        (Printf.sprintf "\nWARNING: %s is full, %s\n" dir line1));
+        (Printf.sprintf "\nWARNING: %s %s, %s\n" dir status line1));
       if !!mail <> "" then
         let module M = Mailer in
-        let subject = Printf.sprintf "[mldonkey] AUTOMATED WARNING: %s is full" dir in
+        let subject = Printf.sprintf "[mldonkey@%s] AUTOMATED WARNING: %s %s" (Unix.gethostname ()) dir status in
         let mail = {
           M.mail_to = !!mail; M.mail_from = !!mail;
 	  M.mail_subject = subject; M.mail_body = line1;
@@ -124,22 +127,31 @@ let send_dirfull_warning dir line1 =
     end
 
 let file_commited_name incoming_dir file =
-  let best_name = file_best_name file in
   (try Unix2.safe_mkdir incoming_dir with _ -> ());
+  let fs = Unix32.filesystem incoming_dir in
+  let namemax =
+    match Unix32.fnamelen incoming_dir with
+      None -> 0
+    | Some v -> Int64.to_int v in
+
   let new_name =
-    Filename.concat incoming_dir (canonize_basename best_name)
-  in
+    Filename2.filesystem_compliant 
+      (canonize_basename (file_best_name file)) fs namemax in
+
   let new_name =
-    if Sys.file_exists new_name then
+    if Sys.file_exists (Filename.concat incoming_dir new_name) then
       let rec iter num =
-        let new_name = Printf.sprintf "%s.%d" new_name num in
-        if Sys.file_exists new_name then
-          iter (num+1)
-        else new_name
+        let new_name =
+	  Filename2.filesystem_compliant
+            (Printf.sprintf "%s_%d" new_name num) fs namemax in
+          if Sys.file_exists (Filename.concat incoming_dir new_name) then
+            iter (num+1)
+          else new_name
       in
       iter 1
     else new_name in
-  new_name
+  set_file_best_name file (file_best_name file) "" 0;
+  Filename.concat incoming_dir new_name
 
 let script_for_file file incoming new_name =
   let info = file_info file in
@@ -151,13 +163,6 @@ let script_for_file file incoming new_name =
   in
   let network = network_find_by_num info.G.file_network in
   let filename = Filename.basename new_name in
-  let ed2k_hash =
-    if info.G.file_md4 = Md4.null then "" else
-       Printf.sprintf "ed2k://|file|%s|%s|%s|/"
-        (Url.encode filename)
-        (Int64.to_string (file_size file))
-        (Md4.to_string info.G.file_md4)
-  in
   begin try
   MlUnix.fork_and_exec !!file_completed_cmd
       [|  (* keep those for compatibility *)
@@ -175,7 +180,15 @@ let script_for_file file incoming new_name =
 	    ("DLFILES",   string_of_int (List.length !!files)); 
 	    ("INCOMING",  incoming);
 	    ("NETWORK",   network.network_name);
-	    ("ED2K_HASH", ed2k_hash)]
+	    ("ED2K_HASH", (file_print_ed2k_link filename (file_size file) info.G.file_md4));
+	    ("FILE_OWNER",(file_owner file).user_name);
+	    ("FILE_GROUP",user2_print_group (file_group file));
+	    ("USER_MAIL", ( if (file_owner file).user_mail <> "" then
+                              (file_owner file).user_mail
+                            else
+                              if !!mail <> "" then !!mail else ""));
+	    ]
+
   with e -> 
       lprintf_nl "Exception %s while executing %s"
         (Printexc2.to_string e) !!file_completed_cmd
@@ -191,42 +204,27 @@ file has already been moved to the incoming/ directory under its new
 name.
 *)
 
-exception Incoming_full
-
 let file_commit file =
   let impl = as_file_impl file in
   if impl.impl_file_state = FileDownloaded then
     let subfiles = file_files file in
     match subfiles with
-      file :: secondary_files ->
+      primary :: secondary_files ->
+	if primary == file then
        (try
 	  let file_name = file_disk_name file in
 	  let incoming =
-            if Unix2.is_directory file_name then
-              incoming_directories ()
-            else
-              incoming_files ()
+            incoming_dir
+            (Unix2.is_directory file_name)
+            ~needed_space:(file_size file)
+            ~user:(file_owner file)
+            ()
           in
 
-(* check if temp_directory and incoming are on different partitions *)
-	  if (Unix.stat incoming.shdir_dirname).Unix.st_dev <>
-	     (Unix.stat !!temp_directory).Unix.st_dev
-	  then
-	    begin
-	      match Unix32.diskfree incoming.shdir_dirname with
-	        Some v -> if v < (file_size file) then begin
-		    send_dirfull_warning incoming.shdir_dirname
-		      (Printf.sprintf "can not commit %s" (file_best_name file));
-		    raise Incoming_full
-		  end
-	      | _ -> ()
-	    end;
-
-	  let new_name = file_commited_name
-            incoming.shdir_dirname file in
+	  let new_name = file_commited_name incoming.shdir_dirname file in
 	    if Unix2.is_directory file_name then begin
 	      Unix2.safe_mkdir new_name;
-	      Unix2.chmod new_name (Misc.int_of_octal_string !!create_dir_mask)
+	      Unix2.chmod new_name !Unix32.create_dir_mode;
 	    end;
 
 (*          the next line really moves the file *)
@@ -271,11 +269,15 @@ let file_commit file =
                 with e ->
                     lprintf_nl "Exception %s in file_commit secondaries" (Printexc2.to_string e);
             ) secondary_files
-        with e ->
-              lprintf_nl "Exception in file_commit: %s" (Printexc2.to_string e))
+	with
+	  Incoming_full ->
+	    send_dirfull_warning "" true
+	      (Printf.sprintf "all incoming dirs are full, can not commit %s" (file_best_name file))
+	| e -> lprintf_nl "Exception in file_commit: %s" (Printexc2.to_string e))
     | _ -> assert false
 
-let file_cancel file =
+let file_cancel file user =
+  if user2_allow_file_admin file user then
   try
     let impl = as_file_impl file in
     if impl.impl_file_state <> FileCancelled then
@@ -304,7 +306,8 @@ let file_cancel file =
       lprintf_nl "Exception in file_cancel: %s" (Printexc2.to_string e)
 
 let mail_for_completed_file file =
-  if !!mail <> "" then
+  let usermail = (file_owner file).user_mail in
+  if !!mail <> "" || usermail <> "" then begin
     let module M = Mailer in
     let info = file_info file in
     let line1 = "mldonkey has completed the download of:\r\n\r\n" in
@@ -321,37 +324,44 @@ let mail_for_completed_file file =
     in
 
     let subject = if !!filename_in_subject then
-        Printf.sprintf "[mldonkey] file received - %s" (file_best_name file)
+        Printf.sprintf "[mldonkey@%s] file received - %s" (Unix.gethostname ()) (file_best_name file)
       else
-        Printf.sprintf "mldonkey, file received"
+        Printf.sprintf "mldonkey@%s, file received" (Unix.gethostname ())
     in
 
-    let incoming =
-      if Unix2.is_directory (file_disk_name file) then
-        incoming_directories ()
-      else
-        incoming_files ()
-    in
+(* TODO: This information can be wrong *)
+    let incoming = incoming_dir (Unix2.is_directory (file_disk_name file)) () in
 
     let line4 = if !!url_in_mail = "" then "" else
-      Printf.sprintf "\r\n<%s/%s/%s>\r\n" !!url_in_mail incoming.shdir_dirname (Url.encode (file_best_name file))
+      Printf.sprintf "\r\n<%s/%s%s/%s>\r\n"
+        !!url_in_mail
+        incoming.shdir_dirname
+        (if (file_owner file).user_commit_dir = "" then ""
+         else Printf.sprintf "/%s" (file_owner file).user_commit_dir)
+        (Url.encode (file_best_name file))
     in
 
     let line5 = if !!auto_commit then "" else
         Printf.sprintf "\r\nauto_commit is disabled, file is not committed to incoming"
     in
 
-    let mail = {
-        M.mail_to = !!mail;
-        M.mail_from = !!mail;
+    let line6 =
+      Printf.sprintf "\r\nUser/Group: %s:%s\r\n" (file_owner file).user_name (user2_print_group (file_group file))
+    in
+      
+    let send_mail address admin =
+      let mail = {
+        M.mail_to = address;
+        M.mail_from = address;
         M.mail_subject = subject;
-        M.mail_body = line1 ^ line2 ^ line3 ^ line4 ^ line5;
+        M.mail_body = line1 ^ line2 ^ line3 ^ line4 ^ line5 ^ (if admin then line6 else "");
       } in
-    M.sendmail !!smtp_server !!smtp_port !!add_mail_brackets mail
+        M.sendmail !!smtp_server !!smtp_port !!add_mail_brackets mail
+    in
+    if !!mail <> "" then send_mail !!mail true; (* Multiuser ToDo: this mail is for the admin user, optional? *)
+    if usermail <> "" && usermail <> !!mail then (try send_mail usermail false with Not_found -> ())
+  end
 
-let chat_for_completed_file file =
-  CommonChat.send_warning_for_downloaded_file (file_best_name file)
-  
 let file_completed (file : file) =
   try
     let impl = as_file_impl file in
@@ -362,9 +372,6 @@ let file_completed (file : file) =
         (try mail_for_completed_file file with e ->
               lprintf_nl "Exception %s in sendmail" (Printexc2.to_string e);
               );
-        if !!CommonOptions.chat_warning_for_downloaded then
-          chat_for_completed_file file;
-
       end
   with e ->
       lprintf_nl "Exception in file_completed: %s" (Printexc2.to_string e)
@@ -470,22 +477,17 @@ let contact_remove c =
   with e ->
       lprintf_nl "Exception in contact_remove: %s" (Printexc2.to_string e)
 
-let exit_counter = ref 0
-let exit_timer = ref false
-
-let rec clean_exit n =
-  let can_exit = networks_for_all (fun n -> network_clean_exit n) in
-  if can_exit || (!exit_counter > !!shutdown_timeout) then 
-    exit_properly n
+let clean_exit n =
+  let can_exit = networks_for_all network_clean_exit in
+  if can_exit then exit_properly n
   else 
-    if not !exit_timer then begin
-      exit_timer := true;
+    let rec retry_later retry_counter =
       add_timer 1. (fun _ ->
-        incr exit_counter;
-        exit_timer := false;
-        clean_exit n;
-      );
-    end
+        let can_exit = networks_for_all network_clean_exit in
+        if can_exit || retry_counter > !!shutdown_timeout then
+	  exit_properly n
+	else retry_later (retry_counter + 1)) in
+    retry_later 0
 
 let time_of_sec sec =
   let hours = sec / 60 / 60 in
@@ -501,7 +503,7 @@ let display_vd = ref false
 let display_bw_stats = ref false
 
 let start_download file =
-  if !!pause_new_downloads then file_pause file;
+  if !!pause_new_downloads then file_pause file admin_user;
   if !!file_started_cmd <> "" then
       MlUnix.fork_and_exec  !!file_started_cmd
       [|
@@ -520,7 +522,7 @@ let download_file o arg =
       | Some s ->
           let result = List.assoc (int_of_string arg) user.ui_last_results in
           let files = CommonResult.result_download
-            result [] false in
+            result [] false user.ui_user in
           List.iter start_download files;
           "download started"
     with
@@ -538,8 +540,7 @@ let start_search user query buf =
         networks_iter (fun r ->
             if query.GuiTypes.search_network = 0 ||
               r.network_num = query.GuiTypes.search_network
-            then
-              r.op_network_search s buf);
+            then network_search r s buf);
   end;
   s
 
@@ -547,11 +548,11 @@ let start_search user query buf =
 let network_must_update n =
    CommonEvent.add_event (Network_info_event n)
 
-let network_display_stats buf o =
+let network_display_stats o =
   networks_iter_all (fun r ->
     try
       if List.mem NetworkHasStats r.network_flags then
-        r.op_network_display_stats buf o
+        network_display_stats r o
     with _ -> ())
 
 let print_connected_servers o =
@@ -565,11 +566,11 @@ let print_connected_servers o =
            if use_html_mods o then begin
                Printf.bprintf buf "\\<div class=servers\\>";
                html_mods_table_one_row buf "serversTable" "servers" [
-                 ("", "srh", Printf.sprintf "--- Connected to %d servers on the %s network ---\n"
+                 ("", "srh", Printf.sprintf (_b "--- Connected to %d servers on the %s network ---\n")
                    (List.length list) r.network_name); ]
              end
            else
-             Printf.bprintf buf "--- Connected to %d servers on the %s network ---\n"
+             Printf.bprintf buf (_b "--- Connected to %d servers on the %s network ---\n")
                (List.length list) r.network_name;
            if use_html_mods o then Printf.bprintf buf "\\</div\\>";
           end;
@@ -584,8 +585,8 @@ let print_connected_servers o =
         if Autoconf.donkey = "yes" && r.network_name = "Donkey" && not !!enable_servers then
           begin
             if use_html_mods o then Printf.bprintf buf "\\<div class=servers\\>";
-            Printf.bprintf buf "You disabled server usage, therefore you are not able to connect ED2K servers.\n";
-            Printf.bprintf buf "To use servers again 'set enable_servers true'\n";
+            Printf.bprintf buf (_b "You disabled server usage, therefore you are not able to connect ED2K servers.\n");
+            Printf.bprintf buf (_b "To use servers again 'set enable_servers true'\n");
             if use_html_mods o then Printf.bprintf buf "\\</div\\>"
           end;
        with e ->
@@ -720,7 +721,7 @@ let send_custom_query user buf query args =
       | Q_MP3_BITRATE _ ->
           let bitrate = get_arg "bitrate" in
           if bitrate = "" then raise Not_found;
-          QHasMinVal(Field_UNKNOWN "bitrate", Int64.of_string bitrate)
+          QHasMinVal(Field_KNOWN "bitrate", Int64.of_string bitrate)
 
     in
     try
@@ -758,12 +759,11 @@ let sort_options l =
 
 let opfile_args r opfile =
   let prefix = r.network_shortname ^ "-" in
-  let args = simple_options prefix opfile in
-  args
+  simple_options prefix opfile true
 
 let all_simple_options () =
   let options = ref (sort_options
-      (simple_options "" downloads_ini)
+      (simple_options "" downloads_ini true)
     )
   in
   networks_iter_all (fun r ->
@@ -813,7 +813,7 @@ let all_active_network_opfile_network_names () =
 let apply_on_fully_qualified_options name f =
   if !verbose then lprintf_nl "Change option %s" name;
   let rec iter prefix opfile =
-    let args = simple_options prefix opfile in
+    let args = simple_options prefix opfile true in
     List.iter (fun o ->
 (*        lprintf "Compare [%s] [%s]\n" o.option_name name; *)
         if o.option_name = name then
@@ -880,14 +880,14 @@ let keywords_of_query query =
     | QHasMinVal (field, value) ->
         begin
           match field with
-            Field_UNKNOWN "bitrate"
+            Field_KNOWN "bitrate"
           | Field_Size
           | _ -> ()
         end
     | QHasMaxVal (field, value) ->
         begin
           match field with
-            Field_UNKNOWN "bitrate"
+            Field_KNOWN "bitrate"
           | Field_Size
           | _ -> ()
         end
@@ -939,92 +939,123 @@ files with the highest priority are in FileDownloading state,
 and the ones with lowest priority in FileQueued state, if there
 is a max_concurrent_downloads constraint.
 
-In the future, we could try to mix this with the multi-users
-system to give some fairness between downloads of different
-users.
-
 **************************************************************)
 
 open CommonFile
 
+type user_file_list = {
+  file_list : file list;
+  downloads_allowed : int option;
+}
+
 let force_download_quotas () =
-  let files = List.sort (fun f1 f2 ->
-    let v = file_priority f2 - file_priority f1 in
-    if v <> 0 then v else
-      (**
-	 * [egs] do not start downloading
-         * a small file against an already active download
-      **)
-      let d1 = file_downloaded f1 in
-      let d2 = file_downloaded f2 in
-      if d1 = 0L && d2 > 0L then 1
-      else 
-	if d1 > 0L && d2 = 0L then -1
-        else 
-          (* Try to download in priority files with fewer bytes missing
-             Rationale: once completed, it may allow to recover some disk space *)
-          let r1 = file_size f1 -- d1 in
-          let r2 = file_size f2 -- d2 in
-          if r1 = r2 then 0 else
-            if r2 < r1 then 1 else -1
-    ) !!CommonComplexOptions.files in
 
-  (** move running and queued downloads from [list] to [files]
-      accumulator, until a drop of priority (or end of list) is
-      encountered; Then submit the batches of downloads with same
-      priority to [iter_line].
+  let queue_files files =
+    List.iter (fun file ->
+      if file_state file = FileDownloading then
+	set_file_state file FileQueued
+    ) files in
 
-      @param ndownloads number of running downloads no longer in [list]
-      @param nqueued number of queued downloads in [files]
-  *)
-  let rec iter list priority files ndownloads nqueued =
-    match list, files with
-    | [], [] -> ()
-    | [], _ ->
-        iter_line list priority files ndownloads nqueued
-    | f :: tail , _ :: _ when file_priority f < priority ->
-        iter_line list priority files ndownloads nqueued
-    | f :: tail, files ->
-        match file_state f with
-        | FileDownloading ->
-            iter tail (file_priority f) (f :: files) (ndownloads+1) nqueued
-        | FileQueued ->
-            iter tail (file_priority f) (f :: files) ndownloads (nqueued+1)
-        | _ ->
-            iter tail (file_priority f) files ndownloads nqueued
+  let queue_user_file_list (_user, user_file_list) =
+    queue_files user_file_list.file_list in
 
-  (** queue or unqueue downloads from [files] list to match quotas *)
-  and iter_line list priority files ndownloads nqueued =
-    if ndownloads > !!max_concurrent_downloads then
-      match files with
-      | [] -> assert false
-      | f :: tail ->
-          match file_state f with
-          | FileDownloading ->
-              set_file_state f FileQueued;
-              iter_line list priority tail (ndownloads-1) nqueued
-          | _ -> iter_line list priority tail ndownloads (nqueued-1)
-    else
-    if ndownloads < !!max_concurrent_downloads && nqueued > 0 then
-      match files with
-      | [] -> assert false
-      | f :: tail ->
-          match file_state f with
-	  | FileQueued ->
-              set_file_state f FileDownloading;
-              iter_line list priority tail (ndownloads+1) (nqueued-1)
-          | _ -> iter_line list priority tail ndownloads nqueued
-    else
-      iter list priority [] ndownloads 0
-
-  in
-  if not !all_temp_queued then
-    iter files max_int [] 0 0
+  if !all_temp_queued then
+    queue_files !!CommonComplexOptions.files
   else
-    List.iter (fun f ->
-      if file_state f = FileDownloading then
-	set_file_state f FileQueued
-    ) files
+
+  (* create the assoc list of downloads of each user *)
+    let files_by_user = List.fold_left (fun acc f ->
+      let owner = CommonFile.file_owner f in
+      try
+	let owner_file_list = List.assoc owner acc in
+	(owner, { owner_file_list with 
+	  file_list = f :: owner_file_list.file_list }) :: 
+	  List.remove_assoc owner acc
+      with Not_found ->
+	(owner, { 
+	  downloads_allowed = 
+	    (match owner.user_max_concurrent_downloads with
+	    | 0 -> None
+	    | i -> Some i);
+	  file_list = [f] }) :: acc
+    ) [] !!CommonComplexOptions.files in
+
+    (* sort each user's list separately *)
+    let files_by_user = List.map (fun (owner, owner_file_list) ->
+      owner, { owner_file_list with
+	file_list = List.sort (fun f1 f2 ->
+	  let v = compare (file_priority f2) (file_priority f1) in
+	  if v <> 0 then v else
+	    (* [egs] do not start downloading a small file 
+	       against an already active download *)
+	    let d1 = file_downloaded f1 in
+	    let d2 = file_downloaded f2 in
+	    let active1 = d1 > 0L in
+	    let active2 = d2 > 0L in
+	    if not active1 && active2 then 1
+	    else if active1 && not active2 then -1
+            else 
+              (* Try to download in priority files with fewer bytes missing
+		 Rationale: once completed, it may allow to recover some disk space *)
+              let remaining1 = file_size f1 -- d1 in
+              let remaining2 = file_size f2 -- d2 in
+	      compare remaining1 remaining2
+	) owner_file_list.file_list }
+    ) files_by_user in
+
+    (* sort the assoc list itself with user with highest quota first *)
+    let files_by_user = 
+      List.sort (fun (_owner1, { downloads_allowed = allowed1 }) 
+		     (_owner2, { downloads_allowed = allowed2 }) ->
+        match allowed1, allowed2 with
+	| None, None -> 0
+	| None, _ -> -1
+        | _, None -> 1
+        | Some allowed1, Some allowed2 -> compare allowed2 allowed1
+      ) files_by_user in
+
+    (* serve users round-robin, starting with the one with highest quota *)
+    let rec iter downloads_left to_serve served =
+      if downloads_left = 0 then begin
+	List.iter queue_user_file_list to_serve;
+	List.iter queue_user_file_list served
+      end else
+	match to_serve with
+	| [] ->
+	    if served = [] then () (* nothing left to rotate *)
+	    else (* new round *)
+	      iter downloads_left served []
+	| (_owner, { file_list = [] }) :: others ->
+	    (* user satisfied, remove from lists *)
+	    iter downloads_left others served
+	| ((_owner, { downloads_allowed = Some 0 }) as first) :: others ->
+	    (* reached quota, remove from future rounds *)
+	    queue_user_file_list first;
+	    iter downloads_left others served
+	| (owner, { file_list = first_file :: other_files; 
+	            downloads_allowed = allowed }) :: others ->
+	    let is_downloading =
+	      match file_state first_file with
+	      | FileDownloading -> true
+	      | FileQueued ->
+		  set_file_state first_file FileDownloading;
+		  true
+	      | _ -> false in
+	    if is_downloading then
+	      iter (downloads_left - 1) others 
+		((owner, { 
+		  file_list = other_files;
+		  downloads_allowed = match allowed with
+		    | None -> None
+		    | Some i -> Some (i - 1)
+		}) :: served)
+	    else 
+	      iter downloads_left others
+		((owner, {
+		  file_list = other_files;
+		  downloads_allowed = allowed
+		}) :: served) in
+    iter !!max_concurrent_downloads files_by_user []
 
 let _ =
   option_hook max_concurrent_downloads (fun _ ->
