@@ -139,13 +139,16 @@ let dummy_result = {
     result_size = zero;
     result_modified = true;
     result_time = BasicSocket.last_time ();
+    result_source_network = 0;
   }
   
-let result_download rs names force =
+let result_download rs names force user =
   let r = IndexedResults.get_result rs in
   let files = ref [] in
   CommonNetwork.networks_iter (fun n ->
-      files := (n.op_network_download r) :: !files
+      (* Temporarily download results only from the network that returned the result *)
+      if (n.network_num = r.result_source_network) then
+      files := (CommonNetwork.network_download n r user) :: !files
   );
   !files  
     
@@ -166,7 +169,39 @@ let update_result2 rs r =
       r.result_modified <- false;
       IndexedResults.update_result rs r
     end
-  
+
+let rec find_avail tags =
+  match tags with
+    [] -> raise Not_found
+  | tag :: tail -> begin
+    match tag with
+      { tag_name = Field_Availability ; tag_value = tag_value } ->
+      tag
+    | _ -> find_avail tail
+  end
+
+let increment_avail r =
+  let rr = IndexedResults.get_result r in
+  begin
+    try
+      let tag = find_avail rr.result_tags in
+      let x = int64_of_tagvalue tag.tag_value in
+      tag.tag_value <- Uint64 (x ++ 1L);
+     with Not_found -> ();
+    end;
+  update_result_num rr
+
+let update_or_create_avail tags =
+  let tag =
+    try
+      let tag = find_avail tags in
+      let x = int64_of_tagvalue tag.tag_value in
+      tag.tag_value <- Uint64 (x ++ 1L);
+      tag
+   with Not_found ->
+      { tag_name = Field_Availability; tag_value = Uint64 1L } 
+  in
+  tag :: tags
   
 let _ = 
   Heap.add_memstat "CommonResult" (fun level buf ->
