@@ -43,12 +43,14 @@ type 'a shared_impl = {
     mutable impl_shared_size : int64;
     mutable impl_shared_id : Md4.t;
     mutable impl_shared_requests : int;
-    mutable impl_shared_magic : string option
+    mutable impl_shared_file : CommonTypes.file option;
+    mutable impl_shared_servers : CommonTypes.server list;
   }
   
 and 'a shared_ops = {
     mutable op_shared_info : ('a -> GuiTypes.shared_info);
     mutable op_shared_unshare : ('a -> unit);
+    mutable op_shared_state : (CommonTypes.file -> CommonTypes.ui_conn -> string);
   }
   
 let as_shared  (shared : 'a shared_impl) =
@@ -166,6 +168,10 @@ let shared_fullname s =
   let impl = as_shared_impl s in
   impl.impl_shared_fullname  
 
+let shared_size s =
+  let impl = as_shared_impl s in
+  impl.impl_shared_size
+
 let shared_codedname s =
   let impl = as_shared_impl s in
   impl.impl_shared_codedname  
@@ -175,6 +181,14 @@ let shared_unshare s =
   shared_remove impl;
   try impl.impl_shared_ops.op_shared_unshare impl.impl_shared_val with _ -> ()
 
+let shared_state s o =
+  let impl = as_shared_impl s in
+  try
+    match impl.impl_shared_file with
+    | None -> ""
+    | Some f -> impl.impl_shared_ops.op_shared_state f o
+  with _ -> ""
+
 let shared_dir = function
   | None	-> ""
   | Some sh	-> 
@@ -193,6 +207,7 @@ let shared_prio impl =
 let new_shared_ops network = {
     op_shared_unshare = (fun _ -> ni_ok network "shared_unshare");
     op_shared_info = (fun _ -> fni network "shared_info");
+    op_shared_state = (fun _ _ -> fni network "shared_state");
   }
 
 let dummy_shared = {
@@ -204,12 +219,14 @@ let dummy_shared = {
     impl_shared_ops = {
       op_shared_unshare = (fun _ -> raise Not_found);
       op_shared_info = (fun _ -> raise Not_found);
+      op_shared_state = (fun _ _ -> raise Not_found);
     };
     impl_shared_uploaded = zero;
     impl_shared_size = zero;
     impl_shared_id = Md4.null;
     impl_shared_requests = 0;
-    impl_shared_magic = None;
+    impl_shared_file = None;
+    impl_shared_servers = []
   }
   
   
@@ -230,18 +247,19 @@ let shared_add_directory shared_dir local_dir =
   waiting_directories := (shared_dir, local_dir) :: !waiting_directories
   
 let shared_scan_directory shared_dir local_dir =
+  let module U = Unix.LargeFile in
   let incoming_files_inode = 
-    ((Unix.stat ((CommonComplexOptions.incoming_files ()).shdir_dirname)).Unix.st_ino)
+    ((U.stat ((CommonComplexOptions.incoming_dir false ()).shdir_dirname)).U.st_ino)
   in
   let incoming_directories_inode = 
-    ((Unix.stat ((CommonComplexOptions.incoming_directories ()).shdir_dirname)).Unix.st_ino)
+    ((U.stat ((CommonComplexOptions.incoming_dir true ()).shdir_dirname)).U.st_ino)
   in
   let temp_directory_inode =
-    ((Unix.stat !!temp_directory).Unix.st_ino)
+    ((U.stat !!temp_directory).U.st_ino)
   in
   let dirname = shared_dir.shdir_dirname in
   let strategy = 
-    CommonComplexOptions.sharing_strategies shared_dir.shdir_strategy in
+    CommonComplexOptions.sharing_strategy shared_dir.shdir_strategy in
   let dirname = 
     if Filename.is_relative dirname then
       Filename.concat file_basedir dirname
@@ -348,6 +366,10 @@ let impl_shared_info impl =
     T.shared_requests = impl.impl_shared_requests; 
     T.shared_uids = [];
     T.shared_sub_files = [];
+    T.shared_magic =
+      match impl.impl_shared_file with
+      | None -> None
+      | Some f -> CommonFile.file_magic f;
   }
   
 let shared_info s =
