@@ -260,11 +260,13 @@ type field =
 | Field_Completesources (* "completesources" *)
 | Field_Filename (* "filename" *)
 | Field_Size
+| Field_Size_Hi
 | Field_Uid
 | Field_Filerating
 | Field_Lastseencomplete
 | Field_Mediacodec
 | Field_Medialength
+| Field_KNOWN of string
 | Field_UNKNOWN of string
   
 type tag = {
@@ -531,7 +533,7 @@ and network = {
     mutable op_network_gui_message : (string -> userdb -> unit);
     
     mutable op_network_download : (result_info -> userdb -> file);
-    mutable op_network_display_stats : (Buffer.t -> ui_conn -> unit);
+    mutable op_network_display_stats : (ui_conn -> unit);
     mutable op_network_stat_info_list : unit -> (string * int * (network_stat_info list)) list;
     mutable op_network_clean_exit : (unit -> bool);
     mutable op_network_reset : (unit -> unit);
@@ -925,6 +927,19 @@ let dummy_stats =
     brand_upload = 0L;
   }
 
+type country_stats =
+  {
+    country_code : string;
+    country_name : string;
+    country_continent : string;
+    mutable country_session_upload : int64;
+    mutable country_session_download : int64;
+    mutable country_session_seen : int64;
+    mutable country_total_upload : int64;
+    mutable country_total_download : int64;
+    mutable country_total_seen : int64;
+  }
+
 type kind_type = {
   f : string -> string -> unit;
   description : string
