@@ -226,6 +226,7 @@ let _ =
   network.op_network_connected <- (fun _ -> false);
   network.op_network_is_enabled <- (fun _ -> raise IgnoreNetwork);
   network.op_network_update_options <- (fun _ -> raise IgnoreNetwork);
+  (* op_network_info defined twice ?? *)
   network.op_network_info <- (fun _ -> raise Not_found);
   network.op_network_info <- (fun n ->
       {
@@ -237,8 +238,15 @@ let _ =
         network_enabled = true;
         network_uploaded = Int64.zero;
         network_downloaded = Int64.zero;
-        network_connected = 0;
+        network_connected_servers = 0;
       });
+  network.op_network_ports <- (fun _ ->
+    [
+    !!http_port, "http_port";
+    !!telnet_port, "telnet_port";
+    !!gui_port, "gui_port";
+    !!gift_port, "gift_port GUI";
+    ]);
   network.op_network_connected_servers <- (fun _ -> [])
 
 let (shared_ops : shared_file CommonShared.shared_ops) =
@@ -272,15 +280,6 @@ let shareds_counter = ref 1
 let shared_counter = ref (Int64.zero)
 let shared_files = Hashtbl.create 13
 
-let _ =
-  Heap.add_memstat "CommonUploads" (fun level buf ->
-   Printf.bprintf buf "  infos_by_name: %d\n" (Hashtbl.length infos_by_name);
-   Printf.bprintf buf "  shareds_by_uid: %d\n" (Hashtbl.length shareds_by_uid);
-   Printf.bprintf buf "  shareds_by_id: %d\n" (Hashtbl.length shareds_by_id);
-   Printf.bprintf buf "  shared_files: %d\n" (Hashtbl.length shared_files);
-  )
-
-
 let new_shared_dir dirname = {
     shared_dirname = dirname;
     shared_files = [];
@@ -688,7 +687,8 @@ let add_shared full_name codedname size =
           impl_shared_ops = shared_ops;
           impl_shared_val = sh;
           impl_shared_requests = 0;
-          impl_shared_magic = None
+          impl_shared_file = None;
+          impl_shared_servers = [];
         }
       and sh = {
           shared_info = index;
@@ -831,14 +831,7 @@ let add_pending_slot c =
     end
   else
   if not (Intmap.mem (client_num c) !pending_slots_map) then
-    begin
-(* This is useless since it is the goal of the pending_slots_map
-        else if Fifo.mem pending_slots_fifo (client_num c) then begin
-	lprintf "Avoided inserting a client twice in pending slots\n";
-	
-      end else *)
-      pending_slots_map := Intmap.add (client_num c) c !pending_slots_map;
-    end
+    pending_slots_map := Intmap.add (client_num c) c !pending_slots_map
 
 let remove_pending_slot c =
   if Intmap.mem (client_num c) !pending_slots_map then
@@ -846,11 +839,11 @@ let remove_pending_slot c =
 
 let rec give_a_slot c =
   remove_pending_slot c;
-  if not (client_is_connected c) then begin
-      find_pending_slot ()
-    end
-  else begin
-      set_client_has_a_slot c true;
+  if not (client_is_connected c) then
+    find_pending_slot ()
+  else
+    begin
+      set_client_has_a_slot c NormalSlot;
       client_enter_upload_queue c
     end
 
@@ -865,36 +858,47 @@ and find_pending_slot () =
   with _ -> ()
 
 let add_pending_slot c =
-  let csh = client_upload c in
+  let client_upload c =
+    match client_upload c with
+      None -> raise Not_found
+    | Some file -> file
+  in
+  let csh = file_shared (client_upload c) in
   let cdir = shared_dir csh in
   let cprio = ref (shared_prio csh) in
   let cfriend = ref (if is_friend c && !!friends_upload_slot then 1 else 0) in
   let csmallfiles = ref (match csh with 
     | None -> 0
     | Some sh -> if shared_size sh <= !!small_files_slot_limit then 1 else 0) in
-  (* if cdir <> "" then
-    lprintf "Testing cdir %s\n" cdir; *)
+  let allowed_release_slots =
+    ref (Misc.percentage_of_ints !!max_upload_slots !!max_release_slots) in
+
+(* check current upload slots for already used special slots *)
   Intmap.iter (fun _ c ->
-    let sh = client_upload c in
-    if shared_dir sh = cdir then decr cprio;
-    if client_has_a_friend_slot c then decr cfriend;
-    match sh with
-      | None -> ()
-      | Some sh ->
-	  if shared_size sh <= !!small_files_slot_limit then
-	    decr csmallfiles;
-  ) !CommonClient.uploaders;
-  (* if cdir <> "" then
-     lprintf "Testing cprio %d cfriend %d csmallfiles\n" 
-     !cprio !cfriend !csmallfiles; *)
-  if !cprio > 0 || !cfriend > 0 || !csmallfiles > 0 then begin
-    remove_pending_slot c;
-    if client_is_connected c then begin
-      set_client_has_a_slot c true;
-      client_enter_upload_queue c
-    end
-  end else
-    add_pending_slot c
+    if shared_dir (file_shared (client_upload c)) = cdir then
+      decr cprio;
+    match client_slot c with
+      ReleaseSlot -> decr allowed_release_slots
+    | FriendSlot -> decr cfriend
+    | SmallFileSlot -> decr csmallfiles
+    | _ -> ()) !CommonClient.uploaders;
+
+  let slot_type =
+    if file_release (client_upload c) && !allowed_release_slots > 0 then Some ReleaseSlot else
+    if !cfriend > 0 then Some FriendSlot else
+    if !csmallfiles > 0 then Some SmallFileSlot else
+    if !cprio > 0 then Some (PrioSlot cdir) else
+    None
+  in
+  match slot_type with
+    Some slot ->
+	remove_pending_slot c;
+	if client_is_connected c then
+	  begin
+	    set_client_has_a_slot c slot;
+	    client_enter_upload_queue c
+	  end
+  | None -> add_pending_slot c
 
 let static_refill_upload_slots () =
   let len = Intmap.length !CommonClient.uploaders in
@@ -931,7 +935,7 @@ let dynamic_refill_upload_slots () =
     estimated_capacity
   else 
     (* max_hard_upload_rate lowered manually,... *)
-    mini estimated_capacity (!!max_hard_upload_rate * 1024) in
+    min estimated_capacity (!!max_hard_upload_rate * 1024) in
   if !verbose_upload then
     lprintf_nl "usage: %d(%d) capacity: %d"
       (short_delay_upload_usage ())
@@ -959,34 +963,18 @@ let dynamic_refill_upload_slots () =
   end
 
 let turn = ref (-1)
-let turn_h = ref (-1)
 
 let refill_upload_slots () =
   incr turn;
   if !turn = 5 then
     turn := 0;
-  if !turn_h = 360 then
-    turn_h := 0;
   if !!dynamic_slots then begin
     if !turn = 0 then
       (* call every 5s *)
       dynamic_refill_upload_slots ()
   end else
     (* call every 1s *)
-    static_refill_upload_slots ();
-
-  if !turn = 0 then begin
-    (* call every 5s *)
-    incr turn_h;
-    update_download_history ();
-    update_upload_history ()
-    end;
-
-  if !turn_h = 0 then begin
-    (* call every 720 * 5s *)
-    update_h_download_history ();
-    update_h_upload_history ()
-    end
+    static_refill_upload_slots ()
 
 let consume_bandwidth len =
   streaming_left := !streaming_left - len
@@ -1060,3 +1048,12 @@ let words_of_filename =
     | l -> l
   in
   get_name_keywords
+
+let _ =
+  Heap.add_memstat "CommonUploads" (fun level buf ->
+   Printf.bprintf buf "  infos_by_name: %d\n" (Hashtbl.length infos_by_name);
+   Printf.bprintf buf "  shareds_by_uid: %d\n" (Hashtbl.length shareds_by_uid);
+   Printf.bprintf buf "  shareds_by_id: %d\n" (Hashtbl.length shareds_by_id);
+   Printf.bprintf buf "  shared_files: %d\n" (Hashtbl.length shared_files);
+   Printf.bprintf buf "  pending_slots: %d\n" (Intmap.length !pending_slots_map);
+  )
