@@ -506,13 +506,29 @@ let get_int_date proto s pos =
 let get_int_pos s pos =
   get_int s pos, pos + 4
   
-let get_sub_files s pos =
+let get_sub_files proto s pos =
   get_list (fun s pos ->
     let name, pos = get_string s pos in
     let size, pos = get_int64 s pos, pos+8 in
-    (name, size, Some ""), pos
+    let magic, pos = 
+      if proto > 40 then
+        get_string s pos
+      else
+        "", pos
+    in
+    (name, size, Some magic), pos
   ) s pos
   
+let get_file_comments proto s pos = 
+  get_list (fun s pos ->
+     let ip, pos = get_ip2 proto s pos in
+     let name, pos = get_string s pos in
+     let rating, pos = get_uint8 s pos, pos+1 in
+     let comment, pos = get_string s pos in
+     (ip, name, rating, comment), pos
+  ) s pos
+
+  
 let get_file proto s pos = 
   let num = get_int s pos in
   let net = get_int s (pos+4) in
@@ -548,7 +564,6 @@ let get_file proto s pos =
   let comment, pos = if proto > 21 then
       get_string s pos
     else "", pos in
-  let names = List.map (fun name -> name, noips()) names in
   let last_seen = BasicSocket.last_time () - last_seen in
   let uids, pos = 
     if proto < 31 then
@@ -558,10 +573,31 @@ let get_file proto s pos =
   in
   let sub_files, pos =
     if proto > 35 then
-      get_sub_files s pos
+      get_sub_files proto s pos
     else [], pos
   in
-
+  let magic, pos = 
+    if proto > 40 then
+      let ms, pos = get_string s pos
+      in Some ms, pos 
+    else
+      Some "", pos
+  in
+  let comments, pos = 
+    if proto > 40 then
+      get_file_comments proto s pos
+    else [], pos
+  in
+  let user, pos = 
+    if proto > 40 
+      then get_string s pos
+      else "", pos
+  in
+  let group, pos = 
+    if proto > 40 
+      then get_string s pos
+      else "", pos
+  in
   (*
   assert (num = file_info_test.file_num);
   assert (net = file_info_test.file_network);
@@ -610,6 +646,11 @@ let get_file proto s pos =
     file_priority = priority;
     file_uids = uids;
     file_sub_files = sub_files;
+    file_magic = magic;
+    file_comments = comments;
+    file_user = user;
+    file_group = group;
+    file_release = false;
   }, pos
 
 let get_host_state proto s pos =
@@ -710,12 +751,15 @@ let get_server proto s pos =
     server_banner = "";
     server_users = None;
     server_preferred = preferred;
+    server_master = false;
     server_version = ve; 
     server_max_users = ma;
     server_lowid_users = lo;
     server_soft_limit = so;
     server_hard_limit = ha;
     server_ping = pi;
+    server_published_files = 0;
+    server_features = None;
   }, pos
 
 let get_client_type s pos = 
@@ -766,16 +810,18 @@ let get_client proto s pos =
       client_name = name;
       client_rating = rating;
       client_chat_port = chat_port;
-      client_files = None;
       client_connect_time = 0;
       client_software = "";
+      client_os = None;
       client_release = "";
       client_emulemod = "";
-      client_downloaded = zero;
-      client_uploaded = zero;
+      client_total_downloaded = zero;
+      client_total_uploaded = zero;
+      client_session_downloaded = zero;
+      client_session_uploaded = zero;
       client_upload = None;
       client_sui_verified = None;
-(*      client_sock_addr = ""; *)
+      client_file_queue = [];
     }, pos+8
   else
   let num = get_int s pos in
@@ -824,16 +870,18 @@ let get_client proto s pos =
     client_name = name;
     client_rating = rating;
     client_chat_port = 0;
-    client_files = None;
     client_connect_time = connect_time;
     client_software = software;
+    client_os = None;
     client_release = release;
     client_emulemod = emulemod;
-    client_downloaded = downloaded;
-    client_uploaded = uploaded;
+    client_total_downloaded = downloaded;
+    client_total_uploaded = uploaded;
+    client_session_downloaded = 0L;
+    client_session_uploaded = 0L;
     client_upload = upload;
     client_sui_verified = verified;
-(*    client_sock_addr = sock_addr; *)
+    client_file_queue = [];
   }, pos
 
 let default_flags = [
@@ -879,7 +927,7 @@ let get_network proto s pos =
     network_config_filename = config_file;
     network_uploaded = uploaded;
     network_downloaded = downloaded;
-    network_connected = connected;
+    network_connected_servers = connected;
   }, pos
 
 
@@ -889,7 +937,7 @@ let get_user proto s pos =
   let name, pos = get_string s (pos+20) in
   let ip, pos = get_ip2 proto s pos in
   let port = get_int16 s pos in
-  let tags, pos = get_list2 proto get_tag s (pos+6) in
+  let tags, pos = get_list2 proto get_tag s (pos+2) in
   let server = get_int s pos in
   {
     user_num = num;
@@ -941,6 +989,7 @@ let get_shared_info proto s pos =
     shared_requests = requests;
     shared_uids = [];
     shared_sub_files = [];
+    shared_magic = Some "";
   }
 
 let get_shared_info_version_10 proto s pos =
@@ -958,9 +1007,16 @@ let get_shared_info_version_10 proto s pos =
   in
   let sub_files, pos =
     if proto > 36 then
-      get_sub_files s pos
+      get_sub_files proto s pos
     else [], pos
   in
+  let magic, pos = 
+    if proto > 40 then
+      let ms, pos = get_string s (pos) in
+      Some ms, pos
+    else
+      Some "", pos
+  in
   {
     shared_num = num;
     shared_network = network;
@@ -970,6 +1026,7 @@ let get_shared_info_version_10 proto s pos =
     shared_requests = requests;
     shared_uids = uids;
     shared_sub_files = sub_files;
+    shared_magic = magic;
   }
 
 
@@ -999,7 +1056,7 @@ let from_gui (proto : int array) opcode s =
     | 5
     | 52 ->
         if proto < 14 then
-           let pass = fst (get_string s 2) in Password (CommonUserDb.admin_user, pass)
+           let pass = fst (get_string s 2) in Password (CommonUserDb.admin_user.CommonTypes.user_name, pass)
         else
         let pass,pos = get_string s 2 in
         let login,pos = get_string s pos in
@@ -1263,6 +1320,10 @@ let from_gui (proto : int array) opcode s =
          let b = get_bool s 6 in
          ServerSetPreferred (num, b)
 
+    | 68 ->
+         let num = get_int s 2 in
+         GetStats num
+
     | _ -> 
         lprintf_nl "FROM GUI:Unknown message %d" opcode; 
         raise FromGuiMessageNotImplemented
@@ -1290,6 +1351,9 @@ let dummy_option =
     M.option_help = "";
     M.option_type = "";
     M.option_advanced = false;
+    M.option_restart = false;
+    M.option_public = false;
+    M.option_internal = false;
   }
 
 let to_gui (proto : int array)  opcode s =
@@ -1508,6 +1572,9 @@ let to_gui (proto : int array)  opcode s =
               M.option_value = value;
               M.option_default = default;
               M.option_advanced = advanced;
+              M.option_restart = false;
+              M.option_public = false;
+              M.option_internal = false;
             }
           else
           let optype = 
@@ -1569,6 +1636,9 @@ let to_gui (proto : int array)  opcode s =
               M.option_value = value;
               M.option_default = default;
               M.option_advanced = advanced;
+              M.option_restart = false;
+              M.option_public = false;
+              M.option_internal = false;
             }
           else
           let optype = 
