@@ -20,13 +20,6 @@
 open Md4
 open CommonTypes
 
-type ips_list = {
-    mutable nips : int;
-    mutable ips : Ip.t list;
-  }
-
-let noips () = { ips = []; nips = 0; }
-  
 type 'a search_request = {
     mutable search_num : int;
     mutable search_type : search_type;
@@ -97,7 +90,7 @@ which fields it is interesting in (why send the availability when the
     | File_network of int
     | File_comment of string
     | File_name of string
-    | File_names of (string * ips_list) list
+    | File_names of string list
     | File_md4 of Md4.t        
     | File_size of int64
     | File_downloaded of int64
@@ -125,7 +118,7 @@ type file_info = {
     file_network : int;
     mutable file_comment : string;
     mutable file_name : string;
-    mutable file_names : (string * ips_list) list;
+    mutable file_names : string list;
     mutable file_md4 : Md4.t;        
     mutable file_size : int64;
     mutable file_downloaded : int64; (* LOT OF CHANGES *)
@@ -143,6 +136,11 @@ type file_info = {
     mutable file_priority : int;
     mutable file_uids : Uid.t list;
     mutable file_sub_files : (string * int64 * string option) list;
+    mutable file_magic : string option;
+    mutable file_comments : (Ip.t * string * int * string) list;
+    mutable file_user : string;
+    mutable file_group : string;
+    mutable file_release : bool;
   }
   
 type user_info = {
@@ -172,12 +170,15 @@ type server_info = {
     mutable server_users : int list option;
     mutable server_banner : string;
     mutable server_preferred : bool;
+    mutable server_master : bool;
     mutable server_version : string;
     mutable server_max_users : int64;
     mutable server_soft_limit : int64;
     mutable server_hard_limit : int64;
     mutable server_lowid_users : int64;
     mutable server_ping : int;
+    mutable server_published_files : int;
+    mutable server_features : string option;
 
   } 
 
@@ -210,18 +211,20 @@ type client_info = {
     mutable client_type : client_type;
     mutable client_tags: CommonTypes.tag list;
     mutable client_name : string;
-    mutable client_files:  file_tree option;
     mutable client_rating : int;
     mutable client_chat_port : int;
     mutable client_connect_time : int;
     mutable client_software : string;
+    mutable client_os : string option;
     mutable client_release : string;
     mutable client_emulemod : string;
-    mutable client_downloaded : int64;
-    mutable client_uploaded : int64;
+    mutable client_total_downloaded : int64;
+    mutable client_total_uploaded : int64;
+    mutable client_session_downloaded : int64;
+    mutable client_session_uploaded : int64;
     mutable client_upload : string option;
     mutable client_sui_verified : bool option;
-(*  mutable client_sock_addr : string;  *)
+    mutable client_file_queue : CommonTypes.file list;
   }
 
 type client_stats = {
@@ -247,9 +250,32 @@ type shared_info = {
     mutable shared_requests : int;
     mutable shared_uids : Uid.t list; (* net file UID *)
     mutable shared_sub_files : (string * int64 * string option) list;
+    mutable shared_magic : string option;
   }
   
   
+let osinfo_short i =
+  match i with
+    Some s when
+      s = "linux" ||
+      s = "netbsd" ||
+      s = "macos" ||
+      s = "freebsd" ||
+      s = "windows" -> Some (String.sub s 0 1)
+  | _ -> i
+
+let client_software_short software os =
+  software ^
+  match osinfo_short os with
+    Some s -> "/" ^ s
+  | None -> ""
+
+let client_software software os =
+  software ^
+  match os with
+    Some s -> "/" ^ s
+  | None -> ""
+
 let add_file tree dirname r =
   let path = Filename2.path_of_filename dirname in
   
@@ -315,7 +341,7 @@ let file_info_test =
     file_name = "tratra";
     file_num = 356;
     file_network = 873;
-    file_names = ["toto", noips(); "tutu", noips()];
+    file_names = ["toto"; "tutu"];
     file_md4 = Md4.random ();
     file_size = Int64.of_string "68758765";
     file_active_sources = 12;
@@ -333,5 +359,10 @@ let file_info_test =
     file_priority = 0;
    file_uids = [];
     file_sub_files = [];
+    file_magic = Some "";
+    file_comments = [];
+    file_user = "";
+    file_group = "";
+    file_release = false;
  } 
   
