@@ -103,9 +103,10 @@ let rec dollar_escape o with_frames s =
 
 let eval auth cmd o =
   let buf = o.conn_buf in
-  let cmd = Url.decode cmd in
+  let cmd = Url.decode ~raw:false cmd in
   let cmd =
     if String2.check_prefix cmd "ed2k://" ||
+       String2.check_prefix cmd "ftp://" ||
        String2.check_prefix cmd "http://" then "dllink " ^ cmd
     else if String2.check_prefix cmd "fha://" then "ovlink " ^ cmd
     else cmd in
@@ -143,7 +144,7 @@ let eval auth cmd o =
           Printf.bprintf buf "\\<tr class=\\\"dl-1\\\"\\>";
           html_mods_td buf [
             ("", "sr", "< > : required parameter");
-            ("", "sr", "[< >] : optionnal parameter");
+            ("", "sr", "[< >] : optional parameter");
             ("", "sr", "< 1 | 2 > : alternative parameter"); ];
           Printf.bprintf buf "\\</table\\>\\</div\\>\\</div\\>"
         end else
@@ -270,7 +271,7 @@ $bDownloads:$n
           $rcancel/pause/resume <num>$n : cancel/pause/resume download <num>
 
 $bSearches:$n
-          $rs  <keywords>$n : start a search for keywords <keywords> on the network
+          $rs <keywords>$n : start a search for keywords <keywords> on the network
           $rvr$n : view results of the last search
           $rd <num>$n : download result number <num>
           $rvs$n : view previous searches
@@ -284,7 +285,7 @@ $bGeneral:$n
 Use '$rlonghelp$n' or '$r??$n' for all commands.
 Use '$rhelp command$n' or '$r? command$n' for help on a command.
             ";
-    | "?" :: args | "help" :: args ->
+    | "?" :: args | "help" :: args | "man" :: args ->
           List.iter (fun arg ->
               List.iter (fun (cmd, _, _, help) ->
                   if cmd = arg then
@@ -309,7 +310,7 @@ Use '$rhelp command$n' or '$r? command$n' for help on a command.
         let user, pass =
           match args with
             [] -> failwith "Usage: auth <user> <password>"
-          | [s1] -> admin_user, s1
+          | [s1] -> admin_user.CommonTypes.user_name, s1
           | user :: pass :: _ -> user, pass
         in
         if valid_password user pass then begin
@@ -317,7 +318,10 @@ Use '$rhelp command$n' or '$r? command$n' for help on a command.
             o.conn_user <- find_ui_user user;
             if not !verbose_no_login then lprintf_nl "Authenticated user: %s" user;
             let module M = CommonMessages in
-            Buffer.add_string buf M.full_access
+            Buffer.add_string buf M.full_access;
+            (match DriverInteractive.real_startup_message () with
+               Some s -> Buffer.add_string buf ("\n" ^ s);
+             | None -> ());
           end else
         let module M = CommonMessages in
         Buffer.add_string buf M.bad_login
@@ -448,7 +452,7 @@ type telnet_conn = {
     telnet_auth : bool ref;
   }
 
-
+let iac_will_8bit = "\255\253\000"
 let iac_will_naws = "\255\253\031"
 
 let user_reader o telnet sock nread =
@@ -476,7 +480,7 @@ let user_reader o telnet sock nread =
 
       let i = int_of_char c in
       telnet.telnet_iac <- false;
-      let is_normal_char = i > 31 && i < 127 in
+      let is_normal_char = i > 31 in
 
       if telnet.telnet_wait = 1 then begin
           Buffer.add_char telnet.telnet_buffer c;
@@ -577,7 +581,7 @@ let telnet_handler t event =
           "telnet connection"
           s in
         let telnet = {
-            telnet_auth = ref (empty_password admin_user);
+            telnet_auth = ref (has_empty_password admin_user);
             telnet_iac = false;
             telnet_wait = 0;
             telnet_buffer = Buffer.create 100;
@@ -591,28 +595,27 @@ let telnet_handler t event =
             conn_width = 80;
             conn_height = 0;
           } in
-      if Ip.matches from_ip !!allowed_ips then begin
+	(match Ip_set.match_ip !allowed_ips_set from_ip with
+	| Some br -> 
         TcpBufferedSocket.prevent_close sock;
         TcpBufferedSocket.set_max_output_buffer sock !!interface_buffer;
         TcpBufferedSocket.set_reader sock (user_reader o telnet);
         TcpBufferedSocket.set_closer sock user_closed;
         user_socks := sock :: !user_socks;
 
+        TcpBufferedSocket.write_string sock iac_will_8bit;
         TcpBufferedSocket.write_string sock iac_will_naws;
 
         before_telnet_output o sock;
         TcpBufferedSocket.write_string sock
 	   (Printf.sprintf "Welcome to MLDonkey %s\n" Autoconf.current_version);
-	(match DriverInteractive.real_startup_message () with
-	   Some s -> TcpBufferedSocket.write_string sock (s);
-	 | None -> ());
 
         TcpBufferedSocket.write_string sock (dollar_escape o false
             "$cWelcome on mldonkey command-line$n\n\nUse $r?$n for help\n\n");
 
         after_telnet_output o sock
-	end
-      else begin
+
+	| None ->
         before_telnet_output o sock;
 	let reject_message =
 	  Printf.sprintf "Telnet connection from %s rejected (see allowed_ips setting)\n"
@@ -621,87 +624,10 @@ let telnet_handler t event =
 	TcpBufferedSocket.write_string sock (dollar_escape o false reject_message);
 	shutdown sock Closed_connect_failed;
         if not !verbose_no_login then lprintf_n "%s" reject_message;
-	Unix.close s
-      end
+	Unix.close s)
 
   | _ -> ()
 
-(*************************************************************
-
-                  The Chat Server
-
-**************************************************************)
-
-let chat_handler t event =
-  match event with
-    TcpServerSocket.CONNECTION (s, Unix.ADDR_INET (from_ip, from_port)) ->
-      (
-        try
-          let o = {
-              conn_buf = Buffer.create 1000;
-              conn_output = TEXT;
-              conn_sortvd = NotSorted;
-              conn_filter = (fun _ -> ());
-              conn_user = default_user;
-              conn_width = 80; conn_height = 0;
-            } in
-
-         let from_ip = Ip.of_inet_addr from_ip in
-         if Ip.matches from_ip !!allowed_ips then
-           (
-            let chanin = Unix.in_channel_of_descr s in
-            let chanout = Unix.out_channel_of_descr s in
-            let paq = Chat_proto.read_packet_channel chanin in
-            let ret =
-              match paq with
-                ((v,id,(host,port)),iddest,pro) ->
-                  if v <> CommonChat.version then
-                    None
-                  else
-                    Some paq
-            in
-            close_out chanout;
-            (match ret with
-              None -> ()
-            | Some ((v,id,(host,port)),iddest,pro) ->
-                match pro with
-                  Chat_proto.Hello ->
-                    CommonChat.send_hello_ok ()
-                | Chat_proto.HelloOk -> ()
-                | Chat_proto.AddOpen _ -> ()
-                | Chat_proto.Byebye -> ()
-                | Chat_proto.RoomMessage _ ->
-                    (* A VOIR *)
-                    ()
-                | Chat_proto.Message s ->
-                    if iddest = !!CommonOptions.chat_console_id then
-                   (* we must eval the string as a command *)
-                      (
-                            let buf = o.conn_buf in
-                            Buffer.reset buf;
-                       let auth = ref true in
-                       eval auth s o;
-                       CommonChat.send_text !!CommonOptions.chat_console_id None
-                         (dollar_escape o false (Buffer.contents buf));
-                       Buffer.reset buf
-                      )
-                    else
-                   (* we must forward the message *)
-                      (networks_iter (fun r ->
-                        network_private_message r iddest s)
-                      )
-            )
-           )
-         else
-           Unix.close s
-     with
-       Failure mess ->
-         lprintf_nl "%s" mess;
-         Unix.close s
-    )
-  | _ ->
-      ()
-
 (*************************************************************
 
                   The HTTP Server
@@ -839,7 +765,7 @@ let get_theme_page page =
         let fname = Filename.concat theme page in fname
 
 let theme_page_exists page =
-        if Sys.file_exists (get_theme_page page) then true else false
+    Sys.file_exists (get_theme_page page)
 
 (* if files are small really_input should be okay *)
 let read_theme_page page =
@@ -850,14 +776,6 @@ let read_theme_page page =
     really_input file s 0 size;
     s)
 
-let add_simple_commands buf =
-  let this_page = "commands.html" in
-  Buffer.add_string buf (
-    if !!html_mods_theme != "" && theme_page_exists this_page then
-      read_theme_page this_page else
-    if !!html_mods then !!CommonMessages.web_common_header_mods0
-    else !!CommonMessages.web_common_header_old)
-
 let http_add_gen_header r =
   add_reply_header r "Server" "MLdonkey";
   add_reply_header r "Connection" "close"
@@ -902,14 +820,22 @@ let http_add_bin_stream_header r ext =
   add_reply_header r "Accept-Ranges" "bytes"
 
 let http_send_bin r buf filename =
-  let file_to_send = File.to_string filename in
-  let clen = String.length file_to_send in
-  let ext_pos = (String.rindex filename '.') + 1 in
-  let exten = (String.sub filename ext_pos ((String.length filename) - ext_pos)) in
-  if !verbose_msg_servers then
-    lprintf_nl "Extension found [%s] for file: [%s]" exten filename;
-  let ext = extension_to_file_ext exten in
-  http_add_bin_header r ext clen;
+  let file_to_send =
+    try
+      File.to_string filename
+    with _ ->
+      try
+        Hashtbl.find CommonPictures.files filename
+      with Not_found ->
+        try
+          if String.sub filename 0 4 = "flag" then
+            Hashtbl.find CommonPictures.files "flag_--.png"
+          else
+            raise Not_found
+        with _ -> raise Not_found
+  in
+  let ext = extension_to_file_ext (Filename2.last_extension2 filename) in
+  http_add_bin_header r ext (String.length file_to_send);
   Buffer.add_string buf file_to_send
 
 let http_error_no_gd img_type =
@@ -958,9 +884,7 @@ let html_open_page buf t r open_body =
                         else !!CommonMessages.html_header_old);
 
   Buffer.add_string buf "</head>\n";
-  if open_body then Buffer.add_string buf "<body>\n";
-  if not !!use_html_frames then add_simple_commands buf;
-  ()
+  if open_body then Buffer.add_string buf "<body>\n"
 
 let html_close_page buf close_body =
   if close_body then Buffer.add_string buf "</body>\n";
@@ -1023,11 +947,25 @@ let http_handler o t r =
   CommonInteractive.display_vd := false;
   CommonInteractive.display_bw_stats := false;
   clear_page buf;
+  if !Http_server.verbose && r.get_url.Url.short_file <> "" then
+    lprintf_nl "received URL %s %s"
+      r.get_url.Url.short_file
+      (let b = Buffer.create 100 in
+	 List.iter (fun (arg, value) -> Printf.bprintf b " %s %s" arg value) r.get_url.Url.args;
+	 if Buffer.contents b <> "" then Printf.sprintf "(%s)" (Buffer.contents b) else "");
 
-  let user = if r.options.login = "" then admin_user else r.options.login in
+  let user = if r.options.login = "" then admin_user.CommonTypes.user_name else r.options.login in
   if not (valid_password user r.options.passwd) then begin
       clear_page buf;
-      need_auth r !!http_realm
+      http_file_type := HTM;
+      let _, error_text_long, header = Http_server.error_page "401" "" ""
+	(Ip.to_string (TcpBufferedSocket.my_ip r.sock))
+	(string_of_int !!http_port) None in
+      Buffer.add_string buf error_text_long;
+      r.reply_head <- header;
+      r.reply_headers <- [
+        "Connection", "close";
+        "WWW-Authenticate", Printf.sprintf "Basic realm=\"%s\"" !!http_realm]
     end
   else
     begin
@@ -1067,15 +1005,15 @@ let http_handler o t r =
                     "VDC" ->
                       let num = int_of_string value in
                       let file = file_find num in
-                      file_cancel file
+                      file_cancel file o.conn_user.ui_user
                   | "VDP" ->
                       let num = int_of_string value in
                       let file = file_find num in
-                      file_pause file
+                      file_pause file o.conn_user.ui_user
                   | "VDR" ->
                       let num = int_of_string value in
                       let file = file_find num in
-                      file_resume file
+                      file_resume file o.conn_user.ui_user
                   | _ -> ()
               ) r.get_url.Url.args;
 
@@ -1115,7 +1053,6 @@ let http_handler o t r =
               if !!html_mods then !!CommonMessages.multidllink_mods0
               else !!CommonMessages.multidllink_old)
         | "" | "index.html" ->
-            if !!use_html_frames then begin
                 html_open_page buf t r false;
                 let this_page = "frames.html" in
                 if !!html_mods_theme != "" && theme_page_exists this_page then
@@ -1146,9 +1083,7 @@ let http_handler o t r =
 </frameset>
 <frame name=\"output\" src=\"oneframe.html\">
 </frameset>
-" !!commands_frame_height;
-              end else
-              html_open_page buf t r true
+" !!commands_frame_height
         | "complex_search.html" ->
             html_open_page buf t r true;
             CommonSearch.complex_search buf
@@ -1157,7 +1092,10 @@ let http_handler o t r =
 
         | "oneframe.html" ->
             html_open_page buf t r true;
-            Buffer.add_string buf (Printf.sprintf "<br><div align=\"center\"><h3>Welcome to MLDonkey %s</h3></div>" Autoconf.current_version);
+            Buffer.add_string buf (Printf.sprintf "<br><div align=\"center\"><h3>%s %s</h3></div>"
+	      (Printf.sprintf (_b "Welcome to MLDonkey")) Autoconf.current_version);
+	    if !!motd_html <> "" then Buffer.add_string buf !!motd_html;
+	    if user2_is_admin o.conn_user.ui_user then
 	    (match DriverInteractive.real_startup_message () with
 	       Some s -> Buffer.add_string buf (Printf.sprintf "<p><pre><b><h3>%s</b></h3></pre>" s);
 	     | None -> ())
@@ -1260,11 +1198,6 @@ let http_handler o t r =
                 http_send_bin r buf "tag.jpg"
             | true -> raise Not_found)
 
-        | "favicon.ico" ->
-            if !verbose_msg_servers then
-              lprintf_nl "favicon.ico request received by tracker";
-            http_send_bin r buf "favicon.ico"
-
         | "filter" ->
             html_open_page buf t r true;
             let b = Buffer.create 10000 in
@@ -1329,7 +1262,7 @@ let http_handler o t r =
                       try
                         let num = int_of_string value in
                         let r = find_result num in
-                        let files = result_download r [] false in
+                        let files = result_download r [] false o.conn_user.ui_user in
                         List.iter CommonInteractive.start_download files;
 
                         let module M = CommonMessages in
@@ -1349,15 +1282,23 @@ let http_handler o t r =
                   "cancel" ->
                     let num = int_of_string value in
                     let file = file_find num in
-                    file_cancel file
+                    file_cancel file o.conn_user.ui_user
                 | "pause" ->
                     let num = int_of_string value in
                     let file = file_find num in
-                    file_pause file
+                    file_pause file o.conn_user.ui_user
                 | "resume" ->
                     let num = int_of_string value in
                     let file = file_find num in
-                    file_resume file
+                    file_resume file o.conn_user.ui_user
+                | "release" ->
+                    let num = int_of_string value in
+                    let file = file_find num in
+                    set_file_release file true o.conn_user.ui_user
+                | "norelease" ->
+                    let num = int_of_string value in
+                    let file = file_find num in
+                    set_file_release file false o.conn_user.ui_user
                 | "sortby" ->
                     begin
                       match value with
@@ -1376,13 +1317,16 @@ let http_handler o t r =
                       | "A" -> o.conn_sortvd <- ByASources
                       | "N" -> o.conn_sortvd <- ByNet
                       | "Avail" -> o.conn_sortvd <- ByAvail
+                      | "Cm" -> o.conn_sortvd <- ByComments
+                      | "User" -> o.conn_sortvd <- ByUser
+                      | "Group" -> o.conn_sortvd <- ByGroup
                       | _ -> ()
                     end
                 | _ -> ()
             ) r.get_url.Url.args;
             let b = Buffer.create 10000 in
 
-            let list = (List2.tail_map file_info !!files) in
+            let list = List2.tail_map file_info (user2_filter_files !!files o.conn_user.ui_user) in
             DriverInteractive.display_file_list b o list;
             html_open_page buf t r true;
             Buffer.add_string buf (html_escaped (Buffer.contents b))
@@ -1394,9 +1338,10 @@ let http_handler o t r =
                 | [ "jvcmd", "multidllink" ; "links", links] ->
                     html_open_page buf t r true;
                     List.iter (fun url ->
-		      if url <> "\013" && url <> "" then
+		      let url = fst (String2.cut_at url '\013') in
+		      if url <> "" then
 		        begin
-                          Buffer.add_string buf (html_escaped (dllink_parse (o.conn_output = HTML) url));
+                          Buffer.add_string buf (html_escaped (dllink_parse (o.conn_output = HTML) url o.conn_user.ui_user));
                           Buffer.add_string buf (html_escaped "\\<P\\>")
 			end
                     ) (String2.split links '\n')
@@ -1451,13 +1396,13 @@ let http_handler o t r =
 
               | [ "setoption", _ ; "option", name; "value", value ] ->
                   html_open_page buf t r true;
-		  if (o.conn_user == default_user) || !!enable_user_config then
+		  if user2_is_admin o.conn_user.ui_user then
 		    begin
                       CommonInteractive.set_fully_qualified_options name value;
                       Buffer.add_string buf "Option value changed"
 		    end
 		  else
-                    Buffer.add_string buf "Only 'admin' is allowed to change options"
+                    Buffer.add_string buf "You are not allowed to change options"
 
               | args ->
                   List.iter (fun (s,v) ->
@@ -1552,11 +1497,15 @@ let http_handler o t r =
                 read_theme_page this_page else
               if !!html_mods then !!CommonMessages.download_html_js_mods0
               else !!CommonMessages.download_html_js_old)
-        | cmd ->
-            html_open_page buf t r true;
-            Printf.bprintf buf "No page named %s" (html_escaped cmd)
+        | s ->  http_send_bin r buf (String.lowercase s)
       with
-      | Not_found -> Printf.bprintf buf "404 Not found"
+      | Not_found ->
+	  let _, error_text_long, header = Http_server.error_page "404" "" ""
+			(Ip.to_string (TcpBufferedSocket.my_ip r.sock))
+			(string_of_int !!http_port)
+			(Some (Url_not_found r.get_url.Url.full_file)) in
+	  r.reply_head <- header;
+	  Buffer.add_string buf error_text_long
       | e ->
           Printf.bprintf buf "\nException %s\n" (Printexc2.to_string e);
           r.reply_stream <- None
@@ -1564,11 +1513,11 @@ let http_handler o t r =
 
   let s =
     match !http_file_type with
-      HTM -> html_close_page buf false; dollar_escape o !!use_html_frames (Buffer.contents buf)
-    | MLHTM -> html_close_page buf true; dollar_escape o !!use_html_frames (Buffer.contents buf)
+      HTM -> html_close_page buf false; dollar_escape o true (Buffer.contents buf)
+    | MLHTM -> html_close_page buf true; dollar_escape o true (Buffer.contents buf)
     | TXT
+    | UNK
     | BIN -> Buffer.contents buf
-    | UNK -> "Unknown type for content :" ^ (Buffer.contents buf)
   in
   r.reply_content <- 
     if !http_file_type <> BIN && !!html_use_gzip then 
@@ -1589,11 +1538,13 @@ let create_http_handler () =
       bind_addr = Ip.to_inet_addr !!http_bind_addr ;
       port = !!http_port;
       requests = [];
-      addrs = !!allowed_ips;
+      addrs = Ip_set.of_list !!allowed_ips;
+      use_ip_block_list = false;
       base_ref = "";
       default = http_handler http_options;
     } in
-  option_hook allowed_ips (fun _ -> config.addrs <- !!allowed_ips);
+  option_hook allowed_ips (fun _ ->
+    config.addrs <- Ip_set.of_list !!allowed_ips);
   ignore(find_port "http server" !!http_bind_addr http_port
       (Http_server.handler config));
   config.port <- !!http_port
