@@ -313,6 +313,7 @@ let save_config () =
       Options.save_with_help_private users_ini;
       CommonComplexOptions.save ();
       CommonUploads.save ();
+      CommonStats.save ();
       networks_iter_all (fun r ->
           List.iter (fun opfile ->
               Options.save_with_help opfile
@@ -391,13 +392,10 @@ let print_table_text buf alignments titles lines =
 
 let print_table_html_mods buf lines =
 
-  let counter = ref 0 in
+  html_mods_cntr_init ();
 
   List.iter (fun line ->
-      if (!counter mod 2 == 0) then Printf.bprintf buf "\\<tr class=dl-1"
-      else Printf.bprintf buf "\\<tr class=dl-2";
-      incr counter;
-
+      Printf.bprintf buf "\\<tr class=\\\"dl-%d\\\"" (html_mods_cntr ());
       Array.iter (fun data ->
           Printf.bprintf buf "%s" data;
       ) line;
@@ -668,7 +666,7 @@ else "")
 (List.length guifiles) (size_of_int64 !tdl) (size_of_int64 !tsize) (!trate /. 1024.)
 (let unread = ref 0 in
 Fifo.iter (fun (t,i,num,n,s) -> if t > !last_message_log then incr unread) chat_message_fifo;
-if !unread > 0 then Printf.sprintf "\\<td class=downloaded title=\\\"%d unread messages\\\"\\>\\<a onClick=\\\"mSub('fstatus','version');mSub('output','message')\\\"\\>(+%d)\\</a\\>\\&nbsp;\\</td\\>" !unread !unread else "");
+if !unread > 0 then Printf.sprintf "\\<td onMouseOver=\\\"mOvr(this);\\\" onMouseOut=\\\"mOut(this);\\\" class=downloaded title=\\\"%d unread messages\\\"\\>\\<a onClick=\\\"mSub('fstatus','version');mSub('output','message')\\\"\\>(+%d)\\</a\\>\\&nbsp;\\</td\\>" !unread !unread else "");
 
 if !!html_mods_vd_network then Printf.bprintf buf
 "\\<td title=\\\"Sort by network\\\" class=dlheader\\>\\<input style=\\\"padding-left: 0px; padding-right: 0px;\\\" class=headbutton type=submit value=N name=sortby\\>\\</td\\>";
@@ -1115,7 +1113,8 @@ let display_active_file_list buf o list =
 
   if o.conn_output <> HTML && !!improved_telnet then
   begin
-    let list = Sort.list (fun f1 f2 -> percent f1 >= percent f2) list in
+    let list = 
+      List.sort (fun f1 f2 -> compare (percent f2) (percent f1)) list in
     simple_print_file_list false buf list o
   end
   else
@@ -1124,29 +1123,37 @@ let display_active_file_list buf o list =
       let sorter =
         match o.conn_sortvd with
 
-        | BySize -> (fun f1 f2 -> f1.file_size >= f2.file_size)
+        | BySize -> (fun f1 f2 -> compare f2.file_size f1.file_size)
         | ByRate -> (fun f1 f2 ->
-                if stalled f1 then false else
-                if stalled f2 then true else
-                  f1.file_download_rate >= f2.file_download_rate
-            )
-        | ByName -> (fun f1 f2 -> String.lowercase f1.file_name <= String.lowercase f2.file_name)
-        | ByDone -> (fun f1 f2 -> f1.file_downloaded >= f2.file_downloaded)
-	| ByPriority -> (fun f1 f2 -> f1.file_priority >= f2.file_priority)
-		| BySources -> (fun f1 f2 -> (number_of_sources f1) >= (number_of_sources f2))
-		| ByASources -> (fun f1 f2 -> (number_of_active_sources f1) >= (number_of_active_sources f2))
-        | ByPercent -> (fun f1 f2 -> percent f1 >= percent f2)
-        | ByETA -> (fun f1 f2 -> calc_file_eta f1 <= calc_file_eta f2)
-        | ByAge -> (fun f1 f2 -> f1.file_age >= f2.file_age)
-        | ByLast -> (fun f1 f2 -> f1.file_last_seen >= f2.file_last_seen)
-        | ByNet -> (fun f1 f2 -> net_name f1 <= net_name f2)
-        | ByAvail -> (fun f1 f2 -> get_file_availability f1 >= get_file_availability f2)
-        | ByComments -> (fun f1 f2 -> (number_of_comments f1) >= (number_of_comments f2))
-        | ByUser -> (fun f1 f2 -> f1.file_user <= f2.file_user)
-        | ByGroup -> (fun f1 f2 -> f1.file_group <= f2.file_group)
+                if stalled f1 then 1 else
+                if stalled f2 then -1 else
+                  compare f2.file_download_rate f1.file_download_rate)
+        | ByName -> (fun f1 f2 -> String.compare 
+		       (String.lowercase f1.file_name) 
+		       (String.lowercase f2.file_name))
+        | ByDone -> (fun f1 f2 -> 
+		       compare f2.file_downloaded f1.file_downloaded)
+	| ByPriority -> (fun f1 f2 -> 
+			   compare f2.file_priority f1.file_priority)
+	| BySources -> (fun f1 f2 -> compare 
+			  (number_of_sources f2) (number_of_sources f1))
+	| ByASources -> (fun f1 f2 -> 
+			   compare (number_of_active_sources f2) 
+			           (number_of_active_sources f1))
+        | ByPercent -> (fun f1 f2 -> compare (percent f2) (percent f1))
+        | ByETA -> (fun f1 f2 -> compare (calc_file_eta f1) (calc_file_eta f2))
+        | ByAge -> (fun f1 f2 -> compare f2.file_age f1.file_age)
+        | ByLast -> (fun f1 f2 -> compare f2.file_last_seen f1.file_last_seen)
+        | ByNet -> (fun f1 f2 -> compare (net_name f1) (net_name f2))
+        | ByAvail -> (fun f1 f2 -> compare 
+			(get_file_availability f2) (get_file_availability f1))
+        | ByComments -> (fun f1 f2 -> compare 
+			   (number_of_comments f2) (number_of_comments f1))
+        | ByUser -> (fun f1 f2 -> compare f1.file_user f2.file_user)
+        | ByGroup -> (fun f1 f2 -> compare f1.file_group f2.file_group)
         | NotSorted -> raise Not_found
       in
-      Sort.list sorter list
+      List.sort sorter list
     with _ -> list
   in
   simple_print_file_list false buf list o
@@ -1191,6 +1198,7 @@ let old_print_search buf o results =
   let counter = ref 0 in
   if use_html_mods o then
   begin
+    html_mods_cntr_init ();
     if !!html_mods_use_js_tooltips then Printf.bprintf buf "\\<div id=\\\"object1\\\" style=\\\"position:absolute; background-color:#FFFFDD;color:black;border-color:black;border-width:20px;font-size:8pt; visibility:visible; left:25px; top:-100px; z-index:+1\\\" onmouseover=\\\"overdiv=1;\\\"  onmouseout=\\\"overdiv=0; setTimeout(\\\'hideLayer()\\\',1000)\\\"\\>\\&nbsp;\\</div\\>\n";
     html_mods_table_header_colspan buf "resultsTable" "results" [
       ( "1", "0", "srh", "Network", "Network" ) ;
@@ -1211,10 +1219,7 @@ let old_print_search buf o results =
               if !counter >= !!max_displayed_results then raise Exit;
 
               if use_html_mods o then
-                begin
-                  if (!counter mod 2 == 0) then Printf.bprintf buf "\\<tr class=\\\"dl-1\\\"\\>"
-                  else Printf.bprintf buf "\\<tr class=\\\"dl-2\\\"\\>";
-                end;
+                Printf.bprintf buf "\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
 
               user.ui_last_results <- (!counter, rs) :: user.ui_last_results;
               let network_name = 
@@ -1259,7 +1264,7 @@ let old_print_search buf o results =
                       let nl = ref false in
                       List.iter (fun t ->
                           match t.tag_name with
-                          | Field_UNKNOWN "FTH" | Field_UNKNOWN "urn" -> ()
+                          | Field_KNOWN "FTH" | Field_KNOWN "urn" -> ()
                           | _ ->
                               Buffer.add_string buf ((if !nl then "<br>" else begin nl := true;"" end) ^
                                 escaped_string_of_field t ^ ": " ^ get_tag_value t);
@@ -1275,7 +1280,7 @@ let old_print_search buf o results =
                       let nl = ref false in
                       List.iter (fun t ->
                           match t.tag_name with
-                          | Field_UNKNOWN "FTH" | Field_UNKNOWN "urn" -> ()
+                          | Field_KNOWN "FTH" | Field_KNOWN "urn" -> ()
                           | _ ->
                               Buffer.add_string buf ((if !nl then "\n" else begin nl := true;"" end) ^
                                   "|| (" ^
@@ -1331,8 +1336,8 @@ let old_print_search buf o results =
 	      let cformat = ref "" in
               List.iter (fun t ->
                   (match t.tag_name with
-                    | Field_UNKNOWN "urn"
-                    | Field_UNKNOWN "FTH"  -> hash := get_tag_value t
+                    | Field_KNOWN "urn"
+                    | Field_KNOWN "FTH"  -> hash := get_tag_value t
                     | Field_Availability -> cavail := get_tag_value t
                     | Field_Completesources -> csource := get_tag_value t
                     | Field_Length -> clength := get_tag_value t
@@ -1383,9 +1388,9 @@ let old_print_search buf o results =
                         | Field_Format
                         | Field_Bitrate
 (* TODO : "urn" shouldn't be some kind of Field_Uid of Gnutella ? *)
-                        | Field_UNKNOWN "urn"
+                        | Field_KNOWN "urn"
 (* TODO : "FTH" shouldn't be some kind of Field_Uid of Fasttrack ? *)
-                        | Field_UNKNOWN "FTH"  -> ()
+                        | Field_KNOWN "FTH"  -> ()
                         | _ ->
                             Buffer.add_string buf ("\\<span title=\\\"" ^
                                 get_tag_value t ^ "\\\"\\>(" ^
@@ -1490,9 +1495,9 @@ let print_search_html buf results o search_num =
               user.ui_last_results <- (!counter, rs) :: user.ui_last_results;
               files := [|
 
+                (Printf.sprintf "[%5d]\\<input name=d type=checkbox value=%d\\>" !counter r.result_num);
                 (Int64.to_string r.result_size);
                 (string_of_int avail);
-                (Printf.sprintf "[%5d]\\<input name=d type=checkbox value=%d\\>" !counter r.result_num);
 
                 (
                   let names = r.result_names in
@@ -1704,8 +1709,8 @@ let print_search buf s o =
   Intmap.iter (fun r_num (avail,rs) ->
       let r = IndexedResults.get_result rs in
       results := (rs, r, !avail) :: !results) s.search_results;
-  let results = Sort.list (fun (_, r1,_) (_, r2,_) ->
-        r1.result_size > r2.result_size
+  let results = List.sort (fun (_, r1,_) (_, r2,_) ->
+        compare r2.result_size r1.result_size
     ) !results in
 
   Printf.bprintf buf "Result of search %d\n" s.search_num;
@@ -1789,7 +1794,8 @@ let print_network_modules buf o =
           with _ -> ())
     end
 
-let print_gdstats buf o =
+let print_gdstats o =
+  let buf = o.conn_buf in
   let picture_suffix () =
     if !!html_mods_vd_gfx_png then
       Printf.bprintf buf "png\\\"\\>"
@@ -1855,6 +1861,9 @@ let buildinfo html buf =
     (
       "Version:\t", 
       "MLNet Multi-Network p2p client version " ^ Autoconf.current_version 
+      ^ (match Filename.basename (Sys.executable_name) with
+         | "mlnet" | "mlnet.static" -> ""
+         | bin -> Printf.sprintf " (%s)" bin)
     );
   if Autoconf.scm_version <> "" then 
     tack list 
@@ -1877,7 +1886,7 @@ let buildinfo html buf =
     );
   tack list 
     (
-      "Build on:\t", 
+      "Built on:\t", 
       Autoconf.build_system ^ " (" ^ Unix2.endianness () ^ ")" ^
       (if Autoconf.glibc_version = "" then "" 
       else
@@ -1941,12 +1950,11 @@ let buildinfo html buf =
        ( "0", "srh", "", "" ) ]
     else
       Printf.bprintf buf "\n\t--Buildinfo--\n";
-    let counter = ref 0 in
+    html_mods_cntr_init ();
     List.iter (fun (desc, text) ->
-      incr counter;
       if html then 
-	Printf.bprintf buf "\\<tr class=\\\"%s\\\"\\>\\<td class=\\\"sr\\\"\\>%s\\</td\\>\\<td class=\\\"sr\\\"\\>%s\\</td\\>\\</tr\\>"
-          (if !counter mod 2 = 0 then "dl-1" else "dl-2") desc text
+	Printf.bprintf buf "\\<tr class=\\\"dl-%d\\\"\\>\\<td class=\\\"sr\\\"\\>%s\\</td\\>\\<td class=\\\"sr\\\"\\>%s\\</td\\>\\</tr\\>"
+          (html_mods_cntr ()) desc text
       else
 	Printf.bprintf buf "%s %s\n" desc text;
     ) list;
@@ -2036,10 +2044,11 @@ let runinfo html buf o =
   tack list
     (
       "", 
-      Printf.sprintf "\t\t max_string_length: %d - word_size: %d - max_array_length: %d"
+      Printf.sprintf "\t\t max_string_length: %d - word_size: %d - max_array_length: %d - max_int: %d"
 	Sys.max_string_length
 	Sys.word_size
 	Sys.max_array_length
+	Pervasives.max_int
     );
   tack list
     (
@@ -2060,12 +2069,11 @@ let runinfo html buf o =
        ( "0", "srh", "", "" ) ]
     else
       Printf.bprintf buf "\n\t--Runinfo--\n";
-    let counter = ref 0 in
+    html_mods_cntr_init ();
     List.iter (fun (desc, text) ->
-      incr counter;
       if html then 
-	Printf.bprintf buf "\\<tr class=\\\"%s\\\"\\>\\<td class=\\\"sr\\\"\\>%s\\</td\\>\\<td class=\\\"sr\\\"\\>%s\\</td\\>\\</tr\\>"
-          (if !counter mod 2 = 0 then "dl-1" else "dl-2") desc text
+	Printf.bprintf buf "\\<tr class=\\\"dl-%d\\\"\\>\\<td class=\\\"sr\\\"\\>%s\\</td\\>\\<td class=\\\"sr\\\"\\>%s\\</td\\>\\</tr\\>"
+          (html_mods_cntr ()) desc text
       else
 	Printf.bprintf buf "%s %s\n" desc text;
     ) list;
@@ -2136,7 +2144,7 @@ let diskinfo html buf =
   let fill_dir_line = String.make (!len_dir - 9) '-' in
   let fill_strategy = String.make (!len_strategy - 4) ' ' in
   let fill_strategy_line = String.make (!len_strategy - 4) '-' in
-  let counter = ref 0 in
+  html_mods_cntr_init ();
   if html then
       html_mods_table_header buf "sharesTable" "shares" [
        ( "0", "srh", "Directory", "Directory" ) ;
@@ -2154,7 +2162,6 @@ let diskinfo html buf =
         fill_dir_line fill_strategy_line;
     end;
   List.iter (fun (dir, strategy) ->
-	incr counter;
 	let diskused =
     	  match Unix32.diskused dir with
 	  | None -> Printf.sprintf "---"
@@ -2172,11 +2179,10 @@ let diskinfo html buf =
 	in
 	let filesystem = Unix32.filesystem dir in
 	if html then
-	  begin
-	    Printf.bprintf buf "\\<tr class=\\\"%s\\\"\\>\\<td class=\\\"sr\\\"\\>%s\\</td\\>\\<td class=\\\"sr\\\"\\>%s\\</td\\>\\<td class=\\\"sr ar\\\"\\>%s\\</td\\>\\<td class=\\\"sr ar\\\"\\>%s\\</td\\>\\<td class=\\\"sr ar\\\"\\>%s\\</td\\>\\<td class=\\\"sr\\\"\\>%s\\</td\\>\\</tr\\>"
-	    (if !counter mod 2 = 0 then "dl-1" else "dl-2")
-	    dir strategy diskused diskfree percentfree filesystem
-	  end
+	    Printf.bprintf buf "\\<tr class=\\\"dl-%d\\\"\\>\\<td class=\\\"sr\\\"\\>%s\\</td\\>\\<td class=\\\"sr\\\"\\>%s\\</td\\>
+	      \\<td class=\\\"sr ar\\\"\\>%s\\</td\\>\\<td class=\\\"sr ar\\\"\\>%s\\</td\\>
+	      \\<td class=\\\"sr ar\\\"\\>%s\\</td\\>\\<td class=\\\"sr\\\"\\>%s\\</td\\>\\</tr\\>"
+	    (html_mods_cntr ()) dir strategy diskused diskfree percentfree filesystem
 	else
 	  Printf.bprintf buf "%-*s|%-*s|%8s|%8s|%5s|%-s\n"
 	    (max !len_dir (!len_dir - String.length dir)) dir
@@ -2448,7 +2454,9 @@ let print_upstats o list server =
       ( "0", "srh", "Preview", "P" ) ;
       ( "0", "srh", "Filename", "Filename" );
       ( "0", "srh", "Statistic links", "Stats" );
-      ( "0", "srh", "Published on servers", "Publ" ) ]
+      ( "0", "srh", "Published on servers", "Publ" );
+      ( "0", "srh", "Share status", "Status" )
+    ]
   else
     begin
       Printf.bprintf buf " Requests |  Bytes   | Uploaded | File\n";
@@ -2456,10 +2464,10 @@ let print_upstats o list server =
     end;
 
   html_mods_cntr_init ();
-  let list = Sort.list (fun f1 f2 ->
-      (f1.impl_shared_requests = f2.impl_shared_requests &&
-       f1.impl_shared_uploaded > f2.impl_shared_uploaded) ||
-      (f1.impl_shared_requests > f2.impl_shared_requests )
+  let list = List.sort (fun f1 f2 ->
+    let c = compare f2.impl_shared_requests f1.impl_shared_requests in
+    if c <> 0 then c else
+    compare f2.impl_shared_uploaded f1.impl_shared_uploaded
     ) list in
 
   List.iter (fun impl ->
@@ -2474,8 +2482,9 @@ let print_upstats o list server =
 	(if !!html_mods_use_js_tooltips then
 	   Printf.bprintf buf " onMouseOver=\\\"mOvr(this);setTimeout('popLayer(\\\\\'%s<br>%s%s\\\\\')',%d);setTimeout('hideLayer()',%d);return true;\\\" onMouseOut=\\\"mOut(this);hideLayer();setTimeout('hideLayer()',%d)\\\"\\>"
 	    (Http_server.html_real_escaped (Filename.basename (Charset.to_utf8 impl.impl_shared_codedname)))
-	    (match impl.impl_shared_magic with
-	       None -> ""
+	    (match impl.impl_shared_file with
+	       None -> "no file info"
+	     | Some file -> match file_magic file with | None -> "no magic"
 	     | Some magic -> "File type: " ^ (Http_server.html_real_escaped magic) ^ "<br>")
 	    (if impl.impl_shared_servers = [] then "" else
 	       Printf.sprintf "<br>Published on %d %s<br>%s"
@@ -2497,7 +2506,6 @@ let print_upstats o list server =
 
 	let uploaded = Int64.to_float impl.impl_shared_uploaded in
 	let size = Int64.to_float impl.impl_shared_size in
-
 	html_mods_td buf [
 	  ("", "sr ar", Printf.sprintf "%d" impl.impl_shared_requests);
 	  ("", "sr ar", size_of_int64 impl.impl_shared_uploaded);
@@ -2515,7 +2523,9 @@ let print_upstats o list server =
                       (Md4.to_string impl.impl_shared_id) "T1"
                       (Md4.to_string impl.impl_shared_id) "T2"
                       (Md4.to_string impl.impl_shared_id) "B"));
-	  ("", "sr ar", Printf.sprintf "%d" published ) ];
+	  ("", "sr ar", Printf.sprintf "%d" published);
+	  ("", "sr", shared_state (as_shared impl) o);
+        ];
 	Printf.bprintf buf "\\</tr\\>\n";
       end
     else
