@@ -246,6 +246,8 @@ let send_event gui ev =
     
 let send_update_file gui file_num update =
   let file = file_find file_num in
+  if user2_can_view_file gui.gui_conn.conn_user.ui_user (file_owner file) (file_group file) then
+  begin
   let impl = as_file_impl file in
   let file_info = if update then
       P.File_info (file_info file) 
@@ -256,6 +258,7 @@ let send_update_file gui file_num update =
         impl.impl_file_last_seen)
   in
   gui_send gui file_info
+  end
 
 let send_update_user gui user_num update =
   let user = user_find user_num in
@@ -428,7 +431,7 @@ let gui_initialize gui =
                   (File_add_source_event (file,c))
                 :: gui.gui_events.gui_new_events
             ) sources
-      ) !!files;
+      ) (user2_filter_files !!files gui.gui_conn.conn_user.ui_user);
       
       List.iter (fun file ->
           addevent gui.gui_events.gui_files (file_num file) true;
@@ -437,7 +440,7 @@ let gui_initialize gui =
       networks_iter_all (fun n ->
           List.iter (fun s ->
               addevent gui.gui_events.gui_servers (server_num s) true
-          ) (n.op_network_connected_servers ())
+          ) (network_connected_servers n)
       );
       
       server_iter (fun s ->
@@ -458,6 +461,7 @@ let gui_initialize gui =
             end
       );
       
+      if user2_can_view_uploads gui.gui_conn.conn_user.ui_user then
       shared_iter (fun s ->
           addevent gui.gui_events.gui_shared_files (shared_num s) true
       );
@@ -467,24 +471,16 @@ let gui_initialize gui =
       ) console_messages;                
       
       gui_send gui (
-        P.Options_info (simple_options "" downloads_ini));
+        P.Options_info (simple_options "" downloads_ini (user2_is_admin gui.gui_conn.conn_user.ui_user)));
+
       networks_iter_all (fun r ->
           List.iter (fun opfile ->
-(*
-lprintf "options for net %s\n" r.network_name; 
-*)
               let prefix = r.network_shortname ^ "-" in
-              let args = simple_options prefix opfile in
-(*
-lprintf "Sending for %s\n" prefix; 
- *)
-              gui_send gui (P.Options_info args)
-(*                            lprintf "sent for %s\n" prefix; *)
-          )
-          r.network_config_file 
-      );
+              let args = simple_options prefix opfile (user2_is_admin gui.gui_conn.conn_user.ui_user) in
+              gui_send gui (P.Options_info args)) r.network_config_file);
 
 (* Options panels defined in downloads.ini *)
+      if user2_is_admin gui.gui_conn.conn_user.ui_user then
       List.iter (fun s ->
           let section = section_name s in
           List.iter (fun o ->
@@ -494,6 +490,7 @@ lprintf "Sending for %s\n" prefix;
       ) (sections downloads_ini);
 
 (* Options panels defined in users.ini *)
+      if user2_is_admin gui.gui_conn.conn_user.ui_user then
       List.iter (fun s ->
           let section = section_name s in
           List.iter (fun o ->
@@ -503,6 +500,7 @@ lprintf "Sending for %s\n" prefix;
       ) (sections users_ini);
 
 (* Options panels defined in each plugin *)
+      if user2_is_admin gui.gui_conn.conn_user.ui_user then
       networks_iter_all (fun r ->
           let prefix = r.network_shortname ^ "-" in
           List.iter (fun file ->
@@ -564,7 +562,7 @@ let gui_reader (gui: gui_record) t _ =
                     (File_add_source_event (file,c))
                     :: gui.gui_events.gui_new_events
                 ) (file_active_sources file)
-            ) !!files;
+            ) (user2_filter_files !!files gui.gui_conn.conn_user.ui_user);
             
           end
     
@@ -623,14 +621,14 @@ let gui_reader (gui: gui_record) t _ =
               ) list
           
           | P.SetOption (name, value) ->
-	      if (gui.gui_conn.conn_user == default_user) || !!enable_user_config then
+	      if user2_is_admin gui.gui_conn.conn_user.ui_user then
 		CommonInteractive.set_fully_qualified_options name value
 	      else
 	        begin
                   let o = gui.gui_conn in
                   let buf = o.conn_buf in
                   Buffer.reset buf; 
-                  Buffer.add_string buf "\nOnly 'admin' is allowed to change options\n";
+                  Buffer.add_string buf "\nYou are not allowed to change options\n";
                   gui_send gui (P.Console (
                       DriverControlers.dollar_escape o false
                         (Buffer.contents buf)))
@@ -654,8 +652,9 @@ let gui_reader (gui: gui_record) t _ =
               end
           
           | P.EnableNetwork (num, bool) ->
+	      if user2_is_admin gui.gui_conn.conn_user.ui_user then
               let n = network_find_by_num num in
-              if n.op_network_is_enabled () <> bool then
+              if network_is_enabled n <> bool then
                 (try
                     if bool then network_enable n else network_disable n;
                   with e ->
@@ -680,14 +679,14 @@ let gui_reader (gui: gui_record) t _ =
                     network_extend_search r s e)
           
           | P.KillServer -> 
-	      if gui.gui_conn.conn_user == default_user then
+	      if user2_is_admin gui.gui_conn.conn_user.ui_user then
 		CommonInteractive.clean_exit 0
 	      else
 	        begin
                   let o = gui.gui_conn in
                   let buf = o.conn_buf in
                   Buffer.reset buf; 
-                  Buffer.add_string buf "\nOnly 'admin' is allowed to kill MLDonkey\n";
+                  Buffer.add_string buf "\nYou are not allowed to kill MLDonkey\n";
                   gui_send gui (P.Console (
                       DriverControlers.dollar_escape o false
                         (Buffer.contents buf)))
@@ -699,7 +698,7 @@ let gui_reader (gui: gui_record) t _ =
               add_timer 60. (fun _ ->
                   gui_send gui (Search_waiting (search_num, 0))
               );
-              gui.gui_id_counter <- maxi gui.gui_id_counter search_num;
+              gui.gui_id_counter <- max gui.gui_id_counter search_num;
               
               let user = gui.gui_conn.conn_user in
               let query = 
@@ -724,44 +723,22 @@ let gui_reader (gui: gui_record) t _ =
               networks_iter (fun r -> 
                   if search.search_network = 0 ||
                     r.network_num = search.search_network
-                  then
-                    
-                    r.op_network_search search buf);
-(*
-        search.op_search_end_reply_handlers <- 
-          (fun _ -> send_waiting gui num search.search_waiting) ::
-search.op_search_end_reply_handlers;
-  *)
+                  then network_search r search buf);
           
           | P.Download_query (filenames, num, force) ->
-              begin
-                let r = find_result num in
-                let files = result_download r filenames force in
-                List.iter CommonInteractive.start_download files
-              end
+              let r = find_result num in
+              let files = result_download r filenames force gui.gui_conn.conn_user.ui_user in
+              List.iter CommonInteractive.start_download files
           
           | P.ConnectMore_query ->
               networks_iter network_connect_servers
           
           | P.Url url ->
-              (* changed with the new header-check code from dllink, but i
-                 didn't care about the read from file functionality that
-                 perhaps worked before:
-              
-              if not (networks_iter_until_true (fun n -> network_parse_url n url)) then
-                begin
-                  let file = File.to_string url in
-                  let lines = String2.split_simplify file '\n' in
-                  List.iter (fun line ->
-                      ignore (networks_iter_until_true (fun n -> network_parse_url n line))
-                  ) lines
-                end
-              *)
               let query_networks url = 
                 if not (networks_iter_until_true
                     (fun n ->
                        try
-                         let s,r = network_parse_url n url in r
+                         let s,r = network_parse_url n url gui.gui_conn.conn_user.ui_user in r
                        with e ->
                          lprintf "Exception %s for network %s\n"
                            (Printexc2.to_string e) (n.network_name);
@@ -795,11 +772,13 @@ search.op_search_end_reply_handlers;
                 query_networks url
           
           | P.GetUploaders ->
+              if user2_can_view_uploads gui.gui_conn.conn_user.ui_user then
               gui_send gui (P.Uploaders
                   (List2.tail_map (fun c -> client_num c) 
                   (Intmap.to_list !uploaders)))
           
           | P.GetPending ->
+              if user2_can_view_uploads gui.gui_conn.conn_user.ui_user then
               gui_send gui (P.Pending (
                   List2.tail_map (fun c -> client_num c)
                   (Intmap.to_list !CommonUploads.pending_slots_map)))
@@ -808,13 +787,13 @@ search.op_search_end_reply_handlers;
               server_remove (server_find num)
           
           | P.SaveOptions_query list ->
-              
+	      if user2_is_admin gui.gui_conn.conn_user.ui_user then
               List.iter (fun (name, value) ->
                   CommonInteractive.set_fully_qualified_options name value) list;
               DriverInteractive.save_config ()
           
           | P.RemoveDownload_query num ->
-              file_cancel (file_find num)
+              file_cancel (file_find num) gui.gui_conn.conn_user.ui_user
           
           | P.ViewUsers num -> 
               let s = server_find num in
@@ -831,7 +810,7 @@ search.op_search_end_reply_handlers;
           
           | P.SaveFile (num, name) ->
               let file = file_find num in
-              set_file_best_name file name;
+              set_file_best_name file name "" 0;
               (match file_state file with
                   FileDownloaded ->
                     file_save_as file name;
@@ -888,7 +867,7 @@ search.op_search_end_reply_handlers;
           
           | P.RenameFile (num, new_name) ->
               let file = file_find num in
-              set_file_best_name file new_name;
+              set_file_best_name file new_name "" 0;
               addevent gui.gui_events.gui_files num true
           
           | P.ConnectFriend num 
@@ -897,8 +876,8 @@ search.op_search_end_reply_handlers;
               let c = client_find num in
               client_connect c
           
-          | P.DisconnectClient num
-            ->
+          | P.DisconnectClient num ->
+              if user2_can_view_uploads gui.gui_conn.conn_user.ui_user then
               let c = client_find num in
               client_disconnect c
           
@@ -936,9 +915,9 @@ search.op_search_end_reply_handlers;
           | P.SwitchDownload (num, resume) ->
               let file = file_find num in
               if resume then
-                file_resume file          
+                file_resume file gui.gui_conn.conn_user.ui_user
               else
-                file_pause file
+                file_pause file gui.gui_conn.conn_user.ui_user
           
           | P.FindFriend user -> 
               networks_iter (fun n ->
@@ -1057,17 +1036,17 @@ search.op_search_end_reply_handlers;
   
           | NetworkMessage (num, s) ->
               let n = network_find_by_num num in
-              n.op_network_gui_message s
+              network_gui_message n s gui.gui_conn.conn_user.ui_user
                 
           | AddServer_query (num, ip, port) ->
               let n = network_find_by_num num in
 (* [CONTRIBUTE:] change the GUI protocol to transfer an address (ip/name)
   instead of just an ip. *)
               
-              let s = n.op_network_add_server (Ip.addr_of_ip ip) port in
+              let s = network_add_server n (Ip.addr_of_ip ip) port in
               server_connect s
           | RefreshUploadStats ->
-              
+              if user2_can_view_uploads gui.gui_conn.conn_user.ui_user then
               shared_iter (fun s ->
                   update_shared_info s;
               ) 
@@ -1075,6 +1054,11 @@ search.op_search_end_reply_handlers;
           | P.GetVersion ->
               gui_send gui (P.Version Autoconf.current_version)
 
+          | P.GetStats num ->
+              let n = network_find_by_num num in
+              let l = network_stat_info_list n in
+              gui_send gui (P.Stats (num, l))
+
           | P.GiftAttach (profile, version, client) ->
 	      let user, pass =
 		try
@@ -1105,16 +1089,29 @@ search.op_search_end_reply_handlers;
               let s = server_find num in
               server_rename s name
           | P.ServerSetPreferred (num, preferred) ->
-              let s = server_find num in
-              server_set_preferred s preferred
+	      if user2_is_admin gui.gui_conn.conn_user.ui_user then
+                server_set_preferred (server_find num) preferred
+	      else
+	        begin
+                  let o = gui.gui_conn in
+                  let buf = o.conn_buf in
+                  Buffer.reset buf; 
+                  Buffer.add_string buf "\nYou are not allowed to change preferred status\n";
+                  gui_send gui (P.Console (
+                      DriverControlers.dollar_escape o false
+                        (Buffer.contents buf)))
+		end
 
   with 
     Failure s ->
       gui_send gui (Console (Printf.sprintf "Failure: %s\n" s))
+  | Torrent_can_not_be_used ->
+      gui_send gui (Console (Printf.sprintf "\nError: This torrent does not have valid tracker URLs\n"))
+  | Torrent_already_exists ->
+      gui_send gui (Console (Printf.sprintf "\nError: This torrent is already in download queue\n"))
   | e ->
-      gui_send gui (Console (    
-          Printf.sprintf "from_gui: exception %s for message %s\n" (
-            Printexc2.to_string e) (GuiProto.string_of_from_gui t)))
+      gui_send gui (Console (Printf.sprintf "from_gui: exception %s for message %s\n"
+	(Printexc2.to_string e) (GuiProto.string_of_from_gui t)))
 
 let gui_events () = 
   {
@@ -1174,7 +1171,8 @@ let gui_handler t event =
     TcpServerSocket.CONNECTION (s, Unix.ADDR_INET (from_ip, from_port)) ->
       let from_ip = Ip.of_inet_addr from_ip in
       if not !verbose_no_login then lprintf_nl "GUI connection from %s" (Ip.to_string from_ip);
-      if Ip.matches from_ip !!allowed_ips then 
+      (match Ip_set.match_ip !allowed_ips_set from_ip with
+      |	Some br ->
         
         let module P = GuiProto in
         let token = create_token unlimited_connection_manager in
@@ -1201,11 +1199,10 @@ let gui_handler t event =
             close sock Closed_for_overflow);
         (* sort GUIs in increasing order of their num *)
         
-      else begin
+      | None ->
           if not !verbose_no_login then lprintf_nl "GUI connection from %s rejected (see allowed_ips setting)"
             (Ip.to_string from_ip);
-          Unix.close s
-        end
+          Unix.close s)
   | _ -> ()
         
 let gift_handler t event = 
@@ -1213,7 +1210,8 @@ let gift_handler t event =
     TcpServerSocket.CONNECTION (s, Unix.ADDR_INET (from_ip, from_port)) ->
       let from_ip = Ip.of_inet_addr from_ip in
       lprintf "Gift: Connection from %s\n" (Ip.to_string from_ip);
-      if Ip.matches from_ip !!allowed_ips then 
+      (match Ip_set.match_ip !allowed_ips_set from_ip with
+      | Some br ->
         
         let module P = GuiProto in
         let token = create_token unlimited_connection_manager in
@@ -1239,11 +1237,10 @@ let gift_handler t event =
             close sock Closed_for_overflow);
         (* sort GUIs in increasing order of their num *)
         
-      else begin
+      | None ->
           lprintf "Connection from IP %s not allowed\n"
             (Ip.to_string from_ip);
-          Unix.close s
-        end
+          Unix.close s)
   | _ -> ()
 
       
