@@ -34,18 +34,6 @@ open CommonNetwork
 
 open DriverInterface
 
-module Dp500 = DriverLink.DP500(struct
-
-      module CommonTypes = CommonTypes
-      module CommonFile = CommonFile
-      module CommonOptions = CommonOptions
-
-      let incoming_directory () =
-        (CommonComplexOptions.incoming_files ()).shdir_dirname
-      let files () = !!CommonComplexOptions.files
-
-    end)
-
 open Gettext (* open last  as most modules redefine _s and _b *)
 
 let _s x = _s "DriverMain" x
@@ -59,8 +47,6 @@ let lprintf_nl fmt =
 let lprintf_n fmt =
   lprintf2 log_prefix fmt
 
-
-let keep_console_output = ref false
 let pid = ref ""
 
 let do_daily () =
@@ -68,7 +54,6 @@ let do_daily () =
 
 let minute_timer () =
   DriverInteractive.hdd_check ();
-  CommonShared.shared_check_files ();
   CommonUploads.upload_credit_timer ();
   CommonInteractive.force_download_quotas ();
   CommonResult.dummy_result.result_time <- last_time ();
@@ -98,7 +83,7 @@ let second_timer timer =
       update_link_stats ()
     with e ->
         lprintf_nl (_b "Exception %s") (Printexc2.to_string e));
-	  (try
+  (try
      CommonUploads.refill_upload_slots ()
    with e ->
         lprintf_nl (_b "Exception %s") (Printexc2.to_string e));
@@ -108,8 +93,16 @@ let second_timer timer =
 
 let start_interfaces () =
 
+  (* option_hook(s) are not called when ini files are created the first time
+     force re-load of allowed_ips to call option_hook which fills the IP blocklist *)
+  ( 
+  match !created_new_base_directory with
+    None -> ()
+  | Some dir -> allowed_ips =:= !!allowed_ips
+  );
 
-  if !!http_port <> 0 then begin try
+  if !!http_port <> 0 then begin 
+      try
         ignore (DriverControlers.create_http_handler ());
       with e ->
           lprintf_nl (_b "Exception %s while starting HTTP interface")
@@ -119,16 +112,6 @@ let start_interfaces () =
   ignore (find_port  "telnet server" !!telnet_bind_addr
       telnet_port DriverControlers.telnet_handler);
 
-  Dp500.start ();
-
-  if !!chat_port <> 0 then begin
-      ignore (find_port "chat server" !!chat_bind_addr
-          chat_port DriverControlers.chat_handler);
-      try
-        CommonChat.send_hello ()
-      with _ -> if !verbose then lprintf (_b "CommonChat.send_hello failed");
-    end;
-
   gui_server_sock := find_port "gui server"  !!gui_bind_addr
     gui_port gui_handler;
   if !!gift_port <> 0 then
@@ -235,7 +218,6 @@ let load_config () =
   (try
       Options.load downloads_ini;
       Options.load users_ini;
-      ignore (DriverInteractive.verify_user_admin ());
       DriverInteractive.hdd_check ()
     with e ->
         lprintf_nl "Exception %s during options load" (Printexc2.to_string e);
@@ -305,13 +287,13 @@ let load_config () =
           exit 0),
       _s " : display information on the implementations";
       "-stdout", Arg.Unit (fun _ ->
-          keep_console_output := true;
-          log_to_file stdout;
+	  lprintf_original_output := (Some stdout);
+          log_to_file stdout
       ),
        _s ": keep output to stdout after startup";
        "-stderr", Arg.Unit (fun _ ->
-           keep_console_output := true;
-           log_to_file stderr;
+	  lprintf_original_output := (Some stderr);
+          log_to_file stderr
        ),
        _s ": keep output to stderr after startup";
       "-daemon", Arg.Unit (fun _ ->
@@ -323,6 +305,23 @@ let load_config () =
       "-pid", Arg.String (fun s -> pid := s;
       ),
        _s ": directory for pid file";
+      "-useradd", Arg.Rest (fun s ->
+        (match String2.split s ' ' with
+        | user :: pass :: _ ->
+            if user2_user_exists user then
+              begin
+                user2_user_set_password (user2_user_find user) pass;
+                Printf.printf "%sPassword of user %s changed\n%!" (log_time ()) user
+              end
+            else
+              begin
+                user2_user_add user (Md4.Md4.string pass) ();
+                Printf.printf "%sUser %s added\n%!" (log_time ()) user
+              end;
+            Options.save_with_help_private users_ini;
+            Printf.printf "%sSaved changes to users.ini\n%!" (log_time ())
+        | _ -> raise (Arg.Bad "invalid syntax"));
+          exit 0), _s "\"<user> <pass>\" : create user/change password";
     ] @
       !more_args
       @
@@ -331,7 +330,7 @@ let load_config () =
 (*      Files.dump_file file; exit 0 *)
   ) "";
 
-  if not ini_files_exist && not !keep_console_output then log_file =:= "mlnet.log";
+  if not ini_files_exist && not (keep_console_output ()) then log_file =:= "mlnet.log";
 
   (**** CREATE DIRS   ****)
 
@@ -358,15 +357,18 @@ let _ =
       CommonGlobals.exit_properly 71
     end;
 
-  (
-    let hostname = "www.mldonkey.net" in
-    try
-      ignore(Ip.from_name hostname);
-      DriverInteractive.dns_works := true
-    with e ->
-        lprintf (_b "\nDNS resolution does not work! Looking up %s failed with %s.")
-           hostname (Printexc2.to_string e);
-	lprintf "
+  ( let resolve_name hostname =
+      try
+	ignore (Ip.from_name hostname);
+	true
+      with _ -> false
+    in
+    let hostnames =
+      ["www.mldonkey.org"; "mldonkey.sf.net"; "www.mldonkey.net"; "www.google.com"]
+    in
+    DriverInteractive.dns_works := List.exists resolve_name hostnames;
+
+    if not !DriverInteractive.dns_works then lprintf "
 The core therefore is unable to get eDonkey serverlists and loading
 .torrent files via dllink from websites is also impossible.
 If you are using MLDonkey in a chroot environment you should
@@ -384,7 +386,9 @@ or getting a binary compiled with glibc %s.\n\n")
           real_glibc_version Autoconf.glibc_version Autoconf.glibc_version
   );
   (
-    if Autoconf.magic then
+    if Autoconf.magic then begin
+      (if Sys.file_exists "./magic/magic" then
+         try Unix.putenv "MAGIC" "./magic/magic" with _ -> ());
       if Magic.M.magic_works () then
         begin
 	  Autoconf.magic_works := true;
@@ -395,7 +399,11 @@ or getting a binary compiled with glibc %s.\n\n")
 	  Autoconf.magic_works := false;
           lprintf_nl (_b "Libmagic file-type recognition database not present")
 	end
+      end
   );
+  if not !Charset.conversion_enabled then
+    lprintf_nl (_b "Self-test failed, charset conversion disabled.");
+
   load_config ();
   
   add_infinite_option_timer download_sample_rate CommonFile.sample_timer;
@@ -403,6 +411,7 @@ or getting a binary compiled with glibc %s.\n\n")
 (*  lprintf "(1) CommonComplexOptions.load\n"; *)
   CommonComplexOptions.load ();
   CommonUploads.load ();
+  CommonStats.load ();
 
 (*  lprintf "(2) CommonComplexOptions.load done\n"; *)
   begin
@@ -413,19 +422,23 @@ or getting a binary compiled with glibc %s.\n\n")
     save_results =:= old_save_results;
   end;
 
-(*  lprintf "(3) networks_iter load_complex_options\n"; *)
-  lprintf_nl (_b "Check http://www.mldonkey.net/ for updates");
+  CommonGlobals.is_startup_phase := false;
+
+  lprintf_nl (_b "Check http://www.mldonkey.org for updates");
   networks_iter (fun r -> network_load_complex_options r);
-  lprintf (_b "%senabling networks: ") (log_time ());
+  lprintf_nl (_b "enabling networks: ");
   networks_iter (fun r ->
-(*      lprintf "(4) networks_iter enabling\n"; *)
-      lprintf (_b "%s ") r.network_name;
+      lprintf_nl (_b "---- enabling %s ----") r.network_name;
       network_enable r;
+      List.iter (fun (p,s) -> if p <> 0 then lprintf_nl "using port %d (%s)" p s) (network_ports r);
 (* are there drawbacks to start recover_temp unconditionally here ? *)
       if !!recover_temp_on_startup then
         network_recover_temp r;
   );
-  lprintf (_b "\n%sdisabled networks: ") (log_time ());
+  lprintf_nl (_b "---- enabling interfaces ----");
+  List.iter (fun (p,s) -> if p <> 0 then lprintf_nl "using port %d (%s)" p s)
+    (network_ports (network_find_by_name "Global Shares"));
+  lprintf (_b "%s[dMain] disabled networks: ") (log_time ());
   let found = ref false in
     networks_iter_all (fun r ->
         if not (network_is_enabled r) then
@@ -450,7 +463,31 @@ or getting a binary compiled with glibc %s.\n\n")
   add_infinite_timer 10. ten_second_timer;
   add_infinite_timer 3600. hourly_timer;
   add_infinite_timer 0.1 CommonUploads.upload_download_timer;
+  add_infinite_timer !!buffer_writes_delay (fun _ -> Unix32.flush ());
 
+  add_infinite_timer ((float_of_int !!share_scan_interval) *. 60.)
+    (fun _ -> CommonShared.shared_check_files ());
+  CommonShared.shared_check_files ();
+
+  history_timeflag := (Unix.time()); 
+  update_download_history (); 
+  update_upload_history ();
+  history_h_timeflag := (Unix.time()); 
+  update_h_download_history (); 
+  update_h_upload_history ();
+  history_size_for_h_graph := history_size * !!html_mods_vd_gfx_h_intervall / 60;
+  history_h_step := 60 * !!html_mods_vd_gfx_h_intervall;
+	
+  add_infinite_timer (float_of_int history_step) (fun timer -> 
+    history_timeflag := (Unix.time()); 
+    update_download_history (); 
+    update_upload_history ());
+
+  add_infinite_timer (float_of_int !history_h_step) (fun timer -> 
+    history_h_timeflag := (Unix.time()); 
+    update_h_download_history (); 
+    update_h_upload_history ());
+		
   if Autoconf.system = "mingw" then
     add_infinite_timer 1. (fun timer ->
         MlUnix.set_console_title (DriverInteractive.console_topic ()));
@@ -465,7 +502,7 @@ or getting a binary compiled with glibc %s.\n\n")
   Options.prune_file downloads_ini;
   Options.prune_file users_ini;
 (*  Options.prune_file downloads_expert_ini; *)
-  add_timer 5. (fun _ -> try CommonWeb.load_web_infos true false with _ -> ());
+  add_timer 1. (fun _ -> try CommonWeb.load_web_infos true false with _ -> ());
   lprintf_nl  (_b "To command: telnet %s %d")
 	(if !!telnet_bind_addr = Ip.any then "127.0.0.1"
 		else Ip.to_string !!telnet_bind_addr)  !!telnet_port;
@@ -477,7 +514,7 @@ or getting a binary compiled with glibc %s.\n\n")
 	(if !!gui_bind_addr = Ip.any then "127.0.0.1"
 		else Ip.to_string !!gui_bind_addr)  !!gui_port;
   lprintf_nl  (_b "If you connect from a remote machine adjust allowed_ips");
-  if Autoconf.system = "cygwin" && not !keep_console_output then lprintf (_b "%s") win_message;
+  if Autoconf.system = "cygwin" && not (keep_console_output ()) then lprintf (_b "%s") win_message;
 
   add_init_hook (fun _ ->
       if not !gui_included && ( !!start_gui || !!ask_for_gui ) then
@@ -523,37 +560,37 @@ or getting a binary compiled with glibc %s.\n\n")
 
   if not Autoconf.windows then
     MlUnix.set_signal  Sys.sigchld
-      (Sys.Signal_handle (fun _ -> if !verbose then lprintf_nl "Received SIGCHLD, doing nothing"));
+      (Sys.Signal_handle (fun _ -> if !verbose then lprintf_nl (_b "Received SIGCHLD, doing nothing")));
 
   if not Autoconf.windows then
     MlUnix.set_signal  Sys.sighup
-      (Sys.Signal_handle (fun _ ->
-   lprintf_nl "Received SIGHUP, closing all files/sockets";
-         BasicSocket.close_all ();
-	 Unix32.close_all ()
+      (Sys.Signal_handle (fun _ -> lprintf_nl (_b "Received SIGHUP, closing all files and client/server sockets");
+         networks_iter (fun r -> CommonNetwork.network_reset r); (* stop_all_bt *)
+	 CommonServer.disconnect_all_servers ();
+	 CommonClient.disconnect_all_clients ();
+	 Unix32.close_all () (* close all files *)
          ));
 
   if not Autoconf.windows then
     MlUnix.set_signal  Sys.sigpipe
-      (Sys.Signal_handle (fun _ -> if !verbose then lprintf_nl "Received SIGPIPE, doing nothing"));
+      (Sys.Signal_handle (fun _ -> if !verbose then lprintf_nl (_b "Received SIGPIPE, doing nothing")));
 
   MlUnix.set_signal  Sys.sigint
-    (Sys.Signal_handle (fun _ -> lprintf_nl "Received SIGINT, stopping MLDonkey...";
+    (Sys.Signal_handle (fun _ -> lprintf_nl (_b "Received SIGINT, stopping MLDonkey...");
         CommonInteractive.clean_exit 0));
 
   MlUnix.set_signal  Sys.sigterm
-    (Sys.Signal_handle (fun _ -> lprintf_nl "Received SIGTERM, stopping MLDonkey...";
+    (Sys.Signal_handle (fun _ -> lprintf_nl (_b "Received SIGTERM, stopping MLDonkey...");
         CommonInteractive.clean_exit 0));
 
   if not Autoconf.windows then
   MlUnix.set_signal  Sys.sigusr1
-    (Sys.Signal_handle (fun _ -> lprintf_nl "Received SIGUSR1, saving options...";
+    (Sys.Signal_handle (fun _ -> lprintf_nl (_b "Received SIGUSR1, saving options...");
         DriverInteractive.save_config ()));
 
   if not Autoconf.windows then
   MlUnix.set_signal  Sys.sigusr2
-    (Sys.Signal_handle (fun _ ->
-  lprintf_n "Received SIGUSR2, starting garbage collection...";
+    (Sys.Signal_handle (fun _ -> lprintf_n (_b "Received SIGUSR2, starting garbage collection...");
 	Gc.compact ();
 	lprintf " finished";
 	lprint_newline ()));
@@ -561,27 +598,27 @@ or getting a binary compiled with glibc %s.\n\n")
   if !verbose then lprintf_nl (_b "Activated system signal handling")
 
 let _ =
-  let security_space_filename = "config_files_space.tmp" in
+  let security_space_oc = ref None in
   begin
 (* Create a 'config_files_security_space' megabytes file to protect some space
 for config files at the end. *)
     try
-      let security_space_fd = Unix32.create_rw security_space_filename in
-      let _ =
+      let oc = Unix.openfile security_space_filename [Unix.O_WRONLY; Unix.O_CREAT] 0o600 in
         let len = 32768 in
-        let len64 = Int64.of_int len in
-        let s = String.create len in
+        let s = String.make len ' ' in
         let pos = ref zero in
         for i = 1 to !!config_files_security_space do
           for j = 1 to 32 do (* 32 = 1 MB / 32kB *)
-            Unix32.write security_space_fd !pos s 0 len;
-            pos := !pos ++ len64
+	    ignore(Unix2.c_seek64 oc !pos Unix.SEEK_SET);
+            Unix2.really_write oc s 0 len;
+            pos := !pos ++ (Int64.of_int len)
           done
-        done
-      in
-      Unix32.close security_space_fd;
-    with _ ->
-        lprintf_nl (_b "Cannot create Security space file:");
+        done;
+	ignore(Unix2.c_seek64 oc zero Unix.SEEK_SET);
+	(try Unix.lockf oc Unix.F_LOCK (!!config_files_security_space * 1024 * 1024) with _ -> ());
+	security_space_oc := Some oc
+    with e ->
+        lprintf_nl (_b "Cannot create Security space file: %s") (Printexc2.to_string e);
         lprintf_nl (_b " not enough space on device or bad permissions");
         lprintf_nl (_b "Exiting...");
         exit 73;
@@ -589,9 +626,6 @@ for config files at the end. *)
   Unix32.external_start ();
 
   (
-    let pid_filename =
-      Printf.sprintf "%s.pid" (Filename.basename Sys.argv.(0))
-    in
     let pid_file, s =
         Filename.concat !pid pid_filename,
 	Printf.sprintf "%s\n" (string_of_int(Unix.getpid()))
@@ -604,11 +638,10 @@ for config files at the end. *)
 
 (* When a core is spawned from a gui, the only way to know the startup has
    succeeded is the string token "Core started". *)
-  if not !keep_console_output then
+  if not (keep_console_output ()) then
     begin
       try
-        Printf.fprintf Pervasives.stderr "%sCore started\n" (log_time ());
-        Pervasives.flush Pervasives.stderr;
+        Printf.eprintf "%s[dMain] Core started\n%!" (log_time ());
       with _ -> ()
     end;
 
@@ -628,7 +661,10 @@ for config files at the end. *)
       DriverGraphics.G.remove_files ();
       (* In case we have no more space on filesystem for
          config files, remove the security space file *)
-      Sys.remove security_space_filename;
+      (match !security_space_oc with
+	None -> ()
+      | Some oc -> Unix.close oc);
+      (try Sys.remove security_space_filename with _ -> ());
       CommonComplexOptions.allow_saving_ini_files := true;
       DriverInteractive.save_config ();
       CommonComplexOptions.save_sources ();
@@ -638,7 +674,7 @@ for config files at the end. *)
       lprintf_nl (_b "Core stopped")
     );
 
-  if not !keep_console_output then
+  if not (keep_console_output ()) then
     if !!log_file = "" then 
       begin
         lprintf_nl (_b "Option log_file is empty, disable logging completely...");
