@@ -47,6 +47,7 @@ let string_color_of_state state =
   | Connected_initiating -> M.fT_tx_initiating, Some !!O.color_not_connected
   | Connected 0 -> M.fT_tx_queued, Some !!O.color_connected
   | Connected n -> Printf.sprintf Gui_messages.fT_tx_ranked n, Some !!O.color_connected
+  | ServerFull -> "", None
   | NotConnected (_,n) -> 
       if n = -1 then
         "", None
@@ -72,7 +73,7 @@ let shorten maxlen s =
   if len > maxlen then
     (String.sub s 0 (maxlen-3)) ^ "..."
   else if s = "" then
-      "http://www.mldonkey.net/"
+      "http://www.mldonkey.org/"
   else s
 
 let state_pix state =
@@ -82,14 +83,13 @@ let state_pix state =
       | NewHost -> O.gdk_pix M.o_xpm_connect_n
       | Connected_initiating -> O.gdk_pix M.o_xpm_connect_m
       | Connected n -> O.gdk_pix M.o_xpm_connect_y
+      | ServerFull -> O.gdk_pix M.o_xpm_connect_n
       | NotConnected (_,n) -> O.gdk_pix M.o_xpm_connect_n
       | RemovedHost -> O.gdk_pix M.o_xpm_removedhost
       | BlackListedHost -> O.gdk_pix M.o_xpm_blacklistedhost
 
 let client_pix c =
-  match c.client_files with
-    Some l -> O.gdk_pix M.o_xpm_files_listed
-  | _ -> state_pix c.client_state
+  state_pix c.client_state
 
 
 let type_pix t =
@@ -425,17 +425,20 @@ class box_friends box_files friend_tab =
         client_type = c.gclient_type;
         client_tags = c.gclient_tags;
         client_name = c.gclient_name;
-        client_files = c.gclient_files;
         client_rating = c.gclient_rating;
         client_chat_port = 0;
         client_connect_time = c.gclient_connect_time;
         client_software = c.gclient_software;
+        client_os = None;
         client_release = c.gclient_release;
         client_emulemod = c.gclient_emulemod;
-        client_downloaded = c.gclient_downloaded;
-        client_uploaded = c.gclient_uploaded;
+        client_total_downloaded = c.gclient_downloaded;
+        client_total_uploaded = c.gclient_uploaded;
+        client_session_downloaded = 0L;
+        client_session_uploaded = 0L;
         client_upload = c.gclient_upload;
 	client_sui_verified = None;
+        client_file_queue = [];
 (*        client_sock_addr = c.gclient_sock_addr;*)
       }
     
@@ -448,14 +451,14 @@ class box_friends box_files friend_tab =
         gclient_type = c.client_type;
         gclient_tags = c.client_tags;
         gclient_name = c.client_name;
-        gclient_files = c.client_files;
+        gclient_files = None;
         gclient_rating = c.client_rating;
         gclient_connect_time = (BasicSocket.last_time () - c.client_connect_time);
         gclient_software = c.client_software;
         gclient_release = c.client_release;
         gclient_emulemod = c.client_emulemod;
-        gclient_downloaded = c.client_downloaded;
-        gclient_uploaded = c.client_uploaded;
+        gclient_downloaded = c.client_total_downloaded;
+        gclient_uploaded = c.client_total_uploaded;
         gclient_upload = c.client_upload;
         gclient_sock_addr = string_of_kind c.client_kind;
         gclient_net_pixmap =
@@ -470,7 +473,7 @@ class box_friends box_files friend_tab =
       }
     
     method update_friend f_new =
-      if client_browsed_tag land f_new.client_type = 0 then
+      if (client_friend_tag lor client_contact_tag) land f_new.client_type = 0 then
         self#h_remove_friend f_new.client_num
       else
       try
@@ -491,8 +494,8 @@ class box_friends box_files friend_tab =
           f.gclient_software <- f_new.client_software;
           f.gclient_release <- f_new.client_release;
           f.gclient_emulemod <- f_new.client_emulemod;
-          f.gclient_downloaded <- f_new.client_downloaded;
-          f.gclient_uploaded <- f_new.client_uploaded;
+          f.gclient_downloaded <- f_new.client_total_downloaded;
+          f.gclient_uploaded <- f_new.client_total_uploaded;
           f.gclient_upload <- f_new.client_upload;
           f.gclient_sock_addr <- string_of_kind f_new.client_kind;
           if box_friends_is_visible then self#update_row f row
@@ -527,7 +530,7 @@ class box_friends box_files friend_tab =
     method update_friend_type (num, friend_kind) =
       try
         let (row, fi) = self#find_client num in
-        if client_browsed_tag land friend_kind = 0 then
+        if (client_friend_tag lor client_contact_tag) land friend_kind = 0 then
           self#h_remove_friend num
           else begin
             fi.gclient_type <- friend_kind;
@@ -632,17 +635,20 @@ class box_list friend_tab =
         client_type = c.gclient_type;
         client_tags = c.gclient_tags;
         client_name = c.gclient_name;
-        client_files = c.gclient_files;
         client_rating = c.gclient_rating;
         client_chat_port = 0;
         client_connect_time = c.gclient_connect_time;
         client_software = c.gclient_software;
+        client_os = None;
         client_release = c.gclient_release;
         client_emulemod = c.gclient_emulemod;
-        client_downloaded = c.gclient_downloaded;
-        client_uploaded = c.gclient_uploaded;
+        client_total_downloaded = c.gclient_downloaded;
+        client_total_uploaded = c.gclient_uploaded;
+        client_session_downloaded = 0L;
+        client_session_uploaded = 0L;
         client_upload = c.gclient_upload;
 	client_sui_verified = None;
+        client_file_queue = [];
 (*        client_sock_addr = string_of_kind c.gclient_kind; *)
       }
 
@@ -655,14 +661,14 @@ class box_list friend_tab =
         gclient_type = c.client_type;
         gclient_tags = c.client_tags;
         gclient_name = c.client_name;
-        gclient_files = c.client_files;
+        gclient_files = None;
         gclient_rating = c.client_rating;
         gclient_connect_time =  (BasicSocket.last_time () - c.client_connect_time);
         gclient_software = c.client_software;
         gclient_release = c.client_release;
         gclient_emulemod = c.client_emulemod;
-        gclient_downloaded = c.client_downloaded;
-        gclient_uploaded = c.client_uploaded;
+        gclient_downloaded = c.client_total_downloaded;
+        gclient_uploaded = c.client_total_uploaded;
         gclient_upload = c.client_upload;
         gclient_sock_addr = string_of_kind c.client_kind;
         gclient_net_pixmap = 
@@ -707,9 +713,9 @@ class box_list friend_tab =
                     c.gclient_kind <- c_new.client_kind;
                     c.gclient_tags <- c_new.client_tags;
                     c.gclient_software <- c_new.client_software;
-                    c.gclient_downloaded <- c_new.client_downloaded;
+                    c.gclient_downloaded <- c_new.client_total_downloaded;
                     c.gclient_emulemod <- c_new.client_emulemod;
-                    c.gclient_uploaded <- c_new.client_uploaded;
+                    c.gclient_uploaded <- c_new.client_total_uploaded;
                     c.gclient_upload <- c_new.client_upload;
                     c.gclient_sock_addr <- string_of_kind c_new.client_kind;
             (if icons_are_used && (c.gclient_type <> c_new.client_type) 
