@@ -53,6 +53,16 @@ let lprintf' fmt =
 let (dialogs : (int * GuiTemplates.chat_buffer) list ref) = ref []
 let source_has_file = ref []
 
+module H = Weak.Make(struct
+      type t = g_file_tree
+
+      let hash ft = Hashtbl.hash ft.g_file_tree_num
+      let equal x y  = x.g_file_tree_num = y.g_file_tree_num
+
+    end)
+
+let ft_by_num = H.create 107
+
 (*************************************************************************)
 (*                                                                       *)
 (*                         Global variables                              *)
@@ -74,6 +84,104 @@ let folder_closed () =
     then Some (A.get_icon ~icon:M.icon_stock_directory ~size:A.SMALL ())
     else None
 
+let dummy_ft =
+  {
+   g_file_tree_num  = 0;
+   g_file_tree_name = "";
+   g_file_tree_list = [];
+   g_file_tree_pixb = None;
+  }
+
+(*************************************************************************)
+(*                                                                       *)
+(*                         friend_num                                    *)
+(*                                                                       *)
+(*************************************************************************)
+
+let friend_num key =
+  try int_of_string key with _ -> raise Not_found
+
+(*************************************************************************)
+(*                                                                       *)
+(*                         friend_of_key                                 *)
+(*                                                                       *)
+(*************************************************************************)
+
+let friend_of_key key =
+  try
+    let num = friend_num key in
+    Hashtbl.find G.sources num
+  with _ -> raise Not_found
+
+(*************************************************************************)
+(*                                                                       *)
+(*                         keys_to_friends                               *)
+(*                                                                       *)
+(*************************************************************************)
+
+let keys_to_friends keys =
+  let l = ref [] in
+  List.iter (fun k ->
+    try
+      let s = friend_of_key k in
+      l := s :: !l
+    with _ -> ()) keys;
+  !l
+
+(*************************************************************************)
+(*                                                                       *)
+(*                         friend_key                                    *)
+(*                                                                       *)
+(*************************************************************************)
+
+let friend_key friend_num =
+  Printf.sprintf "%d" friend_num
+
+(*************************************************************************)
+(*                                                                       *)
+(*                         folder_num                                    *)
+(*                                                                       *)
+(*************************************************************************)
+
+let folder_num key =
+  try int_of_string key with _ -> raise Not_found
+
+(*************************************************************************)
+(*                                                                       *)
+(*                         folder_of_key                                 *)
+(*                                                                       *)
+(*************************************************************************)
+
+let folder_of_key key =
+  try
+    let num = folder_num key in
+    H.find ft_by_num {dummy_ft with g_file_tree_num = num}
+  with _ -> raise Not_found
+
+(*************************************************************************)
+(*                                                                       *)
+(*                         keys_to_folders                               *)
+(*                                                                       *)
+(*************************************************************************)
+
+let keys_to_folders keys =
+  let l = ref [] in
+  List.iter (fun k ->
+    try
+      let s = friend_of_key k in
+      l := s :: !l
+    with _ -> ()) keys;
+  !l
+
+(*************************************************************************)
+(*                                                                       *)
+(*                         folder_key                                    *)
+(*                                                                       *)
+(*************************************************************************)
+
+let folder_key ft_num =
+  Printf.sprintf "%d" ft_num
+
 (*************************************************************************)
 (*                                                                       *)
 (*                         Templates                                     *)
@@ -85,10 +193,9 @@ module Friends = GuiTemplates.Gview(struct
   module Column = GuiColumns.Friend
 
   type item = source_info
-  type key = int
 
   let columns = O.friends_columns
-  let get_key = (fun s -> s.source_num)
+  let get_key = (fun s -> friend_key s.source_num)
   let module_name = "Friends"
 
 end)
@@ -152,6 +259,7 @@ class g_friend () =
 (*************************************************************************)
 
     method content (col : GTree.view_column) c =
+      let autosize = match col#sizing with `AUTOSIZE -> true | _ -> false in
       match c with
           Col_friend_name ->
             begin
@@ -166,9 +274,12 @@ class g_friend () =
                 end;
               let renderer = GTree.cell_renderer_text [`XALIGN 0.] in
               col#pack ~expand:false renderer;
-              col#set_cell_data_func renderer
-               (fun model row ->
-                  match !view_context with
+              col#add_attribute renderer "foreground" friend_color;
+              if autosize
+                then col#add_attribute renderer "text" friend_name
+                else col#set_cell_data_func renderer
+                 (fun model row ->
+                    match !view_context with
                       Some context when col#width > 0 ->
                         begin
                           let width =
@@ -180,9 +291,8 @@ class g_friend () =
                           let s = GuiTools.fit_string_to_pixels name ~context ~pixels:width in
                           renderer#set_properties [ `TEXT s ]
                         end
-                      | _ -> renderer#set_properties [ `TEXT "" ]
-              );
-              col#add_attribute renderer "foreground" friend_color;
+                    | _ -> renderer#set_properties [ `TEXT "" ]
+                 )
             end
 
         | Col_friend_network ->
@@ -206,10 +316,14 @@ class g_friend () =
 (*                                                                       *)
 (*************************************************************************)
 
-    method sort_items c s1 s2 =
-      match c with
+    method sort_items c k1 k2 =
+      try
+        let s1 = friend_of_key k1 in
+        let s2 = friend_of_key k2 in
+        match c with
           Col_friend_network -> compare s1.source_network s2.source_network
         | Col_friend_name -> compare s1.source_name s2.source_name
+      with _  -> 0
 
 (*************************************************************************)
 (*                                                                       *)
@@ -218,34 +332,38 @@ class g_friend () =
 (*************************************************************************)
 
     method force_update_icons () =
-      List.iter (fun s ->
-        try
-          let (row, _) = self#find_item s.source_num in 
-          store#set ~row ~column:friend_network_pixb (Mi.network_pixb s.source_network ~size:A.SMALL ());
-          store#set ~row ~column:friend_type_pixb (Mi.source_type_to_icon s.source_type ~size:A.SMALL);
-          store#set ~row ~column:friend_state_pixb (Mi.client_state_to_icon false ~size:A.SMALL);
-          let rec iter items =
-            match items with
-                [] -> ()
-              | (GTreeDirectory tree) :: tail ->
-                   begin
-                     tree.g_file_tree_pixb <- folder_closed ();
-                     iter tail
-                   end
-              | (GTreeFile r) :: tail ->
-                  begin
-                    r.res_network_pixb <- Mi.network_pixb r.res_num ~size:A.SMALL ();
-                    r.res_name_pixb <- Mi.file_type_of_name r.res_name ~size:A.SMALL;
-                    iter tail
-                  end
-          in
-          match s.source_files with
-              None -> ()
-            | Some ft ->
+      let f k row =
+        let s = friend_of_key k in
+        store#set ~row ~column:friend_network_pixb (Mi.network_pixb s.source_network ~size:A.SMALL ());
+        store#set ~row ~column:friend_type_pixb (Mi.source_type_to_icon s.source_type ~size:A.SMALL);
+        store#set ~row ~column:friend_state_pixb (Mi.client_state_to_icon false ~size:A.SMALL);
+        let rec iter items =
+          match items with
+              [] -> ()
+            | (GTreeDirectory tree) :: tail ->
+                 begin
+                   tree.g_file_tree_pixb <- folder_closed ();
+                   iter tail
+                 end
+            | (GTreeFile r) :: tail ->
                 begin
-                  ft.g_file_tree_pixb <- folder_closed ();
-                  iter ft.g_file_tree_list
+                  r.res_network_pixb <- Mi.network_pixb r.res_num ~size:A.SMALL ();
+                  r.res_name_pixb <- Mi.file_type_of_name r.res_name ~size:A.SMALL;
+                  iter tail
                 end
+        in
+        match s.source_files with
+            None -> ()
+          | Some ft ->
+              begin
+                ft.g_file_tree_pixb <- folder_closed ();
+                iter ft.g_file_tree_list
+              end
+      in
+      List.iter (fun k ->
+        try
+          let row = self#find_row k in
+          Gaux.may ~f:(f k) row
         with _ -> ()
       ) (self#all_items ())
 
@@ -264,10 +382,9 @@ module FriendFolders = GuiTemplates.Gview(struct
   module Column = GuiColumns.Directory
 
   type item = g_file_tree
-  type key = int
 
   let columns = O.friends_dirs_columns
-  let get_key = (fun ft -> ft.g_file_tree_num)
+  let get_key = (fun ft -> folder_key ft.g_file_tree_num)
   let module_name = "FriendFolders"
 
 end)
@@ -326,8 +443,7 @@ class g_folder () =
 (*                                                                       *)
 (*************************************************************************)
 
-    method sort_items _ ft1 ft2 =
-      compare ft1.g_file_tree_name ft2.g_file_tree_name
+    method sort_items _ k1 k2 = 0
 
   end
 
@@ -412,18 +528,22 @@ let add_chat (notebook : GPack.notebook) (s : source_info) (chat : GuiTemplates.
 (*                                                                       *)
 (*************************************************************************)
 
-let remove sel () = 
-  List.iter (fun s ->
-    GuiCom.send (RemoveFriend s.source_num);
-    (* there is a #bug here or is it volontary ? The core doesn't send back the updated client_type !!! *)
-    GuiCom.send (GuiProto.GetClient_info s.source_num)
+let remove sel () =
+  List.iter (fun k ->
+    try
+      GuiCom.send (RemoveFriend (friend_num k));
+      (* there is a #bug here or is it volontary ? The core doesn't send back the updated client_type !!! *)
+      GuiCom.send (GuiProto.GetClient_info (friend_num k))
+    with _ -> ()
   ) sel
 
 let remove_all_friends () = 
   GuiCom.send RemoveAllFriends;
   (* there is a #bug here or is it volontary ? The core doesn't send back the updated client_type !!! *)
-  List.iter (fun s ->
-    GuiCom.send (GuiProto.GetClient_info s.source_num)
+  List.iter (fun k ->
+    try
+      GuiCom.send (GuiProto.GetClient_info (friend_num k))
+    with _ -> ()
   ) (friendstore#all_items ())
 
 
@@ -435,29 +555,35 @@ let find_friend () =
 let on_entry_return num s =
   GuiCom.send (MessageToClient (num, s))
 
-let download_selected_dir (sel : g_file_tree list) () =
+let download_selected_dir sel () =
   match sel with
       [] -> ()
-    | ft :: tail ->
+    | k :: tail ->
         begin
-          let files = Mi.list_files ft in
-          let len = List.length files in
-          match (GToolbox.question_box
-              (!M.qT_wt_download_selected_dir)
-              [ !M.pW_lb_ok ; !M.pW_lb_cancel]
-              (U.utf8_of (Printf.sprintf !M.qT_lb_confirm_download_dir 
-                 len ft.g_file_tree_name))) 
-          with
-            1 ->
-              List.iter (fun r -> 
-                GuiCom.send (Download_query ([r.res_name], r.res_num, false))
-              ) files
-          | _ -> ()
+          try
+            let ft = folder_of_key k in
+            let files = Mi.list_files ft in
+            let len = List.length files in
+            match (GToolbox.question_box
+                    (!M.qT_wt_download_selected_dir)
+                    [ !M.pW_lb_ok ; !M.pW_lb_cancel]
+                    (U.utf8_of (Printf.sprintf !M.qT_lb_confirm_download_dir 
+                    len ft.g_file_tree_name))) 
+            with
+              1 ->
+                List.iter (fun r -> 
+                  GuiCom.send (Download_query ([r.res_name], r.res_num, false))
+                ) files
+            | _ -> ()
+          with _ -> ()
         end
 
-let show_details s () =
-  let item = Source (s, 0) in
-  GuiInfoWindow.window item ()
+let show_details k () =
+  try
+    let s = friend_of_key k in
+    let item = Source (s, 0) in
+    GuiInfoWindow.window item ()
+  with _ -> ()
 
 (*************************************************************************)
 (*                                                                       *)
@@ -465,17 +591,17 @@ let show_details s () =
 (*                                                                       *)
 (*************************************************************************)
 
-let friend_menu (sel : source_info list) =
+let friend_menu sel =
   [
    `I (!M.fT_me_find_friend, find_friend) ;
    `I (!M.fT_me_remove_all_friends, remove_all_friends)
   ] @
   (match sel with
       [] -> []
-    | s :: tail ->
+    | k :: tail ->
                [
                 `S;
-                `I ((!M.dT_me_show_source_details), show_details s) ;
+                `I ((!M.dT_me_show_source_details), show_details k) ;
                 `I (!M.fT_me_remove, remove sel) ;
                ])
 
@@ -487,7 +613,8 @@ let friend_menu (sel : source_info list) =
 
 let rec insert_dir ft id ?parent () =
   ft.g_file_tree_num <- !id;
-  let parent = folderstore#add_item ft ?parent () in
+  folderstore#add_item ft ?parent ();
+  H.add ft_by_num ft;
   incr id;
   match ft.g_file_tree_list with
       [] -> ()
@@ -495,23 +622,33 @@ let rec insert_dir ft id ?parent () =
         List.iter (fun tree_item ->
           match tree_item with
               GTreeFile _ -> ()
-            | GTreeDirectory d -> insert_dir d id ~parent ()
+            | GTreeDirectory d ->
+                try
+                  let parent = folderstore#find_row (folder_key ft.g_file_tree_num) in
+                  insert_dir d id ~parent ()
+                with _ -> ()
         ) ft.g_file_tree_list
 
-let on_select_friend (sel : source_info list) =
+let on_select_friend sel =
+  H.clear ft_by_num;
   resultstore#clear ();
   folderstore#clear ();
   update_results_label ();
   match sel with
       [] -> ()
-    | s :: tail ->
-        match s.source_files with
-            None -> (GuiCom.send (GetClient_files s.source_num))
-          | Some ft ->
-            begin
-              let id = ref 1 in
-              insert_dir ft id ()
-            end
+    | k :: tail ->
+        begin
+          try
+            let s = friend_of_key k in
+            match s.source_files with
+                None -> (GuiCom.send (GetClient_files s.source_num))
+              | Some ft ->
+                  begin
+                    let id = ref 1 in
+                    insert_dir ft id ()
+                  end
+          with _ -> ()
+        end
 
 (*************************************************************************)
 (*                                                                       *)
@@ -519,16 +656,19 @@ let on_select_friend (sel : source_info list) =
 (*                                                                       *)
 (*************************************************************************)
 
-let on_double_click_friend (s : source_info) =
-  match !wnote_chat with
-      Some w when not (List.mem_assoc s.source_num !dialogs) ->
-        begin
-          let buffer = GuiTemplates.chat_buffer ~on_entry:(on_entry_return s.source_num) () in
-          let chat = GuiTemplates.chat_view ~buffer  ~my_name:!G.client_name () in
-          dialogs := (s.source_num, buffer) :: !dialogs;
-          add_chat w s chat
-        end
-    | _ -> ()
+let on_double_click_friend k =
+  try
+    let s = friend_of_key k in
+    match !wnote_chat with
+        Some w when not (List.mem_assoc s.source_num !dialogs) ->
+          begin
+            let buffer = GuiTemplates.chat_buffer ~on_entry:(on_entry_return s.source_num) () in
+            let chat = GuiTemplates.chat_view ~buffer  ~my_name:!G.client_name () in
+            dialogs := (s.source_num, buffer) :: !dialogs;
+            add_chat w s chat
+          end
+      | _ -> ()
+  with _ -> ()
 
 (*************************************************************************)
 (*                                                                       *)
@@ -536,9 +676,11 @@ let on_double_click_friend (s : source_info) =
 (*                                                                       *)
 (*************************************************************************)
 
-let filter_friend s = not (
-  List.memq s.source_network !G.networks_filtered
-  )
+let filter_friend k =
+  try
+    let s = friend_of_key k in
+    not (List.memq s.source_network !G.networks_filtered)
+  with _ -> true
 
 (*************************************************************************)
 (*                                                                       *)
@@ -546,7 +688,7 @@ let filter_friend s = not (
 (*                                                                       *)
 (*************************************************************************)
 
-let folder_menu (sel : g_file_tree list) =
+let folder_menu sel =
   match sel with
       [] -> []
     | _ ->
@@ -560,19 +702,21 @@ let folder_menu (sel : g_file_tree list) =
 (*                                                                       *)
 (*************************************************************************)
 
-let on_select_folder (sel : g_file_tree list) =
+let on_select_folder sel =
   resultstore#clear ();
   update_results_label ();
   match sel with
       [] -> ()
-    | ft :: tail ->
+    | k :: tail ->
         begin
-          List.iter (fun tree_item ->
-            match tree_item with
-                GTreeFile r -> ignore (resultstore#add_item r)
-              | _ -> ()
-          ) ft.g_file_tree_list;
-          update_results_label ()
+          try
+            let ft = folder_of_key k in
+            List.iter (fun tree_item ->
+              match tree_item with
+                  GTreeFile r -> resultstore#add r ~f:update_results_label ()
+                | _ -> ()
+            ) ft.g_file_tree_list
+          with _ -> ()
         end
 
 (*************************************************************************)
@@ -581,10 +725,14 @@ let on_select_folder (sel : g_file_tree list) =
 (*                                                                       *)
 (*************************************************************************)
 
-let on_expanded_folder path (ft : g_file_tree) =
-  let (row, _) = folderstore#find_item ft.g_file_tree_num in
-  let ft_new = {ft with g_file_tree_pixb = folder_opened ()} in
-  folderstore#update_item row ft ft_new
+let on_expanded_folder path k =
+  try
+    let row = folderstore#find_row k in
+    let ft = folder_of_key k in
+    let ft_new = {ft with g_file_tree_pixb = folder_opened ()} in
+    folderstore#update_item row ft ft_new;
+    ft.g_file_tree_pixb <- ft_new.g_file_tree_pixb
+  with _ -> ()
 
 (*************************************************************************)
 (*                                                                       *)
@@ -592,10 +740,14 @@ let on_expanded_folder path (ft : g_file_tree) =
 (*                                                                       *)
 (*************************************************************************)
 
-let on_collapsed_folder path (ft : g_file_tree) =
-  let (row, _) = folderstore#find_item ft.g_file_tree_num in
-  let ft_new = {ft with g_file_tree_pixb = folder_closed ()} in
-  folderstore#update_item row ft ft_new
+let on_collapsed_folder path k =
+  try
+    let row = folderstore#find_row k in
+    let ft = folder_of_key k in
+    let ft_new = {ft with g_file_tree_pixb = folder_closed ()} in
+    folderstore#update_item row ft ft_new;
+    ft.g_file_tree_pixb <- ft_new.g_file_tree_pixb
+  with _ -> ()
 
 (*************************************************************************)
 (*                                                                       *)
@@ -629,6 +781,7 @@ let clear () =
   in
   dialogs := [];
   source_has_file := [];
+  H.clear ft_by_num;
   resultstore#clear ();
   folderstore#clear ();
   friendstore#clear ();
@@ -642,40 +795,49 @@ let clear () =
 (*************************************************************************)
 
 let remove_friend s =
-  friendstore#remove_item s;
+  friendstore#remove_item (friend_key s.source_num);
+  let _ =
+    match s.source_files with
+        None -> ()
+      | Some ft ->
+          List.iter (fun tree_item ->
+            match tree_item with
+                GTreeFile r -> Hashtbl.remove G.results r.res_num
+              | _ -> ()
+          ) ft.g_file_tree_list
+  in
   update_friends_label ();
   if List.mem_assoc s.source_num !source_has_file
     then source_has_file := List.remove_assoc s.source_num !source_has_file
 
-let h_update_friend s_new =
+let h_update_friend s s_new =
   try
-    let (row, s) = friendstore#find_item s_new.source_num in
-    if client_browsed_tag land s_new.source_type = 0
+    let row = friendstore#find_row (friend_key s_new.source_num) in
+    if (client_friend_tag lor client_contact_tag) land s_new.source_type = 0
       then remove_friend s
-      else friendstore#update_item row s s_new
+      else Gaux.may ~f:(fun r -> friendstore#update_item r s s_new) row
  with _ ->
-    if client_browsed_tag land s_new.source_type <> 0
+   if (client_friend_tag lor client_contact_tag) land s_new.source_type <> 0
       then begin
-        ignore (friendstore#add_item s_new);
-        update_friends_label ()
+        friendstore#add_item s_new ~f:update_friends_label ()
       end
 
 let add_friend_files (source_num , dirname, result_num) =
   try
     let r = Hashtbl.find G.results result_num in
     try
-      let (_, s_new) = friendstore#find_item source_num in
+      let s = Hashtbl.find G.sources source_num  in
       let tree =
-        match s_new.source_files with
+        match s.source_files with
             None -> {g_file_tree_num = 0; g_file_tree_list = []; g_file_tree_name = ""; g_file_tree_pixb = folder_closed ()}
-          | Some tree -> { tree with g_file_tree_list = tree.g_file_tree_list }
+          | Some tree -> {tree with g_file_tree_list = tree.g_file_tree_list }
       in
       Mi.add_file tree dirname r;
-      s_new.source_files <- Some tree;
-      Hashtbl.remove G.results result_num;
+      let s_new = {s with source_files = Some tree} in
       (if not (List.mem_assoc source_num !source_has_file)
          then source_has_file := (source_num, false) :: !source_has_file);
-      h_update_friend s_new;
+      h_update_friend s s_new;
+      s.source_files <- s_new.source_files;
       if not (List.assoc source_num !source_has_file)
         then begin
           source_has_file := List.remove_assoc source_num !source_has_file;
