@@ -40,6 +40,103 @@ let (!!) = Options.(!!)
 let (=:=) = Options.(=:=)
 let (<:>) = GuiTools.(<:>)
 
+
+(*************************************************************************)
+(*                                                                       *)
+(*                         message from the core                         *)
+(*                                                                       *)
+(*************************************************************************)
+
+let hashtbl_update_result r res =
+  res.res_format <- U.simple_utf8_of r.result_format;
+  res.res_type <- U.simple_utf8_of r.result_type;
+  res.res_duration <- Mi.duration_of_tags r.result_tags;
+  res.res_codec <- Mi.codec_of_tags r.result_tags;
+  res.res_bitrate <- Mi.bitrate_of_tags r.result_tags;
+  res.res_availability <- Mi.availability_of_tags r.result_tags;
+  res.res_completesources <- Mi.completesources_of_tags r.result_tags;
+  res.res_tags <- Mi.tags_to_string r.result_tags;
+  res.res_comment <- U.utf8_of r.result_comment;
+  res.res_color <- Mi.color_of_result (Mi.availability_of_tags r.result_tags) r.result_done
+
+let result_info (r : result_info) =
+  try
+    let res = Hashtbl.find G.results r.result_num in
+    hashtbl_update_result r res
+  with _ ->
+    begin
+      let net_num = Mi.neworknum_from_uids r.result_uids in
+      let name = Mi.result_first_name r.result_names in
+      let res =
+        {
+         res_num             = r.result_num;
+         res_network         = net_num;
+         res_name            = name;
+         res_uid             = Mi.uid_list_to_string r.result_uids;
+         res_size            = r.result_size;
+         res_format          = U.simple_utf8_of r.result_format;
+         res_type            = U.simple_utf8_of r.result_type;
+         res_duration        = Mi.duration_of_tags r.result_tags;
+         res_codec           = Mi.codec_of_tags r.result_tags;
+         res_bitrate         = Mi.bitrate_of_tags r.result_tags;
+         res_availability    = Mi.availability_of_tags r.result_tags;
+         res_completesources = Mi.completesources_of_tags r.result_tags;
+         res_tags            = Mi.tags_to_string r.result_tags;
+         res_comment         = U.utf8_of r.result_comment;
+         res_color           = Mi.color_of_result (Mi.availability_of_tags r.result_tags) r.result_done;
+         res_has_query       = [];
+         res_network_pixb    = Mi.network_pixb net_num ~size:A.SMALL ();
+         res_name_pixb       = Mi.file_type_of_name name ~size:A.SMALL;
+        }
+      in
+      Hashtbl.add G.results res.res_num res;
+    end
+
+(*************************************************************************)
+(*                                                                       *)
+(*                         result_num                                    *)
+(*                                                                       *)
+(*************************************************************************)
+
+let result_num key =
+  try int_of_string key with _ -> raise Not_found
+
+(*************************************************************************)
+(*                                                                       *)
+(*                         result_of_key                                 *)
+(*                                                                       *)
+(*************************************************************************)
+
+let result_of_key key =
+  try
+    let num = result_num key in
+    Hashtbl.find G.results num
+  with _ -> raise Not_found
+
+(*************************************************************************)
+(*                                                                       *)
+(*                         keys_to_results                               *)
+(*                                                                       *)
+(*************************************************************************)
+
+let keys_to_results keys =
+  let l = ref [] in
+  List.iter (fun k ->
+    try
+      let s = result_of_key k in
+      l := s :: !l
+    with _ -> ()) keys;
+  !l
+
+(*************************************************************************)
+(*                                                                       *)
+(*                         result_key                                    *)
+(*                                                                       *)
+(*************************************************************************)
+
+let result_key result_num =
+  Printf.sprintf "%d" result_num
+
 (*************************************************************************)
 (*                                                                       *)
 (*                         message to the core                           *)
@@ -47,14 +144,16 @@ let (<:>) = GuiTools.(<:>)
 (*************************************************************************)
 
 let download sel () =
+  let l = keys_to_results sel in
   List.iter (fun r -> 
     GuiCom.send (Download_query ([r.res_name], r.res_num, false))
-  ) sel
+  ) l
 
 let force_download sel () =
+  let l = keys_to_results sel in
   List.iter (fun r -> 
     GuiCom.send (Download_query ([r.res_name], r.res_num, true))
-  ) sel
+  ) l
 
 (*************************************************************************)
 (*                                                                       *)
@@ -62,7 +161,7 @@ let force_download sel () =
 (*                                                                       *)
 (*************************************************************************)
 
-let result_menu (sel : res_info list) =
+let result_menu sel =
   match sel with
       [] -> []
     | _ ->
@@ -77,10 +176,11 @@ let result_menu (sel : res_info list) =
 (*                                                                       *)
 (*************************************************************************)
 
-let filter_result (r : res_info) =
-  not (List.memq r.res_network !G.networks_filtered)
-
-
+let filter_result k =
+  try
+    let r = result_of_key k in
+    not (List.memq r.res_network !G.networks_filtered)
+  with _ -> true
 
 module ResultList(Res:
 
@@ -114,10 +214,9 @@ module ResultList(Res:
     module Column = GuiColumns.Result
 
     type item = res_info
-    type key = int
 
     let columns = Res.columns
-    let get_key = (fun r -> r.res_num)
+    let get_key = (fun r -> result_key r.res_num)
     let module_name = Res.module_name
 
   end)
@@ -144,6 +243,28 @@ module ResultList(Res:
 
       inherit R.g_list result_cols
 
+
+        val res_queue = Queue.create ()
+
+(*************************************************************************)
+(*                                                                       *)
+(*                         from_item                                     *)
+(*                                                                       *)
+(*************************************************************************)
+
+      method add_result_while_idle () =
+        ignore (Glib.Idle.add (fun _ ->
+          try
+            let (res, f) = Queue.take res_queue in
+            self#add_item res ?f ();
+            true
+          with Queue.Empty -> false))
+
+      method add (r : res_info) ?f () =
+        let is_empty = Queue.is_empty res_queue in
+        Queue.add (r, f) res_queue;
+        if is_empty then self#add_result_while_idle ()
+
 (*************************************************************************)
 (*                                                                       *)
 (*                         from_item                                     *)
@@ -184,6 +305,7 @@ module ResultList(Res:
 (*************************************************************************)
 
       method content (col : GTree.view_column) c =
+        let autosize = match col#sizing with `AUTOSIZE -> true | _ -> false in
         match c with
             Col_result_name ->
               begin
@@ -193,11 +315,19 @@ module ResultList(Res:
                     col#pack ~expand:false renderer;
                     col#add_attribute renderer "pixbuf" result_name_pixb
                   end;
-                let renderer = GTree.cell_renderer_text [`XALIGN 0.] in
-                col#pack ~expand:false renderer;
-                col#set_cell_data_func renderer
-                  (fun model row ->
-                     match !Res.view_context with
+                if autosize
+                  then begin
+                    let renderer = GTree.cell_renderer_text [`XALIGN 0.] in
+                    col#pack ~expand:false renderer;
+                    col#add_attribute renderer "text" result_name;
+                    col#add_attribute renderer "foreground" result_color;
+                    col#pack ~expand:false renderer;
+                  end else begin
+                    let renderer = GTree.cell_renderer_text [`XALIGN 0.] in
+                    col#pack ~expand:false renderer;
+                    col#set_cell_data_func renderer
+                      (fun model row ->
+                       match !Res.view_context with
                          Some context when col#width > 0 ->
                            begin
                              let width =
@@ -209,9 +339,10 @@ module ResultList(Res:
                              let s = GuiTools.fit_string_to_pixels name ~context ~pixels:width in
                              renderer#set_properties [ `TEXT s ]
                            end
-                  | _ -> renderer#set_properties [ `TEXT "" ]
-                );
-                col#add_attribute renderer "foreground" result_color
+                       | _ -> renderer#set_properties [ `TEXT "" ]
+                      );
+                    col#add_attribute renderer "foreground" result_color;
+                  end
               end
 
           | Col_result_uid ->
@@ -322,8 +453,11 @@ module ResultList(Res:
 (*                                                                       *)
 (*************************************************************************)
 
-      method sort_items c r1 r2 =
-        match c with
+      method sort_items c k1 k2 =
+        try
+          let r1 = result_of_key k1 in
+          let r2 = result_of_key k2 in
+          match c with
             Col_result_name -> compare (String.lowercase r1.res_name) (String.lowercase r2.res_name)
           | Col_result_uid -> compare r1.res_uid r2.res_uid
           | Col_result_size -> compare r1.res_size r2.res_size
@@ -337,7 +471,7 @@ module ResultList(Res:
           | Col_result_comment -> compare r1.res_comment r2.res_comment
           | Col_result_network -> compare r1.res_network r2.res_network
           | Col_result_completesources -> compare r1.res_completesources r2.res_completesources
-
+        with _ -> 0
 
 (*************************************************************************)
 (*                                                                       *)
@@ -346,13 +480,17 @@ module ResultList(Res:
 (*************************************************************************)
 
     method force_update_icons () =
-      List.iter (fun r ->
+      let f r row =
+        r.res_network_pixb <- Mi.network_pixb r.res_num ~size:A.SMALL ();
+        r.res_name_pixb <- Mi.file_type_of_name r.res_name ~size:A.SMALL;
+        store#set ~row ~column:result_network_pixb r.res_network_pixb;
+        store#set ~row ~column:result_name_pixb r.res_name_pixb;
+      in
+      List.iter (fun k ->
         try
-          let (row, _) = self#find_item r.res_num in
-          r.res_network_pixb <- Mi.network_pixb r.res_num ~size:A.SMALL ();
-          r.res_name_pixb <- Mi.file_type_of_name r.res_name ~size:A.SMALL;
-          store#set ~row ~column:result_network_pixb r.res_network_pixb;
-          store#set ~row ~column:result_name_pixb r.res_name_pixb;
+          let r = result_of_key k in
+          let row = self#find_row k in
+          Gaux.may ~f:(f r) row
         with _ -> ()
       ) (self#all_items ())
 
@@ -368,42 +506,3 @@ module ResultList(Res:
 
   end)
 
-
-(*************************************************************************)
-(*                                                                       *)
-(*                         message from the core                         *)
-(*                                                                       *)
-(*************************************************************************)
-
-let result_info (r : result_info) =
-  try
-    let res = Hashtbl.find G.results r.result_num in
-    ()
-  with _ ->
-    begin
-      let name = Mi.result_first_name r.result_names in
-      let net_num =  Mi.neworknum_from_uids r.result_uids in
-      let res =
-        {
-         res_num = r.result_num;    
-         res_network = net_num;
-         res_network_pixb = Mi.network_pixb net_num ~size:A.SMALL ();
-         res_name = name;
-         res_name_pixb = Mi.file_type_of_name name ~size:A.SMALL;
-         res_uid = Mi.uid_list_to_string r.result_uids;
-         res_size = r.result_size;
-         res_format = U.simple_utf8_of r.result_format;
-         res_type = U.simple_utf8_of r.result_type;
-         res_duration = Mi.duration_of_tags r.result_tags;
-         res_codec = Mi.codec_of_tags r.result_tags;
-         res_bitrate = Mi.bitrate_of_tags r.result_tags;
-         res_availability = Mi.availability_of_tags r.result_tags;
-         res_completesources = Mi.completesources_of_tags r.result_tags;
-         res_tags = Mi.tags_to_string r.result_tags;
-         res_comment = U.utf8_of r.result_comment;
-         res_color = Mi.color_of_result (Mi.availability_of_tags r.result_tags) r.result_done;
-        }
-      in
-      Hashtbl.add G.results res.res_num res
-    end
-
