@@ -131,14 +131,24 @@ let connect_trackers file event f =
   in
 
   let enabled_trackers =
-    let enabled_trackers = List.filter (fun t -> t.tracker_enabled) file.file_trackers in
+    let enabled_trackers = List.filter (fun t -> tracker_is_enabled t) file.file_trackers in
     if enabled_trackers <> [] then enabled_trackers
     else begin
       (* if there is no tracker left, do something ? *)
       if !verbose_msg_servers then
-  lprintf_nl "No trackers left, reenabling all of them...";
-      List.iter (fun t -> t.tracker_enabled <- can_handle_tracker t) file.file_trackers;
-      file.file_trackers
+	lprintf_nl "No trackers left for %s, reenabling all of them..." (file_best_name (as_file file));
+      List.iter (fun t ->
+	match t.tracker_status with
+      (* only re-enable after normal error *)
+        | Disabled _ -> t.tracker_status <- Enabled
+	| _ -> ()) file.file_trackers;
+      let enabled_trackers = List.filter (fun t -> tracker_is_enabled t) file.file_trackers in
+      if enabled_trackers = [] && (file_state file) <> FilePaused then
+	begin
+	  file_pause (as_file file) CommonUserDb.admin_user;
+	  lprintf_file_nl (as_file file) "Paused %s, no usable trackers left" (file_best_name (as_file file))
+	end;
+      enabled_trackers
     end in
 
   List.iter (fun t ->
@@ -155,13 +165,12 @@ let connect_trackers file event f =
                 t.tracker_last_conn + !!min_tracker_reask_interval < last_time() ))
       then
         begin
-          (* if we already tried to connect but failed, remove tracker *)
+          (* if we already tried to connect but failed, disable tracker, but allow re-enabling *)
           if file.file_tracker_connected && t.tracker_last_clients_num = 0 &&
             t.tracker_last_conn < 1 then begin
               if !verbose_msg_servers then
                 lprintf_nl "Request error from tracker: disabling %s" t.tracker_url;
-	      t.tracker_enabled <- false;
-              (* remove_tracker t.tracker_url file *)
+	      t.tracker_status <- Disabled (intern "MLDonkey: Request error from tracker")
             end
           (* Send request to tracker *)
           else begin
@@ -205,6 +214,15 @@ let connect_trackers file event f =
             t.tracker_url (t.tracker_interval - (last_time () - t.tracker_last_conn)) file.file_name
   ) enabled_trackers
 
+let start_upload c =
+  set_client_upload (as_client c) (as_file c.client_file);
+  set_client_has_a_slot (as_client c) NormalSlot;
+  Rate.update_no_change c.client_downloaded_rate;
+  Rate.update_no_change c.client_upload_rate;
+  c.client_last_optimist <- last_time();
+  client_enter_upload_queue (as_client c);
+  send_client c Unchoke
+
 (** In this function we decide which peers will be
   uploaders. We send a choke message to current uploaders
   that are not in the next uploaders list. We send Unchoke
@@ -216,21 +234,14 @@ let recompute_uploaders () =
   (*Send choke if a current_uploader is not in next_uploaders*)
   List.iter ( fun c -> if ((List.mem c !next_uploaders)==false) then
         begin
-          set_client_has_a_slot (as_client c) false;
+          set_client_has_a_slot (as_client c) NoSlot;
           (*we will let him finish his download and choke him on next_request*)
         end
   ) !current_uploaders;
 
   (*don't send Choke if new uploader is already an uploaders *)
-  List.iter ( fun c -> if ((List.mem c !current_uploaders)==false) then
-        begin
-          set_client_has_a_slot (as_client c) true;
-          Rate.update_no_change c.client_downloaded_rate;
-          Rate.update_no_change c.client_upload_rate;
-          c.client_last_optimist <- last_time();
-          client_enter_upload_queue (as_client c);
-          send_client c Unchoke;
-        end
+  List.iter ( fun c ->
+    if not (List.mem c !current_uploaders) then start_upload c
   ) !next_uploaders;
   current_uploaders := !next_uploaders
 
@@ -261,6 +272,8 @@ let disconnect_client c reason =
         try
 (*          List.iter (fun r -> CommonSwarming.free_range r) c.client_ranges; *)
           set_client_disconnected c reason;
+          c.client_session_downloaded <- 0L;
+          c.client_session_uploaded <- 0L;
           (try if c.client_good then count_seen c with _ -> ());
           (* this is not useful already done in the match
           (try close sock reason with _ -> ());   *)
@@ -322,7 +335,7 @@ let disconnect_clients file =
   if not ( !must_keep && (client_has_a_slot (as_client c) || c.client_interested)) then
     begin
       if !verbose_msg_clients then
-        lprintf_nl "disconnect since download is finished";
+        lprintf_file_nl (as_file file) "disconnect since download is finished";
       disconnect_client c Closed_by_user
     end
   ) file.file_clients
@@ -405,7 +418,7 @@ let send_bitfield c =
 	  let bitmap = CommonSwarming.chunks_verified_bitmap swarmer in
 	  if !verbose_download then 
 	    lprintf_nl "Sending verified bitmap: [%s]" (VB.to_string bitmap);
-	  let len = VB.length bitmap in
+	  let len = (VB.length bitmap + 7)/8 in
 	  let s = String.make len '\000' in
 	  VB.iteri (fun i c ->
             if c = VB.State_verified then set_bit s i) bitmap;
@@ -414,6 +427,17 @@ let send_bitfield c =
 
 let counter = ref 0
 
+let parse_reserved rbits c =
+  let has_bit pos h = Char.code rbits.[pos] land h <> 0 in
+  
+  c.client_dht <- has_bit 7 0x01;
+  c.client_cache_extension <- has_bit 7 0x02;
+  c.client_fast_extension <- has_bit 7 0x04;
+
+  c.client_utorrent_extension <- has_bit 5 0x10;
+
+  c.client_azureus_messaging_protocol <- has_bit 0 0x80
+
 
 (** This function is called to parse the first message that
   a client send.
@@ -428,7 +452,7 @@ let counter = ref 0
 (* removed: @param peer_id The hash (sha1) of the client. (Should be checked)
 *)
 let rec client_parse_header counter cc init_sent gconn sock
-    (proto, file_id) =
+    (proto, rbits, file_id) =
   try
     set_lifetime sock 600.;
     if !verbose_msg_clients then
@@ -436,12 +460,12 @@ let rec client_parse_header counter cc init_sent gconn sock
 
     let file = Hashtbl.find files_by_uid file_id in
     if !verbose_msg_clients then
-      lprintf_nl "file found";
+      lprintf_file_nl (as_file file) "file found";
     let c =
       match !cc with
         None ->
           let c = new_client file Sha1.null (TcpBufferedSocket.peer_addr sock) in
-          if !verbose_connect then lprintf_nl "Client %d: incoming connection" (client_num c);
+          if !verbose_connect then lprintf_file_nl (as_file file) "Client %d: incoming connection" (client_num c);
           cc := Some c;
           c
       | Some c ->
@@ -482,6 +506,8 @@ let rec client_parse_header counter cc init_sent gconn sock
             (Ip.to_string ip) port;
       end;
 
+    parse_reserved rbits c;
+
     (match c.client_sock with
         NoConnection ->
           if !verbose_msg_clients then begin
@@ -603,9 +629,8 @@ and get_from_client sock (c: client) =
   
     let num, x,y, r =
 
-      if !verbose_msg_clients then begin
-        lprintf_nl "CLIENT %d: Finding new range to send" (client_num c);
-      end;
+      if !verbose_msg_clients then
+        lprintf_file_nl (as_file file) "CLIENT %d: Finding new range to send" (client_num c);
 
       if !verbose_swarming then begin
         lprintf_n "Current download:\n  Current chunks: ";
@@ -629,15 +654,16 @@ and get_from_client sock (c: client) =
 
         lprint_newline ();
           
-        lprintf_n "Current block: ";
+        lprintf_n "Current blocks: ";
           
-        match c.client_block with
+        match c.client_chunk with
         | None -> lprintf "none"
-        | Some b -> CommonSwarming.print_block b;
+        | Some (chunk, blocks) -> List.iter (fun b -> 
+	    CommonSwarming.print_block b.up_block) blocks;
 
         lprint_newline ();
       
-        lprintf_nl "Finding Range:";
+        lprintf_file_nl (as_file file) "Finding Range:";
       end;
 
       try
@@ -648,20 +674,21 @@ and get_from_client sock (c: client) =
 
         let rec iter () =
 
-          match c.client_block with
+          match c.client_chunk with
 
-            None -> 
+          | None -> 
 
-              if !verbose_swarming then lprintf_nl "No block";
+              if !verbose_swarming then lprintf_file_nl (as_file file) "No block";
               update_client_bitmap c;
               (try CommonSwarming.verify_one_chunk swarmer with _ -> ());
               (*Find a free block in the swarmer*)
-              let b = CommonSwarming.find_block up in
+              let chunk, blocks = CommonSwarming.find_blocks up in
               if !verbose_swarming then begin 
-                lprintf_n "Block Found: "; CommonSwarming.print_block b;
+                lprintf_n "Blocks Found: "; List.iter (fun b ->
+		  CommonSwarming.print_block b.up_block) blocks;
                 lprint_newline ()
               end;
-              c.client_block <- Some b;
+	      c.client_chunk <- Some (chunk, blocks);
 
              (*We put the found block in client_block to
                request range in this block. (Useful for
@@ -670,10 +697,11 @@ and get_from_client sock (c: client) =
 
               iter ()
 
-          | Some b ->
+          | Some (chunk, blocks) ->
 
               if !verbose_swarming then begin
-                lprintf_n "Current Block: "; CommonSwarming.print_block b;
+                lprintf_n "Current Blocks: "; List.iter (fun b ->
+		  CommonSwarming.print_block b.up_block) blocks;
                 lprint_newline ()
               end;
 
@@ -681,7 +709,7 @@ and get_from_client sock (c: client) =
                 (*Given a block find a range inside*)
                 let (x,y,r) =
                   match c.client_range_waiting with
-                    Some (x,y,r) ->
+		  | Some (x,y,r) ->
                         c.client_range_waiting <- None;
                         (x,y,r)
                   | None -> 
@@ -700,12 +728,13 @@ and get_from_client sock (c: client) =
                   c.client_ranges_sent <- c.client_ranges_sent @ [x,y, r];
 (*                CommonSwarming.alloc_range r; *)
                 
-                  let num = CommonSwarming.block_chunk_num swarmer b in
-
+(* naughty, naughty, was computing a block number instead of a chunk
+   number. Only matters with merged downloads, and even then other
+   clients didn't seem to care (?), so the bug remained hidden *)
                   if !verbose_swarming then 
-                    lprintf_nl "Asking %d For Range %Ld-%Ld" num x y;
+                    lprintf_file_nl (as_file file) "Asking %d For Range %Ld-%Ld" chunk x y;
                       
-                  num, x -- file.file_piece_size ** Int64.of_int num, y -- x, r
+                  chunk, x -- file.file_piece_size ** Int64.of_int chunk, y -- x, r
 
               with Not_found ->
   
@@ -715,7 +744,7 @@ and get_from_client sock (c: client) =
                     lprintf_nl "Could not find range in current block";
 (*                  c.client_blocks <- List2.removeq b c.client_blocks; *)
 
-                  c.client_block <- None;
+		  c.client_chunk <- None;
 
                   iter ()
         in
@@ -738,12 +767,12 @@ and get_from_client sock (c: client) =
     send_client c (Request (num,x,y));
 
     if !verbose_msg_clients then
-      lprintf_nl "CLIENT %d: Asking %s For Range %Ld-%Ld"
+      lprintf_file_nl (as_file file) "CLIENT %d: Asking %s For Range %Ld-%Ld"
         (client_num c) (Sha1.to_string c.client_uid) x y
 
   with Not_found ->
         if not (CommonSwarming.check_finished swarmer) && !verbose_download then
-          lprintf_nl "BTClient.get_from_client ERROR: can't find a block to download and file is not yet finished for file : %s..." file.file_name
+          lprintf_file_nl (as_file file) "BTClient.get_from_client ERROR: can't find a block to download and file is not yet finished for file : %s..." file.file_name
 
 
 (** In this function we match a message sent by a client
@@ -795,10 +824,10 @@ and client_to_client c sock msg =
 
             if !verbose_msg_clients then
               (match c.client_ranges_sent with
-                  [] -> lprintf_nl "EMPTY Ranges !!!"
+                  [] -> lprintf_file_nl (as_file file) "EMPTY Ranges !!!"
                 | (p1,p2,r) :: _ ->
                     let (x,y) = CommonSwarming.range_range r in
-                    lprintf_nl "Current range from %s : %Ld [%d] (asked %Ld-%Ld[%Ld-%Ld])"
+                    lprintf_file_nl (as_file file) "Current range from %s : %Ld [%d] (asked %Ld-%Ld[%Ld-%Ld])"
                       (brand_to_string c.client_brand) position len
                       p1 p2 x y
               );
@@ -815,13 +844,13 @@ and client_to_client c sock msg =
             (*Update rate and amount of data received from client*)
             count_download c (new_downloaded -- old_downloaded);
             (* use len here with max_dr quickfix *)
-            Rate.update c.client_downloaded_rate  (float_of_int len);
+            Rate.update c.client_downloaded_rate ~amount:len;
             if !verbose_msg_clients then
               (match c.client_ranges_sent with
-                  [] -> lprintf_nl "EMPTY Ranges !!!"
+                  [] -> lprintf_file_nl (as_file file) "EMPTY Ranges !!!"
                 | (p1,p2,r) :: _ ->
                     let (x,y) = CommonSwarming.range_range r in
-                    lprintf_nl "Received %Ld [%d] %Ld-%Ld[%Ld-%Ld] -> %Ld"
+                    lprintf_file_nl (as_file file) "Received %Ld [%d] %Ld-%Ld[%Ld-%Ld] -> %Ld"
                       position len
                       p1 p2 x y
                       (new_downloaded -- old_downloaded)
@@ -840,56 +869,20 @@ and client_to_client c sock msg =
               c.client_ranges_sent <- tail;
         end;
         get_from_client sock c;
-        if (List.length !current_uploaders < (!!max_bt_uploaders-1)) &&
-          (List.mem c (!current_uploaders)) == false && c.client_interested then
-          begin
-            (*we are probably an optimistic uploaders for this client
-              don't miss the oportunity if we can
-            *)
-            current_uploaders := c::(!current_uploaders);
-            c.client_sent_choke <- false;
-            set_client_has_a_slot (as_client c) true;
-            Rate.update_no_change c.client_downloaded_rate;
-            Rate.update_no_change c.client_upload_rate;
-            c.client_last_optimist <- last_time();
-            client_enter_upload_queue (as_client c);
-            send_client c Unchoke;
-          end;
 
         (* Check if the client is still interesting for us... *)
         check_if_interesting file c
 
     | PeerID p ->
       (* Disconnect if that is ourselves. *)
+      c.client_uid <- Sha1.direct_of_string p;
       if not (c.client_uid = !!client_uid) then
         begin
           let brand, release = parse_software p in
           c.client_brand <- brand;
           c.client_release <- release;
-(* TODO : enable it
-      c.client_release <- (parse_release p c.client_brand);
- *)
-          c.client_uid <- Sha1.direct_of_string p;
-
-          if (List.length !current_uploaders < (!!max_bt_uploaders-1)) &&
-            (List.mem c (!current_uploaders)) == false then
-            begin
-            (*we are probably an optimistic uploader for this client
-              don't miss the opportunity if we can *)
-              current_uploaders := c::(!current_uploaders);
-              c.client_sent_choke <- false;
-              set_client_has_a_slot (as_client c) true;
-              Rate.update_no_change c.client_downloaded_rate;
-              Rate.update_no_change c.client_upload_rate;
-              c.client_last_optimist <- last_time();
-              client_enter_upload_queue (as_client c);
-              send_client c Unchoke;
-            end
-          else
-            begin
-              send_client c Choke;
-              c.client_sent_choke <- true;
-            end
+          send_client c Choke;
+          c.client_sent_choke <- true;
         end
       else
         disconnect_client c Closed_by_user
@@ -907,7 +900,7 @@ and client_to_client c sock msg =
               let nbits = String.length p * 8 in
 
               if nbits < npieces then begin
-                lprintf_nl "Error: expected bitfield of atleast %d but got %d" npieces nbits;
+                lprintf_file_nl (as_file file) "Error: expected bitfield of atleast %d but got %d" npieces nbits;
                 disconnect_client c (Closed_for_error "Wrong bitfield length")
               end else begin
 
@@ -930,7 +923,7 @@ and client_to_client c sock msg =
                   send_interested c;
 
                 if !verbose_msg_clients then
-                  lprintf_nl "New BitField Registered";
+                  lprintf_file_nl (as_file file) "New BitField Registered";
 
                 (*  for i = 1 to max_range_requests - List.length c.client_ranges do
                       (try get_from_client sock c with _ -> ())
@@ -999,7 +992,7 @@ and client_to_client c sock msg =
                 (* Afaik this is no protocol violation and happens if the client
                    didn't send a client bitmap after the handshake. *)
                 let (ip,port) = c.client_host in
-                  if !verbose_msg_clients then lprintf_nl "%s:%d with software %s : Choke send, but no client bitmap"
+                  if !verbose_msg_clients then lprintf_file_nl (as_file file) "%s:%d with software %s : Choke send, but no client bitmap"
                     (Ip.to_string ip) port (brand_to_string c.client_brand)
             | Some up ->
                 CommonSwarming.clear_uploader_intervals up
@@ -1064,10 +1057,10 @@ and client_to_client c sock msg =
           c.client_upload_requests <- List2.remove_first (n, pos, len) c.client_upload_requests
         else
           if !verbose_msg_clients then
-            lprintf_nl "Error: received cancel request but client has no slot"
+            lprintf_file_nl (as_file file) "Error: received cancel request but client has no slot"
 
   with e ->
-      lprintf_nl "Error %s while handling MESSAGE: %s" (Printexc2.to_string e) (TcpMessages.to_string msg)
+      lprintf_file_nl (as_file file) "Error %s while handling MESSAGE: %s" (Printexc2.to_string e) (TcpMessages.to_string msg)
 
 
 (** The function used to connect to a client.
@@ -1132,7 +1125,7 @@ let connect_client c =
                   let file = c.client_file in
 
                   if !verbose_msg_clients then
-                    lprintf_nl "READY TO DOWNLOAD FILE";
+                    lprintf_file_nl (as_file file) "READY TO DOWNLOAD FILE";
 
                   send_init !!client_uid file.file_id sock;
 (* Fabrice: Initialize the client bitmap and uploader fields to <> None *)
@@ -1265,7 +1258,7 @@ let resume_clients file =
       try
         match c.client_sock with
         | Connection sock -> ()
-            (*i think this one is not realy usefull for debugging
+            (*i think this one is not really usefull for debugging
               lprintf_nl "[BT]: RESUME: Client is already connected"; *)
         | _ ->
             (try
@@ -1280,7 +1273,7 @@ let resume_clients file =
              with _ -> ())
       with e ->
           if !verbose_connect then
-            lprintf_nl "Exception %s in resume_clients"   (Printexc2.to_string e)
+            lprintf_file_nl (as_file file) "Exception %s in resume_clients"   (Printexc2.to_string e)
   ) file.file_clients
 
 (** Check if the value replied by the tracker is correct.
@@ -1317,13 +1310,13 @@ let get_sources_from_tracker file =
     let tracker_reply =
       try
         File.to_string filename
-      with e -> lprintf_nl "Empty reply from tracker"; ""
+      with e -> lprintf_file_nl (as_file file) "Empty reply from tracker"; ""
     in
     let v =
        match tracker_reply with
        | "" ->
         if !verbose_connect then
-          lprintf_nl "Empty reply from tracker";
+          lprintf_file_nl (as_file file) "Empty reply from tracker";
         Bencode.decode ""
        | _ -> Bencode.decode tracker_reply
     in
@@ -1333,15 +1326,28 @@ let get_sources_from_tracker file =
     match v with
       Dictionary list ->
         List.iter (fun (key,value) ->
+            (match (key, value) with
+            | String "failure reason", _ -> ()
+            | _ -> (match t.tracker_status with
+                   | Disabled_failure (i, _) ->
+                        lprintf_file_nl (as_file file) "Received good message from Tracker %s in file: %s after %d bad attempts"
+                          t.tracker_url file.file_name i
+                   | _ -> ());
+                   (* Received good message from tracker after failures, re-enable tracker *)
+                   t.tracker_status <- Enabled);
+
             match (key, value) with
             | String "failure reason", String failure ->
-                (* On failure, remove the faulty tracker from file.file_trackers list *)
-		t.tracker_enabled <- false;
-		(* remove_tracker t.tracker_url file; *)
-                lprintf_nl "Failure from Tracker %s in file: %s Reason: %s\nBT: Tracker %s disabled for failure"
-                  t.tracker_url file.file_name (Charset.to_utf8 failure) t.tracker_url
+                (* On failure, disable the tracker, count the attempts and forbid re-enabling *)
+		t.tracker_status <- (match t.tracker_status with
+                  | Disabled_failure (i,_) -> Disabled_failure (i + 1, intern failure)
+                  | _ -> Disabled_failure (1, intern failure));
+                lprintf_file_nl (as_file file) "Failure no. %d%s from Tracker %s in file: %s Reason: %s"
+                  (match t.tracker_status with | Disabled_failure (i,_) -> i | _ -> 1)
+                  (if !!tracker_retries = 0 then "" else Printf.sprintf "/%d" !!tracker_retries)
+                  t.tracker_url file.file_name (Charset.to_utf8 failure)
             | String "warning message", String warning ->
-                lprintf_nl "Warning from Tracker %s in file: %s Reason: %s" t.tracker_url file.file_name warning
+                lprintf_file_nl (as_file file) "Warning from Tracker %s in file: %s Reason: %s" t.tracker_url file.file_name warning
             | String "interval", Int n ->
                 t.tracker_interval <- chk_keyval (Bencode.print key) n t.tracker_url file.file_name;
                 (* in case we don't receive "min interval" *)
@@ -1372,11 +1378,11 @@ let get_sources_from_tracker file =
             | String "key", String n ->
                 t.tracker_key <- n;
                 if !verbose_msg_clients then
-                  lprintf_nl "%s in file: %s has key: %s" t.tracker_url file.file_name n
+                  lprintf_file_nl (as_file file) "%s in file: %s has key: %s" t.tracker_url file.file_name n
             | String "tracker id", String n ->
                 t.tracker_id <- n;
                 if !verbose_msg_clients then
-                  lprintf_nl "%s in file: %s has tracker id %s" t.tracker_url file.file_name n
+                  lprintf_file_nl (as_file file) "%s in file: %s has tracker id %s" t.tracker_url file.file_name n
 
             | String "peers", List list ->
                 List.iter (fun v ->
@@ -1408,13 +1414,13 @@ let get_sources_from_tracker file =
                               None -> true
                             | Some reason ->
                                 if !verbose_connect then
-                                  lprintf_nl "%s:%d blocked: %s"
+                                  lprintf_file_nl (as_file file) "%s:%d blocked: %s"
                                     (Ip.to_string !peer_ip) !port reason;
                                 false)
                         then
                           let _ = new_client file !peer_id (!peer_ip,!port)
                           in
-                          if !verbose_sources > 1 then lprintf_nl "Received %s:%d" (Ip.to_string !peer_ip)
+                          if !verbose_sources > 1 then lprintf_file_nl (as_file file) "Received %s:%d" (Ip.to_string !peer_ip)
                           !port;
                           ()
                     | _ -> assert false
@@ -1435,12 +1441,12 @@ let get_sources_from_tracker file =
             | String "private", Int n -> ()
               (* TODO: if set to 1, disable peer exchange *)
 
-            | _ -> lprintf_nl "received unknown entry in answer from tracker: %s : %s" (Bencode.print key) (Bencode.print value)
+            | _ -> lprintf_file_nl (as_file file) "received unknown entry in answer from tracker: %s : %s" (Bencode.print key) (Bencode.print value)
         ) list;
        (*Now, that we have added new clients to a file, it's time
          to connect to them*)
         if !verbose_sources > 0 then
-          lprintf_nl "get_sources_from_tracker: got %i source(s) for file %s"
+          lprintf_file_nl (as_file file) "get_sources_from_tracker: got %i source(s) for file %s"
             t.tracker_last_clients_num file.file_name;
         resume_clients file
 
@@ -1469,7 +1475,7 @@ let recover_files () =
               (try
                   connect_trackers file "" (fun _ _ -> ()) with _ -> ())
           | FilePaused -> () (*when we are paused we do nothing, not even logging this vvvv*)
-          | s -> lprintf_nl "Other state %s!!" (string_of_state s)
+          | s -> lprintf_file_nl (as_file file) "Other state %s!!" (string_of_state s)
       ) !current_files
 
 let upload_buffer = String.create 100000
@@ -1501,7 +1507,8 @@ let rec iter_upload sock c =
 (*          lprintf "Unix32.read: offset %Ld len %d\n" offset len; *)
           Unix32.read (file_fd file) offset upload_buffer 0 len;
           (* update upload rate from len bytes *)
-          Rate.update c.client_upload_rate  (float_of_int len);
+          Rate.update c.client_upload_rate  ~amount:len;
+          Rate.update c.client_downloaded_rate;
           file.file_uploaded <- file.file_uploaded ++ (Int64.of_int len);
           let _ =
             (* update stats *)
@@ -1550,13 +1557,12 @@ let client_can_upload c allowed =
           iter_upload sock c
   )
 
-
-(** Probably useless now
-*)
 let file_resume file =
-  (*useless with no saving of sources
-     resume_clients file;
-  *)
+  List.iter (fun t ->
+    match t.tracker_status with
+      Enabled -> ()
+    | _ -> t.tracker_status <- Enabled
+  ) file.file_trackers;
   (try get_sources_from_tracker file  with _ -> ())
 
 
@@ -1569,7 +1575,7 @@ let file_stop file =
     if file.file_tracker_connected then
     begin
       connect_trackers file "stopped" (fun _ _ ->
-          lprintf_nl "Tracker return: stopped %s" file.file_name;
+          lprintf_file_nl (as_file file) "Tracker return: stopped %s" file.file_name;
           file.file_tracker_connected <- false)
     end
 
