@@ -41,10 +41,6 @@ module StatsOption = struct
     let value_to_stat v =
       match v with
         Options.Module assocs ->
-          let get_value name conv = conv (List.assoc name assocs) in
-          let get_value_nil name conv =
-            try conv (List.assoc name assocs) with _ -> []
-          in
           {
             brand_seen = value_to_int (List.assoc "seen" assocs);
             brand_banned = value_to_int (List.assoc "banned" assocs);
@@ -75,9 +71,6 @@ module ClientOption = struct
       | Module assocs ->
 
           let get_value name conv = conv (List.assoc name assocs) in
-          let get_value_nil name conv =
-            try conv (List.assoc name assocs) with _ -> []
-          in
           let client_ip = get_value "client_ip" (from_value Ip.option)
           in
           let client_port = get_value "client_port" value_to_int in
@@ -103,10 +96,6 @@ module ClientOption = struct
 
 let value_to_file file_size file_state assocs =
   let get_value name conv = conv (List.assoc name assocs) in
-  let get_value_nil name conv =
-    try conv (List.assoc name assocs) with _ -> []
-  in
-
   let file_trackers =
     try
       get_value "file_trackers" (value_to_list value_to_string)
@@ -194,8 +183,9 @@ let value_to_file file_size file_state assocs =
             (Printf.sprintf "BT-%s" (Sha1.to_string file_id)) in
         file_temp
   in
-  let file = new_file file_id torrent torrent_diskname file_temp file_state in
-
+  let file = new_file file_id torrent torrent_diskname
+	       file_temp file_state CommonUserDb.admin_user in
+    
   let file_uploaded = try
       value_to_int64 (List.assoc "file_uploaded" assocs)
     with _ -> zero
@@ -260,7 +250,7 @@ send us more clients.
         CommonSwarming.frontend_to_value swarmer assocs
   with
     e ->
-      lprintf_nl "exception %s in file_to_value"
+      lprintf_file_nl (as_file file) "exception %s in file_to_value"
         (Printexc2.to_string e); raise e
 
 
