@@ -31,6 +31,7 @@ open CommonComplexOptions
 open CommonServer
 open CommonResult
 open CommonFile
+open CommonShared
 open BasicSocket
 open CommonGlobals
 open Options
@@ -107,6 +108,45 @@ let (ft_ops : ft CommonFile.file_ops) =
 let (client_ops : client CommonClient.client_ops) =
   CommonClient.new_client_ops network
 
+let must_share_file file codedname has_old_impl =
+  match file.file_shared with
+  | Some _ -> ()
+  | None ->
+      begin
+        let impl = {
+          impl_shared_update = 1;
+          impl_shared_fullname = file_disk_name file;
+          impl_shared_codedname = codedname;
+          impl_shared_size = file_size file;
+          impl_shared_id = Md4.null;
+          impl_shared_num = 0;
+          impl_shared_uploaded = Int64.zero;
+          impl_shared_ops = shared_ops;
+          impl_shared_val = file;
+          impl_shared_requests = 0;
+          impl_shared_file = Some (as_file file);
+          impl_shared_servers = [];
+        } in
+        file.file_shared <- Some impl;
+        incr CommonGlobals.nshared_files;
+        CommonShared.shared_calculate_total_bytes ();
+        match has_old_impl with
+          None -> update_shared_num impl
+        | Some old_impl -> replace_shared old_impl impl
+      end
+
+let must_share_file file = must_share_file file (file_best_name (as_file file)) None
+
+let unshare_file file =
+  match file.file_shared with
+    None -> ()
+  | Some s ->
+      begin
+        file.file_shared <- None;
+        decr CommonGlobals.nshared_files;
+        CommonShared.shared_calculate_total_bytes ()
+      end
+
 module DO = CommonOptions
 
 let current_files = ref ([] : BTTypes.file list)
@@ -150,13 +190,14 @@ let check_if_interesting file c =
         | Some (x,y,r) ->
             let x,y = CommonSwarming.range_range r in
             x < y) &&
-(* The current block is also useless *)
-      (match c.client_block with
-          None -> true
-        | Some b ->
-            let chunk_num = CommonSwarming.block_chunk_num swarmer b in
+(* The current blocks are also useless *)
+      (match c.client_chunk with
+      | None -> true
+      | Some (chunk, blocks) ->
+	  List.for_all (fun b ->
+            let chunk_num = CommonSwarming.block_chunk_num swarmer b.up_block in
             let bitmap = CommonSwarming.chunks_verified_bitmap swarmer in
-            VB.get bitmap chunk_num <> VB.State_verified)
+            VB.get bitmap chunk_num <> VB.State_verified) blocks)
     in
     if must_send then
       begin
@@ -212,13 +253,14 @@ let rec set_trackers file file_trackers =
             tracker_torrent_last_dl_req = 0;
             tracker_id = "";
             tracker_key = "";
-	    tracker_enabled = true
+	    tracker_status = Enabled
           } in
-	  t.tracker_enabled <- can_handle_tracker t;
+	  if not (can_handle_tracker t) then
+	    t.tracker_status <- Disabled_mld (intern "Tracker type not supported");
 	  file.file_trackers <-  t :: file.file_trackers;
 	set_trackers file q
 
-let new_file file_id t torrent_diskname file_temp file_state =
+let new_file file_id t torrent_diskname file_temp file_state user =
   try
     Hashtbl.find files_by_uid file_id
   with Not_found ->
@@ -228,7 +270,7 @@ let new_file file_id t torrent_diskname file_temp file_state =
           file_file = file_impl;
           file_piece_size = t.torrent_piece_size;
           file_id = file_id;
-          file_name = t.torrent_name;
+          file_name = Charset.safe_convert t.torrent_encoding t.torrent_name;
           file_comment = t.torrent_comment;
           file_created_by = t.torrent_created_by;
           file_creation_date = t.torrent_creation_date;
@@ -247,6 +289,8 @@ let new_file file_id t torrent_diskname file_temp file_state =
           file_shared = None;
         } and file_impl =  {
           dummy_file_impl with
+          impl_file_owner = user;
+          impl_file_group = user.user_default_group;
           impl_file_fd = Some file_fd;
           impl_file_size = t.torrent_length;
           impl_file_downloaded = Int64.zero;
@@ -291,18 +335,18 @@ let new_file file_id t torrent_diskname file_temp file_state =
       current_files := file :: !current_files;
       Hashtbl.add files_by_uid file_id file;
       file_add file_impl file_state;
-(*      lprintf "ADD FILE TO DOWNLOAD LIST\n"; *)
+      must_share_file file;
       file
 
-let new_download file_id t torrent_diskname =
+let new_download file_id t torrent_diskname user =
   let file_temp = Filename.concat !!DO.temp_directory
       (Printf.sprintf "BT-%s" (Sha1.to_string file_id)) in
-  new_file file_id t torrent_diskname file_temp FileDownloading
+  new_file file_id t torrent_diskname file_temp FileDownloading user
 
 let ft_by_num = Hashtbl.create 13
 let ft_counter = ref 0
 
-let new_ft file_name =
+let new_ft file_name user =
   incr ft_counter;
   let rec ft = {
       ft_file = file_impl;
@@ -311,6 +355,8 @@ let new_ft file_name =
       ft_retry = (fun _ -> ());
     } and file_impl =  {
       dummy_file_impl with
+      impl_file_owner = user;
+      impl_file_group = user.user_default_group;
       impl_file_fd = None;
       impl_file_size = zero;
       impl_file_downloaded = Int64.zero;
@@ -358,10 +404,38 @@ let dot_string_of_list s l =
   List.iter (fun i -> Buffer.add_char buf s.[i]) l;
   dot_string (Buffer.contents buf)
 
+let dot_string_of_string s =
+  let buf = Buffer.create 20 in
+  let found_non_int = ref false in
+  String.iter (fun s ->
+    match s with
+    | '0' .. '9' ->
+	if !found_non_int then Buffer.add_char buf '.';
+	found_non_int := false;
+	Buffer.add_char buf s
+    | _ -> found_non_int := true
+  ) s;
+  Buffer.contents buf
+
+(* check string s for char c (dec) at position l (list) *)
 let check_all s c l =
   let ch = char_of_int c in
   List.for_all (fun i -> s.[i] = ch) l
 
+let check_int s p =
+  try
+    ignore (int_of_string (String.sub s p 1));
+    true
+  with _ -> false
+
+let strip_leading_zeroes s =
+  let l = String.length s in
+  let rec aux i =
+    if i = l then "0"
+    else if s.[i] <> '0' then String.sub s i (l - i)
+    else aux (i + 1) in
+  aux 0
+
 (* from azureus/gpl *)
 let decode_az_style s =
   if check_all s 45 [0;7] then begin
@@ -372,16 +446,18 @@ let decode_az_style s =
       | "AZ" -> Brand_azureus
       | "BB" -> Brand_bitbuddy
       | "BC" -> Brand_bitcomet
+      | "BR" -> Brand_bitrocket
       | "BS" -> Brand_btslave
       | "BX" -> Brand_bittorrentx
-      | "CT" -> Brand_ctorrent
+      | "CT" (* ctorrent *)
       | "CD" -> Brand_ctorrent
       | "LT" -> Brand_libtorrent
       | "MT" -> Brand_moonlighttorrent
       | "SB" -> Brand_swiftbit
       | "SN" -> Brand_sharenet
       | "SS" -> Brand_swarmscope
-      | "SZ" -> Brand_shareaza
+      | "SZ" (* shareaza *)
+      | "S~" -> Brand_shareaza
       | "TN" -> Brand_torrentdotnet
       | "TS" -> Brand_torrentstorm
       | "XT" -> Brand_xantorrent
@@ -395,15 +471,30 @@ let decode_az_style s =
       | "TR" -> Brand_transmission
       | "HN" -> Brand_hydranode
       | "RT" -> Brand_retriever
+      | "PC" -> Brand_cachelogic
+      | "ES" -> Brand_electricsheep
+      | "qB" -> Brand_qbittorrent
+      | "QT" -> Brand_qt4
+      | "UL" -> Brand_uleecher
+      | "XX" -> Brand_xtorrent
+      | "AG" (* ares *)
+      | "A~" -> Brand_ares
+      | "AX" -> Brand_bitpump
+      | "DE" -> Brand_deluge
       | _ -> Brand_unknown
     in
     if brand = Brand_unknown then None else
       let version = 
         match brand with
+          (* 4.56 *)
+          | Brand_bitpump
           | Brand_bitcomet -> (String.sub s 4 1) ^ "." ^ (String.sub s 5 2)
+          (* 3.45 *)
           | Brand_utorrent -> (String.sub s 3 1) ^ "." ^ (String.sub s 4 1) ^ "." ^ (String.sub s 5 1)
-          | Brand_ctorrent 
-          | Brand_transmission -> (String.sub s 3 2) ^ "." ^ (String.sub s 5 2)
+          (* 34.56 *)
+          | Brand_ctorrent
+          | Brand_transmission -> (strip_leading_zeroes (String.sub s 3 2)) ^ "." ^ (strip_leading_zeroes(String.sub s 5 2))
+          (* 3.4.5->[R=RC.6|D=Dev|''] *)
           | Brand_ktorrent -> 
               let x = match s.[5] with 
                 | 'R' -> " RC" ^ (String.sub s 6 1)
@@ -411,6 +502,11 @@ let decode_az_style s =
                 | _ -> ""
               in
               (String.sub s 3 1) ^ "." ^ (String.sub s 4 1) ^ x
+          (* 3.4(56) *)
+          | Brand_bitrocket -> (String.sub s 3 1) ^ "." ^ (String.sub s 4 1) ^ "(" ^ (String.sub s 5 2) ^ ")"
+          (* v 3456 *)
+          | Brand_xtorrent -> "v" ^ (strip_leading_zeroes (String.sub s 3 4))
+          (* 3.4.5.6 *)
           | _ -> (dot_string (String.sub s 3 4)) 
       in
       Some (brand, version)
@@ -418,69 +514,80 @@ let decode_az_style s =
   None
 
 let decode_tornado_style s =
-  if check_all s 45 [4;5] then begin
-    let check_brand s =
-     match s with
-     | "T" -> Brand_bittornado
-     | "S" -> Brand_shadow
-     | "A" -> Brand_abc
-     | "U" -> Brand_upnp
-     | "O" -> Brand_osprey
-     | _ -> Brand_unknown
+  if s.[5] = '-' then begin
+    let check_brand c =
+     match c with
+     | 'T' -> Brand_bittornado
+     | 'S' -> Brand_shadow
+     | 'A' -> Brand_abc
+     | 'U' -> Brand_upnp
+     | 'O' -> Brand_osprey
+     | 'R' -> Brand_tribler
+     | _   -> Brand_unknown
     in
     let bv = ref None in
 
-    if check_all s 45 [6;7;8] then begin
-      let brand = check_brand (String.sub s 0 1) in
+    if s.[5] ='-' && s.[6] ='-' && s.[7] ='-' then begin
+      let brand = check_brand s.[0] in
       if not (brand = Brand_unknown) then
         bv := Some (brand, (dot_string_h (String.sub s 1 3)));
     end
     else if s.[6] = (char_of_int 48) then begin
-      let brand = check_brand (String.sub s 0 1) in
+      let brand = check_brand s.[0] in
       if not (brand = Brand_unknown) then
         bv := Some (brand, ("LM " ^ dot_string_h (String.sub s 1 3)));
     end;
-
     !bv
   end else
   None
 
 let decode_mainline_style s =
-  if check_all s 45 [2;4;6;7] then begin
-    let s_id = String.sub s 0 1 in
+  if check_all s 45 [2;7] && check_int s 1 then begin
+    let c_id = s.[0] in
     let brand =
-      match s_id with
-     | "M" -> Brand_mainline
+      match c_id with
+     | 'M' -> Brand_mainline
      | _ -> Brand_unknown
     in
     if brand = Brand_unknown then None
-    else Some (brand, (dot_string_of_list s [1;3;5]))
+    else Some (brand, (dot_string_of_string (String.sub s 1 6)))
   end else
   None
 
 let decode_simple_style s =
   let simple_list = ref
-    [ (0, "martini", Brand_martiniman, "");
-      (0, "oernu", Brand_btugaxp, "");
-      (0, "BTDWV-", Brand_deadmanwalking, "");
-      (0, "PRC.P---", Brand_btplus, "II");
-      (0, "P87.P---", Brand_btplus, "");
-      (0, "S587Plus", Brand_btplus, "");
-      (5, "Azureus", Brand_azureus, "2.0.3.2");
-      (0, "-G3", Brand_g3torrent, "");
-      (0, "-AR", Brand_arctic, "");
-      (4, "btfans", Brand_simplebt, "");
-      (0, "btuga", Brand_btugaxp, "");
-      (0, "BTuga", Brand_btugaxp, "");
-      (0, "DansClient", Brand_xantorrent, "");
-      (0, "Deadman Walking-", Brand_deadmanwalking, "");
-      (0, "346-", Brand_torrenttopia, "");
-      (0, "271-", Brand_greedbt, "2.7.1");
-      (10, "BG", Brand_btgetit, "");
-      (0, "a00---0", Brand_swarmy, "");
-      (0, "a02---0", Brand_swarmy, "");
-      (0, "10-------", Brand_jvtorrent, "");
-      (0, "T00---0", Brand_teeweety, "") ]
+    [ (0,  "martini",           Brand_martiniman,      "");
+      (0,  "oernu",             Brand_btugaxp,         "");
+      (0,  "BTDWV-",            Brand_deadmanwalking,  "");
+      (0,  "PRC.P---",          Brand_btplus,          "II");
+      (0,  "P87.P---",          Brand_btplus,          "");
+      (0,  "S587Plus",          Brand_btplus,          "");
+      (5,  "Azureus",           Brand_azureus,         "2.0.3.2");
+      (0,  "-G3",               Brand_g3torrent,       "");
+      (0,  "-AR",               Brand_arctic,          "");
+      (4,  "btfans",            Brand_simplebt,        "");
+      (0,  "btuga",             Brand_btugaxp,         "");
+      (0,  "BTuga",             Brand_btugaxp,         "");
+      (0,  "DansClient",        Brand_xantorrent,      "");
+      (0,  "Deadman Walking-",  Brand_deadmanwalking,  "");
+      (0,  "346-",              Brand_torrenttopia,    "");
+      (0,  "271-",              Brand_greedbt,         "2.7.1");
+      (10, "BG",                Brand_btgetit,         "");
+      (0,  "a00---0",           Brand_swarmy,          "");
+      (0,  "a02---0",           Brand_swarmy,          "");
+      (0,  "10-------",         Brand_jvtorrent,       "");
+      (0,  "T00---0",           Brand_teeweety,        "");
+      (0,  "LIME",              Brand_limewire,        "");
+      (0,  "AZ2500BT",          Brand_btyrant,         "");
+      (0,  "Mbrst",             Brand_burst,           (dot_string_of_list s [5;7;9]));
+      (0,  "Plus",              Brand_plus,            (dot_string_of_list s [4;5;6]));
+      (0,  "OP",                Brand_opera,           (dot_string(String.sub s 2 4)));
+      (0,  "eX",                Brand_exeem,           (String.sub s 2 18));
+      (0,  "turbobt",           Brand_turbobt,         (String.sub s 7 5));
+      (0,  "btpd",              Brand_btpd,            (dot_string(String.sub s 5 3)));
+      (0,  "XBT",               Brand_xbt,             (dot_string(String.sub s 3 3)));
+      (0,  "-FG",               Brand_flashget,        (dot_string(String.sub s 4 3)));
+    ]
   in
   let len = List.length !simple_list in
   let rec check pos =
@@ -492,40 +599,23 @@ let decode_simple_style s =
   in
   check 0
 
-let decode_mburst s =
-  if "Mbrst" = String.sub s 0 5 then
-     Some (Brand_burst, (dot_string_of_list s [5;7;9]))
-  else None
-
-let decode_plus s =
-  if "Plus" = String.sub s 0 4 then
-     Some (Brand_plus, (dot_string_of_list s [4;5;6]))
-  else None
-
-let decode_opera s =
-  if "OP" = String.sub s 0 2 then
-     Some (Brand_opera, (dot_string_of_list s [2;3;4;5]))
+let decode_rufus s =
+  let release s =
+    let minor = Char.code s.[1] in
+    Printf.sprintf "%d.%d.%d" (Char.code s.[0]) (minor / 10) (minor mod 10) in
+  if "RS" = String.sub s 2 2 then
+     Some (Brand_rufus, release s)
   else None
 
 let decode_bow s =
   if "BOW" = String.sub s 0 3 ||
-  (check_all s 45 [0;7] && "BOW" = String.sub s 1 4) then
+  (check_all s 45 [0;7] && "BOW" = String.sub s 1 3) then
     Some (Brand_bitsonwheels, (String.sub s 4 3))
   else None
-
-let decode_exeem s =
-  if "eX" = String.sub s 0 2 then
-    Some (Brand_exeem, (String.sub s 2 18))
-  else None
-
-let decode_turbo s =
-  if "turbobt" = String.sub s 0 7 then
-    Some (Brand_turbobt, (String.sub s 7 5))
-  else None
-
-let decode_xbt s =
-  if "XBT" = String.sub s 0 3 then
-    Some (Brand_xbt, (dot_string (String.sub s 3 3)))
+  
+let decode_btuga s =
+  if ("BTM" = String.sub s 0 3) && ("BTuga" = String.sub s 5 5) then
+    Some (Brand_btuga, dot_string(String.sub s 3 2))
   else None
 
 let decode_shadow s =
@@ -553,12 +643,13 @@ let decode_bitspirit s =
     let bv = ref None in
     if s.[1] = (char_of_int 0) then bv := Some (Brand_bitspirit, "v1");
     if s.[1] = (char_of_int 2) then bv := Some (Brand_bitspirit, "v2");
+    if s.[1] = (char_of_int 3) then bv := Some (Brand_bitspirit, "v3");
     !bv
   end else
   None
 
 let decode_upnp s =
-  if "U" = String.sub s 0 1 && s.[8] = '-' then
+  if 'U' = s.[0] && s.[8] = '-' then
     Some (Brand_upnp, (dot_string (String.sub s 1 3)))
   else None
 
@@ -623,7 +714,7 @@ let decode_non_zero s =
 
 (* format is : "-ML" ^ version ( of unknown length) ^ "-" ^ random bytes ( of unknown length) *)
 let decode_mldonkey_style s =
-  if check_all s 45 [0] then begin
+  if '-' = s.[0] then begin
     let s_id = String.sub s 1 2 in
     let brand =
      match s_id with
@@ -631,25 +722,21 @@ let decode_mldonkey_style s =
      | _ -> Brand_unknown
     in
     if brand = Brand_unknown then None else
-      let len = 
-        (try String.index_from s 3 s.[0]
+      let len =
+        (try String.index_from s 3 '-'
         with _ -> 8) - 3
       in
       let version = String.sub s 3 len in
     Some (brand, version)
   end else None
 
+
 let decoder_list = [
     decode_az_style;
     decode_tornado_style;
     decode_mainline_style;
     decode_simple_style;
-    decode_mburst;
-    decode_turbo;
-    decode_plus;
-    decode_xbt;
     decode_bow;
-    decode_exeem;
     decode_shadow;
     decode_bitspirit;
     decode_upnp;
@@ -657,18 +744,22 @@ let decoder_list = [
     decode_shareaza;
     decode_non_zero;
     decode_mldonkey_style;
-    decode_opera;
+    decode_rufus;
+    decode_btuga;
   ]
 
-let parse_software s = 
+let parse_software s =
   let default = (Brand_unknown, "") in
   let rec iter l =
     match l with
       [] -> lprintf_nl "Unknown BT client software version, report the next line to http://mldonkey.sourceforge.net/UnknownBtClients%s\nBTUC:\"%s\"" Autoconf.current_version (String.escaped s);
             default
       | d :: t -> match (d s) with 
-                  | None -> iter t
-                  | Some bv -> bv
+                  | None    -> iter t
+                  | Some bv -> let (brand, version) = bv in
+                               if !verbose_msg_clienttags then
+                                 lprintf_nl "BTKC:\"%s\"; ID: \"%s\"; version:\"%s\"" (String.escaped s) (brand_to_string brand) version;
+                               bv
   in
   if Sha1.direct_of_string s = Sha1.null then
     default
@@ -698,17 +789,19 @@ let new_client file peer_id kind =
           client_chunks = [];
           client_ranges_sent = [];
           client_range_waiting = None;
-          client_block = None;
+          client_chunk = None;
           client_uid = peer_id;
           client_brand = brand;
           client_release = release;
           client_bitmap = None;
           client_allowed_to_write = zero;
-          client_uploaded = zero;
-          client_downloaded = zero;
+          client_total_uploaded = zero;
+          client_total_downloaded = zero;
+          client_session_uploaded = zero;
+          client_session_downloaded = zero;
           client_upload_rate = Rate.new_rate ();
           client_downloaded_rate = Rate.new_rate ();
-          client_optimist_time=0;
+          client_connect_time = last_time ();
           client_blocks_sent = [];
           client_new_chunks = [];
           client_good = false;
@@ -719,6 +812,11 @@ let new_client file peer_id kind =
           client_incoming = false;
           client_registered_bitfield = false;
           client_last_optimist = 0;
+          client_dht = false;
+          client_cache_extension = false;
+          client_fast_extension = false;
+          client_utorrent_extension = false;
+          client_azureus_messaging_protocol = false;
         } and impl = {
           dummy_client_impl with
           impl_client_val = c;
@@ -755,6 +853,13 @@ let remove_tracker url file =
       lprintf_nl "New tracker list :%s" tracker.tracker_url
     ) file.file_trackers
 
+let tracker_is_enabled t =
+  match t.tracker_status with
+  | Enabled -> true
+  | Disabled_failure (i,_) ->
+      if !!tracker_retries = 0 || i < !!tracker_retries then true else false
+  | _ -> false
+
 let torrents_directory = "torrents"
 let new_torrents_directory = Filename.concat torrents_directory "incoming"
 let downloads_directory = Filename.concat torrents_directory "downloads"
