@@ -87,7 +87,8 @@ let enable () =
     add_session_timer enabler 300. (fun _ ->
         BTInteractive.share_files ();
     );
-    add_session_timer enabler 10. (fun _ ->
+    if !!import_new_torrents_interval <> 0. then
+    add_session_timer enabler !!import_new_torrents_interval (fun _ ->
         BTInteractive.scan_new_torrents_directory ();
     );
     add_timer 10. BTInteractive.share_files;
@@ -115,7 +116,6 @@ let enable () =
     BTClients.recover_files ();
     add_session_timer enabler 60.0 (fun timer ->
         BTClients.recover_files ();
-        BTInteractive.file_magic_check ()
     );
 
     add_session_timer enabler 120.0 (fun timer ->
@@ -131,7 +131,6 @@ let enable () =
 	      ) !current_files;
   );
 
-  BTInteractive.file_magic_check ();
   BTClients.listen ();
   ()
 
@@ -161,10 +160,10 @@ let _ =
             [] -> "" | opfile :: _ -> options_file_name opfile);
         network_netflags = network.network_flags;
         network_netname = network.network_name;
-        network_enabled = network.op_network_is_enabled ();
+        network_enabled = network_is_enabled network;
         network_uploaded = !bt_upload_counter;
         network_downloaded = !bt_download_counter;
-        network_connected = 0;
+        network_connected_servers = 0;
       });
   CommonInteractive.register_gui_options_panel "BitTorrent"
   gui_bittorrent_options_panel
