@@ -96,6 +96,7 @@ let tracked_files = Hashtbl.create 13
 let ntracked_files = ref 0
 
 let tracker_port = define_option bittorrent_section ["tracker_port"]
+  ~restart: true
   "The port to bind the tracker to"
     int_option 6881
 
@@ -555,8 +556,9 @@ let start_tracker () =
           bind_addr = if !!force_client_ip then Ip.to_inet_addr !!set_client_ip else Unix.inet_addr_any ;
           port = !!tracker_port;
           requests = [];
-          addrs = [ Ip.of_string "255.255.255.255" ];
+          addrs = Ip_set.of_list [ Ip.RangeCIDR (Ip.null, 0) ];
           base_ref = "";
+	  use_ip_block_list = true;
           default = http_handler;
         } in
       let sock = TcpServerSocket.create "BT tracker"
