@@ -111,8 +111,15 @@ type brand =
 | Brand_qbittorrent
 | Brand_qt4
 | Brand_uleecher
-
-
+| Brand_flashget
+| Brand_xtorrent
+| Brand_limewire
+| Brand_btpd
+| Brand_btyrant
+| Brand_ares
+| Brand_bitpump
+| Brand_deluge
+| Brand_btuga
 
 let brand_list = [ 
    ( Brand_unknown          , "unknown"                , "unk" ) ;
@@ -179,6 +186,15 @@ let brand_list = [
    ( Brand_qbittorrent      , "qBittorrent"            , "qbt" ) ;
    ( Brand_qt4              , "QT4"                    , "qt4" ) ;
    ( Brand_uleecher         , "uLeecher!"              , "ul!" ) ;
+   ( Brand_flashget         , "FlashGet"               , "flg" ) ;
+   ( Brand_xtorrent         , "Xtorrent"               , "xto" ) ;
+   ( Brand_limewire         , "LimeWire"               , "lwi" ) ;
+   ( Brand_btpd             , "BT Protocol Daemon"     , "bpd" ) ;
+   ( Brand_btyrant          , "BitTyrant"              , "bty" ) ;
+   ( Brand_ares             , "Ares"                   , "are" ) ;
+   ( Brand_bitpump          , "BitPump"                , "bpu" ) ;
+   ( Brand_deluge           , "Deluge"                 , "del" ) ;
+   ( Brand_btuga            , "BTuga Revolution"       , "btr" ) ;
   ] 
 
 let brand_count = List.length brand_list
@@ -196,7 +212,7 @@ type tracker_status =
   Enabled
 | Disabled of string
 | Disabled_mld of string
-| Disabled_failure of string
+| Disabled_failure of (int * string)
 
 type client = {
     client_client : client CommonClient.client_impl;
@@ -227,8 +243,10 @@ type client = {
     mutable client_allowed_to_write : int64;
     mutable client_upload_rate : Rate.t;
     mutable client_downloaded_rate :  Rate.t;
-    mutable client_downloaded : int64;
-    mutable client_uploaded : int64;
+    mutable client_total_downloaded : int64;
+    mutable client_total_uploaded : int64;
+    mutable client_session_downloaded : int64;
+    mutable client_session_uploaded : int64;
     mutable client_connect_time : int;
 
     mutable client_blocks_sent : int list;
