@@ -601,7 +601,7 @@ module ConnectToMe = NickAndAddr(struct let msg = "ConnectToMe" end)
 module MultiConnectToMe = NickAndAddr(struct let msg = "MultiConnectToMe" end)
   
 module Msg = struct
-    type t = ()
+    type t = unit
       
     let parse s = ()
       
@@ -950,4 +950,4 @@ let make_shared_list () =
   in
   iter 0 shared_tree;
   Buffer.contents buf 
- 
\ No newline at end of file
+ 
