@@ -68,40 +68,43 @@ let active_lifetime = 1200.
 (*************************************************************************)
 (*              adding a source to the source-management                 *)
 (*************************************************************************)
-let add_source file ip port serverIP serverPort =
-    (* man, we are receiving sources from some clients even when we release *)
-    if (file_state file) = FileDownloading then
-        try
-            let uid = 
-                if low_id ip then
-                  (* indirect address *)
-                  begin
-                    try
-                        (* without server, we can't request a callback *)
-                        let s = Hashtbl.find servers_by_key serverIP in
-                        if serverPort = s.server_port then
-                            Indirect_address ( serverIP, serverPort, id_of_ip ip, 0, Ip.null )
-                        else
-                            raise Not_found
-                    with _ ->
-                        raise Not_found
-                  end
+let add_source file ip tcp_port serverIP serverPort =
+  (* man, we are receiving sources from some clients even when we release *)
+  if (file_state file) = FileDownloading then
+    try
+      let uid = 
+    	if low_id ip then
+          begin
+            try
+              (* without server, we can't request a callback *)
+              let s = Hashtbl.find servers_by_key serverIP in
+                if serverPort = s.server_port then
+                  Indirect_address (serverIP, serverPort, id_of_ip ip, 0, Ip.null)
                 else
-                  (* direct adsdess *)
-                  if Ip.usable ip then
-                    if not ( is_black_address ip port ) then
-                        if not ( Hashtbl.mem banned_ips ip) then
-                            Direct_address ( ip, port )
-                        else
-                            raise Not_found
-                    else
-                        raise Not_found
-                  else
-                    raise Not_found
-            in
-            let s = DonkeySources.find_source_by_uid uid in
-            DonkeySources.set_request_result s file.file_sources File_new_source;
-        with Not_found -> ()
+                  raise Not_found
+            with Not_found ->
+              if !!update_server_list_client then
+                begin
+                  ignore (check_add_server serverIP serverPort);
+                  Indirect_address (serverIP, serverPort, id_of_ip ip, 0, Ip.null)
+                end
+              else raise Not_found
+          end
+        else
+          if Ip.usable ip then
+            if not ( is_black_address ip tcp_port ) then
+              if not ( Hashtbl.mem banned_ips ip) then
+                Direct_address ( ip, tcp_port )
+              else
+                raise Not_found
+            else
+              raise Not_found
+          else
+            raise Not_found
+      in
+      let s = DonkeySources.find_source_by_uid uid in
+      DonkeySources.set_request_result s file.file_sources File_new_source;
+    with Not_found -> ()
 
 let is_banned c sock = 
   c.client_banned <- Hashtbl.mem banned_ips (fst (peer_addr sock))
@@ -112,7 +115,7 @@ let supports_eep cb =
   match cb with
     Brand_lmule | Brand_newemule | Brand_cdonkey |
     Brand_emuleplus | Brand_hydranode | Brand_mldonkey3 |
-    Brand_shareaza | Brand_amule | Brand_lphant | Brand_verycd -> true
+    Brand_shareaza | Brand_amule | Brand_lphant | Brand_verycd | Brand_imp -> true
   | _ -> false
 
 let ban_client c sock msg = 
@@ -245,20 +248,7 @@ let disconnect_client c reason =
       c.client_source.DonkeySources.source_sock <- NoConnection
   | Connection sock ->
       (try
-    let log_print cc =
-            lprintf_nl "Client[%d] %s disconnected, connected %s%s%s"
-        (client_num cc)
-        (full_client_identifier cc)
-        (Date.time_to_string (last_time () - cc.client_connect_time) "verbose")
-        (if cc.client_uploaded > 0L then
-    Printf.sprintf ", send %s" (size_of_int64 cc.client_uploaded) else "")
-        (if cc.client_downloaded > 0L then
-    Printf.sprintf ", rec %s" (size_of_int64 cc.client_downloaded) else "")
-    in
-    if c.client_debug ||
-      (!verbose && (c.client_uploaded > 0L || c.client_downloaded > 0L)) then
-      log_print c;
-
+          DonkeyOneFile.remove_client_slot c;
           c.client_comp <- None;
           (try if c.client_checked then count_seen c with _ -> ());
           (try if !!log_clients_on_console && c.client_name <> "" then 
@@ -266,43 +256,34 @@ let disconnect_client c reason =
           c.client_connect_time <- 0;
           (try Hashtbl.remove connected_clients c.client_md4 with _ -> ());
           (try CommonUploads.remove_pending_slot (as_client c) with _ -> ());
-          set_client_has_a_slot (as_client c) NoSlot;
-(*          connection_failed c.client_connection_control; *)
           (try TcpBufferedSocket.close sock reason with _ -> ());
 
 (* Remove the Connected and NoLimit tags *)
           set_client_type c (client_type c
               land (lnot (client_initialized_tag lor client_nolimit_tag)));
-(*          c.client_chunks <- [||];*)
           c.client_source.DonkeySources.source_sock <- NoConnection;
           save_join_queue c;
           c.client_slot <- SlotNotAsked;
-          let files = c.client_file_queue in
           
-          (try DonkeyOneFile.clean_current_download c with _ -> ());
+(* clean_client_zones: clean all structures related to downloads when
+   a client disconnects *)
+	  (try
+            match c.client_download with
+            | None -> ()
+            | Some (file, up) ->
+                CommonSwarming.unregister_uploader up;
+                c.client_download <- None
+            with _ -> ());
 
           List.iter (fun (file, chunks, up) -> 
               try CommonSwarming.unregister_uploader up with _ -> ()
-          )
-          files;    
+          ) c.client_file_queue;
+
           c.client_file_queue <- [];  
-          if c.client_upload != None then CommonUploads.refill_upload_slots ();
+          c.client_session_downloaded <- 0L;
         
         with e -> lprintf_nl "Exception %s in disconnect_client"
               (Printexc2.to_string e));
-(*      lprintf "Client %d to source:" (client_num c);
-      List.iter (fun r ->
-                lprint_char (
-                  match r.request_result with
-                  | File_chunk ->      'C'
-                  | File_upload ->     'U'
-                  | File_not_found ->  '-'
-                  | File_found ->      '+'
-                  | File_possible ->   '?'
-                  | File_expected ->   '!'
-                  | File_new_source -> 'n'
-                )) c.client_files;      
-      lprintf "\n"; *)
       set_client_disconnected c reason;
       DonkeySources.source_disconnected c.client_source
   
@@ -441,16 +422,24 @@ let client_wants_file c md4 =
         
   
 let new_chunk up begin_pos end_pos =
-  if begin_pos <> end_pos then
-    let pair = (begin_pos, end_pos) in
-    (match up.up_chunks with
-        [] ->
+  let req_size = end_pos -- begin_pos in
+  let req_location = (begin_pos ++ end_pos) // (2L ** block_size) in
+  if !verbose_upload then
+    lprintf_nl "new block: (%Ld,%Ld) size %Ld chunk #%Ld" begin_pos end_pos req_size req_location;
+  if (req_size < Int64.zero) || (req_size > zone_size) || ((up.up_current <> req_location) && (req_size <> Int64.zero)) then
+    up.up_finish <- true;
+  if ((not up.up_finish) || (not !!upload_complete_chunks)) && (req_size > Int64.zero) && (req_size <= zone_size) then
+    let chunk = (begin_pos, end_pos) in
+    (* the zone requested is already "in the pipe" *)
+    if not (List.mem chunk up.up_flying_chunks) then
+      match up.up_chunks with
+      | [] ->
           up.up_pos <- begin_pos;
           up.up_end_chunk <- end_pos;
-          up.up_chunks <- [pair];
-      | chunks ->
-          if not (List.mem pair chunks) then
-            up.up_chunks <- chunks @ [pair])
+          up.up_chunks <- [chunk];
+      | up_chunks ->
+          if not (List.mem chunk up_chunks) then
+            up.up_chunks <- up_chunks @ [chunk]
   
 let identify_client_brand c =
   if c.client_brand = Brand_unknown then
@@ -599,7 +588,7 @@ let identify_client_brand_mod c tags =
       List.iter (fun tag ->
         let s = to_lowercase (string_of_tag_value tag.tag_value) in 
           match tag.tag_name with
-           Field_UNKNOWN "mod_version" ->
+           Field_KNOWN "mod_version" ->
                begin
                let rec iter i len =
                 if i < len then
@@ -647,6 +636,7 @@ let parse_compatible_client num old_brand =
     | 6 -> Brand_hydranode
     | 10 -> Brand_mldonkey3
     | 20 -> Brand_lphant
+    | 60 -> Brand_imp
     | 240 -> Brand_verycd
     | _ -> Brand_unknown
 
@@ -661,33 +651,42 @@ let parse_mod_version s c =
    iter 0 (Array.length mod_array)
 
 let update_client_from_tags c tags =
+  let module M = DonkeyProtoClient in
   List.iter (fun tag ->
       match tag.tag_name with
-      | Field_UNKNOWN "name" -> ()
-      | Field_UNKNOWN "version" -> ()
-      | Field_UNKNOWN "emule_udpports" -> 
+      | Field_KNOWN "emule_udpports" -> 
           for_two_int16_tag tag (fun ed2k_port kad_port ->
 (* Kademlia: we should use this client to bootstrap Kademlia *)
             if kad_port <> 0 && !!enable_kademlia then
               DonkeyProtoKademlia.Kademlia.bootstrap 
                 c.client_ip kad_port
           )
-      | Field_UNKNOWN "emule_miscoptions1" ->
+      | Field_KNOWN "emule_miscoptions1" ->
+          c.client_emule_proto.received_miscoptions1 <- true;
           for_int64_tag tag (fun i ->
-            DonkeyProtoClient.update_emule_proto_from_miscoptions1
-            c.client_emule_proto i
+            M.update_emule_proto_from_miscoptions1
+            c.client_emule_proto i;
+            if !verbose_msg_clients || c.client_debug then
+              lprintf_nl "miscoptions1 from client %s\n%s"
+                (full_client_identifier c)
+                (M.print_emule_proto_miscoptions1 c.client_emule_proto)
           )
-      | Field_UNKNOWN "emule_miscoptions2" ->
+      | Field_KNOWN "emule_miscoptions2" ->
+          c.client_emule_proto.received_miscoptions2 <- true;
           for_int64_tag tag (fun i ->
-            DonkeyProtoClient.update_emule_proto_from_miscoptions2
-            c.client_emule_proto i
+            M.update_emule_proto_from_miscoptions2
+            c.client_emule_proto i;
+            if !verbose_msg_clients || c.client_debug then
+              lprintf_nl "miscoptions2 from client %s\n%s"
+                (full_client_identifier c)
+                (M.print_emule_proto_miscoptions2 c.client_emule_proto)
           )
-      | Field_UNKNOWN "emule_compatoptions" ->
+      | Field_KNOWN "emule_compatoptions" ->
           for_int_tag tag (fun i ->
-            DonkeyProtoClient.update_emule_proto_from_compatoptions
+            M.update_emule_proto_from_compatoptions
             c.client_emule_proto i
           )
-      | Field_UNKNOWN "emule_version" ->
+      | Field_KNOWN "emule_version" ->
           for_int_tag tag (fun i ->
             c.client_emule_proto.emule_version <- i;
             let compatibleclient = (i lsr 24) in
@@ -697,60 +696,62 @@ let update_client_from_tags c tags =
             if c.client_brand = Brand_unknown then
               lprintf_nl "[emule_version] Brand_unknown %s" (full_client_identifier c);
           )
-      | Field_UNKNOWN "mod_version" ->
+      | Field_KNOWN "mod_version" ->
           let s = to_lowercase (string_of_tag_value tag.tag_value) in 
           parse_mod_version s c
-      | _ -> 
-          if !verbose_msg_clienttags then
-            lprintf_nl "Unknown Emule tag: [%s] (update_client_from_tags)" (escaped_string_of_field tag)
+      | Field_KNOWN _ -> if !verbose_unknown_messages then
+            lprintf_nl "update_client_from_tags, known tag: [%s] (%s)" (string_of_tag tag) (full_client_identifier c)
+      | _ -> if not (DonkeySources.source_brand c.client_source) then
+            lprintf_nl "update_client_from_tags, unknown tag: [%s] (%s) %s"
+              (hexstring_of_tag tag) (full_client_identifier c) (string_of_tags_list tags)
   ) tags
     
 let update_emule_proto_from_tags c tags =
   List.iter (fun tag ->
       match tag.tag_name with
-        Field_UNKNOWN "compatibleclient" ->
+        Field_KNOWN "compatibleclient" ->
           for_int_tag tag (fun i ->
             c.client_brand <- parse_compatible_client i c.client_brand;
             if c.client_brand = Brand_unknown then
               lprintf_nl "unknown compatibleclient %d (%s) (please report to dev team)" i (full_client_identifier c)
           )
-      | Field_UNKNOWN "compression" ->
+      | Field_KNOWN "compression" ->
           for_int_tag tag (fun i ->
             c.client_emule_proto.emule_compression <- i
           )
-      | Field_UNKNOWN "udpver" ->
+      | Field_KNOWN "udpver" ->
           for_int_tag tag (fun i ->
             c.client_emule_proto.emule_udpver <- i
           )
-      | Field_UNKNOWN "udpport" -> ()
-      | Field_UNKNOWN "sourceexchange" ->
+      | Field_KNOWN "sourceexchange" ->
           for_int_tag tag (fun i ->
             c.client_emule_proto.emule_sourceexchange <- i
           ) 
-      | Field_UNKNOWN "comments" ->
+      | Field_KNOWN "comments" ->
           for_int_tag tag (fun i ->
             c.client_emule_proto.emule_comments <- i
           )
-      | Field_UNKNOWN "extendedrequest" ->
+      | Field_KNOWN "extendedrequest" ->
           for_int_tag tag (fun i ->
             c.client_emule_proto.emule_extendedrequest <- i
           )
-      | Field_UNKNOWN "features" ->
+      | Field_KNOWN "features" ->
           for_int_tag tag (fun i ->
             c.client_emule_proto.emule_secident <- i land 0x3
           )
-      | Field_UNKNOWN "mod_version" ->
-          let s = to_lowercase (string_of_tag_value tag.tag_value) in 
-          parse_mod_version s c;
+      | Field_KNOWN "mod_version" ->
+          parse_mod_version (to_lowercase (string_of_tag_value tag.tag_value)) c;
 
-      | Field_UNKNOWN "os_info" ->
+      | Field_KNOWN "os_info" ->
           let s = to_lowercase (string_of_tag_value tag.tag_value) in 
 	  (match c.client_osinfo with
 	    Some _ -> ()
 	  | _ ->  if s <> "" then c.client_osinfo <- Some s)
-      | _ -> 
-          if !verbose_msg_clienttags then
-            lprintf_nl "Unknown Emule tag: [%s] (update_emule_proto_from_tags)" (escaped_string_of_field tag)
+      | Field_KNOWN _ -> if !verbose_unknown_messages then
+            lprintf_nl "update_emule_proto_from_tags, known tag: [%s] (%s)" (string_of_tag tag) (full_client_identifier c)
+      | _ -> if not (DonkeySources.source_brand c.client_source) then
+            lprintf_nl "update_emule_proto_from_tags, unknown tag: [%s] (%s) %s"
+              (hexstring_of_tag tag) (full_client_identifier c) (string_of_tags_list tags)
   ) tags
 
 let fight_disguised_mods c =
@@ -771,7 +772,7 @@ let request_osinfo c =
 	emule_info with
 	DonkeyProtoClient.EmuleClientInfo.protversion = 255;
 	DonkeyProtoClient.EmuleClientInfo.tags = [
-	  string_tag (Field_UNKNOWN "os_info") (String2.upp_initial Autoconf.system);
+	  string_tag (Field_KNOWN "os_info") (String2.upp_initial Autoconf.system);
 	]} in
       client_send c (DonkeyProtoClient.EmuleClientInfoReq emule_osinfo);
       c.client_osinfo_sent <- true
@@ -1128,7 +1129,7 @@ let client_to_client for_files c t sock =
   let module M = DonkeyProtoClient in
   
   if !verbose_msg_clients || c.client_debug then begin
-      lprintf_nl "Message from %s" (string_of_client c);
+      lprintf_nl "Message from %s" (full_client_identifier c);
       M.print t;
     end;
   
@@ -1160,7 +1161,7 @@ let client_to_client for_files c t sock =
 
       List.iter (fun tag ->
           match tag with
-            { tag_name = Field_UNKNOWN "name"; tag_value = String s } -> 
+            { tag_name = Field_KNOWN "name"; tag_value = String s } -> 
               set_client_name c s t.CR.md4
           | _ -> ()
       ) c.client_tags;
@@ -1374,7 +1375,7 @@ end; *)
           end;
 
 (*    set_rtimeout sock !!upload_timeout; *)
-          set_lifetime sock one_day;
+          set_lifetime sock (float_of_int Date.day_in_secs);
           add_pending_slot c
         
 with _ -> *)
@@ -1401,7 +1402,7 @@ other one for unlimited sockets.  *)
 	     | Some f -> CommonFile.file_best_name f);
 (*      end *)
   
-  | M.CloseSlotReq _ ->
+  | M.OutOfPartsReq _ ->
       set_client_state c (Connected 0);
       begin
         match c.client_download with
@@ -1411,7 +1412,7 @@ other one for unlimited sockets.  *)
                 lprintf_nl "Slot closed during download";
             CommonSwarming.clear_uploader_ranges up
       end;
-(*      DonkeyOneFile.clean_current_download c; *)
+      c.client_session_downloaded <- 0L;
       c.client_slot <- SlotNotAsked;
 (* OK, the slot is closed, but what should we do now ????? *)
       begin
@@ -1419,7 +1420,7 @@ other one for unlimited sockets.  *)
           [] -> ()
         | _ -> 
             if !verbose_download then
-                lprintf_nl "CloseSlotReq"; 
+                lprintf_nl "OutOfPartsReq"; 
             DonkeyOneFile.request_slot c;
             set_rtimeout sock !!queued_timeout;
       end
@@ -1560,8 +1561,7 @@ is checked for the file.
 (*      if file.file_exists then verify_chunks file *)
       end
   
-  
-  | M.EmuleCompressedPart (md4, statpos, newsize, bloc) ->
+  | M.EmuleCompressedPart t ->
       
       set_lifetime sock active_lifetime;      
       if !!reliable_sources && 
@@ -1572,12 +1572,13 @@ is checked for the file.
           raise Not_found
         end;
       
+      let module Q = M.EmuleCompressedPart in
       let comp = match c.client_comp with
           None ->
             let comp = {
-                comp_md4 = md4;
-                comp_pos = statpos;
-                comp_total = Int64.to_int newsize;
+                comp_md4 = t.Q.md4;
+                comp_pos = t.Q.statpos;
+                comp_total = Int64.to_int t.Q.newsize;
                 comp_len = 0;
                 comp_blocs = [];
               } in
@@ -1585,8 +1586,8 @@ is checked for the file.
             comp
         | Some comp -> comp
       in
-      comp.comp_blocs <- bloc :: comp.comp_blocs;
-      comp.comp_len <- comp.comp_len + String.length bloc;
+      comp.comp_blocs <- t.Q.bloc :: comp.comp_blocs;
+      comp.comp_len <- comp.comp_len + String.length t.Q.bloc;
 
 (*            lprintf "Comp bloc: %d/%d\n" comp.comp_len comp.comp_total; *)
       if comp.comp_len = comp.comp_total then begin
@@ -2008,44 +2009,65 @@ end else *)
   | M.QueryBlocReq t when !CommonUploads.has_upload = 0 &&
     client_has_a_slot (as_client c) ->
       
-      if !verbose_upload then
-          lprintf_nl "donkeyClient: uploader %s ask for block" (full_client_identifier c);
-      
       let module Q = M.QueryBloc in
-      let file = find_file  t.Q.md4 in
+      let file = find_file t.Q.md4 in
+
+      if !verbose_upload then lprintf_nl "donkeyClient: uploader %s asks for %s"
+            (full_client_identifier c) (file_best_name file);
+
       let prio = (file_priority file) in
-      let client_upload_lifetime = ref ((max 0 !!upload_lifetime) * 60) in
-      begin
+      let client_upload_lifetime = ref ((max 0 !!upload_lifetime) * Date.minute_in_secs) in
         
-        if !!dynamic_upload_lifetime
-            && c.client_uploaded > c.client_downloaded
-            && c.client_uploaded > Int64.of_int !!dynamic_upload_threshold ** zone_size
+      if !!dynamic_upload_lifetime && not !!upload_complete_chunks
+            && c.client_session_uploaded > c.client_session_downloaded
+            && c.client_session_uploaded > Int64.of_int !!dynamic_upload_threshold ** zone_size
         then
           client_upload_lifetime :=
           Int64.to_int 
             (Int64.of_int !client_upload_lifetime 
-              ** c.client_downloaded // c.client_uploaded);
-        if last_time() > c.client_connect_time + 
-            !client_upload_lifetime + 5 * prio then
-          begin
+              ** c.client_session_downloaded // c.client_session_uploaded);
 
-(* And what happens if we were downloading from this client also ? *)
-            
-            disconnect_client c (Closed_for_error "Upload lifetime expired");
+      let client_received_enough c =
+        if !!upload_full_chunks && not !!upload_complete_chunks then
+          c.client_session_uploaded > (block_size ++ 20L ** 1024L)
+        else
+          last_time() > c.client_connect_time + !client_upload_lifetime + 5 * prio
+      in
+
+      begin
+        if client_received_enough c then
+          if Intmap.length !CommonUploads.pending_slots_map = 0 then
+            begin
+              if !verbose_upload then lprintf_nl
+                "donkeyClient: not closing upload slot of %s (%s), pending slots empty, sending next block..."
+                  (full_client_identifier c) (file_best_name file)
+            end
+          else begin
+            DonkeyOneFile.remove_client_slot c;
             raise Not_found
           end;
-        
+
         set_lifetime sock active_lifetime;
         set_rtimeout sock !!upload_timeout;
-        
+
         let up, waiting = match c.client_upload with
-            Some ({ up_file = f } as up) when f == file ->  up, up.up_waiting
+          | Some ({ up_file = f } as up) when f == file ->
+	      (* zones are received in the order they're sent, so we
+		 know that the oldest of the zones "in fly" must have
+		 been received when this QueryBlockReq was sent *)
+	      (match up.up_flying_chunks with
+	       | [] -> () 
+	       | _ :: q -> up.up_flying_chunks <- q);
+	      up, up.up_waiting
           | Some old_up ->
               {
                 up_file = file;
                 up_pos = Int64.zero;
                 up_end_chunk = Int64.zero;
                 up_chunks = [];
+		up_flying_chunks = [];
+                up_current = Int64.zero;
+                up_finish = true;
                 up_waiting = old_up.up_waiting;
               }, old_up.up_waiting
           | _ ->
@@ -2054,18 +2076,31 @@ end else *)
                 up_pos = Int64.zero;
                 up_end_chunk = Int64.zero;
                 up_chunks = [];
+		up_flying_chunks = [];
+                up_current = ((t.Q.start_pos1 ++ t.Q.end_pos1) // (2L ** block_size));
+                up_finish = false;
                 up_waiting = false;
               }, false
         in
         new_chunk up t.Q.start_pos1 t.Q.end_pos1;
         new_chunk up t.Q.start_pos2 t.Q.end_pos2;
         new_chunk up t.Q.start_pos3 t.Q.end_pos3;
+        (match up.up_chunks with
+          [] ->
+(* it should never happen here, that a client with up.up_finish = false
+   has an empty block queue *)
+            if up.up_finish && !!upload_complete_chunks then
+              begin
+                DonkeyOneFile.remove_client_slot c;
+                raise Not_found
+              end;
+          | chunks ->
         c.client_upload <- Some up;
         set_client_upload (as_client c) (as_file file);
         if not waiting && !CommonUploads.has_upload = 0 then begin
             CommonUploads.ready_for_upload (as_client c);
             up.up_waiting <- true
-          end
+          end)
       end;
       if !verbose_upload then lprintf_nl "QueryBloc treated"
       
@@ -2141,12 +2176,12 @@ make 1500 connections/10 minutes.  *)
 let init_client sock c =
   set_handler sock WRITE_DONE (fun s ->
       match c.client_upload with
-        None -> ()
-      | Some up ->
+      | Some ({ up_chunks = _ :: _ } as up) ->
           if not up.up_waiting && !CommonUploads.has_upload = 0 then begin
               up.up_waiting <- true;
               CommonUploads.ready_for_upload (as_client c)
             end
+      | _ -> ()
   );
 (*
   set_handler sock (BASIC_EVENT RTIMEOUT) (fun s ->
@@ -2193,7 +2228,7 @@ let read_first_message overnet server m sock =
       let name = ref "" in
       List.iter (fun tag ->
           match tag with
-            { tag_name = Field_UNKNOWN "name"; tag_value = String s } -> name := s
+            { tag_name = Field_KNOWN "name"; tag_value = String s } -> name := s
           | _ ->  ()
       ) t.CR.tags;
 
