@@ -68,12 +68,13 @@ let create_online_sig () =
   let server_ip = ref "" in
   let server_port = ref 0 in
   List.iter (fun s -> 
-      if s.server_nusers > !most_users then begin 
+      match s.server_nusers with
+      | Some v when v > !most_users ->
           server_name := s.server_name;
           server_ip := (Ip.to_string s.server_ip);
           server_port := s.server_port;
-          most_users := s.server_nusers;
-        end
+          most_users := (match s.server_nusers with None -> 0L | Some v -> v)
+      | _ -> ()
   ) (connected_servers());
   
   Unix2.tryopen_write "onlinesig.dat" (fun oc ->
@@ -179,11 +180,8 @@ let value_to_server assocs =
             ip, port
         | _ -> failwith  "Options: Not an server option"
     ) in
-  let l = DonkeyGlobals.new_server ip port !!initial_score in
+  let l = DonkeyGlobals.new_server ip port in
   
-  (try
-      l.server_description <- get_value "server_desc" value_to_string 
-    with _ -> ());
   (try
       l.server_name <- get_value "server_name" value_to_string
     with _ -> ());
@@ -192,7 +190,7 @@ let value_to_server assocs =
     with _ -> ());
   (try
       connection_set_last_conn l.server_connection_control
-        (normalize_time (mini (get_value "server_age" value_to_int) 
+        (normalize_time (min (get_value "server_age" value_to_int) 
           (BasicSocket.last_time ())));
     with _ -> ());
   as_server l.server_server
@@ -206,11 +204,6 @@ let server_to_value c =
   ]
   in
   
-  let fields = 
-    if c.server_description <> "" then 
-      ("server_desc", string_to_value c.server_description) :: fields
-    else fields in
-  
   let fields = 
     if c.server_name <> "" then 
       ("server_name", string_to_value c.server_name) :: fields
@@ -231,10 +224,7 @@ let value_to_int32pair v =
 
 let value_to_file file_size file_state assocs =
   let get_value name conv = conv (List.assoc name assocs) in
-  let get_value_nil name conv = 
-    try conv (List.assoc name assocs) with _ -> []
-  in
-  
+
   let file_md4 = 
     try
       get_value "file_md4" value_to_string
@@ -256,7 +246,7 @@ let value_to_file file_size file_state assocs =
                 !!temp_directory
                 ( string_of_uid ( Ed2k (Md4.of_string file_md4) ) )
             in
-            lprintf_nl "geting file_diskname from ini failed, testing for ed2k-temp-file %s"
+            lprintf_nl "getting file_diskname from ini failed, testing for ed2k-temp-file %s"
               filename;
             if Sys.file_exists filename then
               filename
@@ -270,19 +260,16 @@ let value_to_file file_size file_state assocs =
       (* I think we should die here, to prevent any corruption. *)
       lprintf_nl "ERROR ED2K-TEMP-FILE %s DOES NOT EXIST, THIS WILL PERHAPS LEAD TO CORRUPTION IN THAT DOWNLOAD!"
         filename;
-    if !verbose then lprintf_nl "ed2k-temp-file %s used." filename;
+    if !verbose && (not !CommonGlobals.is_startup_phase) then lprintf_nl "ed2k-temp-file %s used." filename;
     filename
   in
 
-  let filenames = List.map (fun name -> name, GuiTypes.noips()) 
-    (get_value_nil "file_filenames" (value_to_list value_to_string)) in
-  
   let file = DonkeyGlobals.new_file file_diskname file_state
-    (Md4.of_string file_md4) file_size filenames true in
+    (Md4.of_string file_md4) file_size "" true CommonUserDb.admin_user in
 
   (try
       set_file_best_name (as_file file)
-      (get_value "file_filename" value_to_string)
+      (get_value "file_filename" value_to_string) "" 0
     with _ -> update_best_name file);
   
   let md4s = try get_value "file_md4s" (value_to_array value_to_md4) 
@@ -309,8 +296,6 @@ let file_to_value file =
       "file_diskname", string_to_value file.file_diskname;
       ("file_md4s", 
         array_to_value Md4.hash_to_value file.file_computed_md4s);
-      "file_filenames", List
-        (List.map (fun (s,_) -> string_to_value s) file.file_filenames);
     ]
   in
   let fields = 
@@ -439,7 +424,7 @@ let force_add_server ip port =
   try
     DonkeyGlobals.find_server ip port
   with _ ->
-      let s = DonkeyGlobals.new_server ip port !!initial_score in
+      let s = DonkeyGlobals.new_server ip port in
       s        
       
 let check_add_server ip port =
@@ -454,7 +439,7 @@ let safe_add_server ip port =
     try
       ignore (DonkeyGlobals.find_server ip port)
     with _ ->
-        ignore (DonkeyGlobals.new_server ip port !!initial_score)
+        ignore (DonkeyGlobals.new_server ip port)
         
 let config_files_loaded = ref false  
       
@@ -525,7 +510,7 @@ let save _ =
     end
 
 let save_sources _ =
-  if !sources_loaded then begin
+  if !sources_loaded && !!keep_sources then begin
       save_time =:= last_time ();
       let cleaner = DonkeySources.attach_sources_to_file file_sources_section in
       Options.save_with_help file_sources_ini;
@@ -535,17 +520,14 @@ let save_sources _ =
 let guptime () = !!guptime - !diff_time
   
 let load_sources () =
-  (lprintf "(loading sources ";
-   try 
+  if not !!keep_sources then () else
+  (try 
       let cleaner = DonkeySources.attach_sources_to_file file_sources_section in
       cleaner ();
-      lprintf ".";
       Options.load file_sources_ini;
-      lprintf ".";
       cleaner ();
-      lprintf ".";
       sources_loaded := true;
-      lprintf " completed) ";
+      lprintf_nl "loading sources completed"
     with _ -> ())
 
 let check_result r tags =
@@ -573,8 +555,12 @@ let result_of_file md4 tags =
       match tag with
         { tag_name = Field_Filename; tag_value = String s } ->
           r.result_names <- s :: r.result_names
-      | { tag_name = Field_Size; tag_value = Uint64 v } ->
+      | { tag_name = Field_Size; tag_value = (Uint64 v| Fint64 v) } ->
           r.result_size <- v;
+      | { tag_name = Field_Size; tag_value = (Uint16 v| Uint8 v) } ->
+          r.result_size <- Int64.of_int v;
+      | { tag_name = Field_Size_Hi; tag_value = Uint8 v } ->
+          r.result_size <- Int64.logor r.result_size (Int64.shift_left (Int64.of_int v) 32);
       | { tag_name = Field_Format; tag_value = String s } ->
           r.result_tags <- tag :: r.result_tags;
           r.result_format <- s
