@@ -104,6 +104,7 @@ let as_client c = as_client c.client_client
 let as_file file = as_file file.file_file
 let file_priority file = file.file_file.impl_file_priority
 let file_size file = file.file_file.impl_file_size
+let file_is_largefile f = file_size f > old_max_emule_file_size
 let file_downloaded file = file_downloaded (as_file file)
 let file_age file = file.file_file.impl_file_age
 let file_fd file = file_fd (as_file file)
@@ -287,6 +288,12 @@ let remove_connected_server c =
 
 let connected_servers () = !connected_server_list
 
+let logged_in_servers () =
+List.filter (fun s -> 
+  match server_state s with
+  | Connected _ -> true
+  | _ -> false) !connected_server_list
+
 let get_udp_sock () =
   match !udp_sock with
     None -> failwith "No UDP socket"
@@ -414,7 +421,7 @@ let new_file file_diskname file_state md4 file_size filename writable user =
           impl_file_fd = Some t;
           impl_file_best_name = Filename.basename file_diskname;
           impl_file_filenames = (if filename = "" then [] else [filename]);
-          impl_file_last_seen = last_time () - 100 * 24 * 3600;
+          impl_file_last_seen = last_time () - 100 * Date.day_in_secs;
         }
       in
 
@@ -595,16 +602,15 @@ let dummy_client =
       client_osinfo = None;
       client_checked = false;
       client_connected = false;
-      client_downloaded = Int64.zero;
-      client_uploaded = Int64.zero;
+      client_session_downloaded = Int64.zero;
+      client_session_uploaded = Int64.zero;
+      client_total_downloaded = Int64.zero;
+      client_total_uploaded = Int64.zero;
       client_banned = false;
-      client_score = 0;
-      client_next_queue = 0;
       client_rank = 0;
       client_connect_time = 0;
       client_requests_sent = 0;
       client_requests_received = 0;
-      client_indirect_address = None;
       client_slot = SlotNotAsked;
       client_debug = false;
       client_pending_messages = [];
@@ -650,16 +656,15 @@ let create_client key =
       client_osinfo = None;
       client_checked = false;
       client_connected = false;
-      client_downloaded = Int64.zero;
-      client_uploaded = Int64.zero;
+      client_total_downloaded = Int64.zero;
+      client_total_uploaded = Int64.zero;
+      client_session_downloaded = Int64.zero;
+      client_session_uploaded = Int64.zero;
       client_banned = false;
-      client_score = 0;
-      client_next_queue = 0;
       client_rank = 0;
       client_connect_time = 0;
       client_requests_received = 0;
       client_requests_sent = 0;
-      client_indirect_address = None;
       client_slot = SlotNotAsked;
       client_debug = Intset.mem s.DonkeySources.source_num !debug_clients;
       client_pending_messages = [];
@@ -715,17 +720,6 @@ let set_client_type c t=
 let friend_add c =
   friend_add (as_client c)
 
-let string_of_client c =
-  Printf.sprintf "client[%d] %s(%s) %s" (client_num c)
-  c.client_name (brand_to_string c.client_brand)
-  (match c.client_kind with
-      Indirect_address (server_ip, server_port, ip, port, real_ip) ->
-        Printf.sprintf  " I[%s:%d]" (Ip.to_string real_ip) port;
-    | Direct_address (ip,port) ->
-        Printf.sprintf  " D[%s:%d]" (Ip.to_string ip) port;
-    | Invalid_address _ -> ""
-  )
-
 let string_of_server s =
   Printf.sprintf "%s:%d" (Ip.to_string s.server_ip) s.server_port
 
@@ -964,21 +958,20 @@ let save_join_queue c =
       with _ -> ()
     end
 
-let half_hour = 30 * 60
 let clean_join_queue_tables () =
   let current_time = last_time () in
 
   let list = Hashtbl2.to_list2 join_queue_by_md4 in
   Hashtbl.clear join_queue_by_md4;
   List.iter (fun (key, ((v,time) as e)) ->
-      if time + half_hour > current_time then
+      if time + Date.half_hour_in_secs > current_time then
         Hashtbl.add join_queue_by_md4 key e
   ) list;
 
   let list = Hashtbl2.to_list2 join_queue_by_id in
   Hashtbl.clear join_queue_by_id;
   List.iter (fun (key, ((v,time) as e)) ->
-      if time + half_hour > current_time then
+      if time + Date.half_hour_in_secs > current_time then
         Hashtbl.add join_queue_by_id key e
   ) list
 
@@ -1017,7 +1010,13 @@ let _ =
 
 let full_client_identifier c =
     Printf.sprintf "%s (%s%s) '%s'"
-      (Ip.to_string c.client_ip)
-      (brand_to_string_short c.client_brand)
+      (match c.client_kind with
+          Indirect_address (server_ip, server_port, ip, port, real_ip) ->
+            Printf.sprintf "%s:%d(lowID, server:%s:%d]"
+              (Ip.to_string real_ip) port (Ip.to_string server_ip) server_port;
+        | Direct_address (ip,port) ->
+            Printf.sprintf "%s:%d" (Ip.to_string ip) port;
+        | Invalid_address _ -> " invalid IP")
+      (GuiTypes.client_software_short (brand_to_string_short c.client_brand) c.client_osinfo)
       (if c.client_emule_proto.emule_release = "" then "" else " " ^ c.client_emule_proto.emule_release)
       (String.escaped c.client_name)
