@@ -113,44 +113,44 @@ let load_server_met filename =
           let server = check_add_server r.S.ip r.S.port in
           List.iter (fun tag ->
               match tag with
-              |  { tag_name = Field_UNKNOWN "name"; tag_value = String s } ->
+              |  { tag_name = Field_KNOWN "name"; tag_value = String s } ->
                   server.server_name <- s;
-              |  { tag_name = Field_UNKNOWN "description" ; tag_value = String s } ->
+              |  { tag_name = Field_KNOWN "description" ; tag_value = String s } ->
                   server.server_description <- s
-              |  { tag_name = Field_UNKNOWN "version" ; tag_value = Uint64 s } ->
+              |  { tag_name = Field_KNOWN "version" ; tag_value = Uint64 s } ->
                   server.server_version <- Printf.sprintf "%d.%d"
 					    ((Int64.to_int s) lsr 16) ((Int64.to_int s) land 0xFFFF)
-              |  { tag_name = Field_UNKNOWN "ping" ; tag_value = Uint64 s } ->
+              |  { tag_name = Field_KNOWN "ping" ; tag_value = Uint64 s } ->
                   server.server_ping <- (Int64.to_int s)
-              |  { tag_name = Field_UNKNOWN "dynip" ; tag_value = String s } ->
+              |  { tag_name = Field_KNOWN "dynip" ; tag_value = String s } ->
                   server.server_dynip <- s
-              |  { tag_name = Field_UNKNOWN "users" ; tag_value = Uint64 s } ->
+              |  { tag_name = Field_KNOWN "users" ; tag_value = Uint64 s } ->
                   (match server.server_nusers with
 		  | None -> server.server_nusers <- Some s | _ -> ())
-              |  { tag_name = Field_UNKNOWN "files" ; tag_value = Uint64 s } ->
+              |  { tag_name = Field_KNOWN "files" ; tag_value = Uint64 s } ->
                   (match server.server_nfiles with
 		  | None -> server.server_nfiles <- Some s | _ -> ())
-              |  { tag_name = Field_UNKNOWN "maxusers" ; tag_value = Uint64 s } ->
+              |  { tag_name = Field_KNOWN "maxusers" ; tag_value = Uint64 s } ->
                   (match server.server_max_users with
 		  | None -> server.server_max_users <- Some s | _ -> ())
-              |  { tag_name = Field_UNKNOWN "softfiles" ; tag_value = Uint64 s } ->
+              |  { tag_name = Field_KNOWN "softfiles" ; tag_value = Uint64 s } ->
                   (match server.server_soft_limit with
 		  | None -> server.server_soft_limit <- Some s | _ -> ())
-              |  { tag_name = Field_UNKNOWN "hardfiles" ; tag_value = Uint64 s } ->
+              |  { tag_name = Field_KNOWN "hardfiles" ; tag_value = Uint64 s } ->
                   (match server.server_hard_limit with
 		  | None -> server.server_hard_limit <- Some s | _ -> ())
-              |  { tag_name = Field_UNKNOWN "auxportslist" ; tag_value = String s } ->
+              |  { tag_name = Field_KNOWN "auxportslist" ; tag_value = String s } ->
                   server.server_auxportslist <- s
-              |  { tag_name = Field_UNKNOWN "lowusers" ; tag_value = Uint64 s } ->
+              |  { tag_name = Field_KNOWN "lowusers" ; tag_value = Uint64 s } ->
                   (match server.server_lowid_users with
 		  | None -> server.server_lowid_users <- Some s | _ -> ())
-              |  { tag_name = Field_UNKNOWN "tcpportobfuscation" ; tag_value = Uint64 s } ->
+              |  { tag_name = Field_KNOWN "tcpportobfuscation" ; tag_value = Uint64 s } ->
                   server.server_obfuscation_tcp <- Some (Int64.to_int s)
-              |  { tag_name = Field_UNKNOWN "udpportobfuscation" ; tag_value = Uint64 s } ->
+              |  { tag_name = Field_KNOWN "udpportobfuscation" ; tag_value = Uint64 s } ->
                   server.server_obfuscation_udp <- Some (Int64.to_int s)
-              |  { tag_name = Field_UNKNOWN "country" ; tag_value = String s } -> ()
-              |  { tag_name = Field_UNKNOWN "udpflags" ; tag_value = Uint64  s } -> ()
-              |  { tag_name = Field_UNKNOWN "refs" ; tag_value = Uint64  s } -> ()
+              |  { tag_name = Field_KNOWN "country" ; tag_value = String s } -> ()
+              |  { tag_name = Field_KNOWN "udpflags" ; tag_value = Uint64  s } -> ()
+              |  { tag_name = Field_KNOWN "refs" ; tag_value = Uint64  s } -> ()
               | _ -> lprintf_nl "parsing server.met, unknown field %s" (string_of_tag tag)
           ) r.S.tags;
 	  server_must_update server
@@ -275,7 +275,8 @@ let really_query_download filename size md4 location old_file absents user =
         match file.file_swarmer with
           None -> assert false
         | Some swarmer ->
-            let absents = Sort.list (fun (p1,_) (p2,_) -> p1 <= p2) absents in
+            let absents = 
+	      List.sort (fun (p1, _) (p2, _) -> compare p1 p2) absents in
             CommonSwarming.set_absent swarmer absents
   end;
 
@@ -429,6 +430,8 @@ let import_temp temp_dir =
                     filename_met := Some s;
                 | { tag_name = Field_Size; tag_value = Uint64 v } ->
                     size := v
+                | { tag_name = Field_Size_Hi; tag_value = Uint64 v } ->
+                    size := Int64.logor !size (Int64.shift_left v 32)
                 | _ -> ()
             ) f.P.tags;
             ignore (really_query_download
@@ -447,16 +450,16 @@ let import_config dirname =
 
   List.iter (fun tag ->
       match tag with
-      | { tag_name = Field_UNKNOWN "name"; tag_value = String s } ->
+      | { tag_name = Field_KNOWN "name"; tag_value = String s } ->
           login =:=  s
-      | { tag_name = Field_UNKNOWN "port"; tag_value = Uint64 v } ->
+      | { tag_name = Field_KNOWN "port"; tag_value = Uint64 v } ->
           donkey_port =:=  Int64.to_int v
       | _ -> ()
   ) ct;
 
   List.iter (fun tag ->
       match tag with
-      | { tag_name = Field_UNKNOWN "temp"; tag_value = String s } ->
+      | { tag_name = Field_KNOWN "temp"; tag_value = String s } ->
           if Sys.file_exists s then (* be careful on that *)
             temp_dir := s
           else (lprintf_nl "Bad temp directory, using default";
@@ -559,8 +562,9 @@ let parse_donkey_url url user =
   | "ed2k://" :: "file" :: name :: size :: md4 :: "/" :: "sources" :: sources :: _
   | "file" :: name :: size :: md4 :: "/" :: "sources" :: sources :: _ ->
 (*  ed2k://|file|Wikipedia_3.3_noimages.iso|2666311680|747735CD46B61DA92973E9A8840A9C99|/|sources,62.143.4.124:4662|/  *)
-      if Int64.of_string size >= 4294967295L then
-	(Printf.sprintf (_b "Files > 4GB are not allowed")), false
+      if Int64.of_string size >= max_emule_file_size then
+	(Printf.sprintf (_b "Files > %s are not allowed")
+	  (Int64ops.int64_to_human_readable max_emule_file_size)), false
       else
         begin
 	  let md4 = if String.length md4 > 32 then
@@ -599,8 +603,9 @@ let parse_donkey_url url user =
     	end
   | "ed2k://" :: "file" :: name :: size :: md4 :: _
   | "file" :: name :: size :: md4 :: _ ->
-      if Int64.of_string size >= 4294967295L then
-	(Printf.sprintf (_b "Files > 4GB are not allowed")), false
+      if Int64.of_string size >= max_emule_file_size then
+	(Printf.sprintf (_b "Files > %s are not allowed")
+	  (Int64ops.int64_to_human_readable max_emule_file_size)), false
       else
         let md4 = if String.length md4 > 32 then
           String.sub md4 0 32 else md4 in
@@ -797,9 +802,10 @@ let commands = [
             ) !current_files
         ) args;
         ""
-    ) , "<f1> < f2> ... :\t\ttry to recover these files at byte level";
+    ) , "<f1> <f2> ... :\t\ttry to recover these files at byte level";
 
     "preferred", Arg_two (fun arg1 arg2 o ->
+        if CommonUserDb.user2_is_admin o.conn_user.ui_user then
         let preferred = bool_of_string arg1 in
         let ip = Ip.of_string arg2 in
         Hashtbl.iter (fun ip_s s ->
@@ -809,19 +815,28 @@ let commands = [
             end
         ) servers_by_key;
         "ok"
-    ), "<true/false> <ip> :\t\tset the server with this IP as preferred";
+        else
+          _s "You are not allowed to change preferred status"
+    ), "<true|false> <ip> :\t\tset the server with this IP as preferred";
 
     "bs", Arg_multiple (fun args o ->
+        if CommonUserDb.user2_is_admin o.conn_user.ui_user then begin
         List.iter (fun arg ->
             let range = Ip.range_of_string arg in
             server_black_list =:=  range :: !!server_black_list;
         ) args;
         "done"
-    ), "<range1> <range2> ... :\t\t\tadd these IPs to the servers black list (can be single IPs, CIDR ranges or begin-end ranges)";
+        end else
+          _s "You are not allowed to blacklist servers"
+    ), "<range1> <range2> ... :\t\tadd these IPs to the servers black list (can be single IPs, CIDR ranges or begin-end ranges)";
 
     "port", Arg_one (fun arg o ->
+        if CommonUserDb.user2_is_admin o.conn_user.ui_user then begin
         donkey_port =:= int_of_string arg;
-        "new port will change at next restart"),
+        "new port will change at next restart"
+        end else
+          _s "You are not allowed to change connection port"
+    ),
     "<port> :\t\t\t\tchange connection port";
 
     "scan_temp", Arg_none (fun o ->
@@ -925,7 +940,7 @@ parent.fstatus.location.href='submit?q=rename+'+i+'+\\\"'+encodeURIComponent(for
         if use_html_mods o then Printf.bprintf buf "\\</table\\>\\</div\\>";
         "" end
 	else begin
-	  print_command_result o o.conn_buf "You are not allowed to use scan_temp";
+	  print_command_result o "You are not allowed to use scan_temp";
 	"" end
 
     ), ":\t\t\t\tprint temp directory content";
@@ -935,7 +950,7 @@ parent.fstatus.location.href='submit?q=rename+'+i+'+\\\"'+encodeURIComponent(for
           DonkeySources.print o.conn_buf o.conn_output;
           "" end
 	else begin
-	  print_command_result o o.conn_buf "You are not allowed to list sources";
+	  print_command_result o "You are not allowed to list sources";
 	  "" end
     ), ":\t\t\t\tshow sources currently known";
 
@@ -991,11 +1006,14 @@ parent.fstatus.location.href='submit?q=rename+'+i+'+\\\"'+encodeURIComponent(for
     ), ":\t\t\t\treset client_md4/client_private_key to random values";
 
     "bp", Arg_multiple (fun args o ->
+        if CommonUserDb.user2_is_admin o.conn_user.ui_user then begin
         List.iter (fun arg ->
             let port = int_of_string arg in
             port_black_list =:=  port :: !!port_black_list;
         ) args;
         "done"
+        end else
+          _s "You are not allowed to blacklist ports"
     ), "<port1> <port2> ... :\t\tadd these ports to the port black list";
   ]
 
@@ -1195,14 +1213,16 @@ let _ =
         P.client_os = c.client_osinfo;
         P.client_release = c.client_emule_proto.emule_release;
         P.client_emulemod = brand_mod_to_string_short c.client_brand_mod;
-        P.client_downloaded = c.client_downloaded;
-        P.client_uploaded = c.client_uploaded;
-(*        P.client_source.source_sock_addr =    (); *)
+        P.client_total_downloaded = c.client_total_downloaded;
+        P.client_total_uploaded = c.client_total_uploaded;
+        P.client_session_downloaded = c.client_session_downloaded;
+        P.client_session_uploaded = c.client_session_uploaded;
         P.client_upload =
         (match client_upload (as_client c) with
             Some f -> Some (CommonFile.file_best_name f)
           | None -> None);
         P.client_sui_verified = c.client_sui_verified;
+        P.client_file_queue = List.map (fun (file,_,_) -> as_file file) c.client_file_queue
       }
   );
   client_ops.op_client_debug <- (fun c debug ->
@@ -1396,8 +1416,10 @@ parent.fstatus.location.href='submit?q=rename+%d+\\\"'+encodeURIComponent(formID
           ( "0", "srh", "Secure User Identification [N]one, [P]assed, [F]ailed", "S" ) ;
           ( "0", "srh br", "IP address", "IP address" ) ;
           ] @ (if !Geoip.active then [( "0", "srh br", "Country Code/Name", "CC" )] else []) @ [
-          ( "1", "srh ar", "Total UL bytes to this client for all files", "UL" ) ;
-          ( "1", "srh ar br", "Total DL bytes from this client for all files", "DL" ) ;
+          ( "1", "srh ar", "Total UL bytes to this client for all files", "tUL" ) ;
+          ( "1", "srh ar br", "Total DL bytes from this client for all files", "tDL" ) ;
+          ( "1", "srh ar", "Session UL bytes to this client for all files", "sUL" ) ;
+          ( "1", "srh ar br", "Session DL bytes from this client for all files", "sDL" ) ;
           ( "1", "srh ar", "Your queue rank on this client", "Rnk" ) ;
           ( "1", "srh ar br", "Source score", "Scr" ) ;
           ( "1", "srh ar br", "Last ok", "LO" ) ;
@@ -1475,9 +1497,11 @@ parent.fstatus.location.href='submit?q=rename+%d+\\\"'+encodeURIComponent(formID
                   | Some b -> if b then "P" else "F"
                 ));
             ("", "sr br", ip_string);
-            ] @ (if !Geoip.active then [(cn, "sr br", cc)] else []) @ [
-            ("", "sr ar", (size_of_int64 c.client_uploaded));
-            ("", "sr ar br", (size_of_int64 c.client_downloaded));
+            ] @ (if !Geoip.active then [(cn, "sr br", CommonPictures.flag_html cc)] else []) @ [
+            ("", "sr ar", (size_of_int64 c.client_total_uploaded));
+            ("", "sr ar br", (size_of_int64 c.client_total_downloaded));
+            ("", "sr ar", (size_of_int64 c.client_session_uploaded));
+            ("", "sr ar br", (size_of_int64 c.client_session_downloaded));
             ("", "sr ar", Printf.sprintf "%d" c.client_rank);
             ("", "sr ar br", Printf.sprintf "%d" c.client_source.DonkeySources.source_score);
             ("", "sr ar br", (string_of_date (c.client_source.DonkeySources.source_age)));
@@ -1679,6 +1703,68 @@ let _ =
         c.client_name
         (string_of_date (c.client_source.DonkeySources.source_age))
   );
+  client_ops.op_client_print_info <- (fun c o ->
+      let buf = o.conn_buf in
+      let ip_string,cc,cn = get_ips_cc_cn c in
+
+      Printf.bprintf buf "Client %d: %s\n"
+        (client_num c)
+        (full_client_identifier c);
+      (
+      match c.client_osinfo with
+      | Some i -> Printf.bprintf buf "  osinfo: %s\n" i
+      | None -> ()
+      );
+      Printf.bprintf buf "  state: %s, rank: %d\n"
+        (string_of_connection_state (client_state c)) c.client_rank;
+      if !Geoip.active then Printf.bprintf buf "  country: %s: %s\n" cc cn;
+      Printf.bprintf buf "  MD4: %s\n" (Md4.to_string c.client_md4);
+      Printf.bprintf buf "  downloaded\n";
+      Printf.bprintf buf "   - session %s\n" (size_of_int64 c.client_session_downloaded);
+      Printf.bprintf buf "   - total   %s\n" (size_of_int64 c.client_total_downloaded);
+      (
+      match c.client_download with
+      | Some (f,_) -> Printf.bprintf buf "  downloading file %s\n" (file_best_name f)
+      | None -> Printf.bprintf buf "  not downloading\n"
+      );
+      Printf.bprintf buf "  uploaded\n";
+      Printf.bprintf buf "   - session %s\n" (size_of_int64 c.client_session_uploaded);
+      Printf.bprintf buf "   - total   %s\n" (size_of_int64 c.client_total_uploaded);
+      (
+      match c.client_upload with
+      | Some u -> Printf.bprintf buf "  uploading file %s\n" (file_best_name u.up_file)
+      | _ -> Printf.bprintf buf "  not uploading\n"
+      );
+      Printf.bprintf buf "  SUI %s\n" (
+        match c.client_sui_verified with
+        | None -> "not supported"
+        | Some b -> if b then "passed" else "failed"
+        );
+      Printf.bprintf buf "  kind: %s\n" (
+        match c.client_kind with
+        | Direct_address (ip,port) ->
+                    Printf.sprintf "highID %s:%d" (Ip.to_string ip) port
+        | Indirect_address (server_ip, server_port, id, port, real_ip) ->
+	            Printf.sprintf "lowID %s:%d, server %s:%d"
+                      (Ip.to_string real_ip) port (Ip.to_string server_ip) server_port
+        | Invalid_address (name,md4) -> Printf.sprintf "invalid"
+        );
+      if c.client_emule_proto.received_miscoptions1 then
+        Printf.bprintf buf "\nmiscoptions1:\n%s" (DonkeyProtoClient.print_emule_proto_miscoptions1 c.client_emule_proto)
+      else
+        Printf.bprintf buf "no miscoptions1 received\n";
+      if c.client_emule_proto.received_miscoptions2 then
+        Printf.bprintf buf "miscoptions2:\n%s" (DonkeyProtoClient.print_emule_proto_miscoptions2 c.client_emule_proto)
+      else
+        Printf.bprintf buf "no miscoptions2 received\n";
+      List.iter (fun (file,_,_) -> Printf.bprintf buf "\nQueue: %s" (file_best_name file)) c.client_file_queue;
+      List.iter (fun r ->
+        Printf.bprintf buf "\nSource file: %s, score %d, request time %d"
+          (CommonFile.file_best_name (r.DonkeySources.request_file.DonkeySources.manager_file ()))
+          r.DonkeySources.request_score
+          r.DonkeySources.request_time;
+       ) c.client_source.DonkeySources.source_files;
+  );
   client_ops.op_client_dprint <- (fun c o file ->
       let info = file_info file in
       let buf = o.conn_buf in
@@ -1698,10 +1784,10 @@ let _ =
                   Direct_address (ip,port) -> (Ip.to_string ip)
                   |  _ -> (string_of_client_addr c));
                     Printf.bprintf buf "\n%14sDown  : %-10s                  Uploaded: %-10s  Ratio: %s%1.1f (%s)\n" ""
-                    (Int64.to_string c.client_downloaded)
-                    (Int64.to_string c.client_uploaded)
-                    (if c.client_downloaded > c.client_uploaded then "-" else "+")
-                    (if c.client_uploaded > Int64.zero then (Int64.to_float (Int64.div c.client_downloaded c.client_uploaded)) else (1.))
+                    (Int64.to_string c.client_total_downloaded)
+                    (Int64.to_string c.client_total_uploaded)
+                    (if c.client_total_downloaded > c.client_total_uploaded then "-" else "+")
+                    (if c.client_total_uploaded > Int64.zero then (Int64.to_float (Int64.div c.client_total_downloaded c.client_total_uploaded)) else (1.))
                     (brand_to_string c.client_brand);
                     (Printf.bprintf buf "%14sFile  : %s\n" "" info.GuiTypes.file_name);
                   end;
@@ -1753,9 +1839,11 @@ let _ =
                         | Some b -> if b then "P" else "F"
                       )); 
                       ("", "sr", ip_string);
-                      ] @ (if !Geoip.active then [(cn, "sr", cc)] else []) @ [
-                      ("", "sr ar", (size_of_int64 c.client_uploaded));
-                      ("", "sr ar", (size_of_int64 c.client_downloaded));
+                      ] @ (if !Geoip.active then [(cn, "sr", CommonPictures.flag_html cc)] else []) @ [
+                      ("", "sr ar", (size_of_int64 c.client_total_uploaded));
+                      ("", "sr ar", (size_of_int64 c.client_total_downloaded));
+                      ("", "sr ar", (size_of_int64 c.client_session_uploaded));
+                      ("", "sr ar", (size_of_int64 c.client_session_downloaded));
                       ("", "sr", info.GuiTypes.file_name) ]);
 
                     Printf.bprintf buf "\\</tr\\>";
@@ -1774,6 +1862,29 @@ let _ =
   )
 
 let _ =
+  shared_ops.op_shared_state <- (fun f o ->
+    match CommonFile.file_state f with
+    | FileShared ->
+      (match file_shared f with
+      | None -> "no file_shared info"
+      | Some f ->
+            let pre_share1_dir =
+              String2.replace (Filename2.dirname (as_shared_impl f).impl_shared_fullname) '\\' "/" in
+            let pre_share2_dir =
+              try       
+                String2.after pre_share1_dir
+                (String2.search_from
+                (Filename2.dirname (as_shared_impl f).impl_shared_fullname) 0 (Sys.getcwd ()) +
+                String.length (Sys.getcwd ()))
+              with Not_found -> pre_share1_dir
+            in
+            let dir =
+              if String2.check_prefix pre_share2_dir "/" then String2.after pre_share2_dir 1 else pre_share2_dir in
+            if o.conn_output = HTML then
+              Printf.sprintf "\\<a href=\\\"submit?q=debug_dir+%s\\\"\\>%s\\</a\\>" (Http_server.html_real_escaped dir) (Http_server.html_real_escaped dir)
+            else Printf.sprintf "Shared in %s" dir)
+    | state -> string_of_state state
+  );
   shared_ops.op_shared_unshare <- (fun file ->
       unshare_file file;
 (* Should we or not ??? *)
@@ -1819,12 +1930,8 @@ let _ =
       if not !!enable_donkey then
         lprintf_nl "eDonkey module is disabled, ignoring..."
       else
-        lprintf_nl "ED2K-update_server_list_server_met is disabled, ignoring..."
-  );
-  CommonWeb.add_web_kind "comments.met" "List of edonkey files comments" 
-    (fun _ filename ->
-(* TODO      DonkeyIndexer.load_comments filename; *)
-      lprintf_nl "COMMENTS ADDED";
+        lprintf_nl "ED2K-update_server_list_server_met is disabled, ignoring...";
+    CommonWeb.remove_job url
   );
 
   file_ops.op_file_proposed_filenames <- op_file_proposed_filenames;
