@@ -59,17 +59,15 @@ let sort_file_queue c =
   match c.client_download with
     Some _ -> ()
   | None ->
-      match c.client_file_queue with
+      (match c.client_file_queue with
         [] -> ()
       | [ (file, chunks, up) ] ->
-          if !verbose_download || c.client_debug then begin
+          if !verbose_download || c.client_debug then
               lprintf_nl "sort_file_queue: single file. client(%d): %s, file(%d): %s" (client_num c) c.client_name (file_num file) (file_best_name file);
-            end
       | (file, chunks, up) :: _ ->
           let fn = file_num file in
-          if !verbose_download || c.client_debug then begin
+          if !verbose_download || c.client_debug then
               lprintf_nl "sort_file_queue: multiple files. client(%d): %s, file(%d): %s" (client_num c) c.client_name (file_num file) (file_best_name file);
-            end;
           c.client_file_queue <- List.stable_sort (fun (f1, _, _) (f2, _, _) ->
               let v = file_priority f2 - file_priority f1 in
               if v <> 0 then v else
@@ -81,30 +79,37 @@ let sort_file_queue c =
                 else 0 in
               s2 - s1
           ) c.client_file_queue;
-          match c.client_file_queue with
+          (match c.client_file_queue with
             [] -> ()
           | (file, chunks, _) :: _ ->
-              if (file_num file) <> fn then begin
-                  if !verbose_download || c.client_debug then begin
-                      lprintf_nl "sort_file_queue: queue change. client(%d): %s, file(%d): %s" (client_num c) c.client_name (file_num file) (file_best_name file);
-                    end;
-(*
-(*                  c.client_chunks <- chunks; *)
-(*                  c.client_all_chunks <- String.make file.file_nchunks '0'; *)
-(*                  c.client_zones <- []; *)
-                  for i = 0 to file.file_nchunks - 1 do
-                    if c.client_chunks.(i)  then
-                      c.client_all_chunks.[i] <- '1';
-done;
-  *)
-                end
+              if (file_num file) <> fn && (!verbose_download || c.client_debug) then
+                lprintf_nl "sort_file_queue: queue change. client(%d): %s, file(%d): %s" (client_num c) c.client_name (file_num file) (file_best_name file)))
 
 let remove_client_slot c =
+  if c.client_debug || (
+     !verbose &&
+     (c.client_session_uploaded > 0L || c.client_session_downloaded > 0L)) then
+    lprintf_nl "Client[%d] %s disconnected, connected %s%s%s"
+      (client_num c) (full_client_identifier c)
+      (Date.time_to_string (last_time () - c.client_connect_time) "verbose")
+      (if c.client_total_uploaded > 0L then
+        Printf.sprintf ", send %s (%s)%s"
+          (size_of_int64 c.client_session_uploaded)
+          (size_of_int64 c.client_total_uploaded)
+          (match client_upload (as_client c) with | None -> ""
+           | Some f -> " of " ^ (CommonFile.file_best_name f)) else "")
+      (if c.client_total_downloaded > 0L then
+         Printf.sprintf ", rec %s (%s)%s"
+          (size_of_int64 c.client_session_downloaded)
+          (size_of_int64 c.client_total_downloaded)
+          (match c.client_download with | None -> ""
+           | Some (f,_) -> " of " ^ (file_best_name f)) else "");
   set_client_has_a_slot (as_client c) NoSlot;
   client_send c (
     let module M = DonkeyProtoClient in
-    let module Q = M.CloseSlot in
-    M.CloseSlotReq Q.t);
+    let module Q = M.OutOfParts in
+    M.OutOfPartsReq Q.t);
+  c.client_session_uploaded <- 0L;
   c.client_upload <- None
 
 let unshare_file file =
@@ -218,16 +223,17 @@ let add_client_chunks c file client_chunks =
 
 (* let next_file _ = failwith "next_file not implemented" *)
       
-(* clean_client_zones: clean all structures related to downloads when
-   a client disconnects *)
-let clean_current_download c = 
-  match c.client_download with
-    None -> ()
-  | Some (file, up) ->
-      CommonSwarming.unregister_uploader up;
-      c.client_download <- None
-
 let send_get_range_request c file ranges = 
+  let rec check_large (rangelist : (int64 * int64 * range) list) =
+    match rangelist with
+    | [] -> false
+    | (x,y,_ : (int64 * int64 * range))::tail_range ->
+	    (x > old_max_emule_file_size) || (y > old_max_emule_file_size) || (check_large tail_range)
+  in
+  let is_large_request = check_large ranges in
+  if file_is_largefile file && c.client_emule_proto.emule_largefiles <> 1 then
+    lprintf_nl "File %s is too large for %s." (file_best_name file) (full_client_identifier c)
+  else
   match c.client_source.DonkeySources.source_sock with
   | Connection sock ->
       
@@ -239,6 +245,7 @@ let send_get_range_request c file ranges =
           [x1,y1,_] ->
             {
               Q.md4 = file.file_md4;
+              Q.usesixtyfour = is_large_request;
               Q.start_pos1 = x1;
               Q.end_pos1 = y1;
               Q.start_pos2 = zero;
@@ -250,6 +257,7 @@ let send_get_range_request c file ranges =
         | [x1,y1,_; x2,y2,_] ->
             {
               Q.md4 = file.file_md4;
+              Q.usesixtyfour = is_large_request;
               Q.start_pos1 = x1;
               Q.end_pos1 = y1;
               Q.start_pos2 = x2;
@@ -261,6 +269,7 @@ let send_get_range_request c file ranges =
         | [x1,y1,_; x2,y2,_; x3,y3,_ ] ->
             {
               Q.md4 = file.file_md4;
+              Q.usesixtyfour = is_large_request;
               Q.start_pos1 = x1;
               Q.end_pos1 = y1;
               Q.start_pos2 = x2;
@@ -377,9 +386,7 @@ let rec get_from_client c =
   or start querying blocks if already in the queue *)
 let request_slot c = 
     if c.client_slot = SlotNotAsked then begin
-      if !verbose_download then begin
-          lprintf_nl "start_download";
-        end;
+      if !verbose_download then lprintf_nl "start_download";
       do_if_connected c.client_source.DonkeySources.source_sock (fun sock ->
           sort_file_queue c;
           match c.client_file_queue with
@@ -472,8 +479,9 @@ let search_found filter search md4 tags =
       match tag with
         { tag_name = Field_Filename; tag_value = String s } -> file_name := s
       | { tag_name = Field_Size; tag_value = Uint64 v } -> file_size := v
-      | { tag_name = Field_Availability;
-    tag_value = (Uint64 v| Fint64 v) } ->
+      | { tag_name = Field_Size_Hi; tag_value = Uint8 v } ->
+      	  file_size := Int64.logor !file_size (Int64.shift_left (Int64.of_int v) 32)
+      | { tag_name = Field_Availability; tag_value = (Uint64 v| Fint64 v) } ->
           availability := Int64.to_int v;  new_tags := tag :: !new_tags
       | _ -> new_tags := tag :: !new_tags
   ) tags;
