@@ -30,6 +30,7 @@ let max_xs_packets = define_expert_option donkey_section ["max_xs_packets"]
     int_option 30
 
 let donkey_port = define_option donkey_section ["port"]
+  ~restart: true
   "The port used for connection by other donkey clients."
     int_option (2000 + Random.int 20000)
 
@@ -174,6 +175,16 @@ let upload_lifetime = define_expert_option donkey_section ["upload_lifetime"]
   "How long a downloading client can stay in my upload queue (in minutes >5)"
     int_option 90
 
+let upload_full_chunks = define_expert_option donkey_section ["upload_full_chunks"]
+  "If true, each client is allowed to receive one chunk, this setting overrides upload_lifetime"
+    bool_option false
+
+let upload_complete_chunks = define_expert_option donkey_section ["upload_complete_chunks"]
+  "If true, each client is allowed to complete only one chunk, independent, if it is empty or
+   partial. this setting overrides upload_full_chunks and dynamic_upload_lifetime,
+   but is, as a failsafe, limited by upload_lifetime (should be set reasonable high)"
+    bool_option false
+
 let dynamic_upload_lifetime = define_expert_option donkey_section ["dynamic_upload_lifetime"]
   "Each client upload lifetime depends on download-upload ratio"
     bool_option false
@@ -182,6 +193,56 @@ let dynamic_upload_threshold = define_expert_option donkey_section ["dynamic_upl
   "Uploaded zones (1 zone = 180 kBytes) needed to enable the dynamic upload lifetime"
     int_option 10
 
+let upload_compression = define_expert_option donkey_section ["upload_compression"]
+  "Enables compressed upload as part of the protocol"
+    bool_option true
+
+let upload_compression_threshold = define_expert_option donkey_section ["upload_compression_threshold"]
+  "Sizedifference in bytes between one zone (180 kBytes) and its compressed
+  counterpart, which has to occure, to send compressed parts instead of plain."
+    int_option 2000
+
+let upload_compression_ext_exclude = define_expert_option donkey_section ["upload_compression_ext_exclude"]
+  "Disable upload compression based on file extensions (without dot)"
+    string_list_option ["zip"; "7z"; "gz"; "bz2"; "rar"; "ace"; "ogm"; "avi"; "mpg"]
+
+let _ =
+  option_hook upload_compression_threshold (fun _ ->
+    if !!upload_compression_threshold < 0 then
+        upload_compression_threshold =:= 0
+  );
+  option_hook upload_compression_ext_exclude (fun _ ->
+    let l = List.map String.lowercase !!upload_compression_ext_exclude in
+    if !!upload_compression_ext_exclude <> l then upload_compression_ext_exclude =:= l
+  )
+
+let upload_compression_level = define_expert_option donkey_section ["upload_compression_level"]
+  "Level of the used zlibcompression. allowed are values between 0 and 9. higher
+  level means better compression, but higher cpu usage too. (emules default
+  compression level for compressed parts is 9)"
+    int_option 9
+
+let _ =
+  option_hook upload_compression_level (fun _ ->
+    if !!upload_compression_level < 0
+      || !!upload_compression_level > 9 then
+        upload_compression_level =:= 9
+  )
+
+let upload_compression_table_size = define_expert_option donkey_section ["upload_compression_table_size"]
+  ~restart: true
+  "Size of the cache table in entries (ca. 2 * 180 kbytes). zones have to be
+  compressed at once, but only parts of it are sent at a time (10 kbytes).
+  to reduce diskaccess and repeated compression to a minimum, size should be
+  at least the number of total upload slots. restart of core is required."
+    int_option 20
+
+let _ =
+  option_hook upload_compression_table_size (fun _ ->
+    if !!upload_compression_table_size < 1 then
+        upload_compression_table_size =:= 1
+  )
+
 let connected_server_timeout = define_expert_option donkey_section ["connected_server_timeout"]
   "How long can a silent server stay connected"
     float_option 1800.
@@ -200,6 +261,7 @@ let max_server_age = define_expert_option donkey_section ["max_server_age"]
     int_option 2
 
 let remove_old_servers_delay = define_expert_option donkey_section ["remove_old_servers_delay"]
+  ~restart: true
   "How often should remove old donkey servers (see max_server_age) be called
   (in seconds, 0 to disable)"
     float_option 900.
@@ -267,10 +329,12 @@ let overnet_section = file_section donkey_ini [ overnet_options_section_name ]
     "Overnet options"
 
 let overnet_port = define_option overnet_section [overnet_options_section_name; "port"]
+  ~restart: true
   "port for overnet"
     int_option (2000 + Random.int 20000)
 
 let options_version = define_expert_option donkey_section ["options_version"]
+  ~internal: true
   "(internal option)"
     int_option 3
 
