@@ -98,23 +98,23 @@ let cut_messages parse b =
 let update_emule_proto_from_tags e tags = 
   List.iter (fun tag -> 
       match tag.tag_name with
-      | Field_UNKNOWN "compression" ->
+      | Field_KNOWN "compression" ->
           for_int_tag tag (fun i -> 
               e.emule_compression <- i)
-      | Field_UNKNOWN "udpver" ->
+      | Field_KNOWN "udpver" ->
           for_int_tag tag (fun i -> 
               e.emule_udpver <- i)          
-      | Field_UNKNOWN "udpport" -> ()
-      | Field_UNKNOWN "sourceexchange" ->
+      | Field_KNOWN "udpport" -> ()
+      | Field_KNOWN "sourceexchange" ->
           for_int_tag tag (fun i -> 
               e.emule_sourceexchange <- i)          
-      | Field_UNKNOWN "comments" ->
+      | Field_KNOWN "comments" ->
           for_int_tag tag (fun i -> 
               e.emule_comments <- i)          
-      | Field_UNKNOWN "extendedrequest" ->
+      | Field_KNOWN "extendedrequest" ->
           for_int_tag tag (fun i -> 
               e.emule_extendedrequest <- i)          
-      | Field_UNKNOWN "features" ->
+      | Field_KNOWN "features" ->
           for_int_tag tag (fun i -> 
               e.emule_secident <- i land 0x3)          
       | s -> 
@@ -138,12 +138,23 @@ let client_parse c opcode s =
             
             begin
               try
-                let options = find_tag (Field_UNKNOWN "emule_miscoptions1") tags in
+                let options = find_tag (Field_KNOWN "emule_miscoptions1") tags in
+                (
                 match options with
-                  Uint64 v | Fint64 v ->
+                | Uint64 v | Fint64 v ->
                     update_emule_proto_from_miscoptions1 emule v
                 | _ -> 
                     lprintf "CANNOT INTERPRETE EMULE OPTIONS\n"
+                );
+
+                let options2 = find_tag (Field_KNOWN "emule_miscoptions2") tags in
+                (
+                match options2 with
+                | Uint64 v | Fint64 v ->
+                    update_emule_proto_from_miscoptions2 emule v
+                | _ -> 
+                    lprintf "CANNOT INTERPRETE EMULE OPTIONS2\n"
+                );
               
               with _ -> ()
             end;
@@ -151,14 +162,14 @@ let client_parse c opcode s =
         | P.UnknownReq (227,_) -> 
             emule.emule_extendedrequest <-  -1
         
-        | P.EmuleCompressedPart (md4, statpos, newsize, bloc) ->
+        | P.EmuleCompressedPart t ->
             
             let comp = match c.client_comp with
                 None ->
                   let comp = {
-                      comp_md4 = md4;
-                      comp_pos = statpos;
-                      comp_total = Int64.to_int newsize;
+                      comp_md4 = t.EmuleCompressedPart.md4;
+                      comp_pos = t.EmuleCompressedPart.statpos;
+                      comp_total = Int64.to_int t.EmuleCompressedPart.newsize;
                       comp_len = 0;
                       comp_blocs = [];
                     } in
@@ -166,8 +177,8 @@ let client_parse c opcode s =
                   comp
               | Some comp -> comp
             in
-            comp.comp_blocs <- bloc :: comp.comp_blocs;
-            comp.comp_len <- comp.comp_len + String.length bloc;
+            comp.comp_blocs <- t.EmuleCompressedPart.bloc :: comp.comp_blocs;
+            comp.comp_len <- comp.comp_len + String.length t.EmuleCompressedPart.bloc;
 
 (*            lprintf "Comp bloc: %d/%d\n" comp.comp_len comp.comp_total; *)
             if comp.comp_len = comp.comp_total then begin
@@ -231,17 +242,13 @@ let client_parse c opcode s =
     
     
 let commit () =  
-
-  let oc = open_out "trace.out" in
-  output_value oc connections;
-  close_out oc
+  Unix2.tryopen_write "trace.out" (fun oc -> output_value oc connections)
 
 exception ServerConnection
   
 let read_trace () =
-  let ic = open_in "trace.out" in
-  let connections = input_value ic in
-  close_in ic;
+  let connections = 
+    Unix2.tryopen_read "trace.out" (fun ic -> input_value ic) in
 
   mldonkey_emule_proto.emule_sourceexchange <- 5;
   
