@@ -32,14 +32,20 @@ open DonkeyOvernet
 
 module Proto = struct
 
-     let lprintf_nl () =
-      lprintf "%s[Overnet] "
-      (log_time ()) ; lprintf_nl2
+    let log_prefix = "[OV]"
 
-    let lprintf_n () =
-      lprintf "[Overnet] " ; lprintf
+    let lprintf_nl fmt =
+      lprintf_nl2 log_prefix fmt
 
-   let names_of_tag = file_common_tags
+    let lprintf_n fmt =
+      lprintf2 log_prefix fmt
+
+    let lprintf fmt =
+      lprintf2 log_prefix fmt
+
+   let names_of_tag = [
+      "loc", Field_KNOWN "loc";
+      ] @ file_common_tags
 
     let buf_peer buf p =
       buf_md4 buf p.peer_md4;
@@ -186,19 +192,18 @@ module Proto = struct
       let peer_tcpport = ref 0 in
       List.iter (fun tag ->
           match tag.tag_name with
-            Field_UNKNOWN "loc" ->
+            Field_KNOWN "loc" ->
               for_string_tag tag (fun bcp ->
-                  if !verbose_overnet then
-                  lprintf_nl () "loc tag : [%s]" bcp;
                   if String2.starts_with bcp "bcp://" then
                     let bcp2 = String.sub bcp 6 (String.length bcp - 6)
                     in
                     match String2.split_simplify bcp2 ':' with
-                    | [_;ip;port] ->
-                        if !verbose_overnet then
-                          lprintf_nl () "Received BCP type 2 %s"
-                            bcp;
+                    | [_;ip;udpport;tcpport] ->
+                        peer_ip := Ip.of_string ip;
+                        peer_udpport := int_of_string udpport;
+                        peer_tcpport := int_of_string tcpport;
 
+                    | [_;ip;port] -> ()
 (* FIXME: A firewalled peer...
                                 peer_ip := Ip.of_string ip;
                                 peer_tcpport := int_of_string port;
@@ -209,14 +214,14 @@ module Proto = struct
                         peer_tcpport := int_of_string port;
                     | _ ->
         		if !verbose_overnet then
-                          lprintf_nl () "Ill formed bcp: [%s]" bcp;
+                          lprintf_nl "Ill formed bcp: [%s]" bcp;
                   else
 		    if !verbose_overnet then
-                      lprintf_nl () "Ill formed bcp: [%s]" bcp;
+                      lprintf_nl "Ill formed bcp: [%s]" bcp;
               )
           | _ ->
             if !verbose_unknown_messages then
-              lprintf_nl () "Unused source tag [%s]"
+              lprintf_nl "Unused source tag [%s]"
                 (escaped_string_of_field tag)
       ) r_tags;
       {
@@ -288,7 +293,7 @@ module Proto = struct
             let r_tags, pos = get_tags s 32 names_of_tag in
             let sources = ref false in
             List.iter (fun tag ->
-                if tag.tag_name = Field_UNKNOWN "loc" then sources := true;
+                if tag.tag_name = Field_KNOWN "loc" then sources := true;
             ) r_tags;
             if !sources then
               let peer = get_peer_from_result ip port r_md4 r_tags in
@@ -304,7 +309,7 @@ module Proto = struct
             let r_tags, pos = get_tags s 32 names_of_tag in
             let sources = ref false in
             List.iter (fun tag ->
-                if tag.tag_name = Field_UNKNOWN "loc" then sources := true;
+                if tag.tag_name = Field_KNOWN "loc" then sources := true;
             ) r_tags;
             if !sources then
               let peer = get_peer_from_result ip port r_md4 r_tags in
@@ -317,7 +322,7 @@ module Proto = struct
         | 21 ->
 (* idem as 33, but IP seem to be a low ID *)
             if !verbose_overnet then begin
-                lprintf_nl () "Received code %d message. Dump:" opcode;
+                lprintf_nl "Received code %d message. Dump:" opcode;
                 dump s;
                 lprint_newline ();
               end;
@@ -325,42 +330,42 @@ module Proto = struct
             OvernetUnknown21 peer
         | 24 ->
             if !verbose_overnet then
-              lprintf_nl () "RCVD: OVERNET FIREWALL CONNECTION (24)";
+              lprintf_nl "RCVD: OVERNET FIREWALL CONNECTION (24)";
             let md4 = get_md4 s 0 in
             let port = get_int16 s 16 in
             OvernetFirewallConnection(md4,port)
         | 25 ->
             if !verbose_overnet then
-              lprintf_nl () "RCVD: OVERNET FIREWALL CONNECTION ACK (25)";
+              lprintf_nl "RCVD: OVERNET FIREWALL CONNECTION ACK (25)";
             let md4 = get_md4 s 0 in
             OvernetFirewallConnectionACK(md4)
         | 26 ->
             if !verbose_overnet then
-              lprintf_nl () "RCVD: OVERNET FIREWALL CONNECTION NACK (26)";
+              lprintf_nl "RCVD: OVERNET FIREWALL CONNECTION NACK (26)";
             let md4 = get_md4 s 0 in
             OvernetFirewallConnectionNACK(md4)
         | 27 ->
             if !verbose_overnet then
-              lprintf_nl () "RCVD: GETMYIP MESSAGE (27)";
+              lprintf_nl "RCVD: GETMYIP MESSAGE (27)";
             OvernetGetMyIP (get_int16 s 0)
         | 28 ->
             if !verbose_overnet then
-              lprintf_nl () "RCVD: GETMYIPRESULT MESSAGE (28)";
+              lprintf_nl "RCVD: GETMYIPRESULT MESSAGE (28)";
             let ip = get_ip s 0 in
             OvernetGetMyIPResult (ip)
         | 29 ->
             if !verbose_overnet then
-              lprintf_nl () "RCVD: GETMYIPDONE MESSAGE (29)";
+              lprintf_nl "RCVD: GETMYIPDONE MESSAGE (29)";
             OvernetGetMyIPDone
         | 33 ->
             if !verbose_overnet then
-              lprintf_nl () "RCVD: PEER NOT FOUND (33)";
+              lprintf_nl "RCVD: PEER NOT FOUND (33)";
             let peer, _ = get_peer s 0 in
             OvernetPeerNotFound peer
         | _ ->
             if !verbose_unknown_messages then
               begin
-                lprintf_nl () "unknown opcode %d" opcode;
+                lprintf_nl "unknown opcode %d" opcode;
                 dump s;
                 lprint_newline ();
               end;
@@ -368,7 +373,7 @@ module Proto = struct
       with e ->
           if !verbose_unknown_messages then
             begin
-              lprintf_nl () "Error %s while parsing opcode %d" (Printexc2.to_string e) opcode;
+              lprintf_nl "Error %s while parsing opcode %d" (Printexc2.to_string e) opcode;
               dump s;
               lprint_newline ();
             end;
@@ -385,7 +390,7 @@ module Proto = struct
                   int_of_char pbuf.[0] <> 227 then
                   begin
                     if !verbose_unknown_messages then begin
-                        lprintf_nl () "Received unknown UDP packet";
+                        lprintf_nl "Received unknown UDP packet";
                         dump pbuf;
                       end
                   end
@@ -397,13 +402,20 @@ module Proto = struct
                           Ip.of_inet_addr inet, port
                       | _ -> assert false
                     in
-
                     let t = parse ip port (int_of_char pbuf.[1]) (String.sub pbuf 2 (len-2)) in
-                    f t p
+                    let is_not_banned ip =
+                      match !Ip.banned ip with
+                         None -> true
+                       | Some reason ->
+                      if !verbose_overnet then
+                        lprintf_nl "%s blocked: %s" (Ip.to_string ip) reason;
+                      false
+                    in
+                    if is_not_banned ip then f t p
                   end
               with e ->
                 if !verbose_unknown_messages then begin
-                  lprintf_nl () "Error %s in udp_handler, dump of packet:"
+                  lprintf_nl "Error %s in udp_handler, dump of packet:"
                     (Printexc2.to_string e);
                   dump p.UdpSocket.udp_content;
                   lprint_newline ()
@@ -429,12 +441,12 @@ module Proto = struct
         let s = Buffer.contents udp_buf in
         if !verbose_overnet then
           begin
-            lprintf_nl () "Sending UDP to %s:%d (opcode 0x%02X len %d) type %s"
+            lprintf_nl "UDP to %s:%d op 0x%02X len %d type %s"
               (Ip.to_string ip) port (get_uint8 s 1) (String.length s) (message_to_string msg);
           end;
         UdpSocket.write sock ping s ip port
       with e ->
-          lprintf_nl () "Exception %s in udp_send" (Printexc2.to_string e)
+          lprintf_nl "Exception %s in udp_send" (Printexc2.to_string e)
 
     let udp_send sock ip port ping msg =
       match msg with
@@ -464,6 +476,7 @@ module Proto = struct
     let command_prefix = "ov_"
     let source_brand = true
     let web_info = "ocl"
+    let web_info_descr = "Overnet network boot peers"
   end
 
 module Overnet_initial = Make(Proto)
@@ -492,6 +505,7 @@ module Overnet = struct
 
     let enable enabler =
       enable enabler;
+      if !!enable_overnet then begin
       try
         let sock = TcpServerSocket.create
             "overnet client server"
@@ -508,6 +522,7 @@ module Overnet = struct
       with e ->
           lprintf_nl "[Overnet] Could not assign TCP port %d for Overnet" !!overnet_port;
           tcp_sock := None
+      end
 
     let disable () =
       disable ();
