@@ -44,12 +44,7 @@ let must_share_file file codedname has_old_impl =
   | Some _ -> ()
   | None ->
       let full_name = file_disk_name file in
-      let magic =
-        match Magic.M.magic_fileinfo full_name false with
-          None -> None
-        | Some magic -> Some (intern magic)
-      in
-
+      check_magic (as_file file);
       let impl = {
           impl_shared_update = 1;
           impl_shared_fullname = full_name;
@@ -61,7 +56,7 @@ let must_share_file file codedname has_old_impl =
           impl_shared_ops = shared_ops;
           impl_shared_val = file;
           impl_shared_requests = 0;
-          impl_shared_magic = magic;
+          impl_shared_file = Some (as_file file);
           impl_shared_servers = []
         } in
       file.file_shared <- Some impl;
@@ -189,12 +184,17 @@ let send_new_shared () =
    build a list of files_to_send with yet unpublished files *)
       begin
 	let files_to_send = ref [] in
+        let can_publish f = not (file_is_largefile f && not s.server_has_largefiles) in
         List.iter (fun f ->
           match f.file_shared with
 	    Some impl ->
 	      if not (List.mem (CommonServer.as_server s.server_server) impl.impl_shared_servers)
-		&& List.length !files_to_send < !!max_published_files then
+		&& List.length !files_to_send < !!max_published_files && can_publish f then
 	      files_to_send := f :: !files_to_send
+	      else
+		if not (can_publish f) then
+		  lprintf_nl "Can not publish largefile %s because server %s does not support largefiles"
+		    (file_best_name f) (string_of_server s)
 	    | _ -> () (* this case never happens *)
 	) all_shared;
 
@@ -221,7 +221,7 @@ let send_new_shared () =
 
 	end
       end
-  ) (connected_servers ());
+  ) (logged_in_servers ());
 
 (*
 The problem: sh.shared_fd might be closed during the execution of the
@@ -333,12 +333,6 @@ lprintf "Searching %s" fullname; lprint_newline ();
     let found = ref false in
     List.iter (fun sh -> if sh.shared_name = fullname then found := true) !shared_files;
     if not !found then begin
-        let magic =
-          match Magic.M.magic_fileinfo fullname false with
-            None -> None
-          | Some magic -> Some (intern magic)
-        in
-
           let rec impl = {
               impl_shared_update = 1;
               impl_shared_fullname = fullname;
@@ -350,7 +344,7 @@ lprintf "Searching %s" fullname; lprint_newline ();
               impl_shared_id = Md4.null;
               impl_shared_val = pre_shared;
               impl_shared_requests = 0;
-              impl_shared_magic = magic;
+              impl_shared_file = None;
               impl_shared_servers = [];
             } and
             pre_shared = {
