@@ -32,126 +32,37 @@ let lprintf_nl fmt =
 let lprintf_n fmt =
   lprintf2 log_prefix fmt
 
-  (*
-
-module MyList = struct
-    type 'a mylist =
-      Nil
-    | Cons of int * 'a * 'a mylist
-
-    type ('a,'b) assoc =
-      {
-        v1 : 'a;
-        mutable v2 : 'b;
-      }
-
-    let nil = Nil
-    let length list =
-      match list with Nil -> 0 | Cons (len,_,_) -> len
-
-    let add v list = Cons (1+ length list, v, list)
-    let head list =
-      match list with
-        Nil -> raise Not_found
-      | Cons (_,v,_) -> v
-
-    let tail list =
-      match list with
-        Nil -> raise Not_found
-      | Cons (_,_,tail) -> tail
-
-    let rec mem v list =
-      match list with
-        Nil -> raise Not_found
-      | Cons (_,vv,tail) ->
-          vv = v || mem v tail
-
-    let rec memq v list =
-      match list with
-        Nil -> raise Not_found
-      | Cons (_,vv,tail) ->
-          vv == v || memq v tail
-
-    let assoc x y = { v1 = x; v2 = y; }
-    let assoc_get x = x.v2
-    let assoc_set x y = x.v2 <- y
-
-    let rec find_assoc v list =
-      match list with
-        Nil -> raise Not_found
-      | Cons (_,vv,tail) ->
-          if vv.v1 = v then vv else find_assoc v tail
-
-    let rec find_assq v list =
-      match list with
-        Nil -> raise Not_found
-      | Cons (_,vv,tail) ->
-          if vv.v1 == v then vv else find_assq v tail
-
-    let rec mem_assoc v list =
-      match list with
-        Nil -> raise Not_found
-      | Cons (_,vv,tail) ->
-          vv.v1 = v || mem_assoc v tail
-
-    let rec mem_assq v list =
-      match list with
-        Nil -> raise Not_found
-      | Cons (_,vv,tail) ->
-          vv.v1 == v || mem_assq v tail
-
-  end
-
-open MyList
-    *)
-
-  (*
-type client_tag_name =
-  CT_CLIENT_NAME
-| CT_CLIENT_PORT
-| CT_CLIENT_VERSION
-| CT_CLIENT_EXTENDED
-| CT_CLIENT_UDPPORT
-
-type server_tag_name =
-| ST_SERVER_NAME
-| ST_SERVER_DESCRIPTION
-| ST_SERVER_PORT
-| ST_SERVER_IP
-| ST_SERVER_PING
-| ST_SERVER_PROF
-| ST_SERVER_HISTORY
-
-type file_tag_name =
-| FT_FILE_FILENAME
-| FT_FILE_SIZE
-| FT_FILE_TYPE
-| FT_FILE_FORMAT
-| FT_FILE_AVAILABILITY
-| FT_FILE_TITLE
-| FT_FILE_ARTIST
-| FT_FILE_ALBUM
-| FT_FILE_LOC
-| FT_FILE_DOWNLOADED
-| FT_FILE_DISKNAME
-| FT_FILE_PRIORITY
-| FT_FILE_STATUS
-    *)
+exception Donkey_large_file
 
 type emule_proto = {
-    mutable emule_comments : int;
     mutable emule_version : int;
     mutable emule_release : string;
+    mutable emule_osinfosupport : int;
+    mutable emule_features : int;
+
+(* emule_miscoptions1 *)
+    mutable received_miscoptions1 : bool;
+    mutable emule_aich : int;
+    mutable emule_unicode : int;
+    mutable emule_udpver : int;
+    mutable emule_compression : int;
     mutable emule_secident : int;
+    mutable emule_sourceexchange : int;
+    mutable emule_extendedrequest : int;
+    mutable emule_comments : int;
+    mutable emule_peercache : int;
     mutable emule_noviewshared : int;
+    mutable emule_multipacket : int;
     mutable emule_supportpreview : int;
 
-    mutable emule_compression : int;
-    mutable emule_sourceexchange : int;
-    mutable emule_multipacket : int;
-    mutable emule_extendedrequest : int;
-    mutable emule_features : int;
-    mutable emule_udpver : int;
+(* emule_miscoptions2 *)
+    mutable received_miscoptions2 : bool;
+    mutable emule_require_crypt : int;
+    mutable emule_request_crypt : int;
+    mutable emule_support_crypt : int;
+    mutable emule_extmultipacket : int;
+    mutable emule_largefiles : int;
+    mutable emule_kad_version : int;
   }
 
 type emule_tag_name =
@@ -168,27 +79,12 @@ type emule_tag_name =
 | ET_MOD_TAROD
 | ET_TAROD_VERSION
 | ET_MOD_PLUS
-(*
-type pref_tag_name =
-| PT_PREF_NAME
-| PT_PREF_PORT
-| PT_PREF_VERSION
-| PT_PREF_TEMP
-    *)
+
 type request_record = {
   mutable last_request : int;
   mutable nwarnings : int;
 }
 
-type client_score =
-  Client_not_connected
-| Client_has_file
-| Client_has_priority_file
-| Client_has_chunk
-| Client_has_priority_chunk
-| Client_has_upload
-| Client_has_priority_upload
-
 type reliability =
   Reliability_neutral
 | Reliability_reliable
@@ -210,6 +106,8 @@ type brand =
 | Brand_lphant
 | Brand_emuleplus
 | Brand_hydranode
+| Brand_verycd
+| Brand_imp
 
 let brand_list = [
    ( Brand_unknown   , "unknown"      , "unk" ) ;
@@ -227,6 +125,8 @@ let brand_list = [
    ( Brand_lphant    , "lPhant"       , "lPH" ) ;
    ( Brand_emuleplus , "ePlus"        , "eM+" ) ;
    ( Brand_hydranode , "Hydra"        , "Hyd" ) ;
+   ( Brand_verycd    , "VeryCD"       , "VCD" ) ;
+   ( Brand_imp       , "IMPmule"      , "IMP" ) ;
   ]
 
 let brand_count = List.length brand_list
@@ -540,21 +440,34 @@ module DonkeySources = CommonSources.Make(struct
         | Invalid_address _ -> failwith "Invalid address"
     end)
 
-type server = (*[]*){
+type file = {
+    file_file : file CommonFile.file_impl;
+    file_md4 : Md4.t;
+    mutable file_swarmer : CommonSwarming.t option;
+    mutable file_nchunks : int;
+    mutable file_nchunk_hashes : int;
+    mutable file_diskname : string;
+    mutable file_computed_md4s : Md4.t array;
+    mutable file_format : format;
+    mutable file_shared : file CommonShared.shared_impl option;
+    mutable file_sources : DonkeySources.file_sources_manager;
+    mutable file_comments : (Ip.t * string * int * string) list;
+  }
+
+and server = (*[]*){
     mutable server_server : server CommonServer.server_impl;
     mutable server_ip : Ip.t;
     mutable server_cid : Ip.t option;
     mutable server_port : int;
     mutable server_realport : int option; (* in case we connect through auxport; this is the true one *)
     mutable server_sock : tcp_connection;
-    mutable server_nqueries : int;
     mutable server_search_queries : CommonTypes.search Fifo.t;
     mutable server_users_queries : bool Fifo.t;
     mutable server_connection_control : connection_control;
     mutable server_score : int;
     mutable server_tags : CommonTypes.tag list;
-    mutable server_nusers : int64;
-    mutable server_nfiles : int64;
+    mutable server_nusers : int64 option;
+    mutable server_nfiles : int64 option;
     mutable server_name : string;
     mutable server_description : string;
     mutable server_banner : string;
@@ -562,10 +475,14 @@ type server = (*[]*){
     mutable server_next_udp : int;
     mutable server_master : bool;
     mutable server_preferred : bool;
-    mutable server_last_message : int; (* used only by mldonkey server *)
-
-    mutable server_last_ping : float;
+    mutable server_last_ping : float; (* time last PingServerUdpReq was sent *)
+    mutable server_next_ping : float; (* time next PingServerUdpReq will be sent *)
+    mutable server_descping_counter : int;
     mutable server_ping : int;
+    mutable server_udp_ping_challenge : int64 option;
+    mutable server_udp_desc_challenge : int64 option;
+(* reset after answer received, increased after UDP ping sent, server deleted after 10 attempts *)
+    mutable server_failed_count : int; 
 
     mutable server_id_requests : file option Fifo.t;
 
@@ -573,13 +490,26 @@ type server = (*[]*){
     mutable server_waiting_queries : file list;
     mutable server_sent_all_queries : bool;
     mutable server_has_zlib : bool;
+    mutable server_has_newtags : bool;
+    mutable server_has_unicode : bool;
+    mutable server_has_related_search : bool;
+    mutable server_has_tag_integer : bool;
+    mutable server_has_largefiles : bool;
+    mutable server_obfuscation_tcp : int option;
+    mutable server_obfuscation_udp : int option;
+    mutable server_dynip : string;
+    mutable server_auxportslist : string;
+    mutable server_has_get_sources : bool;
+    mutable server_has_get_files : bool;
+    mutable server_has_get_sources2 : bool;
 
     mutable server_flags : int;
     mutable server_version : string;
-    mutable server_lowid_users : int64;
-    mutable server_soft_limit : int64;
-    mutable server_hard_limit : int64;
-    mutable server_max_users : int64;
+    mutable server_lowid_users : int64 option;
+    mutable server_soft_limit : int64 option;
+    mutable server_hard_limit : int64 option;
+    mutable server_max_users : int64 option;
+    mutable server_sent_shared : file list
   }
 
 
@@ -593,29 +523,6 @@ and user = {
     user_server : server;
   }
 
-(*
-and server_search = {
-    search_search : CommonTypes.search;
-    mutable nhits : int;
-  }
-*)
-
-(*
-and local_search = {
-    search_search : CommonTypes.search;
-    mutable search_handler : (search_event -> unit);
-    mutable search_xs_servers : server list;
-    mutable search_overnet : bool;
-  }
-*)
-
-  (*
-and result = {
-    result_result : result CommonResult.result_impl;
-    mutable result_index : Store.index;
-  }
-*)
-
 and search_event =
 (*  Result of result *)
 | Waiting of int
@@ -639,65 +546,39 @@ and server_change_kind =
 | ServerInfoChange
 | ServerBusyChange
 
-  (*
-and source = {
-(*    source_num : int; *)
-    source_addr : Ip.t * int;
-(*    mutable source_client: client_kind;      *)
-(* This field is not kept up-to-date when source_client = SourceClient c,
-  change c.client_source_for *)
-    mutable source_files : file_request list;
-    mutable source_overnet : bool;
-    mutable source_score : int;
-    mutable source_last_score : int;
-    mutable source_last_age : int;
-    mutable source_client_num : int;
-    mutable source_age : int;
-    mutable source_in_queues : file list;
-    mutable source_sock : tcp_connection;
-  }
-*)
-
 and client = {
     client_client : client CommonClient.client_impl;
     mutable client_kind : source_uid;
     client_source : DonkeySources.source;
     mutable client_md4 : Md4.t;
-(*    mutable client_chunks : availability; *)
-(*     mutable client_sock : tcp_connection; *)
     mutable client_ip : Ip.t;
     mutable client_download : (file * CommonSwarming.uploader) option;
-(*    mutable client_block : CommonSwarming.block option; *)
-(*    mutable client_zones : (int64 * int64 * CommonSwarming.range) list; *)
-(*    mutable client_connection_control : connection_control; *)
     mutable client_file_queue : (
       file * (* has displayed when connected *)
       Bitv.t *
       CommonSwarming.uploader
       ) list;
-    mutable client_next_view_files :  int;
     mutable client_all_files : result list option;
     mutable client_tags: CommonTypes.tag list;
     mutable client_name : string;
-(*    mutable client_all_chunks : string; *)
     mutable client_rating : int ;
     mutable client_upload : upload_info option;
     mutable client_checked : bool;
     mutable client_connected : bool;
 (* statistics *)
-(*    mutable client_last_filereqs : int;     *)
-    mutable client_downloaded : Int64.t;
-    mutable client_uploaded : Int64.t;
+    mutable client_session_downloaded : Int64.t;
+    mutable client_session_uploaded : Int64.t;
+    mutable client_total_downloaded : Int64.t;
+    mutable client_total_uploaded : Int64.t;
     mutable client_brand : brand;
     mutable client_brand_mod : brand_mod;
+    mutable client_osinfo_sent : bool;
+    mutable client_osinfo : string option;
     mutable client_banned : bool;
-    mutable client_score : int;
-    mutable client_next_queue : int;
     mutable client_rank : int;
     mutable client_connect_time : int;
     mutable client_requests_received : int;
     mutable client_requests_sent: int;
-    mutable client_indirect_address : (Ip.t * Ip.t * int) option;
     mutable client_slot : slot_status;
     mutable client_debug : bool;
     mutable client_pending_messages: string list;
@@ -708,6 +589,7 @@ and client = {
     mutable client_req_challenge : Int64.t;
     mutable client_public_key : string option;
     mutable client_sui_verified : bool option;
+    mutable client_last_file_req_md4 : Md4.t option;
   }
 
 and compressed_parts = {
@@ -728,56 +610,13 @@ and upload_info = {
     mutable up_pos : int64;
     mutable up_end_chunk : int64;
     mutable up_chunks : (int64 * int64) list;
+    (* zones sent but not yet received by other peer, oldest first *)
+    mutable up_flying_chunks : (int64 * int64) list; 
+    mutable up_current : int64;
+    mutable up_finish : bool;
     mutable up_waiting : bool;
   }
 
-(*
-
-and chunk =
-  PresentTemp
-| AbsentTemp
-| PartialTemp of block
-| PresentVerified
-| AbsentVerified
-| PartialVerified of block
-
-and block = {
-    mutable block_present: bool;
-    block_begin : int64;
-    block_end : int64;
-    mutable block_zones : zone list;
-    mutable block_nclients : int;
-    mutable block_contributors : Ip.t list;
-    mutable block_legacy : bool;
-    mutable block_pos : int;
-    block_file : file;
-  }
-
-and zone = {
-    mutable zone_begin : int64;
-    mutable zone_end : int64;
-    mutable zone_nclients : int;
-  }
-*)
-
-
-  (*
-and file_request = {
-    request_file : file;
-    mutable request_time : int;
-    mutable  request_result : request_result;
-  }
-
-and request_result =
-| File_possible   (* we asked, but didn't know *)
-| File_not_found  (* we asked, the file is not there *)
-| File_expected   (* we asked, because it was announced *)
-| File_new_source (* we never asked *)
-| File_found      (* the file was found *)
-| File_chunk      (* the file has chunks we want *)
-| File_upload     (* we uploaded from this client *)
-*)
-
 and client_kind =
   SourceClient of client
 | SourceLastConnection of
@@ -785,36 +624,6 @@ and client_kind =
   int * (* last connection attempt *)
   int   (* booked client num *)
 
-and file = {
-    file_file : file CommonFile.file_impl;
-    file_md4 : Md4.t;
-(*    file_exists : bool; *)
-
-    mutable file_swarmer : CommonSwarming.t option;
-
-    mutable file_nchunks : int;
-    mutable file_nchunk_hashes : int;
-    mutable file_diskname : string;
-(*    mutable file_chunks : chunk array; *)
-(*    mutable file_chunks_order : int array; *)
-(*    mutable file_chunks_age : int array; *)
-(*    mutable file_all_chunks : string; *)
-(*    mutable file_absent_chunks : (int64 * int64) list; *)
-    mutable file_filenames : (string * GuiTypes.ips_list) list;
-(*    mutable file_nsources : int; *)
-    mutable file_computed_md4s : Md4.t array;
-    mutable file_format : format;
-(*    mutable file_available_chunks : int array; *)
-    mutable file_shared : file CommonShared.shared_impl option;
-(*    mutable file_locations : client Intmap.t;  *)
-(*    mutable file_mtime : float; *)
-(*    mutable file_initialized : bool; *)
-(* Source management number 3 !! *)
-(*    mutable file_clients : (client * int) Fifo.t;  *)
-(*    mutable file_sources : source Queue.t array; *)
-    mutable file_sources : DonkeySources.file_sources_manager;
-  }
-
 and file_to_share = {
     shared_name : string;
     shared_size : int64;
@@ -890,198 +699,42 @@ let client_new_file client r =
 let server_state server =
   CommonServer.server_state (as_server server.server_server)
 
-(*
-module SourcesQueueCreate = Queues.Make (struct
-
-      type t = source
-      let compare s1 s2 = compare s1.source_addr s2.source_addr
-    end)
-
-  (*
-    let lifo = lifo
-    let fifo = fifo
-
-    module SourcesSet = Set.Make (
-        struct
-          type t = int * source
-          let compare (t1,s1) (t2,s2) =
-            if s1.source_addr = s2.source_addr then begin
-                0 end else
-            let x = compare t1 t2 in
-            if x = 0 then compare s1.source_addr s2.source_addr else x
-        end
-      )
-
-    let oldest_first () =
-      let t = ref SourcesSet.empty in
-      of_impl {
-        head = (fun _ -> try SourcesSet.min_elt !t with _ -> raise Fifo.Empty);
-        put = (fun x ->  t := SourcesSet.add x !t);
-        length = (fun _ -> SourcesSet.cardinal !t);
-        take = (fun _ ->
-            try
-              let x = SourcesSet.min_elt !t in
-              t := SourcesSet.remove x !t;
-              x
-            with _ -> raise Fifo.Empty);
-        iter = (fun f ->
-            SourcesSet.iter (fun (_,x) -> f x) !t);
-        put_back = (fun e -> t := SourcesSet.add e !t);
-      }
-
-    let oldest_last () =
-      let t = ref SourcesSet.empty in
-      of_impl {
-        head = (fun _ ->
-            try SourcesSet.max_elt !t with _ -> raise Fifo.Empty);
-        put = (fun x ->  t := SourcesSet.add x !t);
-        length = (fun _ -> SourcesSet.cardinal !t);
-        take = (fun _ ->
-            try
-              let x = SourcesSet.max_elt !t in
-              t := SourcesSet.remove x !t;
-              x
-            with _ -> raise Fifo.Empty);
-        iter = (fun f ->
-            SourcesSet.iter (fun (_,x) -> f x) !t);
-        put_back = (fun e -> t := SourcesSet.add e !t);
-      }
-
-      (*
-    let max_first compare =
-      let module SourceSet = Set.Make(struct
-            type t = source
-            let compare = compare
-          end) in
-      let t = ref SourcesSet.empty in
-      {
-        head = (fun _ -> SourcesSet.max_elt !t);
-        put = (fun x ->  t := SourcesSet.add x !t);
-        length = (fun _ -> SourcesSet.cardinal !t);
-        take = (fun _ ->
-            let x = SourcesSet.max_elt !t in
-            t := SourcesSet.remove x !t;
-            x);
-        iter = (fun f ->
-            SourcesSet.iter f !t);
-        put_back = (fun e -> t := SourcesSet.add e !t);
-        }
-
-    let min_first compare =
-      let module SourceSet = Set.Make(struct
-            type t = source
-            let compare = compare
-          end) in
-      let t = ref SourcesSet.empty in
-      of_impl {
-        head = (fun _ -> SourcesSet.min_elt !t);
-        put = (fun x ->  t := SourcesSet.add x !t);
-        length = (fun _ -> SourcesSet.cardinal !t);
-        take = (fun _ ->
-            let x = SourcesSet.min_elt !t in
-            t := SourcesSet.remove x !t;
-            x);
-        iter = (fun f ->
-            SourcesSet.iter f !t);
-        put_back = (fun e -> t := SourcesSet.add e !t);
-        }
-*)
-  end
-*)
-  *)
-
-  (*
-let string_of_file_tag_name name =
-  match name with
-  | FT_FILE_SIZE -> "size"
-  | FT_FILE_FILENAME -> "filename"
-  | FT_FILE_TYPE -> "type"
-  | FT_FILE_FORMAT -> "format"
-  | FT_FILE_AVAILABILITY -> "availability"
-  | FT_FILE_DOWNLOADED -> "downloaded"
-  | FT_FILE_DISKNAME -> "diskname"
-  | FT_FILE_PRIORITY -> "priority"
-  | FT_FILE_STATUS -> "status"
-  | FT_FILE_ARTIST -> "Artist"
-  | FT_FILE_TITLE -> "Title"
-  | FT_FILE_ALBUM -> "Album"
-  | FT_FILE_LOC -> "loc"
-
-let string_of_server_tag_name name =
-  match name with
-  | ST_SERVER_NAME -> "name"
-  | ST_SERVER_DESCRIPTION -> "description"
-  | ST_SERVER_PORT -> "port"
-  | ST_SERVER_IP -> "ip"
-  | ST_SERVER_PING -> "ping"
-  | ST_SERVER_PROF -> "prof"
-  | ST_SERVER_HISTORY -> "history"
-
-let string_of_client_tag_name name =
-  match name with
-  | CT_CLIENT_NAME -> "name"
-  | CT_CLIENT_PORT -> "port"
-  | CT_CLIENT_VERSION -> "version"
-  | CT_CLIENT_EXTENDED -> "extended"
-  | CT_CLIENT_UDPPORT -> "udpport"
-
-let string_of_emule_tag_name name =
-  match name with
-  | ET_COMPRESSION -> "compression"
-  | ET_UDPPORT -> "udpport"
-  | ET_UDPVER -> "udpver"
-  | ET_SOURCEEXCHANGE -> "sourceexchange"
-  | ET_COMMENTS -> "comments"
-  | ET_EXTENDEDREQUEST -> "extendedrequest"
-  | ET_COMPATIBLECLIENT -> "compatibleclient"
-  | ET_MOD_FEATURESET -> "mod_featureset"
-  | ET_MOD_PROTOCOL -> "mod_protocol"
-  | ET_MOD_VERSION -> "mod_version"
-  | ET_MOD_TAROD -> "mod_tarod"
-  | ET_TAROD_VERSION -> "tarod_version"
-  | ET_MOD_PLUS -> "mod_plus"
-
-let string_of_pref_tag_name name =
-  match name with
-  | PT_PREF_NAME -> ""
-  | PT_PREF_PORT -> ""
-  | PT_PREF_VERSION -> ""
-  | PT_PREF_TEMP -> ""
- *)
-  (*
-let dummy_source = {
-(*    source_num = 0; *)
-    source_addr = (Ip.null, 0);
-    source_files = [];
-
-    source_last_score = 0;
-    source_last_age = 0;
-    source_client_num = 0;
-
-    source_overnet = false;
-    source_score = 0;
-    source_age = 0;
-    source_in_queues = [];
-    source_sock = NoConnection;
-  }
-    *)
-
-
 let dummy_emule_proto = {
-    emule_comments = 0;
     emule_version = 0;
     emule_release = "";
+    emule_osinfosupport = 0;
+    emule_features = 0;
+
+(* emule_miscoptions1 *)
+    received_miscoptions1 = false;
+    emule_aich = 0;
+    emule_unicode = 0;
+    emule_udpver = 0;
+    emule_compression = 0; (* 1 *)
     emule_secident = 0;
+    emule_sourceexchange = 0; (* 3 *)
+    emule_extendedrequest = 0; (* 2 *)
+    emule_comments = 0;
+    emule_peercache = 0;
     emule_noviewshared = 0;
+    emule_multipacket = 0;
     emule_supportpreview = 0;
 
-    emule_compression = 0; (* 1 *)
-    emule_sourceexchange = 0; (* 3 *)
-    emule_multipacket = 0; (* 1 *)
-    emule_extendedrequest = 0; (* 2 *)
-    emule_features = 0; (* 3 *)
-    emule_udpver = 0; (* 4 *)
+(* emule_miscoptions2 *)
+    received_miscoptions2 = false;
+    emule_require_crypt = 0;
+    emule_request_crypt = 0;
+    emule_support_crypt = 0;
+    emule_extmultipacket = 0;
+    emule_largefiles = 0;
+    emule_kad_version = 0;
   }
 
 let emule_proto () =
   { dummy_emule_proto with emule_version = 0 }
+
+let old_max_emule_file_size = 4290048000L
+(* #define OLD_MAX_EMULE_FILE_SIZE 4290048000ui64  // (4294967295/PARTSIZE)*PARTSIZE = ~4GB *)
+
+let max_emule_file_size = 0x4000000000L
+(* #define MAX_EMULE_FILE_SIZE             0x4000000000ui64 // = 2^38 = 256GB *)
