@@ -78,13 +78,12 @@ let value_to_file file_size file_state assocs =
       Md4.of_string (get_value "file_id" value_to_string)
     with _ -> failwith "Bad file_id"
   in
-
-  let file = new_file file_id file_name file_size in
+  let file = new_file file_id file_name file_size CommonUserDb.admin_user in
 
   (match file.file_swarmer with
       None -> ()
     | Some swarmer ->
-        CommonSwarming.value_to_swarmer swarmer assocs;
+        CommonSwarming.value_to_frontend swarmer assocs;
   );
 
   (try
@@ -118,7 +117,7 @@ let file_to_value file =
   match file.file_swarmer with
     None -> assocs
   | Some swarmer ->
-      CommonSwarming.swarmer_to_value swarmer assocs
+      CommonSwarming.frontend_to_value swarmer assocs
 
 let old_files =
   define_option fileTP_section ["old_urls"]
