@@ -192,7 +192,8 @@ let new_client proto hostname port referer =
           client_hostname = hostname;
           client_referer = referer;
           client_port = port;
-	  client_downloaded = zero;
+	  client_total_downloaded = zero;
+	  client_session_downloaded = zero;
           client_reconnect = false;
           client_in_queues = [];
           client_connected_for = None;
