@@ -174,7 +174,7 @@ let rec client_parse_header c gconn sock header =
       end;
 
     if code < 200 || code > 299 then begin
-      pause_for_cause file (Printf.sprintf "%d" code);
+      pause_for_cause file (Printf.sprintf "%d" code) CommonUserDb.admin_user;
       failwith "Bad HTTP code";
     end;
 
@@ -288,7 +288,8 @@ let rec client_parse_header c gconn sock header =
             CommonSwarming.received up !counter_pos b.buf b.pos to_read_int;
             let new_downloaded = CommonSwarming.downloaded swarmer in
 
-            c.client_downloaded <- c.client_downloaded ++ (new_downloaded -- old_downloaded);
+            c.client_total_downloaded <- c.client_total_downloaded ++ (new_downloaded -- old_downloaded);
+            c.client_session_downloaded <- c.client_session_downloaded ++ (new_downloaded -- old_downloaded);
             client_must_update (as_client c);
 
             if new_downloaded = file_size file then
@@ -333,10 +334,8 @@ lprintf "READ: buf_used %d\n" to_read_int;
 (*                CommonSwarming.free_range r; *)
                 d.download_ranges <- tail;
 (* If we have no more range to receive, disconnect *)
-                if d.download_ranges = [] then begin
-                  lprintf_nl "No more ranges";
+                if d.download_ranges = [] then
                   raise Exit;
-                end;
                 gconn.gconn_handler <- HttpHeader (client_parse_header c);
           end)
 
@@ -375,7 +374,6 @@ let http_check_size file url start_download_file =
     } in
 
   H.whead2 r (fun headers ->
-      if !verbose then lprintf_nl "RECEIVED HEADERS";
       let content_length = ref None in
       List.iter (fun (name, content) ->
           if String.lowercase name = "content-length" then
@@ -389,7 +387,7 @@ let http_check_size file url start_download_file =
   (fun c ->
     match c with 
      x when x < 200 || x > 299 -> 
-       pause_for_cause file (string_of_int x);
+       pause_for_cause file (string_of_int x) CommonUserDb.admin_user;
     | _ -> ()
   )
 
