@@ -51,6 +51,7 @@ module VB = VerificationBitmap
 
 let _ =
   network.op_network_connected <- (fun _ -> true);
+  network.op_network_ports <- (fun _ -> []);
   network.op_network_connected_servers <- (fun _ -> [])
 
 
@@ -90,7 +91,6 @@ let _ =
       { (impl_file_info file.file_file) with
         P.file_fields = P.Fields_file_info.all;
         P.file_network = network.network_num;
-        P.file_names = file.file_filenames;
         P.file_all_sources = (List.length file.file_clients);
         P.file_active_sources = (List.length file.file_clients);
         P.file_chunks = (match file.file_swarmer with
@@ -123,7 +123,8 @@ let _ =
         P.client_num = (client_num (as_client c));
         P.client_connect_time = BasicSocket.last_time ();
         P.client_software = c.client_software;
-        P.client_downloaded = c.client_downloaded;
+        P.client_total_downloaded = c.client_total_downloaded;
+        P.client_session_downloaded = c.client_session_downloaded;
       }
   );
     client_ops.op_client_bprint <- (fun c buf ->
@@ -141,8 +142,8 @@ let _ =
         client_print cc o;
         Printf.bprintf buf "client: %s downloaded: %s uploaded: %s"
           cinfo.GuiTypes.client_software
-          (Int64.to_string cinfo.GuiTypes.client_downloaded)
-        (Int64.to_string cinfo.GuiTypes.client_uploaded);
+          (Int64.to_string cinfo.GuiTypes.client_total_downloaded)
+        (Int64.to_string cinfo.GuiTypes.client_total_uploaded);
         Printf.bprintf buf "\nfilename: %s\n\n" info.GuiTypes.file_name;
     );
     client_ops.op_client_dprint_html <- (fun c o file str ->
@@ -182,9 +183,11 @@ let _ =
           ("", "sr", "D");
           ("", "sr", "N");
           ("", "sr", Printf.sprintf "%s:%d" (Ip.to_string client_ip) client_port);
-          ] @ (if !Geoip.active then [(cname, "sr", ccode)] else []) @ [
-          ("", "sr ar", (size_of_int64 cinfo.GuiTypes.client_uploaded));
-          ("", "sr ar", (size_of_int64 cinfo.GuiTypes.client_downloaded));
+          ] @ (if !Geoip.active then [(cname, "sr", CommonPictures.flag_html ccode)] else []) @ [
+          ("", "sr ar", (size_of_int64 cinfo.GuiTypes.client_total_uploaded));
+          ("", "sr ar", (size_of_int64 cinfo.GuiTypes.client_total_downloaded));
+          ("", "sr ar", (size_of_int64 cinfo.GuiTypes.client_session_uploaded));
+          ("", "sr ar", (size_of_int64 cinfo.GuiTypes.client_session_downloaded));
           ("", "sr", info.GuiTypes.file_name); ]);
         true
     )
@@ -234,7 +237,7 @@ and find_mirrors file u =
 
 let previous_url = ref ""  
   
-let download_file url referer = 
+let download_file url referer user =
   let u = Url.of_string url in
 
   if List.mem u !!old_files && !previous_url <> url then begin
@@ -242,7 +245,7 @@ let download_file url referer =
       failwith "URL already downloaded: repeat command again to force";
     end;
 
-  let file = new_file (Md4.random ()) u.Url.full_file zero in
+  let file = new_file (Md4.random ()) u.Url.full_file zero user in
   
   if !verbose then
     lprintf_nl "Started new download: %s from %s" (file_best_name file) url; 
@@ -250,6 +253,7 @@ let download_file url referer =
       current_files := file :: !current_files;
     end;
   
+  CommonInteractive.start_download (as_file file);
   download_file_from_mirror file u referer;
   find_mirrors file u
 
@@ -257,10 +261,8 @@ let download_file url referer =
    bt-url-handler is called first. *)
 let is_http_torrent headers url =
   let ext = String.lowercase (Filename2.last_extension url) in
-  if ext = ".torrent" || ext = ".tor"
+  ext = ".torrent" || ext = ".tor"
      || (String2.contains headers "content-type application/x-bittorrent")
-    then true
-  else false
 
 let get_regexp_int text r =
   ignore (Str.search_forward r text 0);
@@ -278,18 +280,17 @@ let get_regexp_string text r =
    It returns true if this file can be handled by fileTP,
    and false otherwise.
  *)
-let op_network_parse_url url =
+let op_network_parse_url url user =
   let location_regexp = "Location: \\(.*\\)" in
   let real_url = get_regexp_string url (Str.regexp location_regexp) in
-  if !verbose then
-    lprintf_nl "real url: %s\n" real_url;
-  if (is_http_torrent url real_url) then "", false
+  if (is_http_torrent url real_url) && !!enable_bittorrent then
+    "this seems to be a torrent, ignoring...", false
   else
     if (String2.check_prefix real_url "http://") then (
       let length_regexp = "Content-Length: \\(.*\\)" in
        try let length = get_regexp_int url (Str.regexp length_regexp) in
          if (length > 0) then begin
-           download_file real_url ""; "started FileTP download", true
+           download_file real_url "" user; "started FileTP download", true
          end
          else "can not parse Content-Length", false
        with Not_found -> 
@@ -298,7 +299,7 @@ let op_network_parse_url url =
     else 
       if (String2.check_prefix real_url "ftp://") ||
          (String2.check_prefix real_url "ssh://") then (
-      download_file real_url "";
+      download_file url "" user;
       "started FileTP download", true)
     else
       "invalid URL", false
@@ -313,8 +314,8 @@ let commands = [
     "http", "Network/FileTP", Arg_multiple (fun args o ->
         try
         (match args with
-          url :: [referer] -> download_file url referer
-        | [url] -> download_file url ""
+          url :: [referer] -> download_file url referer o.conn_user.ui_user
+        | [url] -> download_file url "" o.conn_user.ui_user
         | _ -> raise Not_found);
         let buf = o.conn_buf in
         if o.conn_output = HTML then
@@ -408,7 +409,7 @@ let _ =
   CommonNetwork.register_commands commands;
   network.op_network_share <- (fun fullname codedname size -> ());
   network.op_network_search <- (fun ss buf -> ());
-  network.op_network_download <- (fun r -> dummy_file);
+  network.op_network_download <- (fun r user -> dummy_file);
   file_ops.op_file_commit <- (fun file new_name -> clean_stop file);
   file_ops.op_file_pause <- (fun file -> 
     List.iter (fun c ->
@@ -419,7 +420,10 @@ let _ =
     ) file.file_clients
   );
   file_ops.op_file_resume <- (fun file -> ());
-  file_ops.op_file_print_html <- (fun file buf -> ());
+  file_ops.op_file_print <- (fun file buf -> ());
+  network.op_network_close_search <- (fun s -> ());
   network.op_network_forget_search <- (fun s -> ());
   network.op_network_connect_servers <- (fun s -> ());
+  network.op_network_reset <- (fun _ -> ());
+  network.op_network_porttest_result <- (fun _ -> PorttestNotAvailable);
   network.op_network_recover_temp <- (fun s -> ())
