@@ -442,8 +442,8 @@ let _ =
         client_print cc o; 
         Printf.bprintf buf "client: %s downloaded: %s uploaded: %s"
           "gN" (* cinfo.GuiTypes.client_software *)
-          (Int64.to_string cinfo.GuiTypes.client_downloaded)
-        (Int64.to_string cinfo.GuiTypes.client_uploaded);
+          (Int64.to_string cinfo.GuiTypes.client_total_downloaded)
+        (Int64.to_string cinfo.GuiTypes.client_total_uploaded);
         Printf.bprintf buf "\nfilename: %s\n\n" info.GuiTypes.file_name;
     );    
     client_ops.op_client_dprint_html <- (fun c o file str ->
@@ -451,6 +451,11 @@ let _ =
         let buf = o.conn_buf in
         let cc = as_client c in
         let cinfo = client_info cc in
+        let ccode,cname =
+          match c.client_host with
+          | None -> "?", "?"
+          | Some (ip, _) -> Geoip.get_country ip
+        in
         Printf.bprintf buf " \\<tr onMouseOver=\\\"mOvr(this);\\\"
     onMouseOut=\\\"mOut(this);\\\" class=\\\"%s\\\"\\>" str;
         
@@ -476,9 +481,11 @@ let _ =
           ("", "sr", "D");
           ("", "sr", "N");
           ("", "sr", (string_of_kind cinfo.GuiTypes.client_kind));
-          ] @ (if !Geoip.active then [("?", "sr", "?")] else []) @ [
-          ("", "sr ar", (size_of_int64 cinfo.GuiTypes.client_uploaded));
-          ("", "sr ar", (size_of_int64 cinfo.GuiTypes.client_downloaded));
+          ] @ (if !Geoip.active then [( cname, "sr br", CommonPictures.flag_html ccode)] else []) @ [
+          ("", "sr ar", (size_of_int64 cinfo.GuiTypes.client_total_uploaded));
+          ("", "sr ar", (size_of_int64 cinfo.GuiTypes.client_total_downloaded));
+          ("", "sr ar", (size_of_int64 cinfo.GuiTypes.client_session_uploaded));
+          ("", "sr ar", (size_of_int64 cinfo.GuiTypes.client_session_downloaded));
           ("", "sr", info.GuiTypes.file_name); ]);
         true
     )     
