@@ -170,7 +170,7 @@ let gnutella_503_shielded = "GNUTELLA/0.6 503 I am a shielded leaf node"
 let add_header_fields header sock trailer =
   let buf = Buffer.create 100 in
   Printf.bprintf buf "%s" header;
-  Printf.bprintf buf "User-Agent: %s\r\n" user_agent;
+  Printf.bprintf buf "User-Agent: %s\r\n" (get_user_agent ());
   Printf.bprintf buf "X-My-Address: %s:%d\r\n"
     (Ip.to_string (client_ip (Some sock))) !!client_port;
   Printf.bprintf buf "X-Ultrapeer: False\r\n";
@@ -242,7 +242,7 @@ let cached_qrt_table = ref ""
 
 let update_shared_words () = 
   if !verbose_share then
-    lprintf_nl () "update_shared_words";
+    lprintf_nl "update_shared_words";
   all_shared_words := [];
   cached_qrt_table := "";
   let module M = CommonUploads in
@@ -257,7 +257,7 @@ let update_shared_words () =
     List.iter (fun sh ->
         let info = IndexedSharedFiles.get_result sh.shared_info in
         if !verbose_share then
-          lprintf_nl () "CODED name: %s" sh.M.shared_codedname;
+          lprintf_nl "CODED name: %s" sh.M.shared_codedname;
         register_words sh.M.shared_codedname;
         List.iter (fun uid ->
             words := WordSet.add (Uid.to_string uid) !words
@@ -311,16 +311,31 @@ let parse_headers c first_line headers =
 
       try
         
+        begin
+          try 
+            let (server,_) = List.assoc "server" headers in
+            c.client_user.user_software <- server;
+            client_must_update c;
+          with Not_found -> ()
+        end;
+        begin
+          try 
+            let (useragent,_) = List.assoc "user-agent" headers in
+            c.client_user.user_software <- useragent;
+            client_must_update c;
+          with Not_found -> ()
+        end;
+
         let (locations,_) = 
           List.assoc "x-gnutella-alternate-location" headers in
         let locations = String2.split locations ',' in
         
-        lprintf_nl () "Alternate locations";
+        lprintf_nl "Alternate locations";
         let urls = List.map (fun s ->
               match String2.split_simplify s ' ' with
-                [] -> lprintf_nl () "Cannot parse : %s" s; ""
+                [] -> lprintf_nl "Cannot parse : %s" s; ""
               | url :: _ ->
-                  lprintf_nl () "  Location: %s" url; url
+                  lprintf_nl "  Location: %s" url; url
           ) locations in
         lprint_newline ();
         
@@ -403,7 +418,7 @@ let request_of_download request d =
   
 let make_download_request c s headers =
   let headers =
-    ("User-Agent", user_agent) ::
+    ("User-Agent", get_user_agent ()) ::
     ("Connection", "Keep-Alive") ::
     (match c.client_host with
         None -> headers
@@ -452,7 +467,7 @@ an upload request *)
                                 None -> assert false | Some sw -> sw
                             in
                             let chunks = [ Int64.zero, file_size file ] in
-                            let bs = Int64Swarmer.register_uploader swarmer 
+                            let bs = CommonSwarming.register_uploader swarmer 
                                 (as_client c)
                                 (AvailableRanges chunks) in
                             d.download_uploader <- Some bs
@@ -523,7 +538,7 @@ let find_file_to_upload gconn url =
       match url.Url.args with
         [(urn,_)] ->
           if !verbose_msg_clients then
-            lprintf_nl () "Found /uri-res/N2R request";
+            lprintf_nl "Found /uri-res/N2R request";
           find_by_uid (Uid.of_string urn)
       
       | _ -> failwith "Cannot parse /uri-res/N2R request"
@@ -534,7 +549,7 @@ let find_file_to_upload gconn url =
     let num = String.sub file 5 (pos - 5) in
     let filename = String.sub file (pos+1) (String.length file - pos - 1) in
     if !verbose_msg_clients then
-      lprintf_nl () "Download of file %s, filename = %s" num filename;
+      lprintf_nl "Download of file %s, filename = %s" num filename;
     let num = int_of_string num in
     Hashtbl.find shareds_by_id  num
   in
