@@ -19,6 +19,9 @@
 
 
 open Autoconf
+open Syslog
+
+let syslog_oc = ref None
 
 external format_int: string -> int -> string = "caml_format_int"
 external format_int32: string -> int32 -> string = "caml_int32_format"
@@ -190,24 +193,24 @@ let cprintf kont fmt =
     Buffer.add_string dest (printer ()); doprn i
   in doprn 0
 
-let lprintf_handler = ref (fun s ->
-      Printf.printf "Message [%s] discarded\n" s;
+let lprintf_handler = ref (fun s time ->
+      Printf.printf "%sMessage [%s] discarded\n" time s;
   )
 
 let lprintf fmt =
-  cprintf (fun s -> try !lprintf_handler s with _ -> ())
+  cprintf (fun s -> try !lprintf_handler "" s with _ -> ())
   (fmt : ('a,unit, unit) format )
 
 let lprintf2 m fmt =
-  cprintf (fun s -> try !lprintf_handler ((log_time ())^m^" "^s) with _ -> ())
+  cprintf (fun s -> try !lprintf_handler (log_time ()) (m^" "^s) with _ -> ())
   (fmt : ('a,unit, unit) format )
 
 let lprintf_nl fmt =
-  cprintf (fun s -> try !lprintf_handler ((log_time ())^s^"\n") with _ -> ())
+  cprintf (fun s -> try !lprintf_handler (log_time ()) (s^"\n") with _ -> ())
   (fmt : ('a,unit, unit) format )
 
 let lprintf_nl2 m fmt =
-  cprintf (fun s -> try !lprintf_handler ((log_time ())^m^" "^s^"\n") with _ -> ())
+  cprintf (fun s -> try !lprintf_handler (log_time ()) (m^" "^s^"\n") with _ -> ())
   (fmt : ('a,unit, unit) format )
 
 let lprint_newline () = lprintf "\n"
@@ -221,21 +224,30 @@ let set_lprintf_handler f =
 let lprintf_max_size = ref 100
 let lprintf_size = ref 0
 let lprintf_fifo = Fifo.create ()
-let lprintf_to_stdout = ref true
+let lprintf_to_channel = ref true
 
 let lprintf_output = ref (Some stderr)
+let lprintf_original_output = ref None
+
+let keep_console_output () =
+  match !lprintf_original_output with
+    Some c when c = stderr || c = stdout -> true
+  | _ -> false
 
 let _ =
-  set_lprintf_handler (fun s ->
+  set_lprintf_handler (fun time s ->
+      (match !syslog_oc with
+	None -> ()
+      | Some oc -> Syslog.syslog oc `LOG_INFO s);
       match !lprintf_output with
-        Some out when !lprintf_to_stdout ->
-          Printf.fprintf out "%s" s; flush out
+        Some out when !lprintf_to_channel ->
+          Printf.fprintf out "%s" (time ^ s); flush out
       | _ ->
           if !lprintf_size >= !lprintf_max_size then
             ignore (Fifo.take lprintf_fifo)
           else
             incr lprintf_size;
-          Fifo.put lprintf_fifo s
+          Fifo.put lprintf_fifo (time ^ s)
   )
 
 let detach () =
@@ -244,7 +256,7 @@ let detach () =
   | _ -> ()
 
 let close_log () =
-  lprintf_to_stdout := false;
+  lprintf_to_channel := false;
   match !lprintf_output with
     None -> ()
   | Some oc ->
@@ -255,7 +267,7 @@ let close_log () =
 let log_to_file oc =
   close_log ();
   lprintf_output := Some oc;
-  lprintf_to_stdout := true
+  lprintf_to_channel := true
 
 let log_to_buffer buf =
   try
@@ -266,9 +278,6 @@ let log_to_buffer buf =
     done
   with _ -> ()
 
-let set_logging b =
-  lprintf_to_stdout := b
-
 (* html_mods *)
 
 let html_mods_commands buf n c l =
@@ -386,3 +395,6 @@ let html_mods_cntr () =
 
 let html_mods_cntr_init () =
   html_mods_counter := true
+
+let print_plural_s v =
+  if v > 1 then "s" else ""
