@@ -172,7 +172,7 @@ external is_utf8 : string -> bool = "ml_utf8_validate"
 (**********************************************************************************)
 
 (* taken from camomile *)
-(* $Id: charset.ml,v 1.6 2005/11/19 17:19:45 spiralvoice Exp $ *)
+(* $Id: charset.ml,v 1.9 2006/10/08 14:12:13 spiralvoice Exp $ *)
 (* Copyright 2002, 2003 Yamagata Yoriyuki. distributed with LGPL *)
 
 let utf8_look s i =
@@ -254,7 +254,7 @@ let utf8_get s n = utf8_look s (nth s n)
 (**********************************************************************************)
 
 (* taken from camomile *)
-(* $Id: charset.ml,v 1.6 2005/11/19 17:19:45 spiralvoice Exp $ *)
+(* $Id: charset.ml,v 1.9 2006/10/08 14:12:13 spiralvoice Exp $ *)
 (* Copyright 2002, 2003 Yamagata Yoriyuki. distributed with LGPL *)
 
 let rec length_aux s c i =
@@ -281,7 +281,7 @@ let utf8_length s = length_aux s 0 0
 
 
 (* taken from camomile *)
-(* $Id: charset.ml,v 1.6 2005/11/19 17:19:45 spiralvoice Exp $ *)
+(* $Id: charset.ml,v 1.9 2006/10/08 14:12:13 spiralvoice Exp $ *)
 (* Copyright 2002, 2003 Yamagata Yoriyuki. distributed with LGPL *)
 
 external uint_code : uchar -> int = "%identity"
@@ -1303,10 +1303,15 @@ let charset_from_string s =
 (**********************************************************************************)
 
 let normalize_language s =
+  let s = String.uppercase s in
   if String.length s > 1
   then begin
-    let s = String.sub s 0 2 in
-    String.uppercase s
+(* We have to distinguish between ZH_tw and ZH_cn here
+   ZH_tw = BIG5/Chinese traditional -- ZH_cn = GBK/Chinese simplified *)
+    if String.sub s 0 2 = "ZH" && String.length s > 4 then
+      String.sub s 0 5
+    else
+      String.sub s 0 2
   end else "EN"
 
 (**********************************************************************************)
@@ -1726,6 +1731,8 @@ let charset_list_from_language lang =
       | "SL" -> li := central_european :: !li
       | "SH"
       | "SR" -> li := central_european :: cyrillic ::!li
+      | "ZH_CN" -> li := chinese_simplified :: !li
+      | "ZH_TW" -> li := chinese_traditional :: !li
       | "ZH" -> li := chinese_traditional :: chinese_simplified :: !li
       | "BE"
       | "BG"
@@ -1803,6 +1810,8 @@ let set_default_charset_list (lang : string) =
 (*                                                                                *)
 (**********************************************************************************)
 
+let conversion_enabled = ref true
+
 let convert ~from_charset ~to_charset s =
   if s <> "" then begin
     let t = charset_to_string to_charset in
@@ -1810,6 +1819,17 @@ let convert ~from_charset ~to_charset s =
     convert_string s t f
   end else s
 
+let safe_convert enc s =
+  match enc with
+    "" -> s
+  | enc ->
+    try
+      convert
+        ~from_charset: (charset_from_string enc)
+        ~to_charset: (charset_from_string "UTF-8")
+        s
+    with _ -> s
+
 (**********************************************************************************)
 (*                                                                                *)
 (*                          slow_encode_from_utf8                                 *)
@@ -1891,7 +1911,7 @@ let to_utf8 s =
 (**********************************************************************************)
 
 let to_locale s =
-  if s = ""
+  if s = "" || not !conversion_enabled
   then s
   else begin
     let s = to_utf8 s in
