@@ -31,15 +31,14 @@ let chunk_min_size = ref 65000L
   
 let max_buffered = ref (Int64.of_int (1024 * 1024))
   
-let create_dir_mask = ref "755"
+let create_file_mode = ref 0o664
+let create_dir_mode = ref 0o755
 let verbose = ref false
 let max_cache_size = ref 50
   
 let mini (x: int) (y: int) =
   if x > y then y else x
     
-let rights = 0o664
-  
 let ro_flag =  [Unix.O_RDONLY]
 let rw_flag =  [Unix.O_RDWR]
 let rw_creat_flag =  [Unix.O_CREAT; Unix.O_RDWR]
@@ -121,7 +120,8 @@ module FDCache = struct
 		try
 		  if t.writable then
                     Unix.openfile t.filename 
-		      (if creat then rw_creat_flag else rw_flag) rights
+		      (if creat then rw_creat_flag else rw_flag) 
+		      !create_file_mode
 		  else
 		    Unix.openfile t.filename ro_flag 0o400
 		with e ->
@@ -204,16 +204,24 @@ module FDCache = struct
       raise e
 
     let owner t =
+      let module U = Unix.LargeFile in
       try
       check_destroyed t;
-      let s = Unix.fstat (local_force_fd t) in
-      let user = Unix.getpwuid s.Unix.st_uid in
-      let group = Unix.getgrgid s.Unix.st_gid in
+      let s =
+        try
+          U.fstat (local_force_fd t)
+        with
+ (* Unix.fstat is not supported on MinGW, Unix.stat is supported *)
+        | Invalid_argument _ -> U.stat t.filename
+      in
+      let user = Unix.getpwuid s.U.st_uid in
+      let group = Unix.getgrgid s.U.st_gid in
       user.Unix.pw_name, group.Unix.gr_name
-      with e ->
-      if !verbose then lprintf_nl "Exception in FDCache.owner %s: %s"
-        t.filename
-        (Printexc2.to_string e);
+      with
+      | Not_found -> "", ""
+      | e -> if !verbose then
+            lprintf_nl "Exception in FDCache.owner %s: %s"
+              t.filename (Printexc2.to_string e);
       raise e
 
     let mtime64 t =
@@ -254,9 +262,9 @@ module FDCache = struct
       check_destroyed t;
       close t;
       (let d = (Filename.dirname (Filename.concat f file)) in
-        Unix2.safe_mkdir d;
-        Unix2.chmod d (Misc.int_of_octal_string !create_dir_mask);
-	Unix2.can_write_to_directory d);
+       Unix2.safe_mkdir d;
+       Unix2.chmod d !create_dir_mode;
+       Unix2.can_write_to_directory d);
       (try
         Unix2.rename t.filename (Filename.concat f file);
       with
@@ -1325,64 +1333,16 @@ let buffered_write t offset s pos_s len_s =
 let buffered_write_copy t offset s pos_s len_s =
   buffered_write t offset (String.sub s pos_s len_s) 0 len_s
 
-(* like String.compare, but on the first n characters (at most) *)
-let string_comparen s1 s2 n =
-  let len1 = String.length s1 in
-  let len2 = String.length s2 in
-  let rec aux i =
-    if i = n then 0 
-    else 
-      match i = len1, i = len2 with
-      | true, true -> 0
-      | false, true -> 1
-      | true, false -> -1
-      | false, false ->
-	  let cmp = (int_of_char s1.[i]) - (int_of_char s2.[i]) in
-	  if cmp <> 0 then cmp
-	  else aux (i+1) in
-  aux 0
-
 let copy_chunk t1 t2 pos1 pos2 len =
   flush_fd t1;
   flush_fd t2;
   let buffer_size = 128 * 1024 in
   let buffer = String.make buffer_size '\001' in
-  (* to prevent allocating disk space by writting zeroes over
-     otherwise sparse files, we check if writes would change
-     target file content before proceeding; 
-     Since failures are expensive (one additionnal read of target
-     chunk), after 3 failures we assume source and
-     target are different and revert to plain old copying *)
-  let buffer_tgt = String.make buffer_size '\001' in
-  let maybe_sparse = ref true in
-  let found_different_credit = ref 3 in
-
   let rec iter remaining pos1 pos2 =
     let len = mini remaining buffer_size in
     if len > 0 then begin
       read t1 pos1 buffer 0 len;
-      let is_different = 
-	if not !maybe_sparse then true
-	else
-	  try
-	    read t2 pos2 buffer_tgt 0 len;
-	    let result = string_comparen buffer buffer_tgt len <> 0 in
-	    if result && !verbose then
-	      lprintf_nl "copy_chunk: target is different";
-	    result
-	  with _ -> 
-	    (* reached end of target file, or any other oddity ? 
-	       revert to plain copying immediately *)
-	    maybe_sparse := false;
-	    true  in
-      if is_different then begin
-	if !found_different_credit > 0 then begin
-	  decr found_different_credit;
-	  (* too many differences ? give up cleverness *)
-	  if !found_different_credit = 0 then maybe_sparse := false
-	end;
-        write t2 pos2 buffer 0 len;
-      end;
+      write t2 pos2 buffer 0 len;
       let len64 = Int64.of_int len in
       iter (remaining - len) (pos1 ++ len64) (pos2 ++ len64)
     end
@@ -1483,13 +1443,15 @@ let owner_fd t =
   | Destroyed -> "", ""
 
 let owner s =
-  try
     let old_fd_exists = fd_exists s in
     let fd = create_ro s in
-    let user,group = owner_fd fd in
-    if not old_fd_exists then close fd;
-    user,group
+  let user,pass =
+    try
+      owner_fd fd
   with _ -> "", ""
+  in
+  if not old_fd_exists then close fd;
+  user, pass
 
 let rename t f =
   flush_fd t;
@@ -1794,14 +1756,22 @@ let percentused dir =
 (* percentage of used disk space *)
   match diskfree dir, disktotal dir with
   | Some dfree, Some dtotal ->
-      Some (Int64.to_int (100L -- (dfree ** 100L // dtotal)))
+      begin
+	try
+	  Some (Int64.to_int (100L -- (dfree ** 100L // dtotal)))
+        with Division_by_zero -> None
+      end
   | _ -> None
 
 let percentfree dir =
 (* percentage of free disk space *)
   match diskfree dir, disktotal dir with
   | Some dfree, Some dtotal ->
-      Some (Int64.to_int (dfree ** 100L // dtotal))
+      begin
+	try
+          Some (Int64.to_int (dfree ** 100L // dtotal))
+	with Division_by_zero -> None
+      end
   | _ -> None
 
 let filesystem dir =
