@@ -26,7 +26,8 @@ val external_exit : unit -> unit
 val uname : unit -> string
 val os_supported : unit -> bool
 
-val create_dir_mask : string ref
+val create_file_mode : int ref
+val create_dir_mode : int ref
 val close : t -> unit
 (* val force_fd : t -> Unix.file_descr *)
   
@@ -44,6 +45,7 @@ val rename : t -> string -> unit
 val max_cache_size : int ref
 val mtime : string -> float
 val mtime64 : t -> float
+val owner : string -> (string * string)
   
 val flush : unit -> unit
 val flush_fd : t -> unit
