@@ -255,13 +255,8 @@ let rec get_page r content_handler f ferr =
         TcpBufferedSocket.set_reader sock (http_reply_handler nread
             (default_headers_handler url level));
         set_rtimeout sock 5.;
-      (*
-        TcpBufferedSocket.set_closer sock (fun _ _ -> ()
-        lprintf "Connection closed nread:%b\n" !nread; 
-        )
-      *)
-
     )
+    ferr;
   with e -> 
     lprintf_nl "error in get_url"; 
     raise Not_found
@@ -334,6 +329,14 @@ let rec get_page r content_handler f ferr =
           raise Not_found
         end
           
+    | 400 when r.req_request = HEAD ->
+        lprintf_nl "Error 400 received for HEAD %s, re-try GET" (Url.to_string_no_args r.req_url);
+        let r2 = {
+          r with
+          req_request = GET;
+        } in
+        get_page r2 content_handler f ferr
+
     | 404 ->
         lprintf_nl "404: Not found for: %s" (Url.to_string_no_args r.req_url);
         close sock (Closed_for_error "bad reply");
@@ -355,6 +358,7 @@ let rec get_page r content_handler f ferr =
     end
         else begin
           lprintf_nl "more than %d retries, aborting." r.req_max_retry;
+          ferr ans_code;
           raise Not_found
         end
           
