@@ -20,6 +20,7 @@
 open Printf2
 open BasicSocket
 open TcpBufferedSocket
+open Ip_set
 
 let verbose = ref false
 
@@ -91,6 +92,10 @@ type auth =
 | Read_auth
 | Write_auth
 
+type error_reason =
+| Blocked
+| Not_allowed
+| Url_not_found of string
 
 type header =
   Unknown of string * string
@@ -143,7 +148,8 @@ and config = {
     bind_addr : Unix.inet_addr;
     mutable port : int;
     requests : (string * handler) list;
-    mutable addrs : Ip.t list;
+    mutable addrs : Ip_set.blocking_list;
+    use_ip_block_list : bool;
     base_ref : string;
     default : handler;
   }
@@ -222,6 +228,34 @@ let split_head s =
   in
   iter 0 []
 
+let error_page code from_ip from_port my_ip my_port reason =
+  let error_text, error_text_long =
+    match code with
+    | "401" -> "Unauthorized", "<p>Please login using a valid username and password</p>"
+    | "403" -> "Forbidden", 
+		(match reason with
+		   Some Not_allowed -> Printf.sprintf
+"<p>Connection from %s rejected (see downloads.ini, <a href=\"http://mldonkey.sourceforge.net/Allowed_ips\">allowed_ips</a>)</p>"
+				    from_ip
+		 | Some Blocked -> Printf.sprintf "IP %s is blocked, its part of the used IP blocklist " from_ip
+		 | _ -> "")
+    | "404" -> "Not found", Printf.sprintf "The requested URL %swas not found on this server."
+			      (match reason with Some (Url_not_found url) -> url ^ " " | _ -> "")
+    | _ -> Printf.sprintf "Unknown error %s" code, ""
+  in
+  let reject_message = Printf.sprintf
+"<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n<html>
+<head><title>%s %s</title></head>\n<h1>%s %s</h1>\n%s
+<hr><address>MLDonkey/%s at %s Port %s</address></html>"
+    code error_text code error_text error_text_long
+    Autoconf.current_version my_ip my_port
+  in
+  Printf.sprintf
+"HTTP/1.1 %s %s\nMLDonkey/%s\nConnection: close
+Content-Type: text/html; charset=iso-8859-1\nContent-length: %d\r\n"
+    code error_text Autoconf.current_version (String.length reject_message), reject_message,
+  Printf.sprintf "%s %s" code error_text
+
 let parse_head sock s =
   let h = split_head s in
 (*  List.iter (fun s -> lprintf_http_nl () "LINE: [%s]" (escaped s)) h; *)
@@ -651,13 +685,6 @@ let give_doc buf request =
       at_write_end buf.fd_task shutdown
 *)
 
-let need_auth r name =
-  r.reply_head <- "401 Unauthorized";
-  r.reply_headers <- [
-    "Connection", "close";
-    "WWW-Authenticate", Printf.sprintf "Basic realm=\"%s\"" name
-  ]
-
 (*
 let simple_give_auth psread pswrite request  =
   try
@@ -758,7 +785,7 @@ let request_handler config sock nread =
   let b = TcpBufferedSocket.buf sock in
   let end_pos = b.pos + b.len in
   let new_pos = end_pos - nread in
-  let new_pos = maxi 0 (new_pos - 1) in
+  let new_pos = max 0 (new_pos - 1) in
 (*  lprintf "received [%s]\n" (String.escaped
       (String.sub b.buf new_pos nread)); - log commented out *)
   let rec iter i =
@@ -795,20 +822,26 @@ let request_closer sock msg =
   ()
 
 
-open Ip_set
-
 let handler config t event =
   match event with
     TcpServerSocket.CONNECTION (s, Unix.ADDR_INET(from_ip, from_port)) ->
     (* check here if ip is OK *)
       let from_ip = Ip.of_inet_addr from_ip in
-      if Ip.matches from_ip config.addrs &&
-        (match !Ip.banned from_ip with
-           None -> true
-         | Some reason ->
-             lprintf_nl "%s:%d blocked: %s\n"
-               (Ip.to_string from_ip) from_port reason;
-             false) then
+      let ip_is_allowed from_ip =
+	match Ip_set.match_ip config.addrs from_ip with
+	| Some br -> true
+	| None -> false
+      in
+      let ip_is_blocked from_ip =
+	if config.use_ip_block_list then
+	  match !Ip.banned from_ip with
+            None -> false
+           | Some reason -> lprintf_nl "%s:%d blocked: %s"
+               (Ip.to_string from_ip) from_port reason; true
+        else
+	  false
+      in 
+      if ip_is_allowed from_ip && not (ip_is_blocked from_ip) then
         let token = create_token unlimited_connection_manager in
         let sock = TcpBufferedSocket.create_simple
             token "http connection" s in
@@ -819,8 +852,20 @@ let handler config t event =
           (fun _ -> TcpBufferedSocket.close sock Closed_for_overflow;
          lprintf_nl "BUFFER OVERFLOW" );  ()
       else begin
-         lprintf_nl "connection from %s rejected (see allowed_ips setting)"
-          (Ip.to_string from_ip);
+        lprintf_nl "connection from %s rejected (%s)"
+          (Ip.to_string from_ip)
+	  (if ip_is_blocked from_ip then "IP is blocked" else "see allowed_ips setting");
+        let token = create_token unlimited_connection_manager in
+        let sock = TcpBufferedSocket.create_simple token "http connection" s in
+	let s1,s2,_ = error_page "403"
+	    (Ip.to_string from_ip)
+	    (string_of_int from_port)
+	    (Ip.to_string (TcpBufferedSocket.my_ip sock))
+	    (string_of_int config.port)
+	    (if not (ip_is_allowed from_ip) then Some Not_allowed else Some Blocked)
+	in
+	TcpBufferedSocket.write_string sock (Printf.sprintf "%s\n%s" s1 s2);
+        shutdown sock Closed_connect_failed;
         Unix.close s
       end
   | _ -> ()
