@@ -19,8 +19,13 @@
 
 (* This module uses 2 ints to save IPv4 numbers. *)
 
+open Gettext
 open Int64ops
 open Printf2
+
+let _s x = _s "Ip" x
+let _b x = _b "Ip" x
+
 type t = { hi: int; lo: int }
 
 let of_ints (a,b,c,d) = 
@@ -141,14 +146,25 @@ let banned = ref (fun (ip:t) -> None)
 
 let localhost = of_string "127.0.0.1"
 
+let mask_of_shift shift =
+  of_int64 (Int64.logand 0xffffffffL (Int64.shift_left 0xffffffffL (32 - shift)))
+
+let network_address ip mask =
+  of_int64 (Int64.logand (to_int64 ip) (to_int64 mask))
+
+let broadcast_address ip mask =
+  of_int64 (Int64.logor (to_int64 ip) (Int64.logand 0xffffffffL (Int64.lognot (to_int64 mask))))
+
 let to_sockaddr ip port =
   Unix.ADDR_INET (to_inet_addr ip, port)
 
 let get_non_local_ip list =
-  let list = List.filter ((<>) localhost) list in
   match list with
   | [] -> raise Not_found
-  | l -> l
+  | _ :: _ ->
+      match List.filter ((<>) localhost) list with
+      | (_ :: _) as l -> l
+      | [] -> list (* there's *only* local addresses, give up *)
 
 let gethostbyname name = 
   let h = Unix.gethostbyname name in
@@ -203,7 +219,7 @@ let get_entry_cached name =
       cache_entry
   with Not_found ->
     (* not in cache *)
-    lprintf_nl "[DNS] Resolving [%s] ..." name;
+    lprintf_nl (_b "[DNS] Resolving [%s] ...") name;
     let new_entry =
       make_entry_from_name ~time_limit:(current_time +. 3600.) name in
     Hashtbl.add ip_cache name new_entry;
@@ -269,6 +285,7 @@ type job = {
     mutable entries : Unix.inet_addr array;
     mutable error : bool;
     handler : (t -> unit);
+    err_handler : (int -> unit);
   }
 
 
@@ -278,13 +295,13 @@ external job_start : job -> unit = "ml_ip_job_start"
 let current_job = ref None
 let ip_fifo = Fifo.create ()
 
-let async_ip name f =
+let async_ip name f ferr =
   try
 (*    lprintf "async_ip [%s]\n" name; *)
     let ip = resolve_name_immediate name in
     (try f ip with _ -> ())
   with Not_found ->
-    Fifo.put ip_fifo (name, f)
+    Fifo.put ip_fifo (name, f, ferr)
 
 (* We check for names every 1/10 second. Too long ? *)
 let _ =
@@ -299,7 +316,7 @@ let _ =
     while true do
       match !current_job with
       | None ->
-          let (name, f) = Fifo.take ip_fifo in
+          let (name, f, ferr) = Fifo.take ip_fifo in
           (try
 	    let ip = resolve_name_immediate name in
             (try f ip with _ -> ())
@@ -309,6 +326,7 @@ let _ =
               BasicSocket.has_threads () then
                 let job = {
                   handler = f;
+                  err_handler = ferr;
                   name = name;
                   entries = [||];
                   error = false;
@@ -334,7 +352,8 @@ let _ =
 (*        lprintf "Ip found for %s: %s\n" job.name (to_string ip);  *)
               job.handler ip
             end else begin
-              lprintf_nl "[DNS] could not resolve %s, check URL" job.name;
+              lprintf_nl (_b "[DNS] could not resolve %s, check URL") job.name;
+              job.err_handler 0;
               raise Not_found
             end
           end else raise Exit
@@ -360,10 +379,10 @@ let ip_of_addr addr =
     AddrIp ip -> ip
   | AddrName name -> from_name name
 
-let async_ip_of_addr addr f =
+let async_ip_of_addr addr f ferr =
   match addr with
     AddrIp ip -> f ip
-  | AddrName name -> async_ip name f
+  | AddrName name -> async_ip name f ferr
 
 
 let value_to_addr v = addr_of_string (value_to_string v)
@@ -371,3 +390,35 @@ let value_to_addr v = addr_of_string (value_to_string v)
 let addr_to_value ip = string_to_value (string_of_addr ip)
 
 let addr_option = define_option_class "Addr" value_to_addr addr_to_value
+
+type ip_range = 
+| RangeSingleIp of t
+| RangeRange of t * t
+| RangeCIDR of t * int 
+
+let localhost_range = RangeCIDR ((of_string "127.0.0.1"), 8)
+
+let single_ip = "\\([0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+\\)"
+let single_ip_regexp = Str.regexp ("^" ^ single_ip ^ "$")
+let range_regexp = Str.regexp ("^" ^ single_ip ^ "-" ^ single_ip ^ "$")
+let cidr_regexp = Str.regexp ("^" ^ single_ip ^ "/" ^ "\\([0-9]+\\)$")
+
+let string_of_range ip =
+  match ip with
+  | RangeSingleIp ip -> to_string ip
+  | RangeRange (ip1, ip2) -> Printf.sprintf "%s-%s" (to_string ip1) (to_string ip2)
+  | RangeCIDR (ip, shift) -> Printf.sprintf "%s/%d" (to_string ip) shift
+
+let range_of_string s =
+  if Str.string_match single_ip_regexp s 0 then
+    RangeSingleIp (of_string (Str.matched_group 1 s))
+  else if Str.string_match range_regexp s 0 then
+    RangeRange (of_string (Str.matched_group 1 s), of_string (Str.matched_group 2 s))
+  else if Str.string_match cidr_regexp s 0 then
+    RangeCIDR (of_string (Str.matched_group 1 s), int_of_string (Str.matched_group 2 s))
+  else failwith (Printf.sprintf "Unknown IP range syntax: %s" s)
+
+let value_to_iprange v = range_of_string (value_to_string v)
+let iprange_to_value ip = string_to_value (string_of_range ip)
+
+let range_option = define_option_class "Range" value_to_iprange iprange_to_value
