@@ -30,7 +30,7 @@ let max_connections_per_second = ref (fun () -> 50)
 let opened_connections = ref 0
 let opened_connections_this_second = ref 0
 
-let max_buffer_size = ref 10000000
+let max_buffer_size = ref 50000
 
 let bind_address = ref Unix.inet_addr_any
 let ip_packet_size = ref 40
@@ -319,7 +319,7 @@ let accept_connection_bandwidth t =
   forecast_download t 0
 
 let best_packet_size nbytes =
-  let nbytes = maxi nbytes !minimal_packet_size in
+  let nbytes = max nbytes !minimal_packet_size in
   let nip_packets = 1 + nbytes / !mtu_packet_size in
   let headers = nip_packets * !ip_packet_size in
   let nframes = 1 + (nbytes + headers) / packet_frame_size in
@@ -411,7 +411,7 @@ let buf_add t b s pos1 len =
         lprintf "[TCP_BS]: BUFFER OVERFLOW %d+%d> %d " b.len len b.max_buf_size ;
 
         lprintf "MESSAGE: [";
-        for i = pos1 to pos1 + (mini len 20) - 1 do
+        for i = pos1 to pos1 + (min len 20) - 1 do
           lprintf "(%d)" (int_of_char s.[i]);
         done;
         if len > 20 then lprintf "...";
@@ -422,7 +422,7 @@ let buf_add t b s pos1 len =
   socket !!! *)
       end
     else
-    let new_len = mini (maxi (2 * max_len) (b.len + len)) b.max_buf_size  in
+    let new_len = min (max (2 * max_len) (b.len + len)) b.max_buf_size  in
 (*    if t.monitored then
       (lprintf "Allocate new for %d\n" len; ); *)
     let new_buf = String.create new_len in
@@ -464,7 +464,6 @@ let can_write_len t len =
       t.wbuf.max_buf_size t.wbuf.len len; *)
   b
 let not_buffer_more t max =  t.wbuf.len < max
-let can_fill t = t.wbuf.len < (t.wbuf.max_buf_size / 2)
 let get_rtimeout t = get_rtimeout t.sock_in
 let max_refill t = t.wbuf.max_buf_size - t.wbuf.len
 
@@ -631,8 +630,8 @@ let can_read_handler t sock max_len =
             buf_len - b.len
           )
         else
-        let new_len = mini
-            (maxi
+        let new_len = min
+            (max
               (2 * buf_len) (b.len + min_read_size)) b.max_buf_size
         in
         let new_buf = String.create new_len in
@@ -645,7 +644,7 @@ let can_read_handler t sock max_len =
     in
     b.buf, b.pos+b.len, can_write_in_buffer
   in
-  let can_read = mini max_len buffer_len in
+  let can_read = min max_len buffer_len in
   if can_read > 0 then
     let old_len = b.len in
     let nread = try
@@ -684,8 +683,8 @@ let can_read_handler t sock max_len =
       end else
       b.len <- b.len + nread;
 (*    lprintf " %d\n" nread; *)
-    b.min_buf_size <- mini b.max_buf_size (
-      maxi (nread + nread / 2) min_read_size);
+    b.min_buf_size <- min b.max_buf_size (
+      max (nread + nread / 2) min_read_size);
 
     (*
     if nread = can_read then begin
@@ -804,12 +803,9 @@ let tcp_handler_write t sock =
     | Some bc ->
         if bc.total_bytes = 0 then
           can_write_handler t sock t.wbuf.len
-        else  begin
-(*                lprintf "DELAYED\n";  *)
-            if bc.remaining_bytes > 0 then begin
-                bc.connections <- t :: bc.connections;
-                bc.nconnections <- t.write_power + bc.nconnections
-              end
+        else begin
+            bc.connections <- t :: bc.connections;
+            bc.nconnections <- t.write_power + bc.nconnections
           end
   end
 
@@ -1680,9 +1676,9 @@ to an option 'packet_unit' (default 250). Ask Simon if it makes sense.
 
               List.iter (fun t ->
                   if bc.remaining_bytes > 0 then
-                    let nconnections = maxi bc.nconnections 1 in
-                    let can_read = maxi 1 (bc.remaining_bytes / nconnections) in
-                    let can_read = maxi !ip_packet_size (can_read * t.read_power) in
+                    let nconnections = max bc.nconnections 1 in
+                    let can_read = max 1 (bc.remaining_bytes / nconnections) in
+                    let can_read = max !ip_packet_size (can_read * t.read_power) in
                     (try
 (*                    lprintf "allow to read %d\n" can_read; *)
                         can_read_handler t t.sock_in can_read
@@ -1731,14 +1727,14 @@ to an option 'packet_unit' (default 250). Ask Simon if it makes sense.
                 end;
               List.iter (fun t ->
                   if bc.remaining_bytes > 0 then
-                    let nconnections = maxi bc.nconnections 1 in
-                    let can_write = maxi 1 (bc.remaining_bytes / nconnections) in
+                    let nconnections = max bc.nconnections 1 in
+                    let can_write = max 1 (bc.remaining_bytes / nconnections) in
                     let can_write = best_packet_size (can_write * t.write_power)
                     in
                     let old_nwrite = t.nwrite in
                     (try
 (*                    lprintf "WRITE\n";  *)
-                        can_write_handler t t.sock_out (mini can_write t.wbuf.len)
+                        can_write_handler t t.sock_out (min can_write t.wbuf.len)
                       with _ -> ());
                     bc.remaining_bytes <- bc.remaining_bytes -
                     t.nwrite + old_nwrite;
@@ -1749,7 +1745,7 @@ to an option 'packet_unit' (default 250). Ask Simon if it makes sense.
                         t.name (sock_num t.sock_out) (remaining_to_write t)
                     end
               ) bc.connections;
-(*          if bc.remaining_bytes > 0 then bc.allow_io := false; *)
+(*            if bc.remaining_bytes > 0 then bc.allow_io := false; *)
             end;
           if !verbose_bandwidth > 2 then begin
 
@@ -1806,27 +1802,20 @@ let join_stats titles values =
 
 let load_stats filename =
   try
-    let ic = open_in filename in
-    let rec iter_first ic =
-      let titles = input_line ic in
-      iter_second ic titles
-    and iter_second ic titles =
-      let values = input_line ic in
-      join_stats titles values;
-      iter_first ic
-
-    in
-    try
-      iter_first ic
-    with
-      End_of_file -> close_in ic
-    | e ->
-        lprintf "[BWS] Error %s reading %s\n" (Printexc2.to_string e) filename;
-        close_in ic; proc_net_fs := false
-  with
-  | e ->
-      lprintf "[BWS] Error %s opening %s\n" (Printexc2.to_string e) filename;
-      proc_net_fs := false
+    Unix2.tryopen_read filename (fun ic ->
+      try
+	let rec iter_first ic =
+	  let titles = input_line ic in
+	  iter_second ic titles
+	and iter_second ic titles =
+	  let values = input_line ic in
+	  join_stats titles values;
+	  iter_first ic in
+	iter_first ic
+      with End_of_file -> ())
+  with e ->
+    lprintf "[BWS] Error %s opening %s\n" (Printexc2.to_string e) filename;
+    proc_net_fs := false
 
 let proc_net_timer _ =
   if !proc_net_fs && !verbose_bandwidth > 0 then begin
@@ -1846,6 +1835,7 @@ let _ =
       Printf.bprintf buf "  to_deflate: %d\n" (List.length !to_deflate);
       Printf.bprintf buf "  max_opened_connections: %d\n" (!max_opened_connections ());
       Printf.bprintf buf "  max_connections_per_second: %d\n" (!max_connections_per_second ());
+      Printf.bprintf buf "  max_buffer_size: %d\n" (!max_buffer_size);
   );
   add_infinite_timer 1.0 proc_net_timer
 
